@@ -5,7 +5,7 @@
 // Implementation of Drawable (Graphic objects)
 //
 
-#define MAGICK_IMPLEMENTATION
+#define MAGICK_IMPLEMENTATION  1
 #define MAGICK_DRAWABLE_IMPLEMENTATION
 
 #include "Magick++/Include.h"
@@ -357,7 +357,7 @@ Magick::DrawableClipPath::~DrawableClipPath( void )
 }
 void Magick::DrawableClipPath::operator()( MagickLib::DrawContext context_ ) const
 {
-	DrawSetClipPath( context_, _id.c_str());
+	(void) DrawSetClipPath( context_, _id.c_str());
 }
 Magick::DrawableBase* Magick::DrawableClipPath::copy() const
 {
@@ -536,8 +536,13 @@ std::string Magick::DrawableCompositeImage::magick( void )
 void Magick::DrawableCompositeImage::operator()
   ( MagickLib::DrawContext context_ ) const
 {
-  DrawComposite( context_, _composition, _x, _y, _width, _height,
-                 _image->constImage() );
+  MagickWand
+    *magick_wand;
+
+  magick_wand=NewMagickWandFromImage(_image->constImage());
+  (void) DrawComposite( context_, _composition, _x, _y, _width, _height,
+     magick_wand );
+  magick_wand=DestroyMagickWand(magick_wand);
 }
 
 Magick::DrawableBase* Magick::DrawableCompositeImage::copy() const
@@ -581,7 +586,7 @@ void Magick::DrawableFillColor::operator()
   PixelWand *pixel_wand=NewPixelWand();
   PixelSetQuantumColor(pixel_wand,&color);
   DrawSetFillColor(context_,pixel_wand);
-  DestroyPixelWand(pixel_wand);
+  pixel_wand=DestroyPixelWand(pixel_wand);
 }
 Magick::DrawableBase* Magick::DrawableFillColor::copy() const
 {
@@ -609,7 +614,7 @@ Magick::DrawableFillOpacity::~DrawableFillOpacity ( void )
 void Magick::DrawableFillOpacity::operator()
   ( MagickLib::DrawContext context_ ) const
 {
-  DrawSetFillOpacity( context_, _opacity );
+  DrawSetFillAlpha( context_, _opacity );
 }
 Magick::DrawableBase* Magick::DrawableFillOpacity::copy() const
 {
@@ -653,13 +658,13 @@ void Magick::DrawableFont::operator()( MagickLib::DrawContext context_ ) const
   // font
   if(_font.length())
     {
-      DrawSetFont( context_, _font.c_str() );
+      (void) DrawSetFont( context_, _font.c_str() );
     }
 
   if(_family.length())
     {
       // font-family
-      DrawSetFontFamily( context_, _family.c_str() );
+      (void) DrawSetFontFamily( context_, _family.c_str() );
 
       // font-style
       DrawSetFontStyle( context_, _style );
@@ -885,7 +890,7 @@ Magick::DrawablePopPattern::~DrawablePopPattern ( void )
 void Magick::DrawablePopPattern::operator()
   ( MagickLib::DrawContext context_ ) const
 {
-  DrawPopPattern( context_ );
+  (void) DrawPopPattern( context_ );
 }
 Magick::DrawableBase* Magick::DrawablePopPattern::copy() const
 {
@@ -919,7 +924,7 @@ Magick::DrawablePushPattern::~DrawablePushPattern ( void )
 void Magick::DrawablePushPattern::operator()
   ( MagickLib::DrawContext context_ ) const
 {
-  DrawPushPattern( context_, _id.c_str(), _x, _y, _width, _height );
+  (void) DrawPushPattern( context_, _id.c_str(), _x, _y, _width, _height );
 }
 Magick::DrawableBase* Magick::DrawablePushPattern::copy() const
 {
@@ -1052,7 +1057,7 @@ Magick::DrawableDashArray& Magick::DrawableDashArray::operator=
 void Magick::DrawableDashArray::operator()
   ( MagickLib::DrawContext context_ ) const
 {
-  DrawSetStrokeDashArray( context_, _size, _dasharray );
+  (void) DrawSetStrokeDashArray( context_, _size, _dasharray );
 }
 Magick::DrawableBase* Magick::DrawableDashArray::copy() const
 {
@@ -1208,7 +1213,7 @@ void Magick::DrawableStrokeColor::operator()
   PixelWand *pixel_wand=NewPixelWand();
   PixelSetQuantumColor(pixel_wand,&color);
   DrawSetStrokeColor(context_,pixel_wand);
-  DestroyPixelWand(pixel_wand);
+  pixel_wand=DestroyPixelWand(pixel_wand);
 }
 Magick::DrawableBase* Magick::DrawableStrokeColor::copy() const
 {
@@ -1222,7 +1227,7 @@ Magick::DrawableStrokeOpacity::~DrawableStrokeOpacity ( void )
 void Magick::DrawableStrokeOpacity::operator()
   ( MagickLib::DrawContext context_ ) const
 {
-  DrawSetStrokeOpacity( context_, _opacity );
+  DrawSetStrokeAlpha( context_, _opacity );
 }
 Magick::DrawableBase* Magick::DrawableStrokeOpacity::copy() const
 {
@@ -1354,7 +1359,7 @@ void Magick::DrawableTextUnderColor::operator()
   PixelWand *pixel_wand=NewPixelWand();
   PixelSetQuantumColor(pixel_wand,&color);
   DrawSetTextUnderColor(context_,pixel_wand);
-  DestroyPixelWand(pixel_wand);
+  pixel_wand=DestroyPixelWand(pixel_wand);
 }
 Magick::DrawableBase* Magick::DrawableTextUnderColor::copy() const
 {
