@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -41,13 +41,16 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
 #include "magick/color.h"
-#include "magick/color_private.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/draw.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/log.h"
 #include "magick/magick.h"
@@ -164,9 +167,10 @@ static MagickBooleanType DecodeImage(Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (unsigned char *) NULL);
-  (void) ResetMagickMemory(pixels,0,(size_t) image->columns*image->rows);
+  (void) ResetMagickMemory(pixels,0,(size_t) image->columns*image->rows*
+    sizeof(*pixels));
   byte=0;
   x=0;
   p=pixels;
@@ -180,14 +184,14 @@ static MagickBooleanType DecodeImage(Image *image,
       break;
     if (count != 0)
       {
-        count=Min(count,q-p);
+        count=Min((unsigned long) count,(size_t) (q-p));
         /*
           Encoded mode.
         */
         byte=(unsigned char) ReadBlobByte(image);
         if (compression == BI_RLE8)
           {
-            for (i=count; i != 0; i--)
+            for (i=0; i < count; i++)
               *p++=(unsigned char) byte;
           }
         else
@@ -233,9 +237,9 @@ static MagickBooleanType DecodeImage(Image *image,
             /*
               Absolute mode.
             */
-            count=Min(count,q-p);
+            count=Min((unsigned long) count,(size_t) (q-p));
             if (compression == BI_RLE8)
-              for (i=(long) count; i != 0; --i)
+              for (i=0; i < count; i++)
                 *p++=(unsigned char) ReadBlobByte(image);
             else
               for (i=0; i < count; i++)
@@ -261,9 +265,11 @@ static MagickBooleanType DecodeImage(Image *image,
           }
         }
       }
-    if (QuantumTick(y,image->rows) != MagickFalse)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
       {
-        status=MagickMonitor(LoadImageTag,y,image->rows,&image->exception);
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -312,6 +318,9 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register const unsigned char
     *p;
 
@@ -328,7 +337,7 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (const unsigned char *) NULL);
   assert(compressed_pixels != (unsigned char *) NULL);
   p=pixels;
@@ -353,9 +362,14 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
     */
     *q++=0x00;
     *q++=0x00;
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   /*
     End of bitmap.
@@ -464,6 +478,9 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   size_t
     length;
 
+  ssize_t
+    count;
+
   unsigned char
     *pixels;
 
@@ -476,14 +493,15 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -557,7 +575,9 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (dib_colormap == (unsigned char *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       packet_size=4;
-      (void) ReadBlob(image,packet_size*image->colors,dib_colormap);
+      count=ReadBlob(image,packet_size*image->colors,dib_colormap);
+      if (count != (ssize_t) (packet_size*image->colors))
+        ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
       p=dib_colormap;
       for (i=0; i < (long) image->colors; i++)
       {
@@ -576,12 +596,16 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     dib_info.bits_per_pixel<<=1;
   bytes_per_line=4*((image->columns*dib_info.bits_per_pixel+31)/32);
   length=bytes_per_line*image->rows;
-  pixels=(unsigned char *)
-    AcquireMagickMemory(Max(bytes_per_line,image->columns+1)*image->rows);
+  pixels=(unsigned char *) AcquireMagickMemory((size_t)
+    Max(bytes_per_line,image->columns+256)*image->rows*sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if ((dib_info.compression == 0) || (dib_info.compression == 3))
-    (void) ReadBlob(image,length,pixels);
+    {
+      count=ReadBlob(image,length,pixels);
+      if (count != (ssize_t) (length))
+        ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+    }
   else
     {
       /*
@@ -638,10 +662,11 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
-                exception);
+              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                image->rows,image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -680,10 +705,11 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
-                exception);
+              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                image->rows,image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -715,10 +741,11 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
-                exception);
+              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                image->rows,image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -763,10 +790,11 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
-                exception);
+              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                image->rows,image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -797,10 +825,11 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
-                exception);
+              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                image->rows,image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -825,10 +854,10 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       flipped_image=FlipImage(image,exception);
       if (flipped_image == (Image *) NULL)
         {
-          DestroyImageList(image);
+          image=DestroyImageList(image);
           return((Image *) NULL);
         }
-      DestroyImage(image);
+      image=DestroyImage(image);
       image=flipped_image;
     }
   CloseBlob(image);
@@ -964,7 +993,7 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -1064,12 +1093,14 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
        for (x=(long) (image->columns+7)/8; x < (long) bytes_per_line; x++)
          *q++=0x00;
        if (image->previous == (Image *) NULL)
-         if (QuantumTick(y,image->rows) != MagickFalse)
-           {
-             status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
-             if (status == MagickFalse)
-               break;
-           }
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
+            {
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
+              if (status == MagickFalse)
+                break;
+            }
       }
       break;
     }
@@ -1090,10 +1121,11 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
         for ( ; x < (long) bytes_per_line; x++)
           *q++=0x00;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -1125,10 +1157,11 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
           for (x=3*image->columns; x < (long) bytes_per_line; x++)
             *q++=0x00;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -1184,7 +1217,7 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
       if (dib_colormap == (unsigned char *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       q=dib_colormap;
-      for (i=0; i < (long) image->colors; i++)
+      for (i=0; i < (long) Min(image->colors,dib_info.number_colors); i++)
       {
         *q++=ScaleQuantumToChar(image->colormap[i].blue);
         *q++=ScaleQuantumToChar(image->colormap[i].green);
