@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -42,11 +42,14 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
 #include "magick/color.h"
-#include "magick/color_private.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -152,7 +155,7 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   packet=(unsigned char *) AcquireMagickMemory(256);
   prefix=(short *) AcquireMagickMemory(MaxStackSize*sizeof(*prefix));
   suffix=(unsigned char *) AcquireMagickMemory(MaxStackSize);
@@ -161,8 +164,18 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
       (prefix == (short *) NULL) ||
       (suffix == (unsigned char *) NULL) ||
       (pixel_stack == (unsigned char *) NULL))
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
+    {
+      if (packet != (unsigned char *) NULL)
+        packet=(unsigned char *) RelinquishMagickMemory(packet);
+      if (prefix != (short *) NULL)
+        prefix=(short *) RelinquishMagickMemory(prefix);
+      if (suffix != (unsigned char *) NULL)
+        suffix=(unsigned char *) RelinquishMagickMemory(suffix);
+      if (pixel_stack != (unsigned char *) NULL)
+        pixel_stack=(unsigned char *) RelinquishMagickMemory(pixel_stack);
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
+    }
   /*
     Initialize GIF data stream decoder.
   */
@@ -259,6 +272,8 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
             }
           while (code >= clear)
           {
+            if ((top_stack-pixel_stack) >= MaxStackSize)
+              break;
             *top_stack++=suffix[code];
             code=(unsigned long) prefix[code];
           }
@@ -266,6 +281,8 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
           /*
             Add a new string to the string table,
           */
+          if ((top_stack-pixel_stack) >= MaxStackSize)
+            break;
           if (available >= MaxStackSize)
             break;
           *top_stack++=(unsigned char) first;
@@ -338,9 +355,11 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
     if (x < (long) image->columns)
       break;
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(LoadImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(LoadImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -480,7 +499,17 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   if ((packet == (unsigned char *) NULL) || (hash_code == (short *) NULL) ||
       (hash_prefix == (short *) NULL) ||
       (hash_suffix == (unsigned char *) NULL))
-    return(MagickFalse);
+    {
+      if (packet != (unsigned char *) NULL)
+        packet=(unsigned char *) RelinquishMagickMemory(packet);
+      if (hash_code != (short *) NULL)
+        hash_code=(short *) RelinquishMagickMemory(hash_code);
+      if (hash_prefix != (short *) NULL)
+        hash_prefix=(short *) RelinquishMagickMemory(hash_prefix);
+      if (hash_suffix != (unsigned char *) NULL)
+        hash_suffix=(unsigned char *) RelinquishMagickMemory(hash_suffix);
+      return(MagickFalse);
+    }
   /*
     Initialize GIF encoder.
   */
@@ -616,9 +645,11 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
         }
       }
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -735,7 +766,7 @@ static ssize_t ReadBlobBlock(Image *image,unsigned char *data)
   assert(image->signature == MagickSignature);
   assert(data != (unsigned char *) NULL);
   count=ReadBlob(image,1,&block_count);
-  if (count == 0)
+  if (count != 1)
     return(0);
   return(ReadBlob(image,(size_t) block_count,data));
 }
@@ -814,25 +845,24 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
     Determine if this is a GIF file.
   */
   count=ReadBlob(image,6,magick);
-  if ((count == 0) || ((LocaleNCompare((char *) magick,"GIF87",5) != 0) &&
+  if ((count != 6) || ((LocaleNCompare((char *) magick,"GIF87",5) != 0) &&
       (LocaleNCompare((char *) magick,"GIF89",5) != 0)))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  global_colors=0;
-  global_colormap=(unsigned char *) NULL;
   page.width=ReadBlobLSBShort(image);
   page.height=ReadBlobLSBShort(image);
   flag=(unsigned char) ReadBlobByte(image);
@@ -844,7 +874,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (global_colormap == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (BitSet((int) flag,0x80) != 0)
-    (void) ReadBlob(image,(size_t) (3*global_colors),global_colormap);
+    count=ReadBlob(image,(size_t) (3*global_colors),global_colormap);
   delay=0;
   dispose=0;
   iterations=1;
@@ -853,7 +883,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   for ( ; ; )
   {
     count=ReadBlob(image,1,&c);
-    if (count == 0)
+    if (count != 1)
       break;
     if (c == (unsigned char) ';')
       break;  /* terminator */
@@ -863,7 +893,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           GIF Extension block.
         */
         count=ReadBlob(image,1,&c);
-        if (count == 0)
+        if (count != 1)
           ThrowReaderException(CorruptImageError,"UnableToReadExtensionBlock");
         switch (c)
         {
@@ -896,7 +926,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
               header[count]='\0';
               (void) ConcatenateString(&comments,(const char *) header);
             }
-            (void) SetImageAttribute(image,"comment",comments);
+            (void) SetImageAttribute(image,"Comment",comments);
             comments=(char *) RelinquishMagickMemory(comments);
             break;
           }
@@ -932,18 +962,21 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             global_colormap=(unsigned char *)
               RelinquishMagickMemory(global_colormap);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
     image_count++;
     /*
@@ -968,6 +1001,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->page.y=page.y;
     image->page.x=page.x;
     image->delay=delay;
+    image->ticks_per_second=100;
     image->dispose=(DisposeType) dispose;
     image->iterations=iterations;
     image->matte=(MagickBooleanType) (opacity >= 0);
@@ -1008,7 +1042,9 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           AcquireMagickMemory((size_t) (3*image->colors));
         if (colormap == (unsigned char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        (void) ReadBlob(image,(size_t) (3*image->colors),colormap);
+        count=ReadBlob(image,(size_t) (3*image->colors),colormap);
+        if (count != (ssize_t) (3*image->colors))
+          ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
         p=colormap;
         for (i=0; i < (long) image->colors; i++)
         {
@@ -1194,7 +1230,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
     *global_colormap;
 
   unsigned long
-    bits_per_pixel;
+    bits_per_pixel,
+    delay;
 
   /*
     Open output image file.
@@ -1204,7 +1241,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -1223,7 +1260,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
       page.width=next_image->columns+page.x;
     if ((next_image->rows+page.y) > page.height)
       page.height=next_image->rows+page.y;
-    next_image=next_image->next;
+    next_image=GetNextImageInList(next_image);
   }
   /*
     Allocate colormap.
@@ -1256,7 +1293,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
     Write images to file.
   */
   interlace=write_info->interlace;
-  if ((write_info->adjoin != MagickFalse) && (image->next != (Image *) NULL))
+  if ((write_info->adjoin != MagickFalse) &&
+      (GetNextImageInList(image) != (Image *) NULL))
     interlace=NoInterlace;
   scene=0;
   do
@@ -1269,31 +1307,36 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
           GIF requires that the image is colormapped.
         */
         GetQuantizeInfo(&quantize_info);
-        quantize_info.dither=
-          IsPaletteImage(image,&image->exception) == MagickFalse;
-        if ((LocaleCompare(write_info->magick,"GIF87") == 0) ||
-            (image->matte == MagickFalse))
-          (void) QuantizeImage(&quantize_info,image);
-        else
+        quantize_info.number_colors=256;
+        quantize_info.dither=IsPaletteImage(image,&image->exception) ==
+          MagickFalse ? MagickTrue : MagickFalse;
+        (void) QuantizeImage(&quantize_info,image);
+        if ((LocaleCompare(write_info->magick,"GIF87") != 0) &&
+            (image->matte != MagickFalse))
           {
+            MagickSizeType
+              length;
+
             /*
               Set transparent pixel.
             */
             quantize_info.number_colors=255;
             (void) QuantizeImage(&quantize_info,image);
             opacity=(long) image->colors++;
-            image->colormap=(PixelPacket *) ResizeMagickMemory(image->colormap,
-              (size_t) image->colors*sizeof(*image->colormap));
+            length=(size_t) image->colors*sizeof(*image->colormap);
+            image->colormap=(PixelPacket *)
+              ResizeMagickMemory(image->colormap,length);
             if (image->colormap == (PixelPacket *) NULL)
               {
                 global_colormap=(unsigned char *)
                   RelinquishMagickMemory(global_colormap);
                 colormap=(unsigned char *) RelinquishMagickMemory(colormap);
-                DestroyImageInfo(write_info);
+                write_info=DestroyImageInfo(write_info);
                 ThrowWriterException(ResourceLimitError,
                   "MemoryAllocationFailed");
               }
-            image->colormap[opacity]=image->background_color;
+            (void) QueryColorDatabase("#cccccc",image->colormap+opacity,
+              &image->exception);
             for (y=0; y < (long) image->rows; y++)
             {
               p=AcquireImagePixels(image,0,y,image->columns,1,
@@ -1303,7 +1346,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
               indexes=GetIndexes(image);
               for (x=0; x < (long) image->columns; x++)
               {
-                if ((double) p->opacity >= (MaxRGB-image->fuzz))
+                if ((double) p->opacity >= (QuantumRange-image->fuzz))
                   indexes[x]=(IndexPacket) opacity;
                 p++;
               }
@@ -1327,7 +1370,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
             indexes=GetIndexes(image);
             for (x=0; x < (long) image->columns; x++)
             {
-              if ((double) p->opacity >= (MaxRGB-image->fuzz))
+              if ((double) p->opacity >= (QuantumRange-image->fuzz))
                 {
                   opacity=(long) indexes[x];
                   break;
@@ -1356,7 +1399,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
       *q++=(unsigned char) 0x0;
       *q++=(unsigned char) 0x0;
     }
-    if ((image->previous == (Image *) NULL) ||
+    if ((GetPreviousImageInList(image) == (Image *) NULL) ||
         (write_info->adjoin == MagickFalse))
       {
         /*
@@ -1366,10 +1409,11 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
         c|=(8-1) << 4;  /* color resolution */
         c|=(bits_per_pixel-1);   /* size of global colormap */
         (void) WriteBlobByte(image,(unsigned char) c);
-        for (j=0; j < (long) Max(image->colors-1,1); j++)
+        for (j=0; j < (long) image->colors; j++)
           if (ColorMatch(&image->background_color,image->colormap+j))
             break;
-        (void) WriteBlobByte(image,(unsigned char) j);  /* background color */
+        (void) WriteBlobByte(image,(unsigned char)
+          j == (long) image->colors ? 0 : j);  /* background color */
         (void) WriteBlobByte(image,(unsigned char) 0x00);  /* reserved */
         length=(size_t) (3*(1 << bits_per_pixel));
         (void) WriteBlob(image,length,colormap);
@@ -1388,12 +1432,13 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
         if (opacity >= 0)
           c|=0x01;
         (void) WriteBlobByte(image,(unsigned char) c);
-        (void) WriteBlobLSBShort(image,(unsigned short) image->delay);
+        delay=100*image->delay/Max(image->ticks_per_second,1);
+        (void) WriteBlobLSBShort(image,(unsigned short) delay);
         (void) WriteBlobByte(image,(unsigned char)
           (opacity >= 0 ? opacity : 0));
         (void) WriteBlobByte(image,(unsigned char) 0x00);
         if ((LocaleCompare(write_info->magick,"GIF87") != 0) &&
-            (GetImageAttribute(image,"comment") != (ImageAttribute *) NULL))
+            (GetImageAttribute(image,"Comment") != (ImageAttribute *) NULL))
           {
             const ImageAttribute
               *attribute;
@@ -1409,7 +1454,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
             */
             (void) WriteBlobByte(image,(unsigned char) 0x21);
             (void) WriteBlobByte(image,(unsigned char) 0xfe);
-            attribute=GetImageAttribute(image,"comment");
+            attribute=GetImageAttribute(image,"Comment");
             p=attribute->value;
             while (strlen(p) != 0)
             {
@@ -1420,8 +1465,9 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
             }
             (void) WriteBlobByte(image,(unsigned char) 0x00);
           }
-        if ((image->previous == (Image *) NULL) &&
-            (image->next != (Image *) NULL) && (image->iterations != 1))
+        if ((GetPreviousImageInList(image) == (Image *) NULL) &&
+            (GetNextImageInList(image) != (Image *) NULL) &&
+            (image->iterations != 1))
           {
             /*
               Write Netscape Loop extension.
@@ -1475,22 +1521,26 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
         global_colormap=(unsigned char *)
           RelinquishMagickMemory(global_colormap);
         colormap=(unsigned char *) RelinquishMagickMemory(colormap);
-        DestroyImageInfo(write_info);
+        write_info=DestroyImageInfo(write_info);
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       }
     (void) WriteBlobByte(image,(unsigned char) 0x00);
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (write_info->adjoin != MagickFalse);
   (void) WriteBlobByte(image,';'); /* terminator */
   global_colormap=(unsigned char *) RelinquishMagickMemory(global_colormap);
   colormap=(unsigned char *) RelinquishMagickMemory(colormap);
-  DestroyImageInfo(write_info);
+  write_info=DestroyImageInfo(write_info);
   CloseBlob(image);
   return(MagickTrue);
 }
