@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -41,16 +41,20 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
-#include "magick/nt_feature.h"
+#include "magick/nt-feature.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #if defined(HasJBIG)
@@ -105,13 +109,13 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   IndexPacket
     index;
 
-  int
-    status;
-
   long
     length,
     y;
 
+  MagickBooleanType
+    status;
+
   register IndexPacket
     *indexes;
 
@@ -141,14 +145,15 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -178,7 +183,7 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
     count=0;
     while ((length > 0) && ((status == JBG_EAGAIN) || (status == JBG_EOK)))
     {
-      unsigned int
+      size_t
         count;
 
       status=jbg_dec_in(&jbig_info,p,length,&count);
@@ -199,9 +204,9 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   image->colormap[0].red=0;
   image->colormap[0].green=0;
   image->colormap[0].blue=0;
-  image->colormap[1].red=MaxRGB;
-  image->colormap[1].green=MaxRGB;
-  image->colormap[1].blue=MaxRGB;
+  image->colormap[1].red=QuantumRange;
+  image->colormap[1].green=QuantumRange;
+  image->colormap[1].blue=QuantumRange;
   image->x_resolution=300;
   image->y_resolution=300;
   if (image_info->ping != MagickFalse)
@@ -235,9 +240,14 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   /*
     Free scale resource.
@@ -381,8 +391,12 @@ static void JBIGEncode(unsigned char *pixels,size_t length,void *data)
   (void) WriteBlob(image,length,pixels);
 }
 
-static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
+  Image *image)
 {
+  double
+    version;
+
   long
     y;
 
@@ -424,10 +438,11 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,Image *image
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
+  version=strtod(JBG_VERSION,(char **) NULL);
   scene=0;
   do
   {
@@ -443,7 +458,7 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,Image *image
       Convert pixels to a bitmap.
     */
     SetImageType(image,BilevelType);
-    polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
+    polarity=PixelIntensityToQuantum(&image->colormap[0]) < (QuantumRange/2);
     if (image->colors == 2)
       polarity=PixelIntensityToQuantum(&image->colormap[0]) >
         PixelIntensityToQuantum(&image->colormap[1]);
@@ -471,9 +486,11 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,Image *image
        }
       if (bit != 0)
         *q++=byte << (8-bit);
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -520,20 +537,24 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,Image *image
       }
     (void) jbg_enc_lrange(&jbig_info,-1,-1);
     jbg_enc_options(&jbig_info,JBG_ILEAVE | JBG_SMID,JBG_TPDON | JBG_TPBON |
-      JBG_DPON,-1,-1,-1);
+      JBG_DPON,version < 1.6 ? -1 : 0,-1,-1);
     /*
       Write JBIG image.
     */
     jbg_enc_out(&jbig_info);
     jbg_enc_free(&jbig_info);
     pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
   return(MagickTrue);
