@@ -17,13 +17,13 @@
 %                                 June 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -40,19 +40,23 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
 #include "magick/colorspace.h"
 #include "magick/color.h"
-#include "magick/color_private.h"
-#include "magick/error.h"
+#include "magick/color-private.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/option.h"
+#include "magick/profile.h"
 #include "magick/static.h"
+#include "magick/statistic.h"
 #include "magick/string_.h"
 #if defined(HasJP2)
 #if !defined(uchar)
@@ -129,8 +133,8 @@ static MagickBooleanType IsJP2(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsJPC()() returns MagickTrue if the image format type, identified by the magick
-%  string, is JPC.
+%  IsJPC()() returns MagickTrue if the image format type, identified by the
+%  magick string, is JPC.
 %
 %  The format of the IsJPC method is:
 %
@@ -243,12 +247,12 @@ static jas_stream_t *JP2StreamManager(Image *image)
   StreamManager
     *source;
 
-  stream=(jas_stream_t *) AcquireMagickMemory(sizeof(*stream));
+  stream=(jas_stream_t *) malloc(sizeof(*stream));
   if (stream == (jas_stream_t *) NULL)
     return((jas_stream_t *) NULL);
   (void) ResetMagickMemory(stream,0,sizeof(*stream));
   stream->rwlimit_=(-1);
-  stream->obj_=(jas_stream_obj_t *) AcquireMagickMemory(sizeof(StreamManager));
+  stream->obj_=(jas_stream_obj_t *) malloc(sizeof(StreamManager));
   if (stream->obj_ == (jas_stream_obj_t *) NULL)
     return((jas_stream_t *) NULL);
   (void) ResetMagickMemory(stream->obj_,0,sizeof(StreamManager));
@@ -272,6 +276,12 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  jas_cmprof_t
+    *cm_profile;
+
+  jas_iccprof_t
+    *icc_profile;
+
   jas_image_t
     *jp2_image;
 
@@ -295,8 +305,10 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
     components[4];
 
   unsigned int
-    channel_scale[4],
-    maximum_component_depth;
+    maximum_component_depth,
+    scale[4],
+    x_step[4],
+    y_step[4];
 
   unsigned long
     number_components;
@@ -307,20 +319,20 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
     Initialize JPEG 2000 API.
   */
-  jas_init();
   jp2_stream=JP2StreamManager(image);
   if (jp2_stream == (jas_stream_t *) NULL)
     ThrowReaderException(DelegateError,"UnableToManageJP2Stream");
@@ -334,12 +346,9 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   {
     case JAS_CLRSPC_FAM_RGB:
     {
-      components[0]=jas_image_getcmptbytype(jp2_image,
-        JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_R));
-      components[1]=jas_image_getcmptbytype(jp2_image,
-        JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_G));
-      components[2]=jas_image_getcmptbytype(jp2_image,
-        JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_B));
+      components[0]=jas_image_getcmptbytype(jp2_image,JAS_IMAGE_CT_RGB_R);
+      components[1]=jas_image_getcmptbytype(jp2_image,JAS_IMAGE_CT_RGB_G);
+      components[2]=jas_image_getcmptbytype(jp2_image,JAS_IMAGE_CT_RGB_B);
       if ((components[0] < 0) || (components[1] < 0) || (components[2] < 0))
         {
           (void) jas_stream_close(jp2_stream);
@@ -347,8 +356,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
           ThrowReaderException(CorruptImageError,"MissingImageChannel");
         }
       number_components=3;
-      components[3]=jas_image_getcmptbytype(jp2_image,
-        JAS_IMAGE_CT_COLOR(JAS_IMAGE_CT_OPACITY));
+      components[3]=jas_image_getcmptbytype(jp2_image,3);
       if (components[3] > 0)
         {
           image->matte=MagickTrue;
@@ -358,8 +366,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     case JAS_CLRSPC_FAM_GRAY:
     {
-      components[0]=jas_image_getcmptbytype(jp2_image,
-        JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_GRAY_Y));
+      components[0]=jas_image_getcmptbytype(jp2_image,JAS_IMAGE_CT_GRAY_Y);
       if (components[0] < 0)
         {
           (void) jas_stream_close(jp2_stream);
@@ -370,6 +377,26 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     }
     case JAS_CLRSPC_FAM_YCBCR:
+    {
+      components[0]=jas_image_getcmptbytype(jp2_image,JAS_IMAGE_CT_YCBCR_Y);
+      components[1]=jas_image_getcmptbytype(jp2_image,JAS_IMAGE_CT_YCBCR_CB);
+      components[2]=jas_image_getcmptbytype(jp2_image,JAS_IMAGE_CT_YCBCR_CR);
+      if ((components[0] < 0) || (components[1] < 0) || (components[2] < 0))
+        {
+          (void) jas_stream_close(jp2_stream);
+          jas_image_destroy(jp2_image);
+          ThrowReaderException(CorruptImageError,"MissingImageChannel");
+        }
+      number_components=3;
+      components[3]=jas_image_getcmptbytype(jp2_image,JAS_IMAGE_CT_UNKNOWN);
+      if (components[3] > 0)
+        {
+          image->matte=MagickTrue;
+          number_components++;
+        }
+      image->colorspace=YCbCrColorspace;
+      break;
+    }
     default:
     {
       (void) jas_stream_close(jp2_stream);
@@ -381,18 +408,20 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   image->rows=jas_image_height(jp2_image);
   for (i=0; i < (long) number_components; i++)
   {
-    if ((jas_image_cmptwidth(jp2_image,components[i]) != image->columns) ||
-        (jas_image_cmptheight(jp2_image,components[i]) != image->rows) ||
+    if ((((jas_image_cmptwidth(jp2_image,components[i])*
+          jas_image_cmpthstep(jp2_image,components[i])) != image->columns)) ||
+        (((jas_image_cmptheight(jp2_image,components[i])*
+          jas_image_cmptvstep(jp2_image,components[i])) != image->rows)) ||
         (jas_image_cmpttlx(jp2_image,components[i]) != 0) ||
         (jas_image_cmpttly(jp2_image,components[i]) != 0) ||
-        (jas_image_cmpthstep(jp2_image,components[i]) != 1) ||
-        (jas_image_cmptvstep(jp2_image,components[i]) != 1) ||
         (jas_image_cmptsgnd(jp2_image,components[i]) != MagickFalse))
       {
         (void) jas_stream_close(jp2_stream);
         jas_image_destroy(jp2_image);
         ThrowReaderException(CoderError,"IrregularChannelGeometryNotSupported");
       }
+    x_step[i]=jas_image_cmpthstep(jp2_image,components[i]);
+    y_step[i]=jas_image_cmptvstep(jp2_image,components[i]);
   }
   /*
     Convert JPEG 2000 pixels.
@@ -409,7 +438,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   {
     maximum_component_depth=Max(jas_image_cmptprec(jp2_image,components[i]),
       maximum_component_depth);
-    pixels[i]=jas_matrix_create(1,(unsigned int) image->columns);
+    pixels[i]=jas_matrix_create(1,((unsigned int) image->columns)/x_step[i]);
     if (pixels[i] == (jas_matrix_t *) NULL)
       {
         jas_image_destroy(jp2_image);
@@ -422,10 +451,9 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
     image->depth=Min(QuantumDepth,16);
   for (i=0; i < (long) number_components; i++)
   {
-    channel_scale[i]=1;
+    scale[i]=1;
     if (jas_image_cmptprec(jp2_image,components[i]) < 16)
-      channel_scale[i]=
-        (1 << (16-jas_image_cmptprec(jp2_image,components[i])))+1;
+      scale[i]=(1 << (16-jas_image_cmptprec(jp2_image,components[i])))+1;
   }
   if (number_components == 1)
     {
@@ -441,7 +469,8 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     for (i=0; i < (long) number_components; i++)
       (void) jas_image_readcmpt(jp2_image,(short) components[i],0,
-         (unsigned int) y,(unsigned int) image->columns,1,pixels[i]);
+        ((unsigned int) y)/y_step[i],((unsigned int) image->columns)/x_step[i],
+        1,pixels[i]);
     switch (number_components)
     {
       case 1:
@@ -460,7 +489,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
           for (x=0; x < (long) image->columns; x++)
           {
             index=ScaleQuantumToChar(ScaleShortToQuantum((unsigned short)
-              jas_matrix_getv(pixels[0],x)*channel_scale[0]));
+              (scale[0]*jas_matrix_getv(pixels[0],x))));
             index=ConstrainColormapIndex(image,index);
             *indexes++=index;
             *q++=image->colormap[index];
@@ -468,8 +497,8 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
         else
           for (x=0; x < (long) image->columns; x++)
           {
-            index=(unsigned short) jas_matrix_getv(pixels[0],x)*
-              channel_scale[0];
+            index=(unsigned short) (scale[0]*
+              jas_matrix_getv(pixels[0],x/x_step[0]));
             index=ConstrainColormapIndex(image,index);
             *indexes++=index;
             *q++=image->colormap[index];
@@ -483,12 +512,12 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         for (x=0; x < (long) image->columns; x++)
         {
-          q->red=ScaleShortToQuantum((unsigned short)
-            jas_matrix_getv(pixels[0],x)*channel_scale[0]);
-          q->green=ScaleShortToQuantum((unsigned short)
-            jas_matrix_getv(pixels[1],x)*channel_scale[1]);
-          q->blue=ScaleShortToQuantum((unsigned short)
-            jas_matrix_getv(pixels[2],x)*channel_scale[2]);
+          q->red=ScaleShortToQuantum((unsigned short) (scale[0]*
+            jas_matrix_getv(pixels[0],x/x_step[0])));
+          q->green=ScaleShortToQuantum((unsigned short) (scale[1]*
+            jas_matrix_getv(pixels[1],x/x_step[1])));
+          q->blue=ScaleShortToQuantum((unsigned short) (scale[2]*
+            jas_matrix_getv(pixels[2],x/x_step[2])));
           q++;
         }
         break;
@@ -500,14 +529,14 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         for (x=0; x < (long) image->columns; x++)
         {
-          q->red=ScaleShortToQuantum((unsigned short)
-            jas_matrix_getv(pixels[0],x)*channel_scale[0]);
-          q->green=ScaleShortToQuantum((unsigned short)
-            jas_matrix_getv(pixels[1],x)*channel_scale[1]);
-          q->blue=ScaleShortToQuantum((unsigned short)
-            jas_matrix_getv(pixels[2],x)*channel_scale[2]);
-          q->opacity=MaxRGB-ScaleShortToQuantum((unsigned short)
-            jas_matrix_getv(pixels[3],x)*channel_scale[3]);
+          q->red=ScaleShortToQuantum((unsigned short) (scale[0]*
+            jas_matrix_getv(pixels[0],x/x_step[0])));
+          q->green=ScaleShortToQuantum((unsigned short) (scale[1]*
+            jas_matrix_getv(pixels[1],x/x_step[1])));
+          q->blue=ScaleShortToQuantum((unsigned short) (scale[2]*
+            jas_matrix_getv(pixels[2],x/x_step[2])));
+          q->opacity=QuantumRange-ScaleShortToQuantum((unsigned short) (scale[3]*
+            jas_matrix_getv(pixels[3],x/x_step[3])));
           q++;
         }
         break;
@@ -516,10 +545,54 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
     if (SyncImagePixels(image) == MagickFalse)
       break;
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
-        if (MagickMonitor(LoadImagesTag,y,image->rows,exception) == MagickFalse)
-          break;
+     if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
+        {
+          status=image->progress_monitor(LoadImagesTag,y,image->rows,
+            image->client_data);
+          if (status == MagickFalse)
+            break;
+        }
   }
+  cm_profile=jas_image_cmprof(jp2_image);
+  icc_profile=(jas_iccprof_t *) NULL;
+  if (cm_profile != (jas_cmprof_t *) NULL)
+    icc_profile=jas_iccprof_createfromcmprof(cm_profile);
+  if (icc_profile != (jas_iccprof_t *) NULL)
+    {
+      jas_stream_t
+        *icc_stream;
+
+      icc_stream=jas_stream_memopen(NULL,0);
+      if ((icc_stream != (jas_stream_t *) NULL) &&
+          (jas_iccprof_save(icc_profile,icc_stream) == 0) &&
+          (jas_stream_flush(icc_stream) == 0))
+        {
+          jas_stream_memobj_t
+            *blob;
+
+          StringInfo
+            *icc_profile,
+            *profile;
+
+          /*
+            Extract the icc profile, handle errors without much noise.
+          */
+          blob=(jas_stream_memobj_t *) icc_stream->obj_;
+          if (image->debug != MagickFalse)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+              "Profile: ICC, %lu bytes",(unsigned long) blob->len_);
+          profile=AcquireStringInfo(blob->len_);
+          SetStringInfoDatum(profile,blob->buf_);
+          icc_profile=GetImageProfile(image,"icc");
+          if (icc_profile == (StringInfo *) NULL)
+            SetImageProfile(image,"icc",profile);
+          else
+            ConcatenateStringInfo(icc_profile,profile);
+          profile=DestroyStringInfo(profile);
+          (void) jas_stream_close(icc_stream);
+        }
+    }
   (void) jas_stream_close(jp2_stream);
   jas_image_destroy(jp2_image);
   for (i=0; i < (long) number_components; i++)
@@ -556,7 +629,7 @@ ModuleExport void RegisterJP2Image(void)
     *entry;
 
   entry=SetMagickInfo("JP2");
-  entry->description=AcquireString("JPEG-2000 JP2 File Format Syntax");
+  entry->description=AcquireString("JPEG-2000 File Format Syntax");
   entry->module=AcquireString("JP2");
   entry->magick=(MagickHandler *) IsJP2;
   entry->adjoin=MagickFalse;
@@ -577,6 +650,18 @@ ModuleExport void RegisterJP2Image(void)
 #if defined(HasJP2)
   entry->decoder=(DecoderHandler *) ReadJP2Image;
   entry->encoder=(EncoderHandler *) WriteJP2Image;
+#endif
+  (void) RegisterMagickInfo(entry);
+  entry=SetMagickInfo("JPX");
+  entry->description=AcquireString("JPEG-2000 File Format Syntax");
+  entry->module=AcquireString("JPX");
+  entry->magick=(MagickHandler *) IsJPC;
+  entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
+#if defined(HasJP2)
+  entry->decoder=(DecoderHandler *) ReadJP2Image;
+  entry->encoder=(EncoderHandler *) WriteJP2Image;
 #endif
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PGX");
@@ -590,6 +675,9 @@ ModuleExport void RegisterJP2Image(void)
   entry->decoder=(DecoderHandler *) ReadJP2Image;
 #endif
   (void) RegisterMagickInfo(entry);
+#if defined(HasJP2)
+  jas_init();
+#endif
 }
 
 /*
@@ -616,6 +704,9 @@ ModuleExport void UnregisterJP2Image(void)
   (void) UnregisterMagickInfo("JP2");
   (void) UnregisterMagickInfo("JPC");
   (void) UnregisterMagickInfo("PGX");
+#if defined(HasJP2)
+  jas_cleanup();
+#endif
 }
 
 #if defined(HasJP2)
@@ -656,12 +747,6 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   const char
     *option;
 
-  const ImageAttribute
-    *attribute;
-
-  double
-    quality;
-
   int
     format;
 
@@ -669,7 +754,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
     y;
 
   jas_image_cmptparm_t
-    component_info;
+    component_info[4];
 
   jas_image_t
     *jp2_image;
@@ -701,7 +786,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -709,7 +794,6 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
     Intialize JPEG 2000 API.
   */
   (void) SetImageColorspace(image,RGBColorspace);
-  jas_init();
   jp2_stream=JP2StreamManager(image);
   if (jp2_stream == (jas_stream_t *) NULL)
     ThrowWriterException(DelegateError,"UnableToManageJP2Stream");
@@ -717,26 +801,21 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   if ((image_info->type != TrueColorType) &&
       IsGrayImage(image,&image->exception))
     number_components=1;
-  jp2_image=jas_image_create0();
-  if (jp2_image == (jas_image_t *) NULL)
-    ThrowWriterException(DelegateError,"UnableToCreateImage");
   for (i=0; i < (long) number_components; i++)
   {
-    (void) ResetMagickMemory((void *) &component_info,0,sizeof(component_info));
-    component_info.tlx=0;
-    component_info.tly=0;
-    component_info.hstep=1;
-    component_info.vstep=1;
-    component_info.width=(unsigned int) image->columns;
-    component_info.height=(unsigned int) image->rows;
-    component_info.prec=(unsigned int) image->depth <= 8 ? 8 : 16;
-    component_info.sgnd=MagickFalse;
-    if (jas_image_addcmpt(jp2_image,i,&component_info))
-      {
-        jas_image_destroy(jp2_image);
-        ThrowWriterException(DelegateError,"UnableToCreateImageComponent");
-      }
+    component_info[i].tlx=0;
+    component_info[i].tly=0;
+    component_info[i].hstep=1;
+    component_info[i].vstep=1;
+    component_info[i].width=(unsigned int) image->columns;
+    component_info[i].height=(unsigned int) image->rows;
+    component_info[i].prec=(unsigned int) image->depth <= 8 ? 8 : 16;
+    component_info[i].sgnd=MagickFalse;
   }
+  jp2_image=jas_image_create(number_components,component_info,
+    JAS_CLRSPC_UNKNOWN);
+  if (jp2_image == (jas_image_t *) NULL)
+    ThrowWriterException(DelegateError,"UnableToCreateImage");
   if (number_components == 1)
     {
       /*
@@ -751,6 +830,8 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
       /*
         sRGB.
       */
+      if (number_components == 4 )
+        jas_image_setcmpttype(jp2_image,3,JAS_IMAGE_CT_OPACITY);
       jas_image_setclrspc(jp2_image,JAS_CLRSPC_SRGB);
       jas_image_setcmpttype(jp2_image,0,
         JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_R));
@@ -758,9 +839,6 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
         JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_G));
       jas_image_setcmpttype(jp2_image,2,
         JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_B));
-      if (number_components == 4 )
-        jas_image_setcmpttype(jp2_image,3,
-          JAS_IMAGE_CT_COLOR(JAS_IMAGE_CT_OPACITY));
     }
   /*
     Convert to JPEG 2000 pixels.
@@ -794,7 +872,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
             jas_matrix_setv(pixels[2],x,ScaleQuantumToChar(p->blue));
             if (number_components > 3)
               jas_matrix_setv(pixels[3],x,
-                ScaleQuantumToChar(MaxRGB-p->opacity));
+                ScaleQuantumToChar(QuantumRange-p->opacity));
           }
         p++;
       }
@@ -811,7 +889,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
             jas_matrix_setv(pixels[2],x,ScaleQuantumToShort(p->blue));
             if (number_components > 3)
               jas_matrix_setv(pixels[3],x,
-                ScaleQuantumToShort(MaxRGB-p->opacity));
+                ScaleQuantumToShort(QuantumRange-p->opacity));
           }
         p++;
       }
@@ -819,9 +897,11 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
       (void) jas_image_writecmpt(jp2_image,(short) i,0,(unsigned int) y,
         (unsigned int) image->columns,1,pixels[i]);
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -829,20 +909,6 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   (void) CopyMagickString(magick,image_info->magick,MaxTextExtent);
   LocaleLower(magick);
   format=jas_image_strtofmt(magick);
-  quality=image_info->quality;
-  if (quality == UndefinedCompressionQuality)
-    {
-      quality=75;
-      attribute=GetImageAttribute(image,"JPEG-Quality");
-      if ((attribute != (const ImageAttribute *) NULL) &&
-          (attribute->value != (char *) NULL))
-        {
-          quality=(unsigned long) atol(attribute->value);
-          if (image->debug != MagickFalse)
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  quality=%lu",
-              quality);
-        }
-    }
   options=(char *) NULL;
   ResetImageOptionIterator(image_info);
   key=GetNextImageOption(image_info);
@@ -864,7 +930,8 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   option=GetImageOption(image_info,"jp2:rate");
   if ((option == (const char *) NULL) &&
       (image->compression != LosslessJPEGCompression) &&
-      (quality < 99.5) && ((image->rows*image->columns) > 2500))
+      (image->quality != UndefinedCompressionQuality) &&
+      (image->quality < 99.5) && ((image->rows*image->columns) > 2500))
     {
       char
         option[MaxTextExtent];
@@ -876,7 +943,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
         rate,
         target_size;
 
-      alpha=115-quality;
+      alpha=115-image->quality;
       rate=100.0/(alpha*alpha);
       header_size=550.0;
       header_size+=(number_components-1)*142;
@@ -888,12 +955,12 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
       ConcatenateString(&options,option);
     }
   status=jas_image_encode(jp2_image,jp2_stream,format,options);
-  if (status != MagickFalse)
-    ThrowWriterException(DelegateError,"UnableToEncodeImageFile");
   (void) jas_stream_close(jp2_stream);
   for (i=0; i < (long) number_components; i++)
     jas_matrix_destroy(pixels[i]);
   jas_image_destroy(jp2_image);
+  if (status != MagickFalse)
+    ThrowWriterException(DelegateError,"UnableToEncodeImageFile");
   return(MagickTrue);
 }
 #endif
