@@ -17,13 +17,13 @@
 %                                 July 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -41,12 +41,16 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/blob-private.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/profile.h"
+#include "magick/splay-tree.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/token.h"
@@ -1050,19 +1054,20 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   image->columns=1;
   image->rows=1;
-  SetImage(image,OpaqueOpacity);
+  SetImageBackgroundColor(image);
   length=1;
   if (LocaleNCompare(image_info->magick,"8BIM",4) == 0)
     {
@@ -1075,7 +1080,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       blob=(unsigned char *) AcquireMagickMemory(length);
       if (blob == (unsigned char *) NULL)
         {
-          DestroyImage(buff);
+          buff=DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       AttachBlob(buff->blob,blob,length);
@@ -1104,12 +1109,12 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       profile=AcquireStringInfo(GetBlobSize(buff));
       SetStringInfoDatum(profile,GetBlobStreamData(buff));
       status=SetImageProfile(image,"iptc",profile);
-      DestroyStringInfo(profile);
+      profile=DestroyStringInfo(profile);
       if (status == MagickFalse)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       blob=DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
-      DestroyImage(buff);
+      buff=DestroyImage(buff);
     }
   if (LocaleNCompare(image_info->magick,"APP1",4) == 0)
     {
@@ -1123,7 +1128,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       blob=(unsigned char *) AcquireMagickMemory(length);
       if (blob == (unsigned char *) NULL)
         {
-          DestroyImage(buff);
+          buff=DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       AttachBlob(buff->blob,blob,length);
@@ -1142,7 +1147,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
             {
               blob=DetachBlob(buff->blob);
               blob=(char *) RelinquishMagickMemory(blob);
-              DestroyImage(buff);
+              buff=DestroyImage(buff);
               ThrowReaderException(CoderError,"NoIPTCProfileAvailable");
             }
           iptc=AllocateImage((ImageInfo *) NULL);
@@ -1150,19 +1155,19 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
             {
               blob=DetachBlob(buff->blob);
               blob=(char *) RelinquishMagickMemory(blob);
-              DestroyImage(buff);
+              buff=DestroyImage(buff);
               ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
             }
           AttachBlob(iptc->blob,profile->datum,profile->length);
           result=jpeg_embed(image,buff,iptc);
           blob=DetachBlob(iptc->blob);
           blob=(char *) RelinquishMagickMemory(blob);
-          DestroyImage(iptc);
+          iptc=DestroyImage(iptc);
           if (result == 0)
             {
               blob=DetachBlob(buff->blob);
               blob=(char *) RelinquishMagickMemory(blob);
-              DestroyImage(buff);
+              buff=DestroyImage(buff);
               ThrowReaderException(CoderError,"JPEGEmbeddingFailed");
             }
         }
@@ -1214,12 +1219,12 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       profile=AcquireStringInfo(GetBlobSize(buff));
       SetStringInfoDatum(profile,GetBlobStreamData(buff));
       status=SetImageProfile(image,name,profile);
-      DestroyStringInfo(profile);
+      profile=DestroyStringInfo(profile);
       if (status == MagickFalse)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       blob=DetachBlob(buff->blob);
       blob=(char *) RelinquishMagickMemory(blob);
-      DestroyImage(buff);
+      buff=DestroyImage(buff);
     }
   if ((LocaleCompare(image_info->magick,"ICC") == 0) ||
       (LocaleCompare(image_info->magick,"ICM") == 0))
@@ -1230,7 +1235,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       blob=(unsigned char *) AcquireMagickMemory(length);
       if (blob == (unsigned char *) NULL)
         {
-          DestroyImage(buff);
+          buff=DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       AttachBlob(buff->blob,blob,length);
@@ -1244,10 +1249,10 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       profile=AcquireStringInfo(GetBlobSize(buff));
       SetStringInfoDatum(profile,GetBlobStreamData(buff));
       (void) SetImageProfile(image,"icc",profile);
-      DestroyStringInfo(profile);
+      profile=DestroyStringInfo(profile);
       blob=DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
-      DestroyImage(buff);
+      buff=DestroyImage(buff);
     }
   if (LocaleCompare(image_info->magick,"IPTC") == 0)
     {
@@ -1257,7 +1262,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       blob=(unsigned char *) AcquireMagickMemory(length);
       if (blob == (unsigned char *) NULL)
         {
-          DestroyImage(buff);
+          buff=DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       AttachBlob(buff->blob,blob,length);
@@ -1291,10 +1296,10 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       profile->datum[11]=length & 0xff;
       SetStringInfoDatum(profile,GetBlobStreamData(buff));
       (void) SetImageProfile(image,"iptc",profile);
-      DestroyStringInfo(profile);
+      profile=DestroyStringInfo(profile);
       blob=DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
-      DestroyImage(buff);
+      buff=DestroyImage(buff);
     }
   CloseBlob(image);
   return(GetFirstImageInList(image));
@@ -2101,9 +2106,7 @@ static int format8BIM(Image *ifile, Image *ofile)
       }
     str=(unsigned char *) RelinquishMagickMemory(str);
     PString=(unsigned char *) RelinquishMagickMemory(PString);
-
     resCount++;
-
     c=ReadBlobByte(ifile);
   }
   return resCount;
@@ -2129,8 +2132,8 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
-  if (image->profiles == (HashmapInfo *) NULL)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  if (image->profiles == (void *) NULL)
     return(MagickFalse);
   length=0;
   if (LocaleCompare(image_info->magick,"8BIM") == 0)
@@ -2138,7 +2141,9 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       /*
         Write 8BIM image.
       */
-      profile=GetImageProfile(image,"8BIM");
+      profile=GetImageProfile(image,"8bim");
+      if (profile == (StringInfo *) NULL)
+        profile=GetImageProfile(image,"iptc");
       if (profile == (StringInfo *) NULL)
         ThrowWriterException(CoderError,"No8BIMDataIsAvailable");
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
@@ -2164,7 +2169,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       length=profile->length;
       length=GetIPTCStream(&info,length);
       if (length == 0)
-        ThrowWriterException(CoderError,"NoIPTCInfoWasFound");
+        ThrowWriterException(CoderError,"NoIPTCProfileAvailable");
       (void) WriteBlob(image,length,info);
       CloseBlob(image);
       return(MagickTrue);
@@ -2174,7 +2179,9 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       Image
         *buff;
 
-      profile=GetImageProfile(image,"iptc");
+      profile=GetImageProfile(image,"8bim");
+      if (profile == (StringInfo *) NULL)
+        profile=GetImageProfile(image,"iptc");
       if (profile == (StringInfo *) NULL)
         ThrowWriterException(CoderError,"No8BIMDataIsAvailable");
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
@@ -2186,7 +2193,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       AttachBlob(buff->blob,profile->datum,profile->length);
       format8BIM(buff,image);
       (void) DetachBlob(buff->blob);
-      DestroyImage(buff);
+      buff=DestroyImage(buff);
       CloseBlob(image);
       return(MagickTrue);
     }
@@ -2209,7 +2216,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       length=profile->length;
       length=GetIPTCStream(&info,length);
       if (length == 0)
-        ThrowWriterException(CoderError,"NoIPTCInfoWasFound");
+        ThrowWriterException(CoderError,"NoIPTCProfileAvailable");
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == MagickFalse)
         return(status);
@@ -2219,7 +2226,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       AttachBlob(buff->blob,info,length);
       formatIPTC(buff,image);
       (void) DetachBlob(buff->blob);
-      DestroyImage(buff);
+      buff=DestroyImage(buff);
       CloseBlob(image);
       return(MagickTrue);
     }
@@ -2250,7 +2257,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       */
       profile=GetImageProfile(image,"icc");
       if (profile == (StringInfo *) NULL)
-        ThrowWriterException(CoderError,"NoColorProfileAvailable");
+        ThrowWriterException(CoderError,"NoColorProfileIsAvailable");
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == MagickFalse)
         return(status);
