@@ -17,13 +17,13 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -43,14 +43,17 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
 #include "magick/cache.h"
 #include "magick/color.h"
-#include "magick/color_private.h"
+#include "magick/color-private.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/geometry.h"
 #include "magick/hashmap.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -58,6 +61,7 @@
 #include "magick/option.h"
 #include "magick/profile.h"
 #include "magick/static.h"
+#include "magick/statistic.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
 
@@ -206,6 +210,9 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   size_t
     length;
 
+  ssize_t
+    count;
+
   StringInfo
     *profile;
 
@@ -219,14 +226,15 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   (void) CopyMagickString(cache_filename,image->filename,MaxTextExtent);
@@ -234,10 +242,11 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   c=ReadBlobByte(image);
   if (c == EOF)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   *id='\0';
+  (void) ResetMagickMemory(keyword,0,sizeof(keyword));
   offset=0;
   do
   {
@@ -285,7 +294,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (comment == (char *) NULL)
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           *p='\0';
-          (void) SetImageAttribute(image,"comment",comment);
+          (void) SetImageAttribute(image,"Comment",comment);
           comment=(char *) RelinquishMagickMemory(comment);
           c=ReadBlobByte(image);
         }
@@ -293,41 +302,51 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (isalnum(c) != MagickFalse)
           {
             /*
-              Determine a keyword and its value.
+              Get the keyword.
             */
             p=keyword;
             do
             {
-              if ((size_t) (p-keyword) < (MaxTextExtent-1))
+              if (isspace((int) ((unsigned char) c)) != 0)
+                break;
+              if (c == (int) '=')
+                break;
+              if ((size_t) (p-keyword) < (MaxTextExtent/2))
                 *p++=(char) c;
               c=ReadBlobByte(image);
-            } while ((isalnum(c) != MagickFalse) || (c == (int) '-'));
+            } while (c != EOF);
             *p='\0';
-            while ((isspace((int) ((unsigned char) c)) != 0) ||
-                   (c == (int) '='))
-              c=ReadBlobByte(image);
             p=options;
-            while ((c != (int) '}') && (c != EOF))
-            {
-              if ((p-options+1) >= (int) length)
-                {
-                  *p='\0';
-                  length<<=1;
-                  options=(char *) ResizeMagickMemory(options,
-                    (length+MaxTextExtent)*sizeof(*options));
-                  if (options == (char *) NULL)
-                    break;
-                  p=options+strlen(options);
-                }
-              if (options == (char *) NULL)
-                ThrowReaderException(ResourceLimitError,
-                  "MemoryAllocationFailed");
-              *p++=(char) c;
+            while (isspace((int) ((unsigned char) c)) != 0)
               c=ReadBlobByte(image);
-              if (*options != '{')
-                if (isspace((int) ((unsigned char) c)) != 0)
-                  break;
-            }
+            if (c == (int) '=')
+              {
+                /*
+                  Get the keyword value.
+                */
+                c=ReadBlobByte(image);
+                while ((c != (int) '}') && (c != EOF))
+                {
+                  if ((size_t) (p-options+1) >= length)
+                    {
+                      *p='\0';
+                      length<<=1;
+                      options=(char *) ResizeMagickMemory(options,
+                        (length+MaxTextExtent)*sizeof(*options));
+                      if (options == (char *) NULL)
+                        break;
+                      p=options+strlen(options);
+                    }
+                  if (options == (char *) NULL)
+                    ThrowReaderException(ResourceLimitError,
+                      "MemoryAllocationFailed");
+                  *p++=(char) c;
+                  c=ReadBlobByte(image);
+                  if (*options != '{')
+                    if (isspace((int) ((unsigned char) c)) != 0)
+                      break;
+                }
+              }
             *p='\0';
             if (*options == '{')
               (void) CopyMagickString(options,options+1,MaxTextExtent);
@@ -380,14 +399,14 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"colorspace") == 0)
                   {
-                    image->colorspace=(ColorspaceType)
-                      ParseMagickOption(MagickColorspaceOptions,MagickFalse,options);
+                    image->colorspace=(ColorspaceType) ParseMagickOption(
+                      MagickColorspaceOptions,MagickFalse,options);
                     break;
                   }
                 if (LocaleCompare(keyword,"compression") == 0)
                   {
-                    image->compression=(CompressionType)
-                      ParseMagickOption(MagickCompressionOptions,MagickFalse,options);
+                    image->compression=(CompressionType) ParseMagickOption(
+                      MagickCompressionOptions,MagickFalse,options);
                     break;
                   }
                 if (LocaleCompare(keyword,"columns") == 0)
@@ -413,8 +432,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"dispose") == 0)
                   {
-                    image->dispose=(DisposeType)
-                      ParseMagickOption(MagickDisposeOptions,MagickFalse,options);
+                    image->dispose=(DisposeType) ParseMagickOption(
+                      MagickDisposeOptions,MagickFalse,options);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,options);
@@ -423,6 +442,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               case 'e':
               case 'E':
               {
+                if (LocaleCompare(keyword,"endian") == 0)
+                  {
+                    image->endian=(EndianType) ParseMagickOption(
+                      MagickEndianOptions,MagickFalse,options);
+                    break;
+                  }
                 if (LocaleCompare(keyword,"error") == 0)
                   {
                     image->error.mean_error_per_pixel=atof(options);
@@ -518,11 +543,11 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   {
                     if (profiles == (LinkedListInfo *) NULL)
                       profiles=NewLinkedList(0);
-                    (void) AppendElementToLinkedList(profiles,
+                    (void) AppendValueToLinkedList(profiles,
                       AcquireString(keyword+8));
                     profile=AcquireStringInfo((size_t) atol(options));
                     (void) SetImageProfile(image,keyword+8,profile);
-                    DestroyStringInfo(profile);
+                    profile=DestroyStringInfo(profile);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,options);
@@ -543,6 +568,11 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               case 'q':
               case 'Q':
               {
+                if (LocaleCompare(keyword,"quality") == 0)
+                  {
+                    image->quality=(unsigned long) atol(options);
+                    break;
+                  }
                 if (LocaleCompare(keyword,"quantum-depth") == 0)
                   {
                     quantum_depth=(unsigned long) atol(options);
@@ -564,8 +594,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"rendering-intent") == 0)
                   {
-                    image->rendering_intent=(RenderingIntent)
-                      ParseMagickOption(MagickIntentOptions,MagickFalse,options);
+                    image->rendering_intent=(RenderingIntent) ParseMagickOption(
+                      MagickIntentOptions,MagickFalse,options);
                     break;
                   }
                 if (LocaleCompare(keyword,"resolution") == 0)
@@ -596,13 +626,24 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 (void) SetImageAttribute(image,keyword,options);
                 break;
               }
+              case 't':
+              case 'T':
+              {
+                if (LocaleCompare(keyword,"ticks-per-second") == 0)
+                  {
+                    image->ticks_per_second=(unsigned long) atol(options);
+                    break;
+                  }
+                (void) SetImageAttribute(image,keyword,options);
+                break;
+              }
               case 'u':
               case 'U':
               {
                 if (LocaleCompare(keyword,"units") == 0)
                   {
-                    image->units=(ResolutionType)
-                      ParseMagickOption(MagickResolutionOptions,MagickFalse,options);
+                    image->units=(ResolutionType) ParseMagickOption(
+                      MagickResolutionOptions,MagickFalse,options);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,options);
@@ -687,15 +728,15 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Read image profiles.
         */
         ResetLinkedListIterator(profiles);
-        name=(const char *) GetNextElementInLinkedList(profiles);
+        name=(const char *) GetNextValueInLinkedList(profiles);
         while (name != (const char *) NULL)
         {
           profile=GetImageProfile(image,name);
           if (profile != (StringInfo *) NULL)
-            (void) ReadBlob(image,profile->length,profile->datum);
-          name=(const char *) GetNextElementInLinkedList(profiles);
+            count=ReadBlob(image,profile->length,profile->datum);
+          name=(const char *) GetNextValueInLinkedList(profiles);
         }
-        DestroyLinkedList(profiles,RelinquishMagickMemory);
+        profiles=DestroyLinkedList(profiles,RelinquishMagickMemory);
       }
     depth=GetImageQuantumDepth(image,MagickFalse);
     if (image->storage_class == PseudoClass)
@@ -724,7 +765,10 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               AcquireMagickMemory(packet_size*image->colors);
             if (colormap == (unsigned char *) NULL)
               ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-            (void) ReadBlob(image,packet_size*image->colors,colormap);
+            count=ReadBlob(image,packet_size*image->colors,colormap);
+            if (count != (ssize_t) (packet_size*image->colors))
+              ThrowReaderException(CorruptImageError,
+                "InsufficientImageDataInFile");
             p=colormap;
             switch (depth)
             {
@@ -802,16 +846,19 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (c != EOF);
   CloseBlob(image);
@@ -928,6 +975,9 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
   register long
     i;
 
+  unsigned long
+    depth;
+
   /*
     Open persistent cache.
   */
@@ -936,7 +986,7 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -949,6 +999,10 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
     /*
       Write persistent cache meta-information.
     */
+    depth=GetImageQuantumDepth(image,MagickTrue);
+    if ((image->storage_class == PseudoClass) &&
+        (image->colors > (1UL << depth)))
+      image->storage_class=DirectClass;
     (void) WriteBlobString(image,"id=MagickCache\n");
     (void) FormatMagickString(buffer,MaxTextExtent,"quantum-depth=%d\n",
       QuantumDepth);
@@ -968,10 +1022,18 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
           MagickOptionToMnemonic(MagickColorspaceOptions,image->colorspace));
         (void) WriteBlobString(image,buffer);
       }
+    if (image->endian != UndefinedEndian)
+      {
+        (void) FormatMagickString(buffer,MaxTextExtent,"endian=%s\n",
+          MagickOptionToMnemonic(MagickEndianOptions,image->endian));
+        (void) WriteBlobString(image,buffer);
+      }
     if (image->compression != UndefinedCompression)
       {
-        (void) FormatMagickString(buffer,MaxTextExtent,"compression=%s\n",
-          MagickOptionToMnemonic(MagickCompressionOptions,image->compression));
+        (void) FormatMagickString(buffer,MaxTextExtent,
+          "compression=%s  quality=%lu\n",
+          MagickOptionToMnemonic(MagickCompressionOptions,image->compression),
+          image->quality);
         (void) WriteBlobString(image,buffer);
       }
     if (image->units != UndefinedResolution)
@@ -999,15 +1061,18 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
             image->page.x,image->page.y);
           (void) WriteBlobString(image,buffer);
         }
-    if ((image->next != (Image *) NULL) || (image->previous != (Image *) NULL))
+    if ((GetNextImageInList(image) != (Image *) NULL) ||
+        (GetPreviousImageInList(image) != (Image *) NULL))
       {
         if (image->scene == 0)
           (void) FormatMagickString(buffer,MaxTextExtent,
-            "iterations=%lu  delay=%lu\n",image->iterations,image->delay);
+            "iterations=%lu  delay=%lu  ticks-per-second=%lu\n",
+            image->iterations,image->delay,image->ticks_per_second);
         else
           (void) FormatMagickString(buffer,MaxTextExtent,
-            "scene=%lu  iterations=%lu  delay=%lu\n",image->scene,
-            image->iterations,image->delay);
+            "scene=%lu  iterations=%lu  delay=%lu  ticks-per-second=%lu\n",
+            image->scene,image->iterations,image->delay,
+            image->ticks_per_second);
         (void) WriteBlobString(image,buffer);
       }
     else
@@ -1030,6 +1095,12 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
               image->delay);
             (void) WriteBlobString(image,buffer);
           }
+        if (image->ticks_per_second != UndefinedTicksPerSecond)
+          {
+            (void) FormatMagickString(buffer,MaxTextExtent,
+              "ticks-per-second=%lu\n",image->ticks_per_second);
+            (void) WriteBlobString(image,buffer);
+          }
       }
     if (image->dispose != UndefinedDispose)
       {
@@ -1037,20 +1108,6 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
           MagickOptionToMnemonic(MagickDisposeOptions,image->dispose));
         (void) WriteBlobString(image,buffer);
       }
-    if (image->compose != UndefinedCompositeOp)
-      {
-        (void) FormatMagickString(buffer,MaxTextExtent,"compose=%s\n",
-          MagickOptionToMnemonic(MagickCompositeOptions,image->compose));
-        (void) WriteBlobString(image,buffer);
-      }
-    if (image->error.mean_error_per_pixel != 0.0)
-      {
-        (void) FormatMagickString(buffer,MaxTextExtent,
-          "error=%g  mean-error=%g  maximum-error=%g\n",
-          image->error.mean_error_per_pixel,image->error.normalized_mean_error,
-          image->error.normalized_maximum_error);
-        (void) WriteBlobString(image,buffer);
-      }
     if (image->rendering_intent != UndefinedIntent)
       {
         (void) FormatMagickString(buffer,MaxTextExtent,
@@ -1081,7 +1138,7 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
           image->chromaticity.white_point.x,image->chromaticity.white_point.y);
         (void) WriteBlobString(image,buffer);
       }
-    if (image->profiles != (HashmapInfo *) NULL)
+    if (image->profiles != (void *) NULL)
       {
         const char
           *name;
@@ -1097,9 +1154,12 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
         while (name != (const char *) NULL)
         {
           profile=GetImageProfile(image,name);
-          (void) FormatMagickString(buffer,MaxTextExtent,
-            "profile-%s=%lu\n",name,profile->length);
-          (void) WriteBlobString(image,buffer);
+          if (profile != (StringInfo *) NULL)
+            {
+              (void) FormatMagickString(buffer,MaxTextExtent,
+                "profile-%s=%lu\n",name,(unsigned long) profile->length);
+              (void) WriteBlobString(image,buffer);
+            }
           name=GetNextImageProfile(image);
         }
       }
@@ -1109,27 +1169,26 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
           image->montage);
         (void) WriteBlobString(image,buffer);
       }
-    attribute=GetImageAttribute(image,(char *) NULL);
-    for ( ; attribute != (const ImageAttribute *) NULL; attribute=attribute->next)
+    ResetImageAttributeIterator(image);
+	  attribute=GetNextImageAttribute(image);
+		while (attribute != (const ImageAttribute *) NULL)
     {
-      if (attribute->value != NULL)
+      if (*attribute->key != '[')
         {
-          long
-            j;
-
-          (void) FormatMagickString(buffer,MaxTextExtent,"%s=",
-            attribute->key);
+          (void) FormatMagickString(buffer,MaxTextExtent,"%s=",attribute->key);
           (void) WriteBlobString(image,buffer);
-          for (j=0; j < (long) strlen(attribute->value); j++)
-            if (isspace((int) ((unsigned char) attribute->value[j])) != 0)
+          for (i=0; i < (long) strlen(attribute->value); i++)
+            if (isspace((int) ((unsigned char) attribute->value[i])) != 0)
               break;
-          if (j < (long) strlen(attribute->value))
+          if (i <= (long) strlen(attribute->value))
             (void) WriteBlobByte(image,'{');
-          (void) WriteBlobString(image,attribute->value);
-          if (j < (long) strlen(attribute->value))
+          (void) WriteBlob(image,strlen(attribute->value),(unsigned char *)
+            attribute->value);
+          if (i <= (long) strlen(attribute->value))
             (void) WriteBlobByte(image,'}');
           (void) WriteBlobByte(image,'\n');
         }
+  	  attribute=GetNextImageAttribute(image);
     }
     (void) WriteBlobString(image,"\f\n:\032");
     if (image->montage != (char *) NULL)
@@ -1173,16 +1232,14 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
           *colormap;
 
         unsigned long
-          depth,
           pixel;
 
         /*
           Allocate colormap.
         */
-        depth=GetImageQuantumDepth(image,MagickTrue);
-        packet_size=(size_t) (depth > 8 ? 6 : 3);
+        packet_size=(size_t) (3*depth/8);
         colormap=(unsigned char *)
-          AcquireMagickMemory(packet_size*image->colors);
+          AcquireMagickMemory(packet_size*image->colors*sizeof(*colormap));
         if (colormap == (unsigned char *) NULL)
           return(MagickFalse);
         /*
@@ -1233,16 +1290,21 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
     /*
       Initialize persistent pixel cache.
     */
-    status=PersistCache(image,cache_filename,MagickFalse,&offset,&image->exception);
-    if (status == MagickFalse)
-      ThrowWriterException(CacheError,"UnableToPersistPixelCache");
-    if (image->next == (Image *) NULL)
-      break;
-    image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+    status=PersistCache(image,cache_filename,MagickFalse,&offset,
       &image->exception);
     if (status == MagickFalse)
+      ThrowWriterException(CacheError,"UnableToPersistPixelCache");
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
+    image=SyncNextImageInList(image);
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
   return(status);
