@@ -21,9 +21,9 @@
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
-   obtain a copy of the License at                                            %
+%   obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -44,16 +44,18 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
 #include "magick/color.h"
-#include "magick/color_private.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
 #include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/paint.h"
+#include "magick/pixel-private.h"
 #include "magick/quantize.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -233,6 +235,9 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
+  MagickPixelPacket
+    transpix;
+
   register IndexPacket
     *indexes;
 
@@ -241,7 +246,6 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
     x;
 
   register PixelPacket
-    transpix,
     *q;
 
   ssize_t
@@ -272,9 +276,20 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
   unsigned short
     color16;
 
+  MagickOffsetType
+    totalOffset,
+    seekNextDepth;
+
   /*
     Open image file.
   */
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -282,160 +297,222 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
       DestroyImageList(image);
       return((Image *) NULL);
     }
-  image->columns=ReadBlobMSBShort(image);
-  image->rows=ReadBlobMSBShort(image);
-  bytes_per_row=ReadBlobMSBShort(image);
-  flags=ReadBlobMSBShort(image);
-  bits_per_pixel=ReadBlobByte(image);
-  version=ReadBlobByte(image);
-  nextDepthOffset=ReadBlobMSBShort(image);
-  transparentIndex=ReadBlobByte(image);
-  compressionType=ReadBlobByte(image);
-  pad=ReadBlobMSBShort(image);
-  /*
-    Initialize image colormap.
-  */
-  if ((bits_per_pixel < 16) &&
-      !AllocateImageColormap(image,1L << bits_per_pixel))
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  if (bits_per_pixel == 16)  /* Direct Color */
-    {
-      redbits=ReadBlobByte(image);  /* # of bits of red */
-      greenbits=ReadBlobByte(image);  /* # of bits of green */
-      bluebits=ReadBlobByte(image);  /* # of bits of blue */
-      ReadBlobByte(image);  /* reserved by Palm */
-      ReadBlobByte(image);  /* reserved by Palm */
-      transpix.red=MaxRGB*ReadBlobByte(image)/31;
-      transpix.green=MaxRGB*ReadBlobByte(image)/63;
-      transpix.blue=MaxRGB*ReadBlobByte(image)/31;
-    }
-  if (bits_per_pixel == 8)
-    {
-      if (flags & PALM_HAS_COLORMAP_FLAG)
-        {
-          count=ReadBlobMSBShort(image);
-          for (i=0; i < (long) count; i++)
-          {
-            ReadBlobByte(image);
-            image->colormap[255-i].red=ScaleCharToQuantum(ReadBlobByte(image));
-            image->colormap[255-i].green=
-              ScaleCharToQuantum(ReadBlobByte(image));
-            image->colormap[255-i].blue=ScaleCharToQuantum(ReadBlobByte(image));
-        }
-      }
-    else
-      {
-        for (i = 0; i < (long) (1L << bits_per_pixel); i++)
-        {
-          image->colormap[255-i].red=ScaleCharToQuantum(PalmPalette[i][0]);
-          image->colormap[255-i].green=ScaleCharToQuantum(PalmPalette[i][1]);
-          image->colormap[255-i].blue=ScaleCharToQuantum(PalmPalette[i][2]);
-        }
-      }
-    }
-  if (flags & PALM_IS_COMPRESSED_FLAG)
-    size=ReadBlobMSBShort(image);
-  image->storage_class=DirectClass;
-  if (bits_per_pixel < 16)
-    {
-      image->storage_class=PseudoClass;
-      image->depth=8;
-    }
-  one_row=(unsigned char *) AcquireMagickMemory(bytes_per_row);
-  if (one_row == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  lastrow=(unsigned char *) NULL;
-  if (compressionType == PALM_COMPRESSION_SCANLINE)
-    lastrow=(unsigned char *) AcquireMagickMemory(bytes_per_row);
-  mask=(1l << bits_per_pixel)-1;
-  for (y = 0; y < (long) image->rows; y++)
+  totalOffset=0;
+  do
   {
-    if ((flags & PALM_IS_COMPRESSED_FLAG) == 0)
+    image->columns=ReadBlobMSBShort(image);
+    image->rows=ReadBlobMSBShort(image);
+    /*
+      Copied from coders/pnm.c. TODO other checks ...
+    */
+    if ((image->columns == 0) || (image->rows == 0))
+      ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
+    bytes_per_row=ReadBlobMSBShort(image);
+    flags=ReadBlobMSBShort(image);
+    bits_per_pixel=ReadBlobByte(image);
+    if (bits_per_pixel > 16)
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    version=ReadBlobByte(image);
+    nextDepthOffset=ReadBlobMSBShort(image);
+    transparentIndex=ReadBlobByte(image);
+    compressionType=ReadBlobByte(image);
+    pad=ReadBlobMSBShort(image);
+    /*
+      Initialize image colormap.
+    */
+    if ((bits_per_pixel < 16) &&
+        !AllocateImageColormap(image,1L << bits_per_pixel))
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    GetMagickPixelPacket(image,&transpix);
+    if (bits_per_pixel == 16)  /* Direct Color */
       {
-        image->compression=NoCompression;
-        (void) ReadBlob(image,bytes_per_row,one_row);
+        redbits=ReadBlobByte(image);  /* # of bits of red */
+        greenbits=ReadBlobByte(image);  /* # of bits of green */
+        bluebits=ReadBlobByte(image);  /* # of bits of blue */
+        ReadBlobByte(image);  /* reserved by Palm */
+        ReadBlobByte(image);  /* reserved by Palm */
+        transpix.red=QuantumRange*ReadBlobByte(image)/31;
+        transpix.green=QuantumRange*ReadBlobByte(image)/63;
+        transpix.blue=QuantumRange*ReadBlobByte(image)/31;
       }
-    else
+    if (bits_per_pixel == 8)
       {
-        if (compressionType == PALM_COMPRESSION_RLE)
+        if (flags & PALM_HAS_COLORMAP_FLAG)
           {
-            image->compression=RLECompression;
-            for (i=0; i < (long) bytes_per_row; )
+            count=ReadBlobMSBShort(image);
+            for (i=0; i < (long) count; i++)
             {
-              count=ReadBlobByte(image);
-              byte=ReadBlobByte(image);
-              (void) ResetMagickMemory(one_row+i,(int) byte,count);
-              i+=count;
-            }
-        }
-      else
-        if (compressionType == PALM_COMPRESSION_SCANLINE)
-          {
-            image->compression=FaxCompression;
-            for (i=0; i < (long) bytes_per_row; i+=8)
-            {
-              count=ReadBlobByte(image);
-              byte=Min(bytes_per_row-i,8);
-              for (bit=0; bit < byte; bit++)
-              {
-                if ((y == 0) || (count & (1 << (7 - bit))))
-                  one_row[i+bit]=ReadBlobByte(image);
-                else
-                  one_row[i+bit]=lastrow[i+bit];
-              }
-            }
-            CopyMagickMemory(lastrow, one_row, bytes_per_row);
+              ReadBlobByte(image);
+              image->colormap[255-i].red=
+                ScaleCharToQuantum(ReadBlobByte(image));
+              image->colormap[255-i].green=
+                ScaleCharToQuantum(ReadBlobByte(image));
+              image->colormap[255-i].blue=
+                ScaleCharToQuantum(ReadBlobByte(image));
           }
-      }
-    ptr=one_row;
-    q=SetImagePixels(image,0,y,image->columns,1);
-    if (q == (PixelPacket *) NULL)
-      break;
-    indexes=GetIndexes(image);
-    if (bits_per_pixel == 16)
-      {
-        for (x=0; x < (long) image->columns; x++)
+        }
+      else
         {
-          color16=(*ptr++ << 8);
-          color16|=(*ptr++);
-          q->red=((MaxRGB*(color16 >> 11) & 0x1f))/31;
-          q->green=((MaxRGB*(color16 >> 5) & 0x3f))/63;
-          q->blue=(MaxRGB*(color16 & 0x1f))/31;
-          q->opacity=OpaqueOpacity;
-          q++;
+          for (i = 0; i < (long) (1L << bits_per_pixel); i++)
+          {
+            image->colormap[255-i].red=ScaleCharToQuantum(PalmPalette[i][0]);
+            image->colormap[255-i].green=ScaleCharToQuantum(PalmPalette[i][1]);
+            image->colormap[255-i].blue=ScaleCharToQuantum(PalmPalette[i][2]);
+          }
         }
       }
-    else
+    if (flags & PALM_IS_COMPRESSED_FLAG)
+      /* TODO unused: cast to void? */
+      size=ReadBlobMSBShort(image);
+    image->storage_class=DirectClass;
+    if (bits_per_pixel < 16)
       {
-        bit=8-bits_per_pixel;
-        for (x=0; x < (long) image->columns; x++)
+        image->storage_class=PseudoClass;
+        image->depth=8;
+      }
+    one_row=(unsigned char *) AcquireMagickMemory(bytes_per_row);
+    if (one_row == (unsigned char *) NULL)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    lastrow=(unsigned char *) NULL;
+    if (compressionType == PALM_COMPRESSION_SCANLINE) {
+      lastrow=(unsigned char *) AcquireMagickMemory(bytes_per_row);
+      if (lastrow == (unsigned char *) NULL)
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    }
+    /* TODO move to point just before first use */
+    mask=(1l << bits_per_pixel)-1;
+    for (y = 0; y < (long) image->rows; y++)
+    {
+      if ((flags & PALM_IS_COMPRESSED_FLAG) == 0)
         {
-          index=(IndexPacket) (mask-(((*ptr) & (mask << bit)) >> bit));
-          indexes[x]=index;
-          *q++=image->colormap[index];
-          if (bit)
-            bit-=bits_per_pixel;
-          else
-            {
-              ptr++;
-              bit=8-bits_per_pixel;
+          /* TODO move out of loop! */
+          image->compression=NoCompression;
+          count=ReadBlob(image,bytes_per_row,one_row);
+        }
+      else
+        {
+          if (compressionType == PALM_COMPRESSION_RLE)
+            { 
+              /* TODO move out of loop! */
+              image->compression=RLECompression;
+              for (i=0; i < (long) bytes_per_row; )
+              {
+                count=ReadBlobByte(image);
+                byte=ReadBlobByte(image);
+                (void) ResetMagickMemory(one_row+i,(int) byte,count);
+                i+=count;
+              }
+          }
+        else
+          if (compressionType == PALM_COMPRESSION_SCANLINE)
+            {  
+              /* TODO move out of loop! */
+              image->compression=FaxCompression;
+              for (i=0; i < (long) bytes_per_row; i+=8)
+              {
+                count=ReadBlobByte(image);
+                byte=Min(bytes_per_row-i,8);
+                for (bit=0; bit < byte; bit++)
+                {
+                  if ((y == 0) || (count & (1 << (7 - bit))))
+                    one_row[i+bit]=ReadBlobByte(image);
+                  else
+                    one_row[i+bit]=lastrow[i+bit];
+                }
+              }
+              CopyMagickMemory(lastrow, one_row, bytes_per_row);
             }
         }
-        if (SyncImagePixels(image) == MagickFalse)
-          break;
-      }
-    }
-  if (flags & PALM_HAS_TRANSPARENCY_FLAG)
-    {
+      ptr=one_row;
+      q=SetImagePixels(image,0,y,image->columns,1);
+      if (q == (PixelPacket *) NULL)
+        break;
+      indexes=GetIndexes(image);
       if (bits_per_pixel == 16)
-        TransparentImage(image,transpix,TransparentOpacity);
+        {
+          for (x=0; x < (long) image->columns; x++)
+          {
+            color16=(*ptr++ << 8);
+            color16|=(*ptr++);
+            q->red=(QuantumRange*((color16 >> 11) & 0x1f))/0x1f;
+            q->green=(QuantumRange*((color16 >> 5) & 0x3f))/0x3f;
+            q->blue=(QuantumRange*((color16 >> 0) & 0x1f))/0x1f;
+            q->opacity=OpaqueOpacity;
+            q++;
+          }
+        }
       else
-        TransparentImage(image,image->colormap[mask-transparentIndex],
-          TransparentOpacity);
-    }
-  one_row=(unsigned char *) RelinquishMagickMemory(one_row);
-  if (compressionType == PALM_COMPRESSION_SCANLINE)
+        {
+          bit=8-bits_per_pixel;
+          for (x=0; x < (long) image->columns; x++)
+          {
+            index=(IndexPacket) (mask-(((*ptr) & (mask << bit)) >> bit));
+            indexes[x]=index;
+            *q++=image->colormap[index];
+            if (bit)
+              bit-=bits_per_pixel;
+            else
+              {
+                ptr++;
+                bit=8-bits_per_pixel;
+              }
+          }
+          if (SyncImagePixels(image) == MagickFalse)
+            break;
+        }
+      }
+    if (flags & PALM_HAS_TRANSPARENCY_FLAG)
+      {
+        if (bits_per_pixel != 16)
+          SetMagickPixelPacket(image->colormap+(mask-transparentIndex),
+            (const IndexPacket *) NULL,&transpix);
+        PaintTransparentImage(image,&transpix,TransparentOpacity);
+      }
+    one_row=(unsigned char *) RelinquishMagickMemory(one_row);
+    if (compressionType == PALM_COMPRESSION_SCANLINE)
     lastrow=(unsigned char *) RelinquishMagickMemory(lastrow);
+    /*
+      Proceed to next image. Copied from coders/pnm.c
+    */
+    if (image_info->number_scenes != 0)
+      if (image->scene >= (image_info->scene+image_info->number_scenes-1))
+        break;
+    if (nextDepthOffset != 0)
+      { 
+        /*
+          Skip to next image. 
+        */
+        totalOffset+=(MagickOffsetType) (nextDepthOffset*4);
+        if (totalOffset >= (MagickOffsetType) GetBlobSize(image))
+          {
+            ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+          }
+        else
+          {
+            seekNextDepth=SeekBlob(image,totalOffset,SEEK_SET);
+          }
+        if (seekNextDepth != totalOffset)
+          ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+        /*
+          Allocate next image structure. Copied from coders/pnm.c
+        */
+        AllocateNextImage(image_info,image);
+        if (GetNextImageInList(image) == (Image *) NULL)
+          {
+            DestroyImageList(image);
+            return((Image *) NULL);
+          }
+        image=SyncNextImageInList(image);
+        /* TODO Analyze this ...
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }*/
+      }
+  } while (nextDepthOffset != 0);
   CloseBlob(image);
   return(GetFirstImageInList(image));
 }
@@ -470,7 +547,6 @@ ModuleExport void RegisterPALMImage(void)
   entry=SetMagickInfo("PALM");
   entry->decoder=(DecoderHandler *) ReadPALMImage;
   entry->encoder=(EncoderHandler *) WritePALMImage;
-  entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("Palm pixmap");
   entry->module=AcquireString("PALM");
@@ -543,6 +619,15 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    currentOffset;
+
+  MagickSizeType
+    cc;
+
+  PixelPacket
+    transpix;
+
   QuantizeInfo
     quantize_info;
 
@@ -553,7 +638,6 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
     x;
 
   register PixelPacket
-    transpix,
     *p;
 
   unsigned char
@@ -571,7 +655,8 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
   unsigned long
     count,
     bits_per_pixel,
-    bytes_per_row;
+    bytes_per_row,
+    nextDepthOffset;
 
   unsigned short
     offset,
@@ -582,221 +667,250 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
   /*
     Open output image file.
   */
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&exception);
   if (status == MagickFalse)
     return(status);
   GetExceptionInfo(&exception);
-  attribute=GetImageAttribute(image, "Comment");
-  if (attribute != (ImageAttribute *)NULL)
-    if (LocaleCompare("COLORMAP",attribute->value) == 0)
-      flags|=PALM_HAS_COLORMAP_FLAG;
-  count = GetNumberColors(image, NULL, &exception);
-  for (bits_per_pixel=1;  (1UL << bits_per_pixel) < count;  bits_per_pixel*=2);
-  if (image_info->depth > 100)
-    bits_per_pixel=image_info->depth-100;
-  if (bits_per_pixel < 16)
-    SetImageColorspace(image,image->colorspace);
-  if (bits_per_pixel < 8)
-    {
-      SetImageColorspace(image,GRAYColorspace);
-      (void) SortColormapByIntensity(image);
-    }
-  if (bits_per_pixel > 8)
-    flags|=PALM_IS_DIRECT_COLOR;
-  (void) WriteBlobMSBShort(image,image->columns);  /* width */
-  (void) WriteBlobMSBShort(image,image->rows );  /* height */
-  bytes_per_row=((image->columns+(16/bits_per_pixel-1))/(16/bits_per_pixel))*2;
-  (void) WriteBlobMSBShort(image, bytes_per_row);
-  if ((image->compression == RLECompression) ||
-      (image->compression == FaxCompression))
-    flags|=PALM_IS_COMPRESSED_FLAG;
-  (void) WriteBlobMSBShort(image, flags);
-  (void) WriteBlobByte(image, bits_per_pixel);
-  if (bits_per_pixel > 1)
-    version=1;
-  if ((image->compression == RLECompression) ||
-      (image->compression == FaxCompression))
-    version=2;
-  (void) WriteBlobByte(image,version);
-  (void) WriteBlobMSBShort(image,0);  /* offset */
-  (void) WriteBlobByte(image,transparentIndex);
-  if (image->compression == RLECompression)
-    (void) WriteBlobByte(image,PALM_COMPRESSION_RLE);
-  else
-    if (image->compression == FaxCompression)
-      (void) WriteBlobByte(image,PALM_COMPRESSION_SCANLINE);
-    else
-      (void) WriteBlobByte(image,PALM_COMPRESSION_NONE);
-  (void) WriteBlobMSBShort(image,0);  /* reserved */
-  offset=16;
-  if (bits_per_pixel == 16)
-    {
-      WriteBlobByte(image,5);  /* # of bits of red */
-      WriteBlobByte(image,6);  /* # of bits of green */
-      WriteBlobByte(image,5);  /* # of bits of blue */
-      WriteBlobByte(image,0);  /* reserved by Palm */
-      WriteBlobMSBLong(image,0);  /* no transparent color, YET */
-      offset+=8;
-    }
-  if (bits_per_pixel == 8)
-    {
-      if (flags & PALM_HAS_COLORMAP_FLAG)  /* Write out colormap */
-        {
-          GetQuantizeInfo(&quantize_info);
-          quantize_info.dither=
-            IsPaletteImage(image,&image->exception) == MagickFalse;
-          quantize_info.number_colors=image->colors;
-          (void) QuantizeImage(&quantize_info,image);
-          WriteBlobMSBShort(image,image->colors);
-          for (count = 0; count < image->colors; count++)
-          {
-            WriteBlobByte(image,count);
-            WriteBlobByte(image,ScaleQuantumToChar(image->colormap[count].red));
-            WriteBlobByte(image,
-              ScaleQuantumToChar(image->colormap[count].green));
-            WriteBlobByte(image,
-              ScaleQuantumToChar(image->colormap[count].blue));
-          }
-          offset+=2+count*4;
-        }
-    else  /* Map colors to Palm standard colormap */
-      {
-        map=ConstituteImage(256,1,"RGB",CharPixel,&PalmPalette,&exception);
-        SetImageColorspace(map,map->colorspace);
-        MapImage(image,map,image_info->dither);
-        for (y=0; y < (long) image->rows; y++)
-        {
-          p=GetImagePixels(image,0,y,image->columns,1);
-          indexes=GetIndexes(image);
-          for (x=0; x < (long) image->columns; x++)
-            indexes[x]=FindColor(&image->colormap[indexes[x]]);
-        }
-        DestroyImage(map);
-      }
-    }
-  if (flags & PALM_IS_COMPRESSED_FLAG)
-    WriteBlobMSBShort(image,0);  /* fill in size later */
-  lastrow=(unsigned char *) NULL;
-  if (image->compression == FaxCompression)
-    lastrow=(unsigned char *) AcquireMagickMemory(bytes_per_row);
-  one_row=(unsigned char *) AcquireMagickMemory(bytes_per_row);
-  if (one_row == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  for (y=0; y < (int) image->rows; y++)
+  currentOffset=0;
+  transpix.red=0;
+  transpix.green=0;
+  transpix.blue=0;
+  transpix.opacity=0;
+  do
   {
-    ptr=one_row;
-    (void) ResetMagickMemory(ptr,0,bytes_per_row);
-    p=GetImagePixels(image,0,y,image->columns,1);
-    if (p == (PixelPacket *) NULL)
-      break;
-    indexes=GetIndexes(image);
-    if (bits_per_pixel == 16)
+    attribute=GetImageAttribute(image, "Comment");
+    if (attribute != (ImageAttribute *)NULL)
+      if (LocaleCompare("COLORMAP",attribute->value) == 0)
+        flags|=PALM_HAS_COLORMAP_FLAG;
+    count = GetNumberColors(image, NULL, &exception);
+    for (bits_per_pixel=1;  (1UL << bits_per_pixel) < count;  bits_per_pixel*=2);
+    if (image_info->depth > 100)
+      bits_per_pixel=image_info->depth-100;
+    if (bits_per_pixel < 16)
+      SetImageColorspace(image,image->colorspace);
+    if (bits_per_pixel < 8)
       {
-        for (x=0; x < (int) image->columns; x++)
-        {
-          color16=(((31*p->red)/MaxRGB) << 11) | (((63*p->green)/MaxRGB) << 5) |
-            ((31*p->blue)/MaxRGB);
-          if (p->opacity == TransparentOpacity)
-            {
-              transpix.red=p->red;
-              transpix.green=p->green;
-              transpix.blue=p->blue;
-              transpix.opacity=p->opacity;
-              flags|=PALM_HAS_TRANSPARENCY_FLAG;
-            }
-          *ptr++=(color16 >> 8) & 0xff;
-          *ptr++=color16 & 0xff;
-          p++;
-        }
-      }
-    else
-      {
-        byte=0x00;
-        bit=(unsigned char) (8-bits_per_pixel);
-        for (x=0; x < (int) image->columns; x++)
-        {
-          if (bits_per_pixel >= 8)
-            color=(unsigned char) indexes[x];
-          else
-            color=(unsigned char) (indexes[x]*((1 << bits_per_pixel)-1)/
-              Max(image->colors-1,1));
-          byte|=color << bit;
-          if (bit != 0)
-            bit-=bits_per_pixel;
-          else
-            {
-              *ptr++=byte;
-              byte=0x00;
-              bit=(unsigned char) (8-bits_per_pixel);
-            }
-        }
-        if ((image->columns % (8/bits_per_pixel)) != 0)
-          *ptr++=byte;
+        SetImageColorspace(image,GRAYColorspace);
+        (void) SortColormapByIntensity(image);
       }
+    if (bits_per_pixel > 8)
+      flags|=PALM_IS_DIRECT_COLOR;
+    (void) WriteBlobMSBShort(image,image->columns);  /* width */
+    (void) WriteBlobMSBShort(image,image->rows );  /* height */
+    bytes_per_row=((image->columns+(16/bits_per_pixel-1))/(16/bits_per_pixel))*2;
+    (void) WriteBlobMSBShort(image, bytes_per_row);
+    if ((image->compression == RLECompression) ||
+        (image->compression == FaxCompression))
+      flags|=PALM_IS_COMPRESSED_FLAG;
+    (void) WriteBlobMSBShort(image, flags);
+    (void) WriteBlobByte(image, bits_per_pixel);
+    if (bits_per_pixel > 1)
+      version=1;
+    if ((image->compression == RLECompression) ||
+        (image->compression == FaxCompression))
+      version=2;
+    (void) WriteBlobByte(image,version);
+    (void) WriteBlobMSBShort(image,0);  /* nextDepthOffset */
+    (void) WriteBlobByte(image,transparentIndex);
     if (image->compression == RLECompression)
-      {
-        x=0;
-        while (x < (long) bytes_per_row)
-        {
-          byte=one_row[x];
-          count=1;
-          while ((one_row[++x] == byte) && (count < 255) &&
-                 (x < (long) bytes_per_row))
-            count++;
-          (void) WriteBlobByte(image,count);
-          (void) WriteBlobByte(image,byte);
-        }
-      }
+      (void) WriteBlobByte(image,PALM_COMPRESSION_RLE);
     else
       if (image->compression == FaxCompression)
-        {
-          char
-            tmpbuf[8],
-            *tptr;
-
-          for (x = 0;  x < (long) bytes_per_row;  x += 8)
+        (void) WriteBlobByte(image,PALM_COMPRESSION_SCANLINE);
+      else
+        (void) WriteBlobByte(image,PALM_COMPRESSION_NONE);
+    (void) WriteBlobMSBShort(image,0);  /* reserved */
+    offset=16;
+    if (bits_per_pixel == 16)
+      {
+        WriteBlobByte(image,5);  /* # of bits of red */
+        WriteBlobByte(image,6);  /* # of bits of green */
+        WriteBlobByte(image,5);  /* # of bits of blue */
+        WriteBlobByte(image,0);  /* reserved by Palm */
+        WriteBlobMSBLong(image,0);  /* no transparent color, YET */
+        offset+=8;
+      }
+    if (bits_per_pixel == 8)
+      {
+        if (flags & PALM_HAS_COLORMAP_FLAG)  /* Write out colormap */
           {
-            tptr = tmpbuf;
-            for (bit = 0, byte = 0; bit < Min(8, bytes_per_row - x); bit++)
+            GetQuantizeInfo(&quantize_info);
+            quantize_info.dither=IsPaletteImage(image,&image->exception);
+            quantize_info.number_colors=image->colors;
+            (void) QuantizeImage(&quantize_info,image);
+            WriteBlobMSBShort(image,image->colors);
+            for (count = 0; count < image->colors; count++)
             {
-              if ((y == 0) || (lastrow[x + bit] != one_row[x + bit]))
-                {
-                byte |= (1 << (7 - bit));
-                *tptr++ = one_row[x + bit];
-              }
+              WriteBlobByte(image,count);
+              WriteBlobByte(image,ScaleQuantumToChar(image->colormap[count].red));
+              WriteBlobByte(image,
+                ScaleQuantumToChar(image->colormap[count].green));
+              WriteBlobByte(image,
+                ScaleQuantumToChar(image->colormap[count].blue));
             }
-            (void) WriteBlobByte(image, byte);
-            (void) WriteBlob(image,tptr-tmpbuf,(unsigned char *) tmpbuf);
+            offset+=2+count*4;
+          }
+      else  /* Map colors to Palm standard colormap */
+        {
+          map=ConstituteImage(256,1,"RGB",CharPixel,&PalmPalette,&exception);
+          SetImageColorspace(map,map->colorspace);
+          MapImage(image,map,image_info->dither);
+          for (y=0; y < (long) image->rows; y++)
+          {
+            p=GetImagePixels(image,0,y,image->columns,1);
+            indexes=GetIndexes(image);
+            for (x=0; x < (long) image->columns; x++)
+              indexes[x]=FindColor(&image->colormap[indexes[x]]);
+          }
+          DestroyImage(map);
+        }
+      }
+    if (flags & PALM_IS_COMPRESSED_FLAG)
+      WriteBlobMSBShort(image,0);  /* fill in size later */
+    lastrow=(unsigned char *) NULL;
+    if (image->compression == FaxCompression)
+      lastrow=(unsigned char *) AcquireMagickMemory(bytes_per_row);
+      /* TODO check whether memory really was acquired? */
+    one_row=(unsigned char *) AcquireMagickMemory(bytes_per_row);
+    if (one_row == (unsigned char *) NULL)
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    for (y=0; y < (int) image->rows; y++)
+    {
+      ptr=one_row;
+      (void) ResetMagickMemory(ptr,0,bytes_per_row);
+      p=GetImagePixels(image,0,y,image->columns,1);
+      if (p == (PixelPacket *) NULL)
+        break;
+      indexes=GetIndexes(image);
+      if (bits_per_pixel == 16)
+        {
+          for (x=0; x < (int) image->columns; x++)
+          {
+            color16=(((31*p->red)/QuantumRange) << 11) | (((63*p->green)/QuantumRange) << 5) |
+              ((31*p->blue)/QuantumRange);
+            if (p->opacity == TransparentOpacity)
+              {
+                transpix.red=p->red;
+                transpix.green=p->green;
+                transpix.blue=p->blue;
+                transpix.opacity=p->opacity;
+                flags|=PALM_HAS_TRANSPARENCY_FLAG;
+              }
+            *ptr++=(color16 >> 8) & 0xff;
+            *ptr++=color16 & 0xff;
+            p++;
           }
-          (void) CopyMagickMemory(lastrow,one_row,bytes_per_row);
         }
       else
-        (void) WriteBlob(image,bytes_per_row,one_row);
-    }
-  if (flags & PALM_HAS_TRANSPARENCY_FLAG)
+        {
+          byte=0x00;
+          bit=(unsigned char) (8-bits_per_pixel);
+          for (x=0; x < (int) image->columns; x++)
+          {
+            if (bits_per_pixel >= 8)
+              color=(unsigned char) indexes[x];
+            else
+              color=(unsigned char) (indexes[x]*((1 << bits_per_pixel)-1)/
+                Max(image->colors-1,1));
+            byte|=color << bit;
+            if (bit != 0)
+              bit-=bits_per_pixel;
+            else
+              {
+                *ptr++=byte;
+                byte=0x00;
+                bit=(unsigned char) (8-bits_per_pixel);
+              }
+          }
+          if ((image->columns % (8/bits_per_pixel)) != 0)
+            *ptr++=byte;
+        }
+      if (image->compression == RLECompression)
+        {
+          x=0;
+          while (x < (long) bytes_per_row)
+          {
+            byte=one_row[x];
+            count=1;
+            while ((one_row[++x] == byte) && (count < 255) &&
+                   (x < (long) bytes_per_row))
+              count++;
+            (void) WriteBlobByte(image,count);
+            (void) WriteBlobByte(image,byte);
+          }
+        }
+      else
+        if (image->compression == FaxCompression)
+          {
+            char
+              tmpbuf[8],
+              *tptr;
+  
+            for (x = 0;  x < (long) bytes_per_row;  x += 8)
+            {
+              tptr = tmpbuf;
+              for (bit = 0, byte = 0; bit < Min(8, bytes_per_row - x); bit++)
+              {
+                if ((y == 0) || (lastrow[x + bit] != one_row[x + bit]))
+                  {
+                  byte |= (1 << (7 - bit));
+                  *tptr++ = one_row[x + bit];
+                }
+              }
+              (void) WriteBlobByte(image, byte);
+              (void) WriteBlob(image,tptr-tmpbuf,(unsigned char *) tmpbuf);
+            }
+            (void) CopyMagickMemory(lastrow,one_row,bytes_per_row);
+          }
+        else
+          (void) WriteBlob(image,bytes_per_row,one_row);
+      }
+    if (flags & PALM_HAS_TRANSPARENCY_FLAG)
+      {
+        SeekBlob(image,currentOffset+6,SEEK_SET);
+        (void) WriteBlobMSBShort(image,flags);
+        SeekBlob(image,currentOffset+12,SEEK_SET);
+        (void) WriteBlobByte(image,transparentIndex);  /* trans index */
+      }
+    if (bits_per_pixel == 16)
+      {
+        SeekBlob(image,currentOffset+20,SEEK_SET);
+        WriteBlobByte(image,0);  /* reserved by Palm */
+        WriteBlobByte(image,(31*transpix.red)/QuantumRange);
+        WriteBlobByte(image,(63*transpix.green)/QuantumRange);
+        WriteBlobByte(image,(31*transpix.blue)/QuantumRange);
+      }
+    if (flags & PALM_IS_COMPRESSED_FLAG)  /* fill in size now */
+      {
+        SeekBlob(image,currentOffset+offset,SEEK_SET);
+        (void) WriteBlobMSBShort(image,GetBlobSize(image)-currentOffset-offset);
+      }
+    if (one_row != (unsigned char *) NULL) 
+      one_row=(unsigned char *) RelinquishMagickMemory(one_row);
+    if (lastrow != (unsigned char *) NULL) 
+      lastrow=(unsigned char *) RelinquishMagickMemory(lastrow);
+    if (GetNextImageInList(image) == (Image *) NULL)
+      break;
+    /* padding to 4 byte word */
+    for (cc = (GetBlobSize(image))%4; cc > 0; cc--)
     {
-      SeekBlob(image,6,SEEK_SET);
-      (void) WriteBlobMSBShort(image,flags);
-      SeekBlob(image,12,SEEK_SET);
-      (void) WriteBlobByte(image,transparentIndex);  /* trans index */
-    }
-  if (bits_per_pixel == 16)
-    {
-      SeekBlob(image,20,SEEK_SET);
-      WriteBlobByte(image,0);  /* reserved by Palm */
-      WriteBlobByte(image,(31*transpix.red)/MaxRGB);
-      WriteBlobByte(image,(63*transpix.green)/MaxRGB);
-      WriteBlobByte(image,(31*transpix.blue)/MaxRGB);
-    }
-  if (flags & PALM_IS_COMPRESSED_FLAG)  /* fill in size now */
-    {
-      SeekBlob(image,offset,SEEK_SET);
-      (void) WriteBlobMSBShort(image,GetBlobSize(image)-16);
-    }
+      (void) WriteBlobByte(image,0);
+    } 
+    /* write nextDepthOffset and return to end of image */
+    SeekBlob(image,currentOffset+10,SEEK_SET);
+    nextDepthOffset=(unsigned long) ((GetBlobSize(image)-currentOffset)/4);
+    (void) WriteBlobMSBShort(image,nextDepthOffset);
+    currentOffset=(MagickOffsetType) GetBlobSize(image);
+    SeekBlob(image,currentOffset,SEEK_SET);
+    image=SyncNextImageInList(image);
+  } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
-  one_row=(unsigned char *) RelinquishMagickMemory(one_row);
-  if (image->compression == FaxCompression)
-    lastrow=(unsigned char *) RelinquishMagickMemory(lastrow);
   DestroyExceptionInfo(&exception);
   return(MagickTrue);
 }
