@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -41,14 +41,17 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
 #include "magick/client.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
 #include "magick/decorate.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/gem.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -84,7 +87,7 @@ static MagickBooleanType
 %
 %  The format of the DecodeImage method is:
 %
-%      static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
+%      MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
 %        unsigned char *chroma1,unsigned char *chroma2)
 %
 %  A description of each parameter follows:
@@ -116,10 +119,11 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   { \
     if (p >= (buffer+0x800)) \
       { \
-        (void) ReadBlob(image,0x800,buffer); \
+        count=ReadBlob(image,0x800,buffer); \
         p=buffer; \
       } \
     sum|=((unsigned int) (*p) << (24-bits)); \
+    bits+=8; \
     p++; \
   } \
   if (EOFBlob(image) != MagickFalse) \
@@ -179,7 +183,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(luma != (unsigned char *) NULL);
   assert(chroma1 != (unsigned char *) NULL);
   assert(chroma2 != (unsigned char *) NULL);
@@ -194,7 +198,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   {
     PCDGetBits(8);
     length=(sum & 0xff)+1;
-    pcd_table[i]=(PCDTable *) AcquireMagickMemory(length*sizeof(*pcd_table));
+    pcd_table[i]=(PCDTable *) AcquireMagickMemory(length*sizeof(**pcd_table));
     if (pcd_table[i] == (PCDTable *) NULL)
       {
         buffer=(unsigned char *) RelinquishMagickMemory(buffer);
@@ -215,7 +219,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
       r->sequence=(sum & 0xffff) << 16;
       PCDGetBits(8);
       r->key=(unsigned char) (sum & 0xff);
-      r->mask=(~((((MagickBooleanType) 1) << (32-r->length))-1));
+      r->mask=(~((1U << (32-r->length))-1));
       r++;
     }
     pcd_length[i]=length;
@@ -282,10 +286,11 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
           }
         }
         length=pcd_length[plane];
-        if (QuantumTick(row,image->rows))
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(row,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(DecodeImageTag,row,image->rows,
-              &image->exception);
+            status=image->progress_monitor(DecodeImageTag,row,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
@@ -301,7 +306,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
         ((size_t) (q-luma) > (image->columns*image->rows)))
       {
         (void) ThrowMagickException(&image->exception,GetMagickModule(),
-          CorruptImageWarning,"SkipToSyncByte",image->filename);
+          CorruptImageWarning,"SkipToSyncByte","`%s'",image->filename);
         while ((sum & 0x00fff000) != 0x00fff000)
           PCDGetBits(8);
         while (IsSync == 0)
@@ -394,7 +399,7 @@ static MagickBooleanType IsPCD(const unsigned char *magick,const size_t length)
 static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   ExceptionInfo *exception)
 {
-  char
+  const char
     *commands[3];
 
   Image
@@ -410,11 +415,11 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
     Create image tiles.
   */
   mogrify_info=CloneImageInfo(image_info);
-  commands[0]=(char *) GetClientName();
-  commands[1]=(char *) "-label";
-  commands[2]=(char *) DefaultTileLabel;
-  (void) MogrifyImages(mogrify_info,3,commands,&image);
-  DestroyImageInfo(mogrify_info);
+  commands[0]=GetClientName();
+  commands[1]="-label";
+  commands[2]=DefaultTileLabel;
+  (void) MogrifyImages(mogrify_info,3,commands,&image,exception);
+  mogrify_info=DestroyImageInfo(mogrify_info);
   /*
     Create the PCD Overview image.
   */
@@ -422,10 +427,10 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   (void) CopyMagickString(montage_info->filename,image_info->filename,
     MaxTextExtent);
   montage_image=MontageImages(image,montage_info,exception);
-  DestroyMontageInfo(montage_info);
+  montage_info=DestroyMontageInfo(montage_info);
   if (montage_image == (Image *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  DestroyImage(image);
+  image=DestroyImage(image);
   return(montage_image);
 }
 
@@ -483,14 +488,15 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -577,8 +583,8 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Image
         *overview_image;
 
-      MonitorHandler
-        handler;
+      MagickProgressMonitor
+        progress_monitor;
 
       register long
         j;
@@ -588,7 +594,8 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       for (j=1; j <= (long) number_images; j++)
       {
-        handler=SetMonitorHandler((MonitorHandler) NULL);
+        progress_monitor=SetImageProgressMonitor(image,
+          (MagickProgressMonitor) NULL,image->client_data);
         (void) FormatMagickString(image->filename,MaxTextExtent,
           "images/img%04ld.pcd",j);
         (void) FormatMagickString(image->magick_filename,MaxTextExtent,
@@ -602,13 +609,13 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         c2=chroma2;
         for (y=0; y < (long) height; y+=2)
         {
-          (void) ReadBlob(image,width,yy);
+          count=ReadBlob(image,width,yy);
           yy+=image->columns;
-          (void) ReadBlob(image,width,yy);
+          count=ReadBlob(image,width,yy);
           yy+=image->columns;
-          (void) ReadBlob(image,width >> 1,c1);
+          count=ReadBlob(image,width >> 1,c1);
           c1+=image->columns;
-          (void) ReadBlob(image,width >> 1,c2);
+          count=ReadBlob(image,width >> 1,c2);
           c2+=image->columns;
         }
         Upsample(image->columns >> 1,image->rows >> 1,image->columns,chroma1);
@@ -637,29 +644,34 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         image->colorspace=YCCColorspace;
         if (LocaleCompare(image_info->magick,"PCDS") == 0)
           image->colorspace=sRGBColorspace;
-        SetImageColorspace(image,RGBColorspace);
         if (j < (long) number_images)
           {
             /*
               Allocate next image structure.
             */
             AllocateNextImage(image_info,image);
-            if (image->next == (Image *) NULL)
+            if (GetNextImageInList(image) == (Image *) NULL)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
             image=SyncNextImageInList(image);
           }
-        (void) SetMonitorHandler(handler);
-        if (MagickMonitor(LoadImageTag,j-1,number_images,exception) == MagickFalse)
-          break;
+        (void) SetImageProgressMonitor(image,progress_monitor,
+          image->client_data);
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(j-1,number_images) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,j-1,number_images,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
       chroma2=(unsigned char *) RelinquishMagickMemory(chroma2);
       chroma1=(unsigned char *) RelinquishMagickMemory(chroma1);
       luma=(unsigned char *) RelinquishMagickMemory(luma);
-      while (image->previous != (Image *) NULL)
-        image=image->previous;
+      image=GetFirstImageInList(image);
       overview_image=OverviewImage(image_info,image,exception);
       return(overview_image);
     }
@@ -671,13 +683,13 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   c2=chroma2;
   for (y=0; y < (long) height; y+=2)
   {
-    (void) ReadBlob(image,width,yy);
+    count=ReadBlob(image,width,yy);
     yy+=image->columns;
-    (void) ReadBlob(image,width,yy);
+    count=ReadBlob(image,width,yy);
     yy+=image->columns;
-    (void) ReadBlob(image,width >> 1,c1);
+    count=ReadBlob(image,width >> 1,c1);
     c1+=image->columns;
-    (void) ReadBlob(image,width >> 1,c2);
+    count=ReadBlob(image,width >> 1,c2);
     c2+=image->columns;
   }
   if (scene >= 4)
@@ -738,9 +750,14 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   chroma2=(unsigned char *) RelinquishMagickMemory(chroma2);
   chroma1=(unsigned char *) RelinquishMagickMemory(chroma1);
@@ -765,7 +782,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         rotate_image=RotateImage(image,degrees,exception);
         if (rotate_image != (Image *) NULL)
           {
-            DestroyImage(image);
+            image=DestroyImage(image);
             image=rotate_image;
           }
       }
@@ -784,7 +801,6 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->colorspace=YCCColorspace;
   if (LocaleCompare(image_info->magick,"PCDS") == 0)
     image->colorspace=sRGBColorspace;
-  SetImageColorspace(image,RGBColorspace);
   return(GetFirstImageInList(image));
 }
 
@@ -886,7 +902,7 @@ ModuleExport void UnregisterPCDImage(void)
 */
 
 static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
-  char *page_geometry,char *tile_geometry)
+  const char *page_geometry,const char *tile_geometry)
 {
   GeometryInfo
     geometry_info;
@@ -951,7 +967,7 @@ static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
       bordered_image=BorderImage(tile_image,&border_info,&image->exception);
       if (bordered_image == (Image *) NULL)
         return(MagickFalse);
-      DestroyImage(tile_image);
+      tile_image=DestroyImage(tile_image);
       tile_image=bordered_image;
     }
   (void) TransformImage(&tile_image,(char *) NULL,tile_geometry);
@@ -992,17 +1008,19 @@ static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
       (void) WriteBlobByte(image,ScaleQuantumToChar(q->blue));
       q++;
     }
-    if (QuantumTick(y,tile_image->rows))
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,tile_image->rows) != MagickFalse))
       {
-        status=MagickMonitor(SaveImageTag,y,tile_image->rows,&image->exception);
+        status=image->progress_monitor(LoadImageTag,y,tile_image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
   }
   for (i=0; i < 0x800; i++)
     (void) WriteBlobByte(image,'\0');
-  DestroyImage(downsample_image);
-  DestroyImage(tile_image);
+  downsample_image=DestroyImage(downsample_image);
+  tile_image=DestroyImage(tile_image);
   return(MagickTrue);
 }
 
@@ -1022,7 +1040,7 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   pcd_image=image;
   if (image->columns < image->rows)
     {
@@ -1080,14 +1098,11 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   /*
     Write PCD tiles.
   */
-  status=WritePCDTile(image_info,pcd_image,(char *) "768x512>",
-    (char *) "192x128");
-  status=WritePCDTile(image_info,pcd_image,(char *) "768x512>",
-    (char *) "384x256");
-  status=WritePCDTile(image_info,pcd_image,(char *) "768x512>",
-    (char *) "768x512");
+  status=WritePCDTile(image_info,pcd_image,"768x512>","192x128");
+  status=WritePCDTile(image_info,pcd_image,"768x512>","384x256");
+  status=WritePCDTile(image_info,pcd_image,"768x512>","768x512");
   CloseBlob(pcd_image);
   if (pcd_image != image)
-    DestroyImage(pcd_image);
+    pcd_image=DestroyImage(pcd_image);
   return(status);
 }
