@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -42,14 +42,19 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
 #include "magick/color.h"
-#include "magick/color_private.h"
+#include "magick/color-private.h"
+#include "magick/colorspace.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
-#include "magick/error.h"
+#include "magick/draw.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -141,7 +146,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 #define CropBox  "CropBox"
 #define DeviceCMYK  "DeviceCMYK"
 #define MediaBox  "MediaBox"
-#define RenderPostscriptText  "  Rendering postscript...  "
+#define RenderPostscriptText  "  Rendering Postscript...  "
 
   char
     command[MaxTextExtent],
@@ -154,10 +159,6 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   const DelegateInfo
     *delegate_info;
 
-  double
-    dx_resolution,
-    dy_resolution;
-
   Image
     *image,
     *next_image;
@@ -172,6 +173,9 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     cmyk,
     status;
 
+  PointInfo
+    delta;
+
   RectangleInfo
     bounding_box,
     page;
@@ -190,12 +194,14 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   unsigned long
     height,
+    scene,
     width;
 
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   /*
@@ -205,14 +211,14 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
     Set the page density.
   */
-  dx_resolution=72.0;
-  dy_resolution=72.0;
+  delta.x=DefaultResolution;
+  delta.y=DefaultResolution;
   if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
     {
       GeometryInfo
@@ -237,6 +243,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) ResetMagickMemory(&bounding_box,0,sizeof(bounding_box));
   (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   (void) ResetMagickMemory(&page,0,sizeof(page));
+  (void) ResetMagickMemory(command,0,sizeof(command));
   for (p=command; (c=ReadBlobByte(image)) != EOF; )
   {
     if (image_info->page != (char *) NULL)
@@ -245,7 +252,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Note PDF elements.
     */
     *p++=(char) c;
-    if ((c != (int) '/') && ((size_t) (p-command) < (MaxTextExtent-1)))
+    if ((c != (int) '/') && ((size_t) (p-command) < (MaxTextExtent/2)))
       continue;
     *p='\0';
     p=command;
@@ -289,7 +296,6 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       page.height=height;
   }
   CloseBlob(image);
-  DestroyImage(image);
   /*
     Create Ghostscript control file.
   */
@@ -298,30 +304,32 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
         image_info->filename);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   (void) write(file," ",1);
-  (void) close(file);
+  file=close(file)-1;
   /*
-    Render postscript with the Ghostscript delegate.
+    Render Postscript with the Ghostscript delegate.
   */
   if ((page.width == 0) || (page.height == 0))
     (void) ParseAbsoluteGeometry(PSPageGeometry,&page);
   if (image_info->page != (char *) NULL)
     (void) ParseAbsoluteGeometry(image_info->page,&page);
   page.width=(unsigned long)
-    (page.width*image->x_resolution/dx_resolution+0.5);
+    (page.width*image->x_resolution/delta.x+0.5);
   page.height=(unsigned long)
-    (page.height*image->y_resolution/dy_resolution+0.5);
+    (page.height*image->y_resolution/delta.y+0.5);
+  image=DestroyImage(image);
   (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",
     page.width,page.height);
   if (image_info->monochrome != MagickFalse)
-    delegate_info=GetDelegateInfo("gs-mono",(char *) NULL,exception);
+    delegate_info=GetDelegateInfo("ps:mono",(char *) NULL,exception);
   else
      if (cmyk != MagickFalse)
-       delegate_info=GetDelegateInfo("gs-cmyk",(char *) NULL,exception);
+       delegate_info=GetDelegateInfo("ps:cmyk",(char *) NULL,exception);
      else
-       delegate_info=GetDelegateInfo("gs-color",(char *) NULL,exception);
+       delegate_info=GetDelegateInfo("ps:color",(char *) NULL,exception);
   if (delegate_info == (const DelegateInfo *) NULL)
     {
       (void) RelinquishUniqueFileResource(postscript_filename);
@@ -353,13 +361,11 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,geometry,density,options,
     read_info->filename,postscript_filename,image_info->filename);
-  (void) MagickMonitor(RenderPostscriptText,0,8,&image->exception);
   status=InvokePostscriptDelegate(read_info->verbose,command);
   image=ReadImage(read_info,exception);
-  (void) MagickMonitor(RenderPostscriptText,7,8,&image->exception);
   (void) RelinquishUniqueFileResource(postscript_filename);
   (void) RelinquishUniqueFileResource(read_info->filename);
-  DestroyImageInfo(read_info);
+  read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     ThrowReaderException(DelegateError,"PostscriptDelegateFailed");
   if (LocaleCompare(image->magick,"BMP") == 0)
@@ -370,14 +376,16 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       cmyk_image=ConsolidateCMYKImages(image,&image->exception);
       if (cmyk_image != (Image *) NULL)
         {
-          DestroyImageList(image);
+          image=DestroyImageList(image);
           image=cmyk_image;
         }
     }
+  scene=0;
   do
   {
     (void) CopyMagickString(image->filename,filename,MaxTextExtent);
     image->page=page;
+    image->scene=scene++;
     next_image=SyncNextImageInList(image);
     if (next_image != (Image *) NULL)
       image=next_image;
@@ -531,14 +539,6 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   const ImageAttribute
     *attribute;
 
-  double
-    dx_resolution,
-    dy_resolution,
-    x_resolution,
-    x_scale,
-    y_resolution,
-    y_scale;
-
   GeometryInfo
     geometry_info;
 
@@ -550,11 +550,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     *tile_image;
 
   MagickBooleanType
+    matte,
     status;
 
   MagickOffsetType
     offset,
-    scene;
+    scene,
+    *xref;
 
   MagickSizeType
     number_pixels;
@@ -562,6 +564,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   MagickStatusType
     flags;
 
+  PointInfo
+    delta,
+    resolution,
+    scale;
+
   RectangleInfo
     geometry,
     media_info,
@@ -596,12 +603,10 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     info_id,
     object,
     pages_id,
+    quality,
     root_id,
     text_size;
 
-  MagickOffsetType
-    *xref;
-
   /*
     Open output image file.
   */
@@ -610,11 +615,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
   compression=image->compression;
+  quality=(unsigned long)
+    (image->quality == UndefinedCompressionQuality ? 75 : image->quality);
   switch (compression)
   {
 #if !defined(HasJPEG)
@@ -622,7 +629,17 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     {
       compression=RLECompression;
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"JPEGLibraryIsNotAvailable",image->filename);
+        MissingDelegateError,"JPEGLibraryIsNotAvailable","`%s'",
+        image->filename);
+      break;
+    }
+#endif
+#if !defined(HasJP2)
+    case JPEG2000Compression:
+    {
+      compression=RLECompression;
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        MissingDelegateError,"JP2LibraryIsNotAvailable","`%s'",image->filename);
       break;
     }
 #endif
@@ -631,7 +648,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     {
       compression=RLECompression;
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"ZipLibraryIsNotAvailable",image->filename);
+        MissingDelegateError,"ZipLibraryIsNotAvailable","`%s'",image->filename);
       break;
     }
 #endif
@@ -649,10 +666,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     Write Info object.
   */
   object=0;
-  if (image->matte == MagickFalse)
-    (void) WriteBlobString(image,"%PDF-1.3 \n");
+  if (compression == JPEG2000Compression)
+    (void) WriteBlobString(image,"%PDF-1.5 \n");
   else
-    (void) WriteBlobString(image,"%PDF-1.4 \n");
+    if (image->matte != MagickFalse)
+      (void) WriteBlobString(image,"%PDF-1.4 \n");
+    else
+      (void) WriteBlobString(image,"%PDF-1.3 \n");
   xref[object++]=TellBlob(image);
   info_id=object;
   (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
@@ -666,8 +686,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   (void) FormatMagickString(date,MaxTextExtent,"D:%04d%02d%02d%02d%02d%02d",
     time_meridian->tm_year+1900,time_meridian->tm_mon+1,time_meridian->tm_mday,
     time_meridian->tm_hour,time_meridian->tm_min,time_meridian->tm_sec);
-  (void) FormatMagickString(buffer,MaxTextExtent,"/CreationDate (%s)\n",
-    date);
+  (void) FormatMagickString(buffer,MaxTextExtent,"/CreationDate (%s)\n",date);
   (void) WriteBlobString(image,buffer);
   (void) FormatMagickString(buffer,MaxTextExtent,"/ModDate (%s)\n",date);
   (void) WriteBlobString(image,buffer);
@@ -710,11 +729,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         Predict page object id's.
       */
       kid_image=image;
-      for ( ; kid_image->next != (Image *) NULL; count+=ObjectsPerImage)
+      for ( ; GetNextImageInList(kid_image) != (Image *) NULL; count+=ObjectsPerImage)
       {
         (void) FormatMagickString(buffer,MaxTextExtent,"%ld 0 R ",count);
         (void) WriteBlobString(image,buffer);
-        kid_image=kid_image->next;
+        kid_image=GetNextImageInList(kid_image);
       }
       xref=(MagickOffsetType *)
         ResizeMagickMemory(xref,(size_t) (count+2048)*sizeof(*xref));
@@ -731,15 +750,33 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   do
   {
     /*
-      Scale image to size of Portable Document page.
+      Scale relative to dots-per-inch.
     */
-    text_size=0;
-    attribute=GetImageAttribute(image,"label");
-    if (attribute != (const ImageAttribute *) NULL)
-      text_size=(unsigned long)
-        (MultilineCensus(attribute->value)*image_info->pointsize+12);
+    delta.x=DefaultResolution;
+    delta.y=DefaultResolution;
+    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+    resolution.x=geometry_info.rho;
+    resolution.y=geometry_info.sigma;
+    if ((flags & SigmaValue) == 0)
+      resolution.y=resolution.x;
+    if (image->x_resolution != 0.0)
+      resolution.x=image->x_resolution;
+    if (image->y_resolution != 0.0)
+      resolution.y=image->y_resolution;
+    if (image_info->density != (char *) NULL)
+      {
+        flags=ParseGeometry(image_info->density,&geometry_info);
+        resolution.x=geometry_info.rho;
+        resolution.y=geometry_info.sigma;
+        if ((flags & SigmaValue) == 0)
+          resolution.y=resolution.x;
+      }
+    if (image->units == PixelsPerCentimeterResolution)
+      {
+        resolution.x*=2.54;
+        resolution.y*=2.54;
+      }
     SetGeometry(image,&geometry);
-    geometry.y=(long) text_size;
     (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
       image->columns,image->rows);
     if (image_info->page != (char *) NULL)
@@ -754,36 +791,22 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
           (void) strcpy(page_geometry,PSPageGeometry);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
+    scale.x=(double) (geometry.width*delta.x)/resolution.x;
+    geometry.width=(unsigned long) (scale.x+0.5);
+    scale.y=(double) (geometry.height*delta.y)/resolution.y;
+    geometry.height=(unsigned long) (scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
     (void) ParseGravityGeometry(image,page_geometry,&page_info);
-    /*
-      Scale relative to dots-per-inch.
-    */
-    dx_resolution=72.0;
-    dy_resolution=72.0;
-    x_resolution=72.0;
-    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    x_resolution=geometry_info.rho;
-    y_resolution=geometry_info.sigma;
-    if ((flags & SigmaValue) == 0)
-      y_resolution=x_resolution;
-    if (image_info->density != (char *) NULL)
+    if (image->gravity != UndefinedGravity)
       {
-        flags=ParseGeometry(image_info->density,&geometry_info);
-        x_resolution=geometry_info.rho;
-        y_resolution=geometry_info.sigma;
-        if ((flags & SigmaValue) == 0)
-          y_resolution=x_resolution;
+        geometry.x=(-page_info.x);
+        geometry.y=(long) (media_info.height+page_info.y-image->rows);
       }
-    if (image->units == PixelsPerCentimeterResolution)
-      {
-        x_resolution*=2.54;
-        y_resolution*=2.54;
-      }
-    x_scale=(double) (geometry.width*dx_resolution)/x_resolution;
-    geometry.width=(unsigned long) (x_scale+0.5);
-    y_scale=(double) (geometry.height*dy_resolution)/y_resolution;
-    geometry.height=(unsigned long) (y_scale+0.5);
+    text_size=0;
+    attribute=GetImageAttribute(image,"Label");
+    if (attribute != (const ImageAttribute *) NULL)
+      text_size=(unsigned long)
+        (MultilineCensus(attribute->value)*image_info->pointsize+12);
     /*
       Write Page object.
     */
@@ -799,9 +822,17 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) FormatMagickString(buffer,MaxTextExtent,
       "/Font << /F%lu %lu 0 R >>\n",image->scene,object+4);
     (void) WriteBlobString(image,buffer);
+    matte=image->matte;
+    if (IsGrayImage(image,&image->exception) != MagickFalse)
+      matte=MagickFalse;
+    if (compression == JPEG2000Compression)
+      {
+	      (void) SetImageColorspace(image,RGBColorspace);
+        matte=MagickFalse;
+      }
     (void) FormatMagickString(buffer,MaxTextExtent,
       "/XObject << /Im%lu %lu 0 R >>\n",image->scene,object+
-      (image->matte != MagickFalse ? 7 : 5));
+      (matte != MagickFalse ? 7 : 5));
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/ProcSet %lu 0 R >>\n",
       object+3);
@@ -817,7 +848,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       object+1);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/Thumb %lu 0 R\n",
-      object+(image->matte != MagickFalse ? 10 : 8));
+      object+(matte != MagickFalse ? 10 : 8));
     (void) WriteBlobString(image,buffer);
     (void) WriteBlobString(image,">>\n");
     (void) WriteBlobString(image,"endobj\n");
@@ -836,7 +867,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     offset=TellBlob(image);
     (void) WriteBlobString(image,"q\n");
     labels=(char **) NULL;
-    attribute=GetImageAttribute(image,"label");
+    attribute=GetImageAttribute(image,"Label");
     if (attribute != (const ImageAttribute *) NULL)
       labels=StringToList(attribute->value);
     if (labels != (char **) NULL)
@@ -848,7 +879,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             image->scene,image_info->pointsize);
           (void) WriteBlobString(image,buffer);
           (void) FormatMagickString(buffer,MaxTextExtent,"%ld %ld Td\n",
-            geometry.x,geometry.y+geometry.height+i*image_info->pointsize+12);
+            geometry.x,(long) (geometry.y+geometry.height+i*
+            image_info->pointsize+12));
           (void) WriteBlobString(image,buffer);
           (void) FormatMagickString(buffer,MaxTextExtent,"(%s) Tj\n",
             labels[i]);
@@ -859,7 +891,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         labels=(char **) RelinquishMagickMemory(labels);
       }
     (void) FormatMagickString(buffer,MaxTextExtent,"%g 0 0 %g %ld %ld cm\n",
-      x_scale,y_scale,geometry.x,geometry.y);
+      scale.x,scale.y,geometry.x,geometry.y);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/Im%lu Do\n",image->scene);
     (void) WriteBlobString(image,buffer);
@@ -909,7 +941,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobString(image,"/Encoding /MacRomanEncoding\n");
     (void) WriteBlobString(image,">>\n");
     (void) WriteBlobString(image,"endobj\n");
-    if (image->matte != MagickFalse)
+    if (matte != MagickFalse)
       {
         /*
           Write softmask object.
@@ -979,7 +1011,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             pixels=(unsigned char *) AcquireMagickMemory(length);
             if (pixels == (unsigned char *) NULL)
               {
-                DestroyImage(image);
+                image=DestroyImage(image);
                 ThrowWriterException(ResourceLimitError,
                   "MemoryAllocationFailed");
               }
@@ -995,14 +1027,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 break;
               for (x=0; x < (long) image->columns; x++)
               {
-                *q++=ScaleQuantumToChar(MaxRGB-p->opacity);
+                *q++=ScaleQuantumToChar(QuantumRange-p->opacity);
                 p++;
               }
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                75UL : image_info->quality,pixels);
+              status=ZLIBEncodeImage(image,length,quality,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1031,7 +1062,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 break;
               for (x=0; x < (long) image->columns; x++)
               {
-                Ascii85Encode(image,ScaleQuantumToChar(MaxRGB-p->opacity));
+                Ascii85Encode(image,ScaleQuantumToChar(QuantumRange-p->opacity));
                 p++;
               }
             }
@@ -1081,6 +1112,15 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         (void) strcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n");
         break;
       }
+      case JPEG2000Compression:
+      {
+        (void) FormatMagickString(buffer,MaxTextExtent,CFormat,"JPXDecode");
+        if (image->colorspace != CMYKColorspace)
+          break;
+        (void) WriteBlobString(image,buffer);
+        (void) strcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n");
+        break;
+      }
       case LZWCompression:
       {
          (void) FormatMagickString(buffer,MaxTextExtent,CFormat,"LZWDecode");
@@ -1119,7 +1159,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) FormatMagickString(buffer,MaxTextExtent,"/BitsPerComponent %d\n",
       compression == FaxCompression ? 1 : 8);
     (void) WriteBlobString(image,buffer);
-    if (image->matte != MagickFalse)
+    if (matte != MagickFalse)
       {
         (void) FormatMagickString(buffer,MaxTextExtent,"/SMask %lu 0 R\n",
           object-2);
@@ -1132,11 +1172,12 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobString(image,"stream\n");
     offset=TellBlob(image);
     number_pixels=(MagickSizeType) image->columns*image->rows;
-    if (number_pixels != (MagickSizeType) ((size_t) number_pixels))
+    if ((4*number_pixels) != (MagickSizeType) ((size_t) (4*number_pixels)))
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     if ((compression == FaxCompression) ||
         ((image_info->type != TrueColorType) &&
          (IsGrayImage(image,&image->exception) != MagickFalse)))
+
       {
         switch (compression)
         {
@@ -1157,6 +1198,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               ThrowWriterException(CoderError,image->exception.reason);
             break;
           }
+          case JPEG2000Compression:
+          {
+            status=JP2EncodeImage(image_info,image);
+            if (status == MagickFalse)
+              ThrowWriterException(CoderError,image->exception.reason);
+            break;
+          }
           case RLECompression:
           default:
           {
@@ -1183,18 +1231,18 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                75UL : image_info->quality,pixels);
+              status=ZLIBEncodeImage(image,length,quality,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1228,10 +1276,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -1243,7 +1292,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       }
     else
       if ((image->storage_class == DirectClass) || (image->colors > 256) ||
-          (compression == JPEGCompression))
+          (compression == JPEGCompression) ||
+          (compression == JPEG2000Compression))
         switch (compression)
         {
           case JPEGCompression:
@@ -1253,6 +1303,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               ThrowWriterException(CoderError,image->exception.reason);
             break;
           }
+          case JPEG2000Compression:
+          {
+            status=JP2EncodeImage(image_info,image);
+            if (status == MagickFalse)
+              ThrowWriterException(CoderError,image->exception.reason);
+            break;
+          }
           case RLECompression:
           default:
           {
@@ -1285,18 +1342,18 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                75UL : image_info->quality,pixels);
+              status=ZLIBEncodeImage(image,length,quality,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1334,10 +1391,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -1378,18 +1436,18 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 for (x=0; x < (long) image->columns; x++)
                   *q++=(unsigned char) indexes[x];
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
 #if defined(HasZLIB)
               if (compression == ZipCompression)
-                status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                  75UL : image_info->quality,pixels);
+                status=ZLIBEncodeImage(image,length,quality,pixels);
               else
 #endif
                 if (compression == LZWCompression)
@@ -1420,13 +1478,14 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 for (x=0; x < (long) image->columns; x++)
                   Ascii85Encode(image,(unsigned char) indexes[x]);
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               Ascii85Flush(image);
               break;
@@ -1461,7 +1520,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
           (void) strcpy(buffer,"/DeviceGray\n");
       else
         if ((image->storage_class == DirectClass) || (image->colors > 256) ||
-            (compression == JPEGCompression))
+            (compression == JPEGCompression) ||
+            (compression == JPEG2000Compression))
           (void) strcpy(buffer,"/DeviceRGB\n");
         else
           (void) FormatMagickString(buffer,MaxTextExtent,
@@ -1499,6 +1559,15 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         (void) strcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n");
         break;
       }
+      case JPEG2000Compression:
+      {
+        (void) FormatMagickString(buffer,MaxTextExtent,CFormat,"JPXDecode");
+        if (image->colorspace != CMYKColorspace)
+          break;
+        (void) WriteBlobString(image,buffer);
+        (void) strcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n");
+        break;
+      }
       case LZWCompression:
       {
         (void) FormatMagickString(buffer,MaxTextExtent,CFormat,"LZWDecode");
@@ -1566,6 +1635,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               ThrowWriterException(CoderError,tile_image->exception.reason);
             break;
           }
+          case JPEG2000Compression:
+          {
+            status=JP2EncodeImage(image_info,tile_image);
+            if (status == MagickFalse)
+              ThrowWriterException(CoderError,tile_image->exception.reason);
+            break;
+          }
           case RLECompression:
           default:
           {
@@ -1576,7 +1652,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             pixels=(unsigned char *) AcquireMagickMemory(length);
             if (pixels == (unsigned char *) NULL)
               {
-                DestroyImage(tile_image);
+                tile_image=DestroyImage(tile_image);
                 ThrowWriterException(ResourceLimitError,
                   "MemoryAllocationFailed");
               }
@@ -1598,8 +1674,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                75UL : image_info->quality,pixels);
+              status=ZLIBEncodeImage(image,length,quality,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1640,7 +1715,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       }
     else
       if ((tile_image->storage_class == DirectClass) ||
-          (tile_image->colors > 256) || (compression == JPEGCompression))
+          (tile_image->colors > 256) || (compression == JPEGCompression) ||
+          (compression == JPEG2000Compression))
         switch (compression)
         {
           case JPEGCompression:
@@ -1650,6 +1726,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               ThrowWriterException(CoderError,tile_image->exception.reason);
             break;
           }
+          case JPEG2000Compression:
+          {
+            status=JP2EncodeImage(image_info,tile_image);
+            if (status == MagickFalse)
+              ThrowWriterException(CoderError,tile_image->exception.reason);
+            break;
+          }
           case RLECompression:
           default:
           {
@@ -1661,7 +1744,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             pixels=(unsigned char *) AcquireMagickMemory(length);
             if (pixels == (unsigned char *) NULL)
               {
-                DestroyImage(tile_image);
+                tile_image=DestroyImage(tile_image);
                 ThrowWriterException(ResourceLimitError,
                   "MemoryAllocationFailed");
               }
@@ -1688,8 +1771,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                75UL : image_info->quality,pixels);
+              status=ZLIBEncodeImage(image,length,quality,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1748,7 +1830,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               pixels=(unsigned char *) AcquireMagickMemory(length);
               if (pixels == (unsigned char *) NULL)
                 {
-                  DestroyImage(tile_image);
+                  tile_image=DestroyImage(tile_image);
                   ThrowWriterException(ResourceLimitError,
                     "MemoryAllocationFailed");
                 }
@@ -1768,8 +1850,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               }
 #if defined(HasZLIB)
               if (compression == ZipCompression)
-                status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                  75UL : image_info->quality,pixels);
+                status=ZLIBEncodeImage(image,length,quality,pixels);
               else
 #endif
                 if (compression == LZWCompression)
@@ -1805,7 +1886,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             }
           }
         }
-    DestroyImage(tile_image);
+    tile_image=DestroyImage(tile_image);
     offset=TellBlob(image)-offset;
     (void) WriteBlobString(image,"\nendstream\n");
     (void) WriteBlobString(image,"endobj\n");
@@ -1819,21 +1900,16 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       (unsigned long) offset);
     (void) WriteBlobString(image,buffer);
     (void) WriteBlobString(image,"endobj\n");
-    if ((image->storage_class == DirectClass) || (image->colors > 256) ||
-        (compression == FaxCompression))
+    /*
+      Write Colormap object.
+    */
+    xref[object++]=TellBlob(image);
+    (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
+    (void) WriteBlobString(image,buffer);
+    (void) WriteBlobString(image,"<<\n");
+    if ((image->storage_class != DirectClass) && (image->colors <= 256) &&
+        (compression != FaxCompression))
       {
-        xref[object++]=0;
-        xref[object++]=0;
-      }
-    else
-      {
-        /*
-          Write Colormap object.
-        */
-        xref[object++]=TellBlob(image);
-        (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
-        (void) WriteBlobString(image,buffer);
-        (void) WriteBlobString(image,"<<\n");
         if (compression == NoCompression)
           (void) WriteBlobString(image,"/Filter [ /ASCII85Decode ]\n");
         (void) FormatMagickString(buffer,MaxTextExtent,"/Length %lu 0 R\n",
@@ -1862,27 +1938,31 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         }
         if (compression == NoCompression)
           Ascii85Flush(image);
-        offset=TellBlob(image)-offset;
-        (void) WriteBlobString(image,"\nendstream\n");
-        (void) WriteBlobString(image,"endobj\n");
-        /*
-          Write Length object.
-        */
-        xref[object++]=TellBlob(image);
-        (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
-        (void) WriteBlobString(image,buffer);
-        (void) FormatMagickString(buffer,MaxTextExtent,"%lu\n",
-          (unsigned long) offset);
-        (void) WriteBlobString(image,buffer);
-        (void) WriteBlobString(image,"endobj\n");
       }
-    if (image->next == (Image *) NULL)
+    offset=TellBlob(image)-offset;
+    (void) WriteBlobString(image,"\nendstream\n");
+    (void) WriteBlobString(image,"endobj\n");
+    /*
+      Write Length object.
+    */
+    xref[object++]=TellBlob(image);
+    (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
+    (void) WriteBlobString(image,buffer);
+    (void) FormatMagickString(buffer,MaxTextExtent,"%lu\n",
+      (unsigned long) offset);
+    (void) WriteBlobString(image,buffer);
+    (void) WriteBlobString(image,"endobj\n");
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   /*
     Write Xref object.
