@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -41,14 +41,17 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
-#include "magick/random.h"
+#include "magick/random_.h"
 #include "magick/signature.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -89,9 +92,9 @@ static void inline PlasmaPixel(Image *image,double x,double y)
   q=GetImagePixels(image,(long) ceil(x-0.5),(long) ceil(y-0.5),1,1);
   if (q == (PixelPacket *) NULL)
     return;
-  q->red=ScaleXToQuantum(GetRandomValue(),1.0);
-  q->green=ScaleXToQuantum(GetRandomValue(),1.0);
-  q->blue=ScaleXToQuantum(GetRandomValue(),1.0);
+  q->red=ScaleAnyToQuantum(GetRandomValue(),1.0);
+  q->green=ScaleAnyToQuantum(GetRandomValue(),1.0);
+  q->blue=ScaleAnyToQuantum(GetRandomValue(),1.0);
   (void) SyncImagePixels(image);
 }
 
@@ -132,12 +135,11 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
     Recursively apply plasma to the image.
   */
   read_info=CloneImageInfo(image_info);
-  read_info->blob=(void *) NULL;
-  read_info->length=0;
+  SetImageInfoBlob(read_info,(void *) NULL,0);
   (void) FormatMagickString(read_info->filename,MaxTextExtent,
     "gradient:%s",image_info->filename);
   image=ReadImage(read_info,exception);
-  DestroyImageInfo(read_info);
+  read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     return(GetFirstImageInList(image));
   image->storage_class=DirectClass;
@@ -148,7 +150,7 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
       break;
     for (x=0; x < (long) image->columns; x++)
     {
-      q->opacity=(Quantum) (MaxRGB/2);
+      q->opacity=(Quantum) (QuantumRange/2);
       q++;
     }
     if (SyncImagePixels(image) == MagickFalse)
@@ -179,12 +181,16 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
     i>>=1;
   for (depth=1; ; depth++)
   {
-    status=MagickMonitor(PlasmaImageTag,(MagickOffsetType) depth,max_depth,
-      &image->exception);
-    if (status == MagickFalse)
-      break;
     if (PlasmaImage(image,&segment_info,0,depth) != MagickFalse)
       break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(depth,max_depth) != MagickFalse))
+      {
+        status=image->progress_monitor(PlasmaImageTag,(MagickOffsetType) depth,
+          max_depth,image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   return(GetFirstImageInList(image));
 }
