@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -42,11 +42,14 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
 #include "magick/color.h"
-#include "magick/color_private.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -129,15 +132,27 @@ static unsigned long PNMInteger(Image *image,const unsigned int base)
 {
 #define P7Comment  "END_OF_COMMENTS\n"
 
+  char
+    *comment;
+
   int
     c;
 
+  register char
+    *p;
+
+  size_t
+    length;
+
   unsigned long
     value;
 
   /*
     Skip any leading whitespace.
   */
+  length=MaxTextExtent;
+  comment=(char *) NULL;
+  p=comment;
   do
   {
     c=ReadBlobByte(image);
@@ -145,30 +160,16 @@ static unsigned long PNMInteger(Image *image,const unsigned int base)
       return(0);
     if (c == (int) '#')
       {
-        char
-          *comment;
-
-        MagickOffsetType
-          offset;
-
-        register char
-          *p,
-          *q;
-
-        size_t
-          length;
-
         /*
           Read comment.
         */
-        length=MaxTextExtent;
-        offset=0;
-        comment=AcquireString((char *) NULL);
-        for (p=comment; (c != EOF) && (c != (int) '\n'); p++)
+        if (comment == (char *) NULL)
+          comment=AcquireString((char *) NULL);
+        p=comment+strlen(comment);
+        for ( ; (c != EOF) && (c != (int) '\n'); p++)
         {
           if ((size_t) (p-comment+1) >= length)
             {
-              *p='\0';
               length<<=1;
               comment=(char *) ResizeMagickMemory(comment,
                 (length+MaxTextExtent)*sizeof(*comment));
@@ -178,18 +179,24 @@ static unsigned long PNMInteger(Image *image,const unsigned int base)
             }
           c=ReadBlobByte(image);
           *p=(char) c;
+          *(p+1)='\0';
         }
-        *p='\0';
         if (comment == (char *) NULL)
           return(0);
-        q=comment+offset;
-        if (LocaleCompare(q,P7Comment) == 0)
-          *q='\0';
-        (void) SetImageAttribute(image,"comment",comment);
-        comment=(char *) RelinquishMagickMemory(comment);
         continue;
       }
   } while (isdigit(c) == MagickFalse);
+  if (comment != (char *) NULL)
+    {
+      if (strlen(comment) > strlen(P7Comment))
+        {
+          p-=strlen(P7Comment);
+          if (LocaleCompare(p,P7Comment) == 0)
+            *p='\0';
+        }
+      (void) SetImageAttribute(image,"Comment",comment);
+      comment=(char *) RelinquishMagickMemory(comment);
+    }
   if (base == 2)
     return((unsigned long) (c-(int) '0'));
   /*
@@ -236,9 +243,6 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  MonitorHandler
-    handler;
-
   Quantum
     *scale;
 
@@ -276,14 +280,15 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -295,7 +300,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Initialize image structure.
     */
-    if ((count == 0) || (format != 'P'))
+    if ((count != 1) || (format != 'P'))
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     format=(char) ReadBlobByte(image);
     if (format == '7')
@@ -323,7 +328,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         if (AllocateImageColormap(image,image->colors) == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        if (format == '7')
+        if ((format == '7') && (image->colors == 256))
           {
             /*
               Initialize 332 colormap.
@@ -333,14 +338,14 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
               for (pixel.green=0; pixel.green < 8; pixel.green++)
                 for (pixel.blue=0; pixel.blue < 4; pixel.blue++)
                 {
-                  image->colormap[i].red=ScaleXToQuantum(pixel.red,0x07);
-                  image->colormap[i].green=ScaleXToQuantum(pixel.green,0x07);
-                  image->colormap[i].blue=ScaleXToQuantum(pixel.blue,0x03);
+                  image->colormap[i].red=ScaleAnyToQuantum(pixel.red,0x07);
+                  image->colormap[i].green=ScaleAnyToQuantum(pixel.green,0x07);
+                  image->colormap[i].blue=ScaleAnyToQuantum(pixel.blue,0x03);
                   i++;
                 }
           }
       }
-    if ((image->storage_class != PseudoClass) || (max_value > MaxRGB))
+    if ((image->storage_class != PseudoClass) || (max_value > QuantumRange))
       {
         /*
           Compute pixel scaling table.
@@ -350,7 +355,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (scale == (Quantum *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         for (i=0; i <= (long) max_value; i++)
-          scale[i]=ScaleXToQuantum(i,max_value);
+          scale[i]=ScaleAnyToQuantum(i,max_value);
       }
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
@@ -377,7 +382,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if ((unsigned long) index >= image->colors)
               {
                 (void) ThrowMagickException(&image->exception,GetMagickModule(),
-                  CorruptImageError,"InvalidColormapIndex",image->filename);
+                  CorruptImageError,"InvalidColormapIndex","`%s'",
+                  image->filename);
                 index=0;
               }
             indexes[x]=(IndexPacket) index;
@@ -386,9 +392,11 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -418,7 +426,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if ((unsigned long) index >= image->colors)
               {
                 (void) ThrowMagickException(&image->exception,GetMagickModule(),
-                  CorruptImageError,"InvalidColormapIndex",image->filename);
+                  CorruptImageError,"InvalidColormapIndex","`%s'",
+                  image->filename);
                 index=0;
               }
             indexes[x]=(IndexPacket) index;
@@ -427,9 +436,11 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -465,9 +476,11 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -506,9 +519,11 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -532,7 +547,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         for (y=0; y < (long) image->rows; y++)
         {
           count=ReadBlob(image,(size_t) packets*image->columns,pixels);
-          if (count == 0)
+          if (count != (ssize_t) (packets*image->columns))
             ThrowReaderException(CorruptImageError,"UnableToReadImageData");
           p=pixels;
           q=SetImagePixels(image,0,y,image->columns,1);
@@ -547,7 +562,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 {
                   (void) ThrowMagickException(&image->exception,
                     GetMagickModule(),CorruptImageError,"InvalidColormapIndex",
-                    image->filename);
+                    "`%s'",image->filename);
                   index=0;
                 }
               indexes[x]=(IndexPacket) index;
@@ -561,7 +576,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 {
                   (void) ThrowMagickException(&image->exception,
                     GetMagickModule(),CorruptImageError,"InvalidColormapIndex",
-                    image->filename);
+                    "`%s'",image->filename);
                   index=0;
                 }
               indexes[x]=(IndexPacket) index;
@@ -570,9 +585,11 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -580,7 +597,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         pixels=(unsigned char *) RelinquishMagickMemory(pixels);
         if (EOFBlob(image) != MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
-            CorruptImageError,"UnexpectedEndOfFile",image->filename);
+            CorruptImageError,"UnexpectedEndOfFile","`%s'",image->filename);
         break;
       }
       case '6':
@@ -595,7 +612,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         for (y=0; y < (long) image->rows; y++)
         {
           count=ReadBlob(image,packets*image->columns,pixels);
-          if (count == 0)
+          if (count != (ssize_t) (packets*image->columns))
             ThrowReaderException(CorruptImageError,"UnableToReadImageData");
           p=pixels;
           q=SetImagePixels(image,0,y,image->columns,1);
@@ -638,19 +655,19 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
         }
         pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-        handler=SetMonitorHandler((MonitorHandler) NULL);
-        (void) SetMonitorHandler(handler);
         if (EOFBlob(image) != MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
-            CorruptImageError,"UnexpectedEndOfFile",image->filename);
+            CorruptImageError,"UnexpectedEndOfFile","`%s'",image->filename);
         break;
       }
       default:
@@ -664,9 +681,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (image_info->number_scenes != 0)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if ((format != '1') && (format != '2') && (format != '3'))
-      count=ReadBlob(image,1,(unsigned char *) &format);
-    else
+    if ((format == '1') || (format == '2') || (format == '3'))
       do
       {
         /*
@@ -678,24 +693,28 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if ((count != 0) && (format == 'P'))
           break;
       } while (format != '\n');
-    if ((count != 0) && (format == 'P'))
+    count=ReadBlob(image,1,(unsigned char *) &format);
+    if ((count == 1) && (format == 'P'))
       {
         /*
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
-  } while ((count != 0) && (format == 'P'));
+  } while ((count == 1) && (format == 'P'));
   CloseBlob(image);
   return(GetFirstImageInList(image));
 }
@@ -857,7 +876,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -893,7 +912,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
         (void) strcpy(buffer,"P7 332\n");
       }
     (void) WriteBlobString(image,buffer);
-    attribute=GetImageAttribute(image,"comment");
+    attribute=GetImageAttribute(image,"Comment");
     if (attribute != (const ImageAttribute *) NULL)
       {
         register char
@@ -934,7 +953,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
         */
         (void) SetImageType(image,BilevelType);
         polarity=(IndexPacket)
-          (PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2));
+          (PixelIntensityToQuantum(&image->colormap[0]) < (QuantumRange/2));
         if (image->colors == 2)
           polarity=(IndexPacket)
             (PixelIntensityToQuantum(&image->colormap[0]) <
@@ -959,10 +978,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
               }
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1005,10 +1025,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
             p++;
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1052,10 +1073,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
             p++;
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1078,7 +1100,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
         */
         (void) SetImageType(image,BilevelType);
         polarity=(IndexPacket)
-          (PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2));
+          (PixelIntensityToQuantum(&image->colormap[0]) < (QuantumRange/2));
         if (image->colors == 2)
           polarity=(IndexPacket)
             (PixelIntensityToQuantum(&image->colormap[0]) <
@@ -1108,10 +1130,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
           if (bit != 0)
             (void) WriteBlobByte(image,(unsigned char) (byte << (8-bit)));
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1147,10 +1170,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
               p++;
             }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1209,10 +1233,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
             }
           (void) WriteBlob(image,(size_t) (q-pixels),pixels);
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1331,10 +1356,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
           i++;
           if (i == 2)
             i=0;
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -1355,13 +1381,17 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
         break;
       }
     }
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
   return(MagickTrue);
