@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -42,14 +42,18 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
 #include "magick/color.h"
-#include "magick/color_private.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
-#include "magick/error.h"
+#include "magick/draw.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -58,6 +62,7 @@
 #include "magick/resource_.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/token.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
 
@@ -165,15 +170,15 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 #define BoundingBox  "BoundingBox:"
 #define BeginDocument  "BeginDocument:"
 #define DocumentMedia  "DocumentMedia:"
-#define DocumentProcessColors  "DocumentProcessColors:"
 #define EndDocument  "EndDocument:"
 #define HiResBoundingBox  "HiResBoundingBox:"
 #define PageBoundingBox  "PageBoundingBox:"
 #define PageMedia  "PageMedia:"
-#define PageProcessColors  "PageProcessColors:"
+#define Pages  "Pages:"
 #define PhotoshopProfile  "BeginPhotoshop:"
-#define PostscriptLevel  "%!PS-"
-#define RenderPostscriptText  "  Rendering postscript...  "
+#define PostscriptLevel  "!PS-"
+#define RenderPostscriptText  "  Rendering Postscript...  "
+#define XMLProfile  "begin_xml_packet:"
 
   char
     command[MaxTextExtent],
@@ -182,15 +187,13 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     geometry[MaxTextExtent],
     options[MaxTextExtent],
     postscript_filename[MaxTextExtent],
+    *q,
+    token[MaxTextExtent],
     translate_geometry[MaxTextExtent];
 
   const DelegateInfo
     *delegate_info;
 
-  double
-    dx_resolution,
-    dy_resolution;
-
   GeometryInfo
     geometry_info;
 
@@ -213,6 +216,9 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickStatusType
     flags;
 
+  PointInfo
+    delta;
+
   RectangleInfo
     page;
 
@@ -232,11 +238,14 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     count;
 
   StringInfo
-    *iptc_profile;
+    *iptc_profile,
+    *xml_profile;
 
   unsigned long
     height,
     length,
+    pages,
+    scene,
     width;
 
   /*
@@ -245,14 +254,15 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -283,26 +293,29 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Set the page density.
   */
-  dx_resolution=72.0;
-  dy_resolution=72.0;
-  if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
+  delta.x=DefaultResolution;
+  delta.y=DefaultResolution;
+  if ((image->y_resolution == 0.0) || (image->y_resolution == 0.0))
     {
       flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-      image->x_resolution=geometry_info.rho;
+      image->y_resolution=geometry_info.rho;
       image->y_resolution=geometry_info.sigma;
       if ((flags & SigmaValue) == 0)
-        image->y_resolution=image->x_resolution;
+        image->y_resolution=image->y_resolution;
     }
-  (void) FormatMagickString(density,MaxTextExtent,"%gx%g",image->x_resolution,
+  (void) FormatMagickString(density,MaxTextExtent,"%gx%g",image->y_resolution,
     image->y_resolution);
   /*
     Determine page geometry from the Postscript bounding box.
   */
   (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   (void) ResetMagickMemory(&page,0,sizeof(page));
+  (void) ResetMagickMemory(command,0,sizeof(command));
   iptc_profile=(StringInfo *) NULL;
+  xml_profile=(StringInfo *) NULL;
   skip=MagickFalse;
   cmyk=MagickFalse;
+  pages=(~0UL);
   for (p=command; (c=ReadBlobByte(image)) != EOF; )
   {
     if (image_info->page != (char *) NULL)
@@ -311,7 +324,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Note document structuring comments.
     */
     *p++=(char) c;
-    if ((c != (int) '%') && ((size_t) (p-command) < (MaxTextExtent-1)))
+    if ((c != (int) '%') && ((size_t) (p-command) < (MaxTextExtent/2)))
       continue;
     *p='\0';
     p=command;
@@ -324,6 +337,11 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       skip=MagickFalse;
     if (skip != MagickFalse)
       continue;
+    if (LocaleNCompare(PostscriptLevel,command,strlen(PostscriptLevel)) == 0)
+      if (GlobExpression(command,"*EPSF-*") != MagickFalse)
+        pages=1;
+    if (LocaleNCompare(Pages,command,strlen(Pages)) == 0)
+      (void) sscanf(command,"Pages: %lu",&pages);
     if (LocaleNCompare(PhotoshopProfile,command,strlen(PhotoshopProfile)) == 0)
       {
         /*
@@ -337,15 +355,30 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           iptc_profile->datum[i]=(unsigned char)
             ProfileInteger(image,hex_digits);
       }
+    if (LocaleNCompare(XMLProfile,command,strlen(XMLProfile)) == 0)
+      {
+        /*
+          Read image profile.
+        */
+        count=(ssize_t) sscanf(command,"begin_xml_packet: %lu",&length);
+        if (count != 1)
+          continue;
+        xml_profile=AcquireStringInfo(length);
+        for (i=0; i < (long) length; i++)
+          xml_profile->datum[i]=(unsigned char)
+            ProfileInteger(image,hex_digits);
+      }
     /*
       Is this a CMYK document?
     */
-    length=strlen(DocumentProcessColors);
-    if (LocaleNCompare(DocumentProcessColors,command,length) == 0)
-      cmyk=MagickTrue;
-    length=strlen(PageProcessColors);
-    if (LocaleNCompare(PageProcessColors,command,length) == 0)
-      cmyk=MagickTrue;
+    for (q=p; *q != '\0'; )
+    {
+      GetMagickToken(q,&q,token);
+      if (LocaleCompare(token,"cmyk") == 0)
+        cmyk=MagickTrue;
+      if (LocaleCompare(token,"cyan") == 0)
+        cmyk=MagickTrue;
+    }
     /*
       Note region defined by bounding box.
     */
@@ -381,7 +414,6 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       page.height=height;
   }
   CloseBlob(image);
-  DestroyImageList(image);
   /*
     Create Ghostscript control file.
   */
@@ -390,32 +422,40 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       ThrowFileException(&image->exception,FileOpenError,"UnableToOpenFile",
         image_info->filename);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  (void) CopyMagickString(command,"/setpagedevice {pop} bind 1 index where {"
+    "dup wcheck {3 1 roll put} {pop def} ifelse} {def} ifelse\n",MaxTextExtent);
+  (void) write(file,command,strlen(command));
   (void) FormatMagickString(translate_geometry,MaxTextExtent,
     "%g %g translate\n",-bounds.x1,-bounds.y1);
   (void) write(file,translate_geometry,strlen(translate_geometry));
-  (void) close(file);
+  file=close(file)-1;
   /*
-    Render postscript with the Ghostscript delegate.
+    Render Postscript with the Ghostscript delegate.
   */
   if ((page.width == 0) || (page.height == 0))
     (void) ParseAbsoluteGeometry(PSPageGeometry,&page);
   if (image_info->page != (char *) NULL)
     (void) ParseAbsoluteGeometry(image_info->page,&page);
   page.width=(unsigned long)
-    (page.width*image->x_resolution/dx_resolution+0.5);
+    (page.width*image->y_resolution/delta.x+0.5);
   page.height=(unsigned long)
-    (page.height*image->y_resolution/dy_resolution+0.5);
+    (page.height*image->y_resolution/delta.y+0.5);
+  image=DestroyImageList(image);
   (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",
     page.width,page.height);
   if (image_info->monochrome != MagickFalse)
-    delegate_info=GetDelegateInfo("gs-mono",(char *) NULL,exception);
+    delegate_info=GetDelegateInfo("ps:mono",(char *) NULL,exception);
   else
     if (cmyk != MagickFalse)
-      delegate_info=GetDelegateInfo("gs-cmyk",(char *) NULL,exception);
+      delegate_info=GetDelegateInfo("ps:cmyk",(char *) NULL,exception);
     else
-      delegate_info=GetDelegateInfo("gs-color",(char *) NULL,exception);
+      if ((pages == 1) && ((image_info->channel & OpacityChannel) != 0))
+        delegate_info=GetDelegateInfo("ps:alpha",(char *) NULL,exception);
+      else
+        delegate_info=GetDelegateInfo("ps:color",(char *) NULL,exception);
   if (delegate_info == (const DelegateInfo *) NULL)
     {
       (void) RelinquishUniqueFileResource(postscript_filename);
@@ -423,6 +463,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   *options='\0';
   read_info=CloneImageInfo(image_info);
+  *read_info->magick='\0';
   if (read_info->number_scenes != 0)
     {
       if (read_info->number_scenes != 1)
@@ -432,6 +473,9 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (void) FormatMagickString(options,MaxTextExtent,
           "-dFirstPage=%lu -dLastPage=%lu",read_info->scene+1,
           read_info->scene+read_info->number_scenes);
+      read_info->number_scenes=0;
+      if (read_info->scenes != (char *) NULL)
+        *read_info->scenes='\0';
     }
   (void) CopyMagickString(filename,read_info->filename,MaxTextExtent);
   (void) AcquireUniqueFilename(read_info->filename);
@@ -440,11 +484,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,geometry,density,options,
     read_info->filename,postscript_filename,image_info->filename);
-  (void) MagickMonitor(RenderPostscriptText,0,8,&image->exception);
   status=InvokePostscriptDelegate(read_info->verbose,command);
-  *read_info->magick='\0';
-  read_info->scene=0;
-  read_info->number_scenes=0;
   image=ReadImage(read_info,exception);
   if (image == (Image *) NULL)
     {
@@ -454,7 +494,6 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=ReadImage(read_info,exception);
     }
   (void) RelinquishUniqueFileResource(postscript_filename);
-  (void) MagickMonitor(RenderPostscriptText,7,8,&image->exception);
   (void) RelinquishUniqueFileResource(read_info->filename);
   if (image == (Image *) NULL)
     {
@@ -468,7 +507,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         return(GetFirstImageInList(image));
       ThrowReaderException(DelegateError,"PostscriptDelegateFailed");
     }
-  DestroyImageInfo(read_info);
+  read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     ThrowReaderException(DelegateError,"PostscriptDelegateFailed");
   if (LocaleCompare(image->magick,"BMP") == 0)
@@ -479,22 +518,28 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       cmyk_image=ConsolidateCMYKImages(image,&image->exception);
       if (cmyk_image != (Image *) NULL)
         {
-          DestroyImageList(image);
+          image=DestroyImageList(image);
           image=cmyk_image;
         }
     }
+  scene=0;
   do
   {
     (void) CopyMagickString(image->filename,filename,MaxTextExtent);
     if (iptc_profile != (StringInfo *) NULL)
       (void) SetImageProfile(image,"iptc",iptc_profile);
+    if (xml_profile != (StringInfo *) NULL)
+      (void) SetImageProfile(image,"xml",xml_profile);
     image->page=page;
+    image->scene=scene++;
     next_image=SyncNextImageInList(image);
     if (next_image != (Image *) NULL)
       image=next_image;
   } while (next_image != (Image *) NULL);
   if (iptc_profile != (StringInfo *) NULL)
-    DestroyStringInfo(iptc_profile);
+    iptc_profile=DestroyStringInfo(iptc_profile);
+  if (xml_profile != (StringInfo *) NULL)
+    xml_profile=DestroyStringInfo(xml_profile);
   return(GetFirstImageInList(image));
 }
 
@@ -532,6 +577,7 @@ ModuleExport void RegisterPSImage(void)
   entry->magick=(MagickHandler *) IsPS;
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=
     AcquireString("Encapsulated PostScript Interchange format");
   entry->module=AcquireString("PS");
@@ -542,6 +588,7 @@ ModuleExport void RegisterPSImage(void)
   entry->magick=(MagickHandler *) IsPS;
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("Encapsulated PostScript");
   entry->module=AcquireString("PS");
   (void) RegisterMagickInfo(entry);
@@ -551,6 +598,7 @@ ModuleExport void RegisterPSImage(void)
   entry->magick=(MagickHandler *) IsPS;
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("Encapsulated PostScript");
   entry->module=AcquireString("PS");
   (void) RegisterMagickInfo(entry);
@@ -560,6 +608,7 @@ ModuleExport void RegisterPSImage(void)
   entry->magick=(MagickHandler *) IsPS;
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=
     AcquireString("Encapsulated PostScript Interchange format");
   entry->module=AcquireString("PS");
@@ -570,6 +619,7 @@ ModuleExport void RegisterPSImage(void)
   entry->magick=(MagickHandler *) IsPS;
   entry->module=AcquireString("PS");
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("PostScript");
   (void) RegisterMagickInfo(entry);
 }
@@ -636,11 +686,11 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
 { \
   if ((image->matte != MagickFalse) && (p->opacity == TransparentOpacity)) \
     (void) FormatMagickString(buffer,MaxTextExtent,"ffffff%02lX", \
-      Min(length,0xff)); \
+      (unsigned long) Min(length,0xff)); \
   else \
     (void) FormatMagickString(buffer,MaxTextExtent,"%02X%02X%02X%02lX", \
       ScaleQuantumToChar(pixel.red),ScaleQuantumToChar(pixel.green), \
-      ScaleQuantumToChar(pixel.blue),Min(length,0xff)); \
+      ScaleQuantumToChar(pixel.blue),(unsigned long) Min(length,0xff)); \
   (void) WriteBlobString(image,buffer); \
 }
 
@@ -914,14 +964,6 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
   const ImageAttribute
     *attribute;
 
-  double
-    dx_resolution,
-    dy_resolution,
-    x_resolution,
-    x_scale,
-    y_resolution,
-    y_scale;
-
   GeometryInfo
     geometry_info;
 
@@ -945,6 +987,11 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
   PixelPacket
     pixel;
 
+  PointInfo
+    delta,
+    resolution,
+    scale;
+
   RectangleInfo
     geometry,
     media_info,
@@ -986,25 +1033,44 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
+  (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   page=1;
   scene=0;
   do
   {
     /*
-      Scale image to size of Postscript page.
+      Scale relative to dots-per-inch.
     */
     (void) SetImageColorspace(image,RGBColorspace);
-    text_size=0;
-    attribute=GetImageAttribute(image,"label");
-    if (attribute != (const ImageAttribute *) NULL)
-      text_size=(unsigned long)
-        (MultilineCensus(attribute->value)*image_info->pointsize+12);
+    delta.x=DefaultResolution;
+    delta.y=DefaultResolution;
+    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+    resolution.x=geometry_info.rho;
+    resolution.y=geometry_info.sigma;
+    if ((flags & SigmaValue) == 0)
+      resolution.y=resolution.x;
+    if (image->x_resolution != 0.0)
+      resolution.x=image->x_resolution;
+    if (image->y_resolution != 0.0)
+      resolution.y=image->y_resolution;
+    if (image_info->density != (char *) NULL)
+      {
+        flags=ParseGeometry(image_info->density,&geometry_info);
+        resolution.x=geometry_info.rho;
+        resolution.y=geometry_info.sigma;
+        if ((flags & SigmaValue) == 0)
+          resolution.y=resolution.x;
+      }
+    if (image->units == PixelsPerCentimeterResolution)
+      {
+        resolution.x*=2.54;
+        resolution.y*=2.54;
+      }
     SetGeometry(image,&geometry);
-    geometry.y=(long) text_size;
     (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
       image->columns,image->rows);
     if (image_info->page != (char *) NULL)
@@ -1019,38 +1085,22 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
           (void) strcpy(page_geometry,PSPageGeometry);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
+    scale.x=(double) (geometry.width*delta.x)/resolution.x;
+    geometry.width=(unsigned long) (scale.x+0.5);
+    scale.y=(double) (geometry.height*delta.y)/resolution.y;
+    geometry.height=(unsigned long) (scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
     (void) ParseGravityGeometry(image,page_geometry,&page_info);
-    geometry.x=(-page_info.x);
-    geometry.y=(long) (media_info.height+page_info.y-image->rows);
-    /*
-      Scale relative to dots-per-inch.
-    */
-    dx_resolution=72.0;
-    dy_resolution=72.0;
-    x_resolution=72.0;
-    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    x_resolution=geometry_info.rho;
-    y_resolution=geometry_info.sigma;
-    if ((flags & SigmaValue) == 0)
-      y_resolution=x_resolution;
-    if (image_info->density != (char *) NULL)
+    if (image->gravity != UndefinedGravity)
       {
-        flags=ParseGeometry(image_info->density,&geometry_info);
-        x_resolution=geometry_info.rho;
-        y_resolution=geometry_info.sigma;
-        if ((flags & SigmaValue) == 0)
-          y_resolution=x_resolution;
+        geometry.x=(-page_info.x);
+        geometry.y=(long) (media_info.height+page_info.y-image->rows);
       }
-    if (image->units == PixelsPerCentimeterResolution)
-      {
-        x_resolution*=2.54;
-        y_resolution*=2.54;
-      }
-    x_scale=(double) (geometry.width*dx_resolution)/x_resolution;
-    geometry.width=(unsigned long) (x_scale+0.5);
-    y_scale=(double) (geometry.height*dy_resolution)/y_resolution;
-    geometry.height=(unsigned long) (y_scale+0.5);
+    text_size=0;
+    attribute=GetImageAttribute(image,"Label");
+    if (attribute != (const ImageAttribute *) NULL)
+      text_size=(unsigned long)
+        (MultilineCensus(attribute->value)*image_info->pointsize+12);
     count=0;
     if (page == 1)
       {
@@ -1075,9 +1125,10 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,buffer);
         bounds.x1=(double) geometry.x;
         bounds.y1=(double) geometry.y;
-        bounds.x2=(double) geometry.x+x_scale;
+        bounds.x2=(double) geometry.x+scale.x;
         bounds.y2=(double) geometry.y+(geometry.height+text_size);
-        if ((image_info->adjoin != MagickFalse) && (image->next != (Image *) NULL))
+        if ((image_info->adjoin != MagickFalse) &&
+            (GetNextImageInList(image) != (Image *) NULL))
           (void) strcpy(buffer,"%%%%BoundingBox: (atend)\n");
         else
           {
@@ -1091,7 +1142,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
               bounds.x2,bounds.y2);
           }
         (void) WriteBlobString(image,buffer);
-        attribute=GetImageAttribute(image,"label");
+        attribute=GetImageAttribute(image,"Label");
         if (attribute != (const ImageAttribute *) NULL)
           (void) WriteBlobString(image,
             "%%DocumentNeededResources: font Times-Roman\n");
@@ -1137,7 +1188,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
             */
             (void) SetImageType(preview_image,BilevelType);
             polarity=(IndexPacket) (PixelIntensityToQuantum(
-              &preview_image->colormap[0]) < (MaxRGB/2));
+              &preview_image->colormap[0]) < (QuantumRange/2));
             if (preview_image->colors == 2)
               polarity=(IndexPacket)
                 (PixelIntensityToQuantum(&preview_image->colormap[0]) >
@@ -1166,7 +1217,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 if (bit == 8)
                   {
                     (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                      byte & 0xff);
+                      (unsigned int) (byte & 0xff));
                     (void) WriteBlobString(image,buffer);
                     count++;
                     if (count == 36)
@@ -1182,7 +1233,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 {
                   byte<<=(8-bit);
                   (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                    byte & 0xff);
+                    (unsigned int) (byte & 0xff));
                   (void) WriteBlobString(image,buffer);
                   count++;
                   if (count == 36)
@@ -1193,7 +1244,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 };
             }
             (void) WriteBlobString(image,"\n%%EndPreview\n");
-            DestroyImage(preview_image);
+            preview_image=DestroyImage(preview_image);
           }
         /*
           Output Postscript commands.
@@ -1203,7 +1254,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
           (void) FormatMagickString(buffer,MaxTextExtent,"%s\n",*q);
           (void) WriteBlobString(image,buffer);
         }
-        attribute=GetImageAttribute(image,"label");
+        attribute=GetImageAttribute(image,"Label");
         if (attribute != (const ImageAttribute *) NULL)
           for (j=(long) MultilineCensus(attribute->value)-1; j >= 0; j--)
           {
@@ -1239,7 +1290,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
       bounds.x2=(double) geometry.x+geometry.width-1;
     if ((double) (geometry.y+(geometry.height+text_size)-1) > bounds.y2)
       bounds.y2=(double) geometry.y+(geometry.height+text_size)-1;
-    attribute=GetImageAttribute(image,"label");
+    attribute=GetImageAttribute(image,"Label");
     if (attribute != (const ImageAttribute *) NULL)
       (void) WriteBlobString(image,"%%%%PageResources: font Times-Roman\n");
     if (LocaleCompare(image_info->magick,"PS") != 0)
@@ -1249,10 +1300,10 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
       Output image data.
     */
     (void) FormatMagickString(buffer,MaxTextExtent,"%ld %ld\n%g %g\n%f\n",
-      geometry.x,geometry.y,x_scale,y_scale,image_info->pointsize);
+      geometry.x,geometry.y,scale.x,scale.y,image_info->pointsize);
     (void) WriteBlobString(image,buffer);
     labels=(char **) NULL;
-    attribute=GetImageAttribute(image,"label");
+    attribute=GetImageAttribute(image,"Label");
     if (attribute != (const ImageAttribute *) NULL)
       labels=StringToList(attribute->value);
     if (labels != (char **) NULL)
@@ -1304,10 +1355,11 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -1323,7 +1375,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
             */
             (void) SetImageType(image,BilevelType);
             polarity=(IndexPacket) (PixelIntensityToQuantum(
-              &image->colormap[0]) < (MaxRGB/2));
+              &image->colormap[0]) < (QuantumRange/2));
             if (image->colors == 2)
               polarity=(IndexPacket)
                 (PixelIntensityToQuantum(&image->colormap[1]) <
@@ -1347,7 +1399,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 if (bit == 8)
                   {
                     (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                      byte & 0xff);
+                      (unsigned int) (byte & 0xff));
                     (void) WriteBlobString(image,buffer);
                     count++;
                     if (count == 36)
@@ -1364,7 +1416,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 {
                   byte<<=(8-bit);
                   (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                    byte & 0xff);
+                    (unsigned int) (byte & 0xff));
                   (void) WriteBlobString(image,buffer);
                   count++;
                   if (count == 36)
@@ -1374,10 +1426,11 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                     };
                 };
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -1387,7 +1440,8 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlobByte(image,'\n');
       }
     else
-      if ((image->storage_class == DirectClass) || (image->matte != MagickFalse))
+      if ((image->storage_class == DirectClass) ||
+          (image->matte != MagickFalse))
         {
           /*
             Dump DirectClass image.
@@ -1436,13 +1490,14 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 }
                 WriteRunlengthPacket(image,pixel,length,p);
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               break;
             }
@@ -1478,13 +1533,14 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                   p++;
                 }
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               break;
             }
@@ -1541,7 +1597,8 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                       if (x > 0)
                         {
                           (void) FormatMagickString(buffer,MaxTextExtent,
-                            "%02X%02lX",index,Min(length,0xff));
+                            "%02lX%02lX",(unsigned long) index,(unsigned long)
+                            Min(length,0xff));
                           (void) WriteBlobString(image,buffer);
                           i++;
                           if (i == 18)
@@ -1556,17 +1613,18 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                   pixel=(*p);
                   p++;
                 }
-                (void) FormatMagickString(buffer,MaxTextExtent,"%02X%02lX",
-                  index,Min(length,0xff));
+                (void) FormatMagickString(buffer,MaxTextExtent,"%02lX%02lX",
+                  (unsigned long) index,(unsigned long) Min(length,0xff));
                 (void) WriteBlobString(image,buffer);
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               break;
             }
@@ -1586,8 +1644,8 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 indexes=GetIndexes(image);
                 for (x=0; x < (long) image->columns; x++)
                 {
-                  (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                    indexes[x]);
+                  (void) FormatMagickString(buffer,MaxTextExtent,"%02lX",
+                    (unsigned long) indexes[x]);
                   (void) WriteBlobString(image,buffer);
                   i++;
                   if (i == 36)
@@ -1598,13 +1656,14 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                   p++;
                 }
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               break;
             }
@@ -1614,13 +1673,17 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
     if (LocaleCompare(image_info->magick,"PS") != 0)
       (void) WriteBlobString(image,"end\n");
     (void) WriteBlobString(image,"%%PageTrailer\n");
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   (void) WriteBlobString(image,"%%Trailer\n");
   if (page > 2)
