@@ -18,13 +18,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -43,13 +43,17 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
 #include "magick/color.h"
-#include "magick/color_private.h"
+#include "magick/color-private.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/draw.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -206,7 +210,7 @@ static MagickBooleanType SerializeImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=MagickTrue;
   *length=(image->colorspace == CMYKColorspace ? 4 : 3)*
     (size_t) image->columns*image->rows;
@@ -238,9 +242,11 @@ static MagickBooleanType SerializeImage(const ImageInfo *image_info,
         p++;
       }
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -279,7 +285,7 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=MagickTrue;
   pack=(unsigned long)
     (IsMonochromeImage(image,&image->exception) != MagickFalse ? 8 : 1);
@@ -319,10 +325,11 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
         }
       }
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,
-            &image->exception);
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -356,7 +363,7 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=MagickTrue;
   *length=(size_t) image->columns*image->rows;
   *pixels=(unsigned char *) AcquireMagickMemory(*length);
@@ -372,19 +379,22 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
     for (x=0; x < (long) image->columns; x++)
       *q++=(unsigned char) indexes[x];
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
-        {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
-          if (status == MagickFalse)
-            break;
-        }
+     if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+         (QuantumTick(y,image->rows) != MagickFalse))
+       {
+         status=image->progress_monitor(SaveImageTag,y,image->rows,
+           image->client_data);
+         if (status == MagickFalse)
+           break;
+       }
   }
   if (status == MagickFalse)
     *pixels=(unsigned char *) RelinquishMagickMemory(*pixels);
   return(status);
 }
 
-static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
+  Image *image)
 {
   char
     buffer[MaxTextExtent];
@@ -411,15 +421,20 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,Image *im
   unsigned char
     *pixels;
 
+  unsigned long
+    quality;
+
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(image->matte != MagickFalse);
   status=MagickTrue;
   compression=image->compression;
+  quality=(unsigned long)
+    (image->quality == UndefinedCompressionQuality ? 75 : image->quality);
   /*
     Note BeginData DSC comment for update later.
   */
@@ -486,7 +501,7 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,Image *im
   status=SeparateImageChannel(mask_image,OpacityChannel);
   if (status == MagickFalse)
     {
-      DestroyImage(mask_image);
+      mask_image=DestroyImage(mask_image);
       return(MagickFalse);
     }
   (void) SetImageType(mask_image,BilevelType);
@@ -545,13 +560,12 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,Image *im
       status=SerializeImageChannel(image_info,mask_image,&pixels,&length);
       if (status == MagickFalse)
         break;
-      status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-        75UL : image_info->quality,pixels);
+      status=ZLIBEncodeImage(image,length,quality,pixels);
       pixels=(unsigned char *) RelinquishMagickMemory(pixels);
       break;
     }
   }
-  DestroyImage(mask_image);
+  mask_image=DestroyImage(mask_image);
   (void) WriteBlobByte(image,'\n');
   length=(size_t) (TellBlob(image)-stop);
   stop=TellBlob(image);
@@ -778,14 +792,6 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
   const ImageAttribute
     *attribute;
 
-  double
-    dx_resolution,
-    dy_resolution,
-    x_resolution,
-    x_scale,
-    y_resolution,
-    y_scale;
-
   GeometryInfo
     geometry_info;
 
@@ -803,6 +809,11 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
   MagickStatusType
     flags;
 
+  PointInfo
+    delta,
+    resolution,
+    scale;
+
   RectangleInfo
     geometry,
     media_info,
@@ -836,7 +847,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(MagickFalse);
@@ -848,7 +859,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     {
       compression=RLECompression;
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"JPEGLibraryIsNotAvailable",image->filename);
+        MissingDelegateError,"JPEGLibraryIsNotAvailable","`%s'",
+        image->filename);
       break;
     }
 #endif
@@ -857,72 +869,77 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     {
       compression=RLECompression;
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"ZipLibraryIsNotAvailable",image->filename);
+        MissingDelegateError,"ZipLibraryIsNotAvailable","`%s'",image->filename);
       break;
     }
 #endif
     default:
       break;
   }
+  (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   page=0;
   scene=0;
   do
   {
     /*
-      Scale image to size of Postscript page.
+      Scale relative to dots-per-inch.
     */
-    page++;
-    text_size=0;
-    attribute=GetImageAttribute(image,"label");
-    if (attribute != (const ImageAttribute *) NULL)
-      text_size=(unsigned long) (MultilineCensus(attribute->value)*
-        image_info->pointsize+12);
+    delta.x=DefaultResolution;
+    delta.y=DefaultResolution;
+    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+    resolution.x=geometry_info.rho;
+    resolution.y=geometry_info.sigma;
+    if ((flags & SigmaValue) == 0)
+      resolution.y=resolution.x;
+    if (image->x_resolution != 0.0)
+      resolution.x=image->x_resolution;
+    if (image->y_resolution != 0.0)
+      resolution.y=image->y_resolution;
+    if (image_info->density != (char *) NULL)
+      {
+        flags=ParseGeometry(image_info->density,&geometry_info);
+        resolution.x=geometry_info.rho;
+        resolution.y=geometry_info.sigma;
+        if ((flags & SigmaValue) == 0)
+          resolution.y=resolution.x;
+      }
+    if (image->units == PixelsPerCentimeterResolution)
+      {
+        resolution.x*=2.54;
+        resolution.y*=2.54;
+      }
     SetGeometry(image,&geometry);
-    geometry.y=(long) text_size;
     (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
       image->columns,image->rows);
     if (image_info->page != (char *) NULL)
       (void) CopyMagickString(page_geometry,image_info->page,MaxTextExtent);
     else
       if ((image->page.width != 0) && (image->page.height != 0))
-        (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu%+ld%+ld",          image->page.width,image->page.height,image->page.x,image->page.y);
+        (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu%+ld%+ld",
+          image->page.width,image->page.height,image->page.x,image->page.y);
       else
-        if (LocaleCompare(image_info->magick,"PS3") == 0)
+        if ((image->gravity != UndefinedGravity) &&
+            (LocaleCompare(image_info->magick,"PS") == 0))
           (void) strcpy(page_geometry,PSPageGeometry);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
+    scale.x=(double) (geometry.width*delta.x)/resolution.x;
+    geometry.width=(unsigned long) (scale.x+0.5);
+    scale.y=(double) (geometry.height*delta.y)/resolution.y;
+    geometry.height=(unsigned long) (scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
     (void) ParseGravityGeometry(image,page_geometry,&page_info);
-    geometry.x=(-page_info.x);
-    geometry.y=(long) (media_info.height+page_info.y-image->rows);
-    /*
-      Scale relative to dots-per-inch.
-    */
-    dx_resolution=72.0;
-    dy_resolution=72.0;
-    x_resolution=72.0;
-    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    x_resolution=geometry_info.rho;
-    y_resolution=geometry_info.sigma;
-    if ((flags & SigmaValue) == 0)
-      y_resolution=x_resolution;
-    if (image_info->density != (char *) NULL)
+    if (image->gravity != UndefinedGravity)
       {
-        flags=ParseGeometry(image_info->density,&geometry_info);
-        x_resolution=geometry_info.rho;
-        y_resolution=geometry_info.sigma;
-        if ((flags & SigmaValue) == 0)
-          y_resolution=x_resolution;
+        geometry.x=(-page_info.x);
+        geometry.y=(long) (media_info.height+page_info.y-image->rows);
       }
-    if (image->units == PixelsPerCentimeterResolution)
-      {
-        x_resolution*=2.54;
-        y_resolution*=2.54;
-      }
-    x_scale=(double) ((geometry.width*dx_resolution)/x_resolution);
-    geometry.width=(unsigned long) (x_scale+0.5);
-    y_scale=(double) ((geometry.height*dy_resolution)/y_resolution);
-    geometry.height=(unsigned long) (y_scale+0.5);
+    text_size=0;
+    attribute=GetImageAttribute(image,"Label");
+    if (attribute != (const ImageAttribute *) NULL)
+      text_size=(unsigned long)
+        (MultilineCensus(attribute->value)*image_info->pointsize+12);
+    page++;
     if (page == 1)
       {
         /*
@@ -948,10 +965,10 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,buffer);
         bounds.x1=(double) geometry.x;
         bounds.y1=(double) geometry.y;
-        bounds.x2=(double) geometry.x+x_scale;
-        bounds.y2=(double) geometry.y+y_scale+text_size;
+        bounds.x2=(double) geometry.x+scale.x;
+        bounds.y2=(double) geometry.y+scale.y+text_size;
         if ((image_info->adjoin != MagickFalse) &&
-            (image->next != (Image *) NULL))
+            (GetNextImageInList(image) != (Image *) NULL))
           {
             (void) WriteBlobString(image,"%%BoundingBox: (atend)\n");
             (void) WriteBlobString(image,"%%HiResBoundingBox: (atend)\n");
@@ -977,7 +994,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         /*
           Font resources
         */
-        attribute=GetImageAttribute(image,"label");
+        attribute=GetImageAttribute(image,"Label");
         if (attribute != (const ImageAttribute *) NULL)
           (void) WriteBlobString(image,
             "%%DocumentNeededResources: font Helvetica\n");
@@ -1011,7 +1028,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         /*
           One label line for each line in label string.
         */
-        attribute=GetImageAttribute(image,"label");
+        attribute=GetImageAttribute(image,"Label");
         if (attribute != (const ImageAttribute *) NULL)
         {
           (void) WriteBlobString(image,"\n  % Labels.\n  /Helvetica findfont "
@@ -1060,14 +1077,14 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       bounds.x1=(double) geometry.x;
     if ((double) geometry.y < bounds.y1)
       bounds.y1=(double) geometry.y;
-    if ((double) (geometry.x+x_scale) > bounds.x2)
-      bounds.x2=(double) geometry.x+x_scale;
-    if ((double) (geometry.y+y_scale+text_size) > bounds.y2)
-      bounds.y2=(double) geometry.y+y_scale+text_size;
+    if ((double) (geometry.x+scale.x) > bounds.x2)
+      bounds.x2=(double) geometry.x+scale.x;
+    if ((double) (geometry.y+scale.y+text_size) > bounds.y2)
+      bounds.y2=(double) geometry.y+scale.y+text_size;
     /*
       Page font resource if there's a label.
     */
-    attribute=GetImageAttribute(image,"label");
+    attribute=GetImageAttribute(image,"Label");
     if (attribute != (const ImageAttribute *) NULL)
       (void) WriteBlobString(image,"%%PageResources: font Helvetica\n");
     /*
@@ -1115,13 +1132,13 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       Translate, scale, and font point size.
     */
     (void) FormatMagickString(buffer,MaxTextExtent,"%ld %ld\n%g %g\n%f\n",
-      geometry.x,geometry.y,x_scale,y_scale,image_info->pointsize);
+      geometry.x,geometry.y,scale.x,scale.y,image_info->pointsize);
     (void) WriteBlobString(image,buffer);
     /*
       Output labels.
     */
     labels=(char **) NULL;
-    attribute=GetImageAttribute(image,"label");
+    attribute=GetImageAttribute(image,"Label");
     if (attribute != (const ImageAttribute *) NULL)
       labels=StringToList(attribute->value);
     if (labels != (char **) NULL)
@@ -1285,8 +1302,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
               }
               case ZipCompression:
               {
-                status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                  75UL : image_info->quality,pixels);
+                status=ZLIBEncodeImage(image,length,image->quality,pixels);
                 break;
               }
             }
@@ -1370,8 +1386,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
                 }
                 case ZipCompression:
                 {
-                  status=ZLIBEncodeImage(image,length,image_info->quality ==
-                    0UL ? 75UL : image_info->quality,pixels);
+                  status=ZLIBEncodeImage(image,length,image->quality,pixels);
                   break;
                 }
               }
@@ -1483,8 +1498,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
               }
               case ZipCompression:
               {
-                status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                  75UL : image_info->quality,pixels);
+                status=ZLIBEncodeImage(image,length,image->quality,pixels);
                 break;
               }
             }
@@ -1514,13 +1528,16 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     if (LocaleCompare(image_info->magick,"PS3") != 0)
       (void) WriteBlobString(image,"end\n");
     (void) WriteBlobString(image,"%%PageTrailer\n");
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
     scene++;
   } while (image_info->adjoin != MagickFalse);
   (void) WriteBlobString(image,"%%Trailer\n");
