@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -41,11 +41,14 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
 #include "magick/color.h"
-#include "magick/color_private.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -250,6 +253,9 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register unsigned char
     *p;
 
+  ssize_t
+    count;
+
   SGIInfo
     iris_info;
 
@@ -265,14 +271,15 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -307,7 +314,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
     iris_info.minimum_value=ReadBlobMSBLong(image);
     iris_info.maximum_value=ReadBlobMSBLong(image);
-    (void) ReadBlob(image,sizeof(iris_info.filler),iris_info.filler);
+    count=ReadBlob(image,sizeof(iris_info.filler),iris_info.filler);
     /*
       Allocate SGI pixels.
     */
@@ -337,8 +344,10 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
           p=iris_pixels+bytes_per_pixel*z;
           for (y=0; y < (long) iris_info.rows; y++)
           {
-            (void) ReadBlob(image,bytes_per_pixel*iris_info.columns,
+            count=ReadBlob(image,bytes_per_pixel*iris_info.columns,
               scanline);
+            if (EOFBlob(image) != MagickFalse)
+              break;
             if (bytes_per_pixel == 2)
               for (x=0; x < (long) iris_info.columns; x++)
               {
@@ -414,8 +423,10 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     offset=offsets[y+z*iris_info.rows];
                     (void) SeekBlob(image,(long) offset,SEEK_SET);
                   }
-                (void) ReadBlob(image,(size_t) runlength[y+z*iris_info.rows],
+                count=ReadBlob(image,(size_t) runlength[y+z*iris_info.rows],
                   max_packets);
+                if (EOFBlob(image) != MagickFalse)
+                  break;
                 offset+=runlength[y+z*iris_info.rows];
                 SGIDecode(iris_info.columns,bytes_per_pixel,max_packets,
                   p+bytes_per_pixel*z);
@@ -439,8 +450,10 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     offset=offsets[y+z*iris_info.rows];
                     (void) SeekBlob(image,(long) offset,SEEK_SET);
                   }
-                (void) ReadBlob(image,(size_t) runlength[y+z*iris_info.rows],
+                count=ReadBlob(image,(size_t) runlength[y+z*iris_info.rows],
                   max_packets);
+                if (EOFBlob(image) != MagickFalse)
+                  break;
                 offset+=runlength[y+z*iris_info.rows];
                 SGIDecode(iris_info.columns,bytes_per_pixel,max_packets,
                   p+bytes_per_pixel*z);
@@ -480,17 +493,21 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 q->red=ScaleShortToQuantum((*(p+0) << 8) | (*(p+1)));
                 q->green=ScaleShortToQuantum((*(p+2) << 8) | (*(p+3)));
                 q->blue=ScaleShortToQuantum((*(p+4) << 8) | (*(p+5)));
-                q->opacity=(Quantum)
-                  (MaxRGB-ScaleShortToQuantum((*(p+6) << 8) | (*(p+7))));
+                q->opacity=OpaqueOpacity;
+                if (image->matte != MagickFalse)
+                  q->opacity=(Quantum) (QuantumRange-
+                    ScaleShortToQuantum((*(p+6) << 8) | (*(p+7))));
                 p+=8;
                 q++;
               }
               if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                    status=image->progress_monitor(LoadImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -508,16 +525,20 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               q->red=ScaleCharToQuantum(*p);
               q->green=ScaleCharToQuantum(*(p+1));
               q->blue=ScaleCharToQuantum(*(p+2));
-              q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*(p+3)));
+              q->opacity=OpaqueOpacity;
+              if (image->matte != MagickFalse)
+                q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(*(p+3)));
               p+=4;
               q++;
             }
             if (SyncImagePixels(image) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows) != MagickFalse)
+              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                  (QuantumTick(y,image->rows) != MagickFalse))
                 {
-                  status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                  status=image->progress_monitor(LoadImageTag,y,image->rows,
+                    image->client_data);
                   if (status == MagickFalse)
                     break;
                 }
@@ -552,9 +573,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                    status=image->progress_monitor(LoadImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -577,9 +600,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (SyncImagePixels(image) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows) != MagickFalse)
+              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                  (QuantumTick(y,image->rows) != MagickFalse))
                 {
-                  status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                  status=image->progress_monitor(LoadImageTag,y,image->rows,
+                    image->client_data);
                   if (status == MagickFalse)
                     break;
                 }
@@ -606,16 +631,19 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (iris_info.magic == 0x01DA);
   CloseBlob(image);
@@ -803,7 +831,7 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((image->columns > 65535UL) || (image->rows > 65535UL))
     ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
@@ -815,7 +843,8 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
     /*
       Initialize SGI raster file header.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace != LogColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     iris_info.magic=0x01DA;
     if (image->compression == NoCompression)
       iris_info.storage=(unsigned char) 0x00;
@@ -839,7 +868,7 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
           iris_info.depth=3;
       }
     iris_info.minimum_value=0;
-    iris_info.maximum_value=(unsigned long) ScaleQuantumToChar(MaxRGB);
+    iris_info.maximum_value=(unsigned long) ScaleQuantumToChar(QuantumRange);
     for (i=0; i < (int) sizeof(iris_info.filler); i++)
       iris_info.filler[i]=(unsigned char) 0;
     /*
@@ -879,13 +908,15 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
         *q++=ScaleQuantumToChar(p->red);
         *q++=ScaleQuantumToChar(p->green);
         *q++=ScaleQuantumToChar(p->blue);
-        *q++=ScaleQuantumToChar(MaxRGB-p->opacity);
+        *q++=ScaleQuantumToChar(QuantumRange-p->opacity);
         p++;
       }
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            status=image->progress_monitor(SaveImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
@@ -975,17 +1006,18 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
         offsets=(ssize_t *) RelinquishMagickMemory(offsets);
       }
     iris_pixels=(unsigned char *) RelinquishMagickMemory(iris_pixels);
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
-  if (image_info->adjoin != MagickFalse)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
   CloseBlob(image);
   return(MagickTrue);
 }
