@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -42,10 +42,13 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
-#include "magick/color_private.h"
+#include "magick/blob-private.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -169,49 +172,81 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
     Read TGA header information.
   */
   count=ReadBlob(image,1,&tga_info.id_length);
-  tga_info.colormap_type=ReadBlobByte(image);
-  tga_info.image_type=ReadBlobByte(image);
+  tga_info.colormap_type=(unsigned char) ReadBlobByte(image);
+  tga_info.image_type=(unsigned char) ReadBlobByte(image);
   do
   {
-    if ((count == 0) || (tga_info.image_type == 0) ||
-        (tga_info.image_type > 11))
+    if ((count != 1) ||
+        ((tga_info.image_type != TGAColormap) &&
+         (tga_info.image_type != TGARGB) &&
+         (tga_info.image_type != TGAMonochrome) &&
+         (tga_info.image_type != TGARLEColormap) &&
+         (tga_info.image_type != TGARLERGB) &&
+         (tga_info.image_type != TGARLEMonochrome)) ||
+        (((tga_info.image_type == TGAColormap) ||
+         (tga_info.image_type == TGARLEColormap)) &&
+         (tga_info.colormap_type == 0)))
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     tga_info.colormap_index=ReadBlobLSBShort(image);
     tga_info.colormap_length=ReadBlobLSBShort(image);
-    tga_info.colormap_size=ReadBlobByte(image);
+    tga_info.colormap_size=(unsigned char) ReadBlobByte(image);
     tga_info.x_origin=ReadBlobLSBShort(image);
     tga_info.y_origin=ReadBlobLSBShort(image);
     tga_info.width=ReadBlobLSBShort(image);
     tga_info.height=ReadBlobLSBShort(image);
-    tga_info.bits_per_pixel=ReadBlobByte(image);
-    tga_info.attributes=ReadBlobByte(image);
+    tga_info.bits_per_pixel=(unsigned char) ReadBlobByte(image);
+    tga_info.attributes=(unsigned char) ReadBlobByte(image);
     /*
       Initialize image structure.
     */
-    image->matte=tga_info.bits_per_pixel == 32 ? MagickTrue : MagickFalse;
+    image->matte=(tga_info.attributes & 0x0FU) != 0 ? MagickTrue : MagickFalse;
     image->columns=tga_info.width;
     image->rows=tga_info.height;
-    image->depth=tga_info.bits_per_pixel <= 8 ? 8 : QuantumDepth;
-    if (tga_info.colormap_type != 0)
+    if ((tga_info.image_type != TGAColormap) &&
+        (tga_info.image_type != TGARLEColormap))
+      image->depth=(unsigned long) ((tga_info.bits_per_pixel <= 8) ? 8 :
+        (tga_info.bits_per_pixel <= 16) ? 5 :
+        (tga_info.bits_per_pixel == 24) ? 8 :
+        (tga_info.bits_per_pixel == 32) ? 8 : 8);
+    else
+      image->depth=(unsigned long) ((tga_info.colormap_size <= 8) ? 8 :
+        (tga_info.colormap_size <= 16) ? 5 :
+        (tga_info.colormap_size == 24) ? 8 :
+        (tga_info.colormap_size == 32) ? 8 : 8);
+    if ((tga_info.image_type == TGAColormap) ||
+        (tga_info.image_type == TGAMonochrome) ||
+        (tga_info.image_type == TGARLEColormap) ||
+        (tga_info.image_type == TGARLEMonochrome))
+      image->storage_class=PseudoClass;
+    image->compression=NoCompression;
+    if ((tga_info.image_type == TGARLEColormap) ||
+        (tga_info.image_type == TGARLEMonochrome))
+      image->compression=RLECompression;
+    if (image->storage_class == PseudoClass)
       {
-        if ((tga_info.image_type == TGARLEColormap) ||
-            (tga_info.image_type == TGARLERGB))
-          image->storage_class=PseudoClass;
-        image->colors=tga_info.colormap_length;
+        if (tga_info.colormap_type != 0)
+          image->colors=tga_info.colormap_length;
+        else
+          {
+            image->colors=0x01U << tga_info.bits_per_pixel;
+            if (AllocateImageColormap(image,image->colors) == MagickFalse)
+              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+          }
       }
     if (tga_info.id_length != 0)
       {
@@ -224,9 +259,9 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         comment=(char *) AcquireMagickMemory(tga_info.id_length+MaxTextExtent);
         if (comment == (char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        (void) ReadBlob(image,tga_info.id_length,(unsigned char *) comment);
+        count=ReadBlob(image,tga_info.id_length,(unsigned char *) comment);
         comment[tga_info.id_length]='\0';
-        (void) SetImageAttribute(image,"comment",comment);
+        (void) SetImageAttribute(image,"Comment",comment);
         comment=(char *) RelinquishMagickMemory(comment);
       }
     (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
@@ -259,12 +294,12 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 5 bits each of red green and blue.
               */
-              j=ReadBlobByte(image);
-              k=ReadBlobByte(image);
-              pixel.red=ScaleXToQuantum((int) (k & 0x7c) >> 2,31);
-              pixel.green=ScaleXToQuantum(((int) (k & 0x03) << 3)+
+              j=(unsigned char) ReadBlobByte(image);
+              k=(unsigned char) ReadBlobByte(image);
+              pixel.red=ScaleAnyToQuantum((int) (k & 0x7c) >> 2,31);
+              pixel.green=ScaleAnyToQuantum(((int) (k & 0x03) << 3)+
                 ((int) (j & 0xe0) >> 5),31);
-              pixel.blue=ScaleXToQuantum((int) (j & 0x1f),31);
+              pixel.blue=ScaleAnyToQuantum((int) (j & 0x1f),31);
               break;
             }
             case 24:
@@ -336,7 +371,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 Gray scale.
               */
-              index=ReadBlobByte(image);
+              index=(IndexPacket) ReadBlobByte(image);
               if (tga_info.colormap_type != 0)
                 pixel=image->colormap[ConstrainColormapIndex(image,index)];
               else
@@ -353,14 +388,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 5 bits each of red green and blue.
               */
-              j=ReadBlobByte(image);
-              k=ReadBlobByte(image);
-              pixel.red=ScaleXToQuantum((int) (k & 0x7c) >> 2,31);
-              pixel.red=ScaleXToQuantum(((int) (k & 0x03) << 3)+
+              j=(unsigned char) ReadBlobByte(image);
+              k=(unsigned char) ReadBlobByte(image);
+              pixel.red=ScaleAnyToQuantum((int) (k & 0x7c) >> 2,31);
+              pixel.red=ScaleAnyToQuantum(((int) (k & 0x03) << 3)+
                 ((int) (j & 0xe0) >> 5),31);
-              pixel.blue=ScaleXToQuantum((int) (j & 0x1f),31);
+              pixel.blue=ScaleAnyToQuantum((int) (j & 0x1f),31);
               if (image->storage_class == PseudoClass)
-                index=ConstrainColormapIndex(image,((unsigned short) k << 8)+j);
+                index=ConstrainColormapIndex(image,((unsigned long) k << 8)+j);
               break;
             }
             case 24:
@@ -373,7 +408,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               pixel.green=ScaleCharToQuantum(ReadBlobByte(image));
               pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
               if (tga_info.bits_per_pixel == 32)
-                pixel.opacity=ScaleCharToQuantum(ReadBlobByte(image));
+                pixel.opacity=QuantumRange-ScaleCharToQuantum(ReadBlobByte(image));
               break;
             }
           }
@@ -398,9 +433,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (SyncImagePixels(image) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != MagickFalse)
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
     }
     if (EOFBlob(image) != MagickFalse)
       {
@@ -415,26 +455,33 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     count=ReadBlob(image,1,&tga_info.id_length);
-    tga_info.colormap_type=ReadBlobByte(image);
-    tga_info.image_type=ReadBlobByte(image);
-    status=((tga_info.image_type != 0) && (tga_info.image_type <= 11)) ?
-      MagickTrue : MagickFalse;
+    tga_info.colormap_type=(unsigned char) ReadBlobByte(image);
+    tga_info.image_type=(unsigned char) ReadBlobByte(image);
+    status=((tga_info.image_type == TGAColormap) ||
+      (tga_info.image_type == TGARGB) ||
+      (tga_info.image_type == TGAMonochrome) ||
+      (tga_info.image_type == TGARLEColormap) ||
+      (tga_info.image_type == TGARLERGB) ||
+      (tga_info.image_type == TGARLEMonochrome)) ? MagickTrue : MagickFalse;
     if (status == MagickTrue)
       {
         /*
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (status == MagickTrue);
   CloseBlob(image);
@@ -626,7 +673,7 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -638,9 +685,9 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
     */
     (void) SetImageColorspace(image,RGBColorspace);
     targa_info.id_length=0;
-    attribute=GetImageAttribute(image,"comment");
+    attribute=GetImageAttribute(image,"Comment");
     if (attribute != (const ImageAttribute *) NULL)
-      targa_info.id_length=Min(strlen(attribute->value),255);
+      targa_info.id_length=(unsigned char) Min(strlen(attribute->value),255);
     targa_info.colormap_type=0;
     targa_info.colormap_index=0;
     targa_info.colormap_length=0;
@@ -651,30 +698,36 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
     targa_info.height=(unsigned short) image->rows;
     targa_info.bits_per_pixel=8;
     targa_info.attributes=0;
-    if ((image->storage_class == DirectClass) || (image->colors > 256))
-      {
-        /*
-          Full color TGA raster.
-        */
-        targa_info.image_type=TargaRGB;
-        targa_info.bits_per_pixel=24;
-        if (image->matte != MagickFalse)
-          {
-            targa_info.bits_per_pixel=32;
-            targa_info.attributes=8;  /* # of alpha bits */
-          }
-      }
+    if ((image_info->type != TrueColorType) &&
+        (image_info->type != TrueColorMatteType) &&
+        (image_info->type != PaletteType) &&
+        (image->matte == MagickFalse) &&
+        (IsGrayImage(image,&image->exception) != MagickFalse))
+      targa_info.image_type=TargaMonochrome;
     else
-      {
-        /*
-          Colormapped TGA raster.
-        */
-        targa_info.image_type=TargaColormap;
-        targa_info.colormap_type=1;
-        targa_info.colormap_index=0;
-        targa_info.colormap_length=(unsigned short) image->colors;
-        targa_info.colormap_size=24;
-      }
+      if ((image->storage_class == DirectClass) || (image->colors > 256))
+        {
+          /*
+            Full color TGA raster.
+          */
+          targa_info.image_type=TargaRGB;
+          targa_info.bits_per_pixel=24;
+          if (image->matte != MagickFalse)
+            {
+              targa_info.bits_per_pixel=32;
+              targa_info.attributes=8;  /* # of alpha bits */
+            }
+        }
+      else
+        {
+          /*
+            Colormapped TGA raster.
+          */
+          targa_info.image_type=TargaColormap;
+          targa_info.colormap_type=1;
+          targa_info.colormap_length=(unsigned short) image->colors;
+          targa_info.colormap_size=24;
+        }
     /*
       Write TGA header.
     */
@@ -718,8 +771,8 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
     /*
       Convert MIFF to TGA raster pixels.
     */
-    count=(long) ((targa_info.bits_per_pixel*targa_info.width) >> 3);
-    targa_pixels=(unsigned char *) AcquireMagickMemory(count);
+    count=(ssize_t) ((targa_info.bits_per_pixel*targa_info.width) >> 3);
+    targa_pixels=(unsigned char *) AcquireMagickMemory((size_t) count);
     if (targa_pixels == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     for (y=(long) (image->rows-1); y >= 0; y--)
@@ -732,36 +785,45 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
       for (x=0; x < (long) image->columns; x++)
       {
         if (targa_info.image_type == TargaColormap)
-          *q++=indexes[x];
+          *q++=(unsigned char) indexes[x];
         else
-          {
-            *q++=ScaleQuantumToChar(p->blue);
-            *q++=ScaleQuantumToChar(p->green);
-            *q++=ScaleQuantumToChar(p->red);
-            if (image->matte != MagickFalse)
-              *q++=ScaleQuantumToChar(p->opacity);
-            if (image->colorspace == CMYKColorspace)
-              *q++=ScaleQuantumToChar(indexes[x]);
-          }
+          if (targa_info.image_type == TargaMonochrome)
+            *q++=(unsigned char) ScaleQuantumToChar(PixelIntensityToQuantum(p));
+          else
+            {
+              *q++=ScaleQuantumToChar(p->blue);
+              *q++=ScaleQuantumToChar(p->green);
+              *q++=ScaleQuantumToChar(p->red);
+              if (image->matte != MagickFalse)
+                *q++=QuantumRange-ScaleQuantumToChar(p->opacity);
+              if (image->colorspace == CMYKColorspace)
+                *q++=ScaleQuantumToChar(indexes[x]);
+            }
         p++;
       }
-      (void) WriteBlob(image,q-targa_pixels,targa_pixels);
+      (void) WriteBlob(image,(size_t) (q-targa_pixels),targa_pixels);
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            status=image->progress_monitor(SaveImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
     }
     targa_pixels=(unsigned char *) RelinquishMagickMemory(targa_pixels);
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
   return(MagickTrue);
