@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -46,21 +46,25 @@
 #include "magick/annotate.h"
 #include "magick/client.h"
 #include "magick/color.h"
+#include "magick/color-private.h"
 #include "magick/composite.h"
-#include "magick/composite_private.h"
+#include "magick/composite-private.h"
 #include "magick/constitute.h"
 #include "magick/draw.h"
-#include "magick/draw_private.h"
-#include "magick/error.h"
+#include "magick/draw-private.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/gem.h"
 #include "magick/geometry.h"
+#include "magick/image-private.h"
 #include "magick/log.h"
 #include "magick/resource_.h"
+#include "magick/statistic.h"
 #include "magick/string_.h"
 #include "magick/transform.h"
 #include "magick/type.h"
 #include "magick/utility.h"
-#include "magick/xwindow.h"
+#include "magick/xwindow-private.h"
 #if defined(HasTTF)
 #if defined(__MINGW32__)
 #  undef interface
@@ -155,7 +159,6 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
 {
   char
     primitive[MaxTextExtent],
-    *text,
     **textlist;
 
   DrawInfo
@@ -187,25 +190,17 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
     height,
     number_lines;
 
-  /*
-    Translate any embedded format characters (e.g. %f).
-  */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(draw_info != (DrawInfo *) NULL);
   assert(draw_info->signature == MagickSignature);
   if (draw_info->text == (char *) NULL)
     return(MagickFalse);
   if (*draw_info->text == '\0')
-    return(MagickFalse);
-  text=TranslateText((ImageInfo *) NULL,image,draw_info->text);
-  if (text == (char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
-  textlist=StringToList(text);
-  text=(char *) RelinquishMagickMemory(text);
+    return(MagickTrue);
+  textlist=StringToList(draw_info->text);
   if (textlist == (char **) NULL)
     return(MagickFalse);
   length=strlen(textlist[0]);
@@ -213,10 +208,6 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
     if (strlen(textlist[i]) > length)
       length=strlen(textlist[i]);
   number_lines=(unsigned long) i;
-  text=(char *) AcquireMagickMemory(length+MaxTextExtent);
-  if (text == (char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
   annotate=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   annotate_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   SetGeometry(image,&geometry);
@@ -379,16 +370,22 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
     }
     if (draw_info->undercolor.opacity != TransparentOpacity)
       {
+        DrawInfo
+          *undercolor_info;
+
         /*
           Text box.
         */
-        annotate_info->fill=draw_info->undercolor;
-        annotate_info->affine.tx=offset.x-draw_info->affine.ry*metrics.ascent;
-        annotate_info->affine.ty=offset.y-draw_info->affine.sy*metrics.ascent;
+        undercolor_info=CloneDrawInfo((ImageInfo *) NULL,(DrawInfo *) NULL);
+        undercolor_info->fill=draw_info->undercolor;
+        undercolor_info->affine=draw_info->affine;
+        undercolor_info->affine.tx=offset.x-draw_info->affine.ry*metrics.ascent;
+        undercolor_info->affine.ty=offset.y-draw_info->affine.sy*metrics.ascent;
         (void) FormatMagickString(primitive,MaxTextExtent,
           "rectangle 0,0 %g,%ld",metrics.width,height);
-        (void) CloneString(&annotate_info->primitive,primitive);
-        (void) DrawImage(image,annotate_info);
+        (void) CloneString(&undercolor_info->primitive,primitive);
+        (void) DrawImage(image,undercolor_info);
+        DestroyDrawInfo(undercolor_info);
       }
     annotate_info->affine.tx=offset.x;
     annotate_info->affine.ty=offset.y;
@@ -406,7 +403,7 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
       if (annotate->decorate == UnderlineDecoration)
         {
           annotate_info->affine.ty-=
-            (draw_info->affine.sy*metrics.underline_position);
+            (draw_info->affine.sy*metrics.underline_position+2);
           (void) CloneString(&annotate_info->primitive,primitive);
           (void) DrawImage(image,annotate_info);
         }
@@ -427,9 +424,8 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
   /*
     Free resources.
   */
-  DestroyDrawInfo(annotate_info);
-  DestroyDrawInfo(annotate);
-  text=(char *) RelinquishMagickMemory(text);
+  annotate_info=DestroyDrawInfo(annotate_info);
+  annotate=DestroyDrawInfo(annotate);
   for (i=0; textlist[i] != (char *) NULL; i++)
     textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
   textlist=(char **) RelinquishMagickMemory(textlist);
@@ -732,6 +728,119 @@ static unsigned short *EncodeUnicode(const Image *image,const char *text,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t M u l t i l i n e T y p e M e t r i c s                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetMultilineTypeMetrics() returns the following information for the
+%  specified font and text:
+%
+%    character width
+%    character height
+%    ascent
+%    descent
+%    text width
+%    text height
+%    maximum horizontal advance
+%    underline position
+%    underline thickness
+%
+%  This method is like GetTypeMetrics() but it returns the maximum text width
+%  and height for multiple lines of text.
+%
+%  The format of the GetMultilineTypeMetrics method is:
+%
+%      MagickBooleanType GetMultilineTypeMetrics(Image *image,
+%        const DrawInfo *draw_info,TypeMetric *metrics)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o draw_info: The draw info.
+%
+%    o metrics: Return the font metrics in this structure.
+%
+*/
+MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
+  const DrawInfo *draw_info,TypeMetric *metrics)
+{
+  char
+    **textlist;
+
+  double
+    max_width;
+
+  DrawInfo
+    *annotate_info;
+
+  MagickBooleanType
+    status;
+
+  PointInfo
+    offset;
+
+  register long
+    i;
+
+  unsigned long
+    number_lines;
+
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(draw_info != (DrawInfo *) NULL);
+  assert(draw_info->text != (char *) NULL);
+  assert(draw_info->signature == MagickSignature);
+  if (*draw_info->text == '\0')
+    return(MagickFalse);
+  annotate_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
+  annotate_info->text=(char *) RelinquishMagickMemory(annotate_info->text);
+  /*
+    Convert newlines to multiple lines of text.
+  */
+  textlist=StringToList(draw_info->text);
+  if (textlist == (char **) NULL)
+    return(MagickFalse);
+  annotate_info->render=MagickFalse;
+  (void) ResetMagickMemory(metrics,0,sizeof(*metrics));
+  offset.x=0.0;
+  offset.y=0.0;
+  /*
+    Find the widest of the text lines.
+  */
+  annotate_info->text = textlist[0];
+  status=RenderType(image,annotate_info,&offset,metrics);
+  max_width = metrics->width;
+  for (i=1; textlist[i] != (char *) NULL; i++)
+  {
+    annotate_info->text = textlist[i];
+    status=RenderType(image,annotate_info,&offset,metrics);
+    if (metrics->width > max_width)
+      max_width=metrics->width;
+  }
+  number_lines= i;
+  metrics->width=max_width;
+  metrics->height=number_lines*(long) (metrics->ascent-metrics->descent+0.5)+1;
+  /*
+    Free resources.
+  */
+  annotate_info->text=(char *) NULL;
+  annotate_info=DestroyDrawInfo(annotate_info);
+  for (i=0; textlist[i] != (char *) NULL; i++)
+    textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
+  textlist=(char **) RelinquishMagickMemory(textlist);
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t T y p e M e t r i c s                                               %
 %                                                                             %
 %                                                                             %
@@ -741,15 +850,15 @@ static unsigned short *EncodeUnicode(const Image *image,const char *text,
 %  GetTypeMetrics() returns the following information for the specified font
 %  and text:
 %
-%    o character width
-%    o character height
-%    o ascent
-%    o descent
-%    o text width
-%    o text height
-%    o maximum horizontal advance
-%    o underline position
-%    o underline thickness
+%    character width
+%    character height
+%    ascent
+%    descent
+%    text width
+%    text height
+%    maximum horizontal advance
+%    underline position
+%    underline thickness
 %
 %  The format of the GetTypeMetrics method is:
 %
@@ -781,7 +890,7 @@ MagickExport MagickBooleanType GetTypeMetrics(Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(draw_info != (DrawInfo *) NULL);
   assert(draw_info->text != (char *) NULL);
   assert(draw_info->signature == MagickSignature);
@@ -791,7 +900,7 @@ MagickExport MagickBooleanType GetTypeMetrics(Image *image,
   offset.x=0.0;
   offset.y=0.0;
   status=RenderType(image,annotate_info,&offset,metrics);
-  DestroyDrawInfo(annotate_info);
+  annotate_info=DestroyDrawInfo(annotate_info);
   return(status);
 }
 
@@ -845,20 +954,31 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
         return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
       if (*draw_info->font == '-')
         return(RenderX11(image,draw_info,offset,metrics));
+      if (IsAccessible(draw_info->font) != MagickFalse)
+        return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
       type_info=GetTypeInfo(draw_info->font,&image->exception);
       if (type_info == (const TypeInfo *) NULL)
-        if (IsAccessible(draw_info->font) != MagickFalse)
-          return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
+        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+          TypeWarning,"UnableToReadFont","`%s'",draw_info->font);
+    }
+  if ((type_info == (const TypeInfo *) NULL) &&
+      (draw_info->family != (const char *) NULL))
+    {
+      type_info=GetTypeInfoByFamily(draw_info->family,draw_info->style,
+        draw_info->stretch,draw_info->weight,&image->exception);
+      if (type_info == (const TypeInfo *) NULL)
+        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+          TypeWarning,"UnableToReadFont","`%s'",draw_info->family);
     }
   if (type_info == (const TypeInfo *) NULL)
-    type_info=GetTypeInfoByFamily(draw_info->family,draw_info->style,
+    type_info=GetTypeInfoByFamily("arial",draw_info->style,
       draw_info->stretch,draw_info->weight,&image->exception);
   if (type_info == (const TypeInfo *) NULL)
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),TypeWarning,
-      "UnableToReadFont",draw_info->font);
+    type_info=GetTypeInfoByFamily("helvetica",draw_info->style,
+      draw_info->stretch,draw_info->weight,&image->exception);
   if (type_info == (const TypeInfo *) NULL)
-    type_info=GetTypeInfoByFamily("Arial",draw_info->style,draw_info->stretch,
-      draw_info->weight,&image->exception);
+    type_info=GetTypeInfoByFamily((const char *) NULL,draw_info->style,
+      draw_info->stretch,draw_info->weight,&image->exception);
   if (type_info == (const TypeInfo *) NULL)
     return(RenderPostscript(image,draw_info,offset,metrics));
   annotate_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
@@ -868,7 +988,7 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
   if (type_info->glyphs != (char *) NULL)
     (void) CloneString(&annotate_info->font,type_info->glyphs);
   status=RenderFreetype(image,annotate_info,type_info->encoding,offset,metrics);
-  DestroyDrawInfo(annotate_info);
+  annotate_info=DestroyDrawInfo(annotate_info);
   return(status);
 }
 
@@ -1070,7 +1190,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     Initialize Truetype library.
   */
   status=FT_Init_FreeType(&library);
-  if (status)
+  if (status != 0)
     ThrowBinaryException(TypeError,"UnableToInitializeFreetypeLibrary",
       image->filename);
   args.flags=FT_OPEN_PATHNAME;
@@ -1078,6 +1198,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     args.pathname=draw_info->font;
   else
     args.pathname=draw_info->font+1;
+  face=(FT_Face) NULL;
   status=FT_Open_Face(library,&args,draw_info->face,&face);
   if (status != 0)
     {
@@ -1087,15 +1208,10 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   if ((draw_info->metrics != (char *) NULL) &&
       (IsAccessible(draw_info->metrics) != MagickFalse))
     (void) FT_Attach_File(face,draw_info->metrics);
-  if (face->num_charmaps != 0)
-    status=FT_Set_Charmap(face,face->charmaps[0]);
   encoding_type=ft_encoding_unicode;
   status=FT_Select_Charmap(face,encoding_type);
-  if (status != 0)
-    {
-      encoding_type=ft_encoding_none;
-      status=FT_Select_Charmap(face,encoding_type);
-    }
+  if ((status != 0) && (face->num_charmaps != 0))
+    status=FT_Set_Charmap(face,face->charmaps[0]);
   if (encoding != (char *) NULL)
     {
       if (LocaleCompare(encoding,"AdobeCustom") == 0)
@@ -1110,6 +1226,12 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
         encoding_type=ft_encoding_big5;
       if (LocaleCompare(encoding,"GB2312") == 0)
         encoding_type=ft_encoding_gb2312;
+      if (LocaleCompare(encoding,"Johab") == 0)
+        encoding_type=ft_encoding_johab;
+      if (LocaleCompare(encoding,"Latin-1") == 0)
+        encoding_type=ft_encoding_latin_1;
+      if (LocaleCompare(encoding,"Latin-2") == 0)
+        encoding_type=ft_encoding_latin_2;
       if (LocaleCompare(encoding,"None") == 0)
         encoding_type=ft_encoding_none;
       if (LocaleCompare(encoding,"SJIScode") == 0)
@@ -1127,8 +1249,8 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   /*
     Set text size.
   */
-  resolution.x=72.0;
-  resolution.y=72.0;
+  resolution.x=DefaultResolution;
+  resolution.y=DefaultResolution;
   if (draw_info->density != (char *) NULL)
     {
       GeometryInfo
@@ -1196,6 +1318,11 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
               break;
             }
         }
+      if (encoding_type == ft_encoding_unicode)
+        {
+          text=EncodeUnicode(image,draw_info->text,&length);
+          break;
+        }
       text=EncodeText(image,draw_info->text,&length);
       break;
     }
@@ -1235,6 +1362,8 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   for (i=0; i < (long) length; i++)
   {
     glyph.id=FT_Get_Char_Index(face,text[i]);
+    if (glyph.id == 0)
+      glyph.id=text[i];
     if ((glyph.id != 0) && (last_glyph.id != 0) && FT_HAS_KERNING(face))
       {
         FT_Vector
@@ -1245,17 +1374,16 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
         origin.x+=kerning.x;
       }
     glyph.origin=origin;
-    if (draw_info->render != MagickFalse)
-      status=FT_Load_Glyph(face,glyph.id,FT_LOAD_DEFAULT);
-    else
-      status=FT_Load_Glyph(face,glyph.id,FT_LOAD_NO_BITMAP);
-    if (status != MagickFalse)
+    status=FT_Load_Glyph(face,glyph.id,FT_LOAD_NO_BITMAP);
+    if (status != 0)
       continue;
     status=FT_Get_Glyph(face->glyph,&glyph.image);
-    if (status != MagickFalse)
+    if (status != 0)
       continue;
-    (void) FT_Outline_Get_BBox(&((FT_OutlineGlyph) glyph.image)->outline,
+    status=FT_Outline_Get_BBox(&((FT_OutlineGlyph) glyph.image)->outline,
       &bounds);
+    if (status != 0)
+      continue;
     if ((i == 0) || (bounds.xMin < metrics->bounds.x1))
       metrics->bounds.x1=bounds.xMin;
     if ((i == 0) || (bounds.yMin < metrics->bounds.y1))
@@ -1280,12 +1408,16 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     (void) FT_Glyph_Transform(glyph.image,&affine,&glyph.origin);
     if (draw_info->render != MagickFalse)
       {
+        long
+          x_offset,
+          y_offset;
+
         /*
           Rasterize the glyph.
         */
         status=FT_Glyph_To_Bitmap(&glyph.image,ft_render_mode_normal,
           (FT_Vector *) NULL,MagickTrue);
-        if (status != MagickFalse)
+        if (status != 0)
           continue;
         bitmap=(FT_BitmapGlyph) glyph.image;
         image->storage_class=DirectClass;
@@ -1294,19 +1426,20 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
         p=bitmap->bitmap.buffer;
         for (y=0; y < (long) bitmap->bitmap.rows; y++)
         {
-          if (((long) (point.y+y+0.5) < 0) ||
-              ((long) (point.y+y+0.5) >= (long) image->rows))
+          x_offset=(long) (point.x+0.5);
+          y_offset=(long) (point.y+y+0.5);
+          if ((y_offset < 0) || (y_offset >= (long) image->rows))
             {
               p+=bitmap->bitmap.width;
               continue;
             }
-          q=GetImagePixels(image,(long) (point.x+0.5),(long) (point.y+y+0.5),
-            bitmap->bitmap.width,1);
+          q=GetImagePixels(image,x_offset,y_offset,bitmap->bitmap.width,1);
           active=q != (PixelPacket *) NULL ? MagickTrue : MagickFalse;
           for (x=0; x < (long) bitmap->bitmap.width; x++)
           {
-            if (((long) (point.x+x+0.5) < 0) || (*p == 0) ||
-                ((unsigned long) (point.x+x+0.5) >= image->columns))
+            x_offset=(long) (point.x+x+0.5);
+            if ((*p == 0) || (x_offset < 0) ||
+                (x_offset >= (long) image->columns))
               {
                 p++;
                 q++;
@@ -1317,17 +1450,16 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
             else
               fill_opacity=(*p) > 127 ? 1.0 : 0.0;
             if (active == MagickFalse)
-              q=GetImagePixels(image,(long) (point.x+x+0.5),(long)
-                (point.y+y+0.5),1,1);
+              q=GetImagePixels(image,x_offset,y_offset,1,1);
             if (q == (PixelPacket *) NULL)
               {
                 p++;
                 q++;
                 continue;
               }
-            fill_color=GetFillColor(draw_info,(long) (point.x+x+0.5),(long)
-              (point.y+y+0.5));
-            fill_opacity=MaxRGB-fill_opacity*(MaxRGB-fill_color.opacity);
+            fill_color=GetFillColor(draw_info,x_offset,y_offset);
+            fill_opacity=QuantumRange-fill_opacity*
+              (QuantumRange-fill_color.opacity);
             MagickCompositeOver(&fill_color,fill_opacity,q,q->opacity,q);
             if (active == MagickFalse)
               (void) SyncImagePixels(image);
@@ -1368,14 +1500,15 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     Free resources.
   */
   text=(unsigned short *) RelinquishMagickMemory(text);
-  DestroyDrawInfo(annotate_info);
+  annotate_info=DestroyDrawInfo(annotate_info);
   (void) FT_Done_Face(face);
   (void) FT_Done_FreeType(library);
   return(MagickTrue);
 }
 #else
-static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
-  const char *encoding,const PointInfo *offset,TypeMetric *metrics)
+static MagickBooleanType RenderFreetype(Image *image,
+  const DrawInfo *magick_unused(draw_info),const char *magick_unused(encoding),
+  const PointInfo *magick_unused(offset),TypeMetric *magick_unused(metrics))
 {
   ThrowBinaryException(MissingDelegateError,"FreeTypeLibraryIsNotAvailable",
     image->filename)
@@ -1528,8 +1661,8 @@ static MagickBooleanType RenderPostscript(Image *image,
     if (point.y > extent.y)
       extent.y=point.y;
   }
-  (void) fprintf(file,"%g %g moveto\n",identity  != MagickFalse ? 0.0 : extent.x/2.0,
-    extent.y/2.0);
+  (void) fprintf(file,"%g %g moveto\n",identity  != MagickFalse ? 0.0 :
+    extent.x/2.0,extent.y/2.0);
   (void) fprintf(file,"%g %g scale\n",draw_info->pointsize,
     draw_info->pointsize);
   if ((draw_info->font == (char *) NULL) || (*draw_info->font == '\0'))
@@ -1559,15 +1692,13 @@ static MagickBooleanType RenderPostscript(Image *image,
     (void) CloneString(&annotate_info->density,draw_info->density);
   annotate_info->antialias=draw_info->text_antialias;
   annotate_image=ReadImage(annotate_info,&image->exception);
-  DestroyImageInfo(annotate_info);
+  CatchException(&image->exception);
+  annotate_info=DestroyImageInfo(annotate_info);
   (void) RelinquishUniqueFileResource(filename);
-  if (image->exception.severity != UndefinedException)
-    MagickError(image->exception.severity,image->exception.reason,
-      image->exception.description);
   if (annotate_image == (Image *) NULL)
     return(MagickFalse);
-  resolution.x=72.0;
-  resolution.y=72.0;
+  resolution.x=DefaultResolution;
+  resolution.y=DefaultResolution;
   if (draw_info->density != (char *) NULL)
     {
       GeometryInfo
@@ -1590,14 +1721,14 @@ static MagickBooleanType RenderPostscript(Image *image,
         crop_info;
 
       crop_info=GetImageBoundingBox(annotate_image,&annotate_image->exception);
-      crop_info.height=(unsigned long) ((resolution.y/72.0)*
+      crop_info.height=(unsigned long) ((resolution.y/DefaultResolution)*
         ExpandAffine(&draw_info->affine)*draw_info->pointsize+0.5);
-      crop_info.y=(long) ((resolution.y/72.0)*extent.y/8.0+0.5);
+      crop_info.y=(long) ((resolution.y/DefaultResolution)*extent.y/8.0+0.5);
       (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu%+ld%+ld",
         crop_info.width,crop_info.height,crop_info.x,crop_info.y);
       (void) TransformImage(&annotate_image,geometry,(char *) NULL);
     }
-  metrics->pixels_per_em.x=(resolution.y/72.0)*
+  metrics->pixels_per_em.x=(resolution.y/DefaultResolution)*
     ExpandAffine(&draw_info->affine)*draw_info->pointsize;
   metrics->pixels_per_em.y=metrics->pixels_per_em.x;
   metrics->ascent=metrics->pixels_per_em.x;
@@ -1614,7 +1745,7 @@ static MagickBooleanType RenderPostscript(Image *image,
   metrics->underline_thickness=1.0;
   if (draw_info->render == MagickFalse)
     {
-      DestroyImage(annotate_image);
+      annotate_image=DestroyImage(annotate_image);
       return(MagickTrue);
     }
   if (draw_info->fill.opacity != TransparentOpacity)
@@ -1643,9 +1774,9 @@ static MagickBooleanType RenderPostscript(Image *image,
               (long) (x-pattern->extract_info.x) % pattern->columns,
               (long) (y-pattern->extract_info.y) % pattern->rows,
               &image->exception);
-          q->opacity=(Quantum) (MaxRGB-(((MaxRGB-(MagickRealType)
-            PixelIntensityToQuantum(q))*(MaxRGB-fill_color.opacity))/
-            MaxRGB)+0.5);
+          q->opacity=(Quantum) (QuantumRange-(((QuantumRange-(MagickRealType)
+            PixelIntensityToQuantum(q))*(QuantumRange-fill_color.opacity))/
+            QuantumRange)+0.5);
           q->red=fill_color.red;
           q->green=fill_color.green;
           q->blue=fill_color.blue;
@@ -1658,7 +1789,7 @@ static MagickBooleanType RenderPostscript(Image *image,
         (long) (offset->x+0.5),(long) (offset->y-(metrics->ascent+
         metrics->descent)+0.5));
     }
-  DestroyImage(annotate_image);
+  annotate_image=DestroyImage(annotate_image);
   return(MagickTrue);
 }
 
