@@ -17,13 +17,13 @@
 %                                 May 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -45,27 +45,28 @@
 #include "magick/coder.h"
 #include "magick/configure.h"
 #include "magick/draw.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/hashmap.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
-#include "magick/nt_feature.h"
 #include "magick/option.h"
 #include "magick/semaphore.h"
 #include "magick/string_.h"
+#include "magick/splay-tree.h"
 #include "magick/token.h"
 #include "magick/utility.h"
 
 /*
   Define declarations.
 */
-#define MagickCoderFilename  "coder.mgk"
+#define MagickCoderFilename  "coder.xml"
 
 /*
   Declare coder map.
 */
 static const char
-  *CoderMap = (char *)
+  *CoderMap = (const char *)
     "<?xml version=\"1.0\"?>"
     "<codermap>"
     "  <coder magick=\"8BIM\" name=\"META\" />"
@@ -82,6 +83,7 @@ static const char
     "  <coder magick=\"C\" name=\"GRAY\" />"
     "  <coder magick=\"CUR\" name=\"ICON\" />"
     "  <coder magick=\"DCX\" name=\"PCX\" />"
+    "  <coder magick=\"DNG\" name=\"TIFF\" />"
     "  <coder magick=\"EMF\" name=\"EMF\" />"
     "  <coder magick=\"EPDF\" name=\"PDF\" />"
     "  <coder magick=\"EPI\" name=\"PS\" />"
@@ -112,6 +114,7 @@ static const char
     "  <coder magick=\"JNG\" name=\"PNG\" />"
     "  <coder magick=\"JPC\" name=\"JP2\" />"
     "  <coder magick=\"JPG\" name=\"JPEG\" />"
+    "  <coder magick=\"JPX\" name=\"JP2\" />"
     "  <coder magick=\"K\" name=\"GRAY\" />"
     "  <coder magick=\"LOGO\" name=\"MAGICK\" />"
     "  <coder magick=\"M2V\" name=\"MPEG\" />"
@@ -166,12 +169,12 @@ static const char
 /*
   Static declarations.
 */
-static LinkedListInfo
-  *coder_list = (LinkedListInfo *) NULL;
-
 static SemaphoreInfo
   *coder_semaphore = (SemaphoreInfo *) NULL;
 
+static SplayTreeInfo
+  *coder_list = (SplayTreeInfo *) NULL;
+
 static volatile MagickBooleanType
   instantiate_coder = MagickFalse;
 
@@ -201,32 +204,14 @@ static MagickBooleanType
 %
 %
 */
-
-static void *DestroyCoderElement(void *coder_info)
-{
-  register CoderInfo
-    *p;
-
-  p=(CoderInfo *) coder_info;
-  if (p->path != (char *) NULL)
-    p->path=(char *) RelinquishMagickMemory(p->path);
-  if (p->name != (char *) NULL)
-    p->name=(char *) RelinquishMagickMemory(p->name);
-  if (p->magick != (char *) NULL)
-    p->magick=(char *) RelinquishMagickMemory(p->magick);
-  p=(CoderInfo *) RelinquishMagickMemory(p);
-  return((void *) NULL);
-}
-
 MagickExport void DestroyCoderList(void)
 {
   AcquireSemaphoreInfo(&coder_semaphore);
-  if (coder_list != (LinkedListInfo *) NULL)
-    DestroyLinkedList(coder_list,DestroyCoderElement);
-  coder_list=(LinkedListInfo *) NULL;
+  if (coder_list != (SplayTreeInfo *) NULL)
+    coder_list=DestroySplayTree(coder_list);
   instantiate_coder=MagickFalse;
-  RelinquishSemaphoreInfo(&coder_semaphore);
-  DestroySemaphoreInfo(&coder_semaphore);
+  RelinquishSemaphoreInfo(coder_semaphore);
+  coder_semaphore=DestroySemaphoreInfo(coder_semaphore);
 }
 
 /*
@@ -262,28 +247,31 @@ MagickExport const CoderInfo *GetCoderInfo(const char *name,
     *p;
 
   assert(exception != (ExceptionInfo *) NULL);
-  if ((coder_list == (LinkedListInfo *) NULL) ||
+  if ((coder_list == (SplayTreeInfo *) NULL) ||
       (instantiate_coder == MagickFalse))
     if (InitializeCoderList(exception) == MagickFalse)
       return((const CoderInfo *) NULL);
-  if ((coder_list == (LinkedListInfo *) NULL) ||
-      (IsLinkedListEmpty(coder_list) == MagickTrue))
+  if ((coder_list == (SplayTreeInfo *) NULL) ||
+      (GetNumberOfNodesInSplayTree(coder_list) == 0))
     return((const CoderInfo *) NULL);
   if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
-    return((const CoderInfo *) GetElementFromLinkedList(coder_list,0));
+    {
+      ResetSplayTreeIterator(coder_list);
+      return((const CoderInfo *) GetNextValueInSplayTree(coder_list));
+    }
   /*
     Search for requested coder.
   */
   AcquireSemaphoreInfo(&coder_semaphore);
-  ResetLinkedListIterator(coder_list);
-  p=(const CoderInfo *) GetNextElementInLinkedList(coder_list);
+  ResetSplayTreeIterator(coder_list);
+  p=(const CoderInfo *) GetNextValueInSplayTree(coder_list);
   while (p != (const CoderInfo *) NULL)
   {
     if (LocaleCompare(name,p->magick) == 0)
       break;
-    p=(const CoderInfo *) GetNextElementInLinkedList(coder_list);
+    p=(const CoderInfo *) GetNextValueInSplayTree(coder_list);
   }
-  RelinquishSemaphoreInfo(&coder_semaphore);
+  RelinquishSemaphoreInfo(coder_semaphore);
   return(p);
 }
 
@@ -303,7 +291,7 @@ MagickExport const CoderInfo *GetCoderInfo(const char *name,
 %  The format of the GetCoderInfoList function is:
 %
 %      const CoderInfo **GetCoderInfoList(const char *pattern,
-%        unsigned long *number_coders)
+%        unsigned long *number_coders,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -311,6 +299,8 @@ MagickExport const CoderInfo *GetCoderInfo(const char *name,
 %
 %    o number_coders:  This integer returns the number of coders in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
+%
 %
 */
 
@@ -328,14 +318,11 @@ static int CoderInfoCompare(const void *x,const void *y)
 }
 
 MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
-  unsigned long *number_coders)
+  unsigned long *number_coders,ExceptionInfo *exception)
 {
   const CoderInfo
     **coders;
 
-  ExceptionInfo
-    exception;
-
   register const CoderInfo
     *p;
 
@@ -346,32 +333,30 @@ MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
     Allocate coder list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_coders != (unsigned long *) NULL);
   *number_coders=0;
-  GetExceptionInfo(&exception);
-  p=GetCoderInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetCoderInfo("*",exception);
   if (p == (const CoderInfo *) NULL)
     return((const CoderInfo **) NULL);
   coders=(const CoderInfo **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(coder_list)+1)*sizeof(*coders));
+    (GetNumberOfNodesInSplayTree(coder_list)+1)*sizeof(*coders));
   if (coders == (const CoderInfo **) NULL)
     return((const CoderInfo **) NULL);
   /*
     Generate coder list.
   */
   AcquireSemaphoreInfo(&coder_semaphore);
-  ResetLinkedListIterator(coder_list);
-  p=(const CoderInfo *) GetNextElementInLinkedList(coder_list);
+  ResetSplayTreeIterator(coder_list);
+  p=(const CoderInfo *) GetNextValueInSplayTree(coder_list);
   for (i=0; p != (const CoderInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
       coders[i++]=p;
-    p=(const CoderInfo *) GetNextElementInLinkedList(coder_list);
+    p=(const CoderInfo *) GetNextValueInSplayTree(coder_list);
   }
-  RelinquishSemaphoreInfo(&coder_semaphore);
+  RelinquishSemaphoreInfo(coder_semaphore);
   qsort((void *) coders,(size_t) i,sizeof(*coders),CoderInfoCompare);
   coders[i]=(CoderInfo *) NULL;
   *number_coders=(unsigned long) i;
@@ -393,7 +378,8 @@ MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
 %
 %  The format of the GetCoderList function is:
 %
-%      char **GetCoderList(const char *pattern,unsigned long *number_coders)
+%      char **GetCoderList(const char *pattern,unsigned long *number_coders,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -401,29 +387,27 @@ MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
 %
 %    o number_coders:  This integer returns the number of coders in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
 static int CoderCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+  p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
 MagickExport char **GetCoderList(const char *pattern,
-  unsigned long *number_coders)
+  unsigned long *number_coders,ExceptionInfo *exception)
 {
   char
     **coders;
 
-  ExceptionInfo
-    exception;
-
   register const CoderInfo
     *p;
 
@@ -434,32 +418,30 @@ MagickExport char **GetCoderList(const char *pattern,
     Allocate coder list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_coders != (unsigned long *) NULL);
   *number_coders=0;
-  GetExceptionInfo(&exception);
-  p=GetCoderInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetCoderInfo("*",exception);
   if (p == (const CoderInfo *) NULL)
     return((char **) NULL);
   coders=(char **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(coder_list)+1)*sizeof(*coders));
+    (GetNumberOfNodesInSplayTree(coder_list)+1)*sizeof(*coders));
   if (coders == (char **) NULL)
     return((char **) NULL);
   /*
     Generate coder list.
   */
   AcquireSemaphoreInfo(&coder_semaphore);
-  ResetLinkedListIterator(coder_list);
-  p=(const CoderInfo *) GetNextElementInLinkedList(coder_list);
+  ResetSplayTreeIterator(coder_list);
+  p=(const CoderInfo *) GetNextValueInSplayTree(coder_list);
   for (i=0; p != (const CoderInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
-      coders[i++]=AcquireString(p->name);
-    p=(const CoderInfo *) GetNextElementInLinkedList(coder_list);
+      coders[i++]=ConstantString(AcquireString(p->name));
+    p=(const CoderInfo *) GetNextValueInSplayTree(coder_list);
   }
-  RelinquishSemaphoreInfo(&coder_semaphore);
+  RelinquishSemaphoreInfo(coder_semaphore);
   qsort((void *) coders,(size_t) i,sizeof(*coders),CoderCompare);
   coders[i]=(char *) NULL;
   *number_coders=(unsigned long) i;
@@ -490,25 +472,19 @@ MagickExport char **GetCoderList(const char *pattern,
 */
 static MagickBooleanType InitializeCoderList(ExceptionInfo *exception)
 {
-  MagickBooleanType
-    active;
-
-  active=MagickFalse;
-  if ((coder_list == (LinkedListInfo *) NULL) &&
+  if ((coder_list == (SplayTreeInfo *) NULL) &&
       (instantiate_coder == MagickFalse))
     {
       AcquireSemaphoreInfo(&coder_semaphore);
-      if ((coder_list == (LinkedListInfo *) NULL) &&
+      if ((coder_list == (SplayTreeInfo *) NULL) &&
           (instantiate_coder == MagickFalse))
         {
+          (void) LoadCoderLists(MagickCoderFilename,exception);
           instantiate_coder=MagickTrue;
-          active=MagickTrue;
         }
-      RelinquishSemaphoreInfo(&coder_semaphore);
-      if (active == MagickTrue)
-        (void) LoadCoderLists(MagickCoderFilename,exception);
+      RelinquishSemaphoreInfo(coder_semaphore);
     }
-  return(coder_list != (LinkedListInfo *) NULL ? MagickTrue : MagickFalse);
+  return(coder_list != (SplayTreeInfo *) NULL ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -556,7 +532,7 @@ MagickExport MagickBooleanType ListCoderInfo(FILE *file,
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  coder_info=GetCoderInfoList("*",&number_coders);
+  coder_info=GetCoderInfoList("*",&number_coders,exception);
   if (coder_info == (const CoderInfo **) NULL)
     return(MagickFalse);
   path=(const char *) NULL;
@@ -617,6 +593,22 @@ MagickExport MagickBooleanType ListCoderInfo(FILE *file,
 %
 %
 */
+
+static void *DestroyCoderNode(void *coder_info)
+{
+  register CoderInfo
+    *p;
+
+  p=(CoderInfo *) coder_info;
+  if (p->path != (char *) NULL)
+    p->path=(char *) RelinquishMagickMemory(p->path);
+  if (p->name != (char *) NULL)
+    p->name=(char *) RelinquishMagickMemory(p->name);
+  if (p->magick != (char *) NULL)
+    p->magick=(char *) RelinquishMagickMemory(p->magick);
+  return(RelinquishMagickMemory(p));
+}
+
 static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
 {
@@ -638,10 +630,11 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
     "Loading coder file \"%s\" ...",filename);
   if (xml == (const char *) NULL)
     return(MagickFalse);
-  if (coder_list == (LinkedListInfo *) NULL)
+  if (coder_list == (SplayTreeInfo *) NULL)
     {
-      coder_list=NewLinkedList(0);
-      if (coder_list == (LinkedListInfo *) NULL)
+      coder_list=NewSplayTree(CompareSplayTreeString,RelinquishMagickMemory,
+        DestroyCoderNode);
+      if (coder_list == (SplayTreeInfo *) NULL)
         {
           ThrowFileException(exception,ResourceLimitError,
             "MemoryAllocationFailed",filename);
@@ -649,23 +642,32 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
         }
     }
   status=MagickTrue;
-  token=AcquireString((char *) xml);
+  token=AcquireString(xml);
   for (q=(char *) xml; *q != '\0'; )
   {
     /*
       Interpret XML.
     */
-    GetToken(q,&q,token);
+    GetMagickToken(q,&q,token);
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+      {
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -676,15 +678,15 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (*token != '=')
             continue;
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
               if (depth > 200)
                 (void) ThrowMagickException(exception,GetMagickModule(),
-                  ConfigureError,"IncludeElementNestedTooDeeply",token);
+                  ConfigureError,"IncludeNodeNestedTooDeeply","`%s'",token);
               else
                 {
                   char
@@ -696,7 +698,7 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
                   (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
+                  xml=FileToString(path,~0,exception);
                   if (LoadCoderList(xml,path,depth+1,exception) == MagickFalse)
                     status=MagickFalse;
                   xml=(char *) RelinquishMagickMemory(xml);
@@ -715,7 +717,7 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
           ThrowMagickFatalException(ResourceLimitFatalError,
             "MemoryAllocationFailed",filename);
         (void) ResetMagickMemory(coder_info,0,sizeof(*coder_info));
-        coder_info->path=AcquireString(filename);
+        coder_info->path=ConstantString(AcquireString(filename));
         coder_info->signature=MagickSignature;
         continue;
       }
@@ -723,17 +725,19 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
       continue;
     if (LocaleCompare(keyword,"/>") == 0)
       {
-        status=AppendElementToLinkedList(coder_list,coder_info);
+        status=AddValueToSplayTree(coder_list,
+          ConstantString(AcquireString(coder_info->magick)),coder_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
-            ResourceLimitError,"MemoryAllocationFailed",coder_info->name);
+            ResourceLimitError,"MemoryAllocationFailed","`%s'",
+            coder_info->magick);
         coder_info=(CoderInfo *) NULL;
       }
-    GetToken(q,(char **) NULL,token);
+    GetMagickToken(q,(char **) NULL,token);
     if (*token != '=')
       continue;
-    GetToken(q,&q,token);
-    GetToken(q,&q,token);
+    GetMagickToken(q,&q,token);
+    GetMagickToken(q,&q,token);
     switch (*keyword)
     {
       case 'M':
@@ -741,7 +745,7 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"magick") == 0)
           {
-            coder_info->magick=AcquireString(token);
+            coder_info->magick=ConstantString(AcquireString(token));
             break;
           }
         break;
@@ -751,7 +755,7 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"name") == 0)
           {
-            coder_info->name=AcquireString(token);
+            coder_info->name=ConstantString(AcquireString(token));
             break;
           }
         break;
@@ -772,7 +776,7 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
     }
   }
   token=(char *) RelinquishMagickMemory(token);
-  if (coder_list == (LinkedListInfo *) NULL)
+  if (coder_list == (SplayTreeInfo *) NULL)
     return(MagickFalse);
   return(status);
 }
@@ -807,6 +811,9 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
 static MagickBooleanType LoadCoderLists(const char *filename,
   ExceptionInfo *exception)
 {
+#if defined(UseEmbeddableMagick)
+  return(LoadCoderList(CoderMap,"built-in",0,exception));
+#else
   const StringInfo
     *option;
 
@@ -818,16 +825,19 @@ static MagickBooleanType LoadCoderLists(const char *filename,
 
   status=MagickFalse;
   options=GetConfigureOptions(filename,exception);
-  option=(const StringInfo *) GetNextElementInLinkedList(options);
+  option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
     status|=LoadCoderList((const char *) option->datum,option->path,0,
       exception);
-    option=(const StringInfo *) GetNextElementInLinkedList(options);
+    option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
-  DestroyConfigureOptions(options);
-  if ((coder_list == (LinkedListInfo *) NULL) || 
-      (IsLinkedListEmpty(coder_list) != MagickFalse))
+  options=DestroyConfigureOptions(options);
+  if ((coder_list == (SplayTreeInfo *) NULL) || 
+      (GetNumberOfNodesInSplayTree(coder_list) == 0))
     status|=LoadCoderList(CoderMap,"built-in",0,exception);
+  else
+    (void) SetExceptionInfo(exception,UndefinedException);
   return(status != 0 ? MagickTrue : MagickFalse);
+#endif
 }
