@@ -17,13 +17,13 @@
 %                              May  1993                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -42,9 +42,14 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/color-private.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
+#include "magick/image-private.h"
+#include "magick/list.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/resource_.h"
@@ -283,7 +288,7 @@ MagickExport void Ascii85Flush(Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(image->ascii85 != (Ascii85Info *) NULL);
   if (image->ascii85->offset > 0)
     {
@@ -292,7 +297,7 @@ MagickExport void Ascii85Flush(Image *image)
       image->ascii85->buffer[image->ascii85->offset+2]='\0';
       tuple=Ascii85Tuple(image->ascii85->buffer);
       (void) WriteBlob(image,(size_t) image->ascii85->offset+1,
-        (unsigned char *) (*tuple == 'z' ? "!!!!" : tuple));
+        (const unsigned char *) (*tuple == 'z' ? "!!!!" : tuple));
     }
   (void) WriteBlobByte(image,'~');
   (void) WriteBlobByte(image,'>');
@@ -449,7 +454,7 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   mb_hash=(HuffmanTable **) AcquireMagickMemory(HashSize*sizeof(*mb_hash));
   mw_hash=(HuffmanTable **) AcquireMagickMemory(HashSize*sizeof(*mw_hash));
   scanline=(unsigned char *) AcquireMagickMemory((size_t) image->columns);
@@ -610,9 +615,11 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
       {
-        status=MagickMonitor(LoadImageTag,y,image->rows,&image->exception);
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -731,7 +738,7 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   width=image->columns;
   if (LocaleCompare(image_info->magick,"FAX") == 0)
     width=Max(image->columns,1728);
@@ -760,7 +767,7 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
     Compress runlength encoded to 1D Huffman pixels.
   */
   polarity=(IndexPacket) (PixelIntensityToQuantum(
-    &huffman_image->colormap[0]) >= (MaxRGB/2) ? 0x00 : 0x01);
+    &huffman_image->colormap[0]) >= (QuantumRange/2) ? 0x00 : 0x01);
   if (huffman_image->colors == 2)
     polarity=(IndexPacket)
       (PixelIntensityToQuantum(&huffman_image->colormap[0]) <
@@ -836,11 +843,12 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
       OutputBit(0);
     OutputBit(1);
     q=scanline;
-    if (huffman_image->previous == (Image *) NULL)
-      if (QuantumTick(y,huffman_image->rows) != MagickFalse)
+    if (GetPreviousImageInList(huffman_image) == (Image *) NULL)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,huffman_image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,huffman_image->rows,
-            &image->exception);
+          status=image->progress_monitor(LoadImageTag,y,huffman_image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -866,7 +874,7 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
     }
   if (LocaleCompare(image_info->magick,"FAX") != 0)
     Ascii85Flush(image);
-  DestroyImage(huffman_image);
+  huffman_image=DestroyImage(huffman_image);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   return(MagickTrue);
 }
@@ -944,7 +952,7 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   huffman_image=CloneImage(image,0,0,MagickTrue,&image->exception);
@@ -957,16 +965,14 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   (void) FormatMagickString(huffman_image->filename,MaxTextExtent,"tiff:%s",
     filename);
   write_info=CloneImageInfo(image_info);
+  SetImageInfoFile(write_info,fdopen(unique_file,"wb"));
   write_info->compression=Group4Compression;
   status=WriteImage(write_info,huffman_image);
-  DestroyImageInfo(write_info);
-  DestroyImage(huffman_image);
+  write_info=DestroyImageInfo(write_info);
   if (status == MagickFalse)
     return(MagickFalse);
-  tiff=(TIFF *) NULL;
-  if (unique_file != -1)
-    tiff=TIFFFdOpen(unique_file,filename,"rb");
-  if ((unique_file == -1) || (tiff == (TIFF *) NULL))
+  tiff=TIFFOpen(filename,"rb");
+  if (tiff == (TIFF *) NULL)
     {
       (void) RelinquishUniqueFileResource(filename);
       ThrowFileException(&image->exception,FileOpenError,"UnableToOpenFile",
@@ -1006,6 +1012,7 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   }
   buffer=(unsigned char *) RelinquishMagickMemory(buffer);
   TIFFClose(tiff);
+  huffman_image=DestroyImage(huffman_image);
   (void) RelinquishUniqueFileResource(filename);
   return(MagickTrue);
 }
@@ -1016,7 +1023,7 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   ThrowBinaryException(MissingDelegateError,"TIFFLibraryIsNotAvailable",
     image->filename);
 }
@@ -1054,6 +1061,9 @@ MagickExport MagickBooleanType JPEGEncodeImage(const ImageInfo *image_info,
   Image
     *jpeg_image;
 
+  ImageInfo
+    *jpeg_info;
+
   size_t
     length;
 
@@ -1068,13 +1078,16 @@ MagickExport MagickBooleanType JPEGEncodeImage(const ImageInfo *image_info,
     ThrowWriterException(CoderError,image->exception.reason);
   DestroyBlob(jpeg_image);
   jpeg_image->blob=CloneBlobInfo((BlobInfo *) NULL);
-  (void) strcpy(jpeg_image->magick,"JPEG");
+  jpeg_info=CloneImageInfo(image_info);
+  *jpeg_info->filename='\0';
+  (void) strcpy(jpeg_info->magick,"JPEG");
   length=0;
-  blob=ImageToBlob(image_info,jpeg_image,&length,&image->exception);
+  blob=ImageToBlob(jpeg_info,jpeg_image,&length,&image->exception);
+  jpeg_info=DestroyImageInfo(jpeg_info);
   if (blob == (void *) NULL)
     ThrowWriterException(CoderError,image->exception.reason);
   (void) WriteBlob(image,length,blob);
-  DestroyImage(jpeg_image);
+  jpeg_image=DestroyImage(jpeg_image);
   blob=(unsigned char *) RelinquishMagickMemory(blob);
   return(MagickTrue);
 }
@@ -1085,12 +1098,87 @@ MagickExport MagickBooleanType JPEGEncodeImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   ThrowBinaryException(MissingDelegateError,"JPEGLibraryIsNotAvailable",
     image->filename);
 }
 #endif
 
+#if defined(HasJP2)
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   J P 2 E n c o d e I m a g e                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  JP2EncodeImage() compresses an image via JPEG-2000 compression.
+%
+%  The format of the JP2EncodeImage method is:
+%
+%      MagickBooleanType JP2EncodeImage(const ImageInfo *image_info,
+%        Image *image)
+%
+%  A description of each parameter follows:
+%
+%    o image_info: The image info..
+%
+%    o image: The image.
+%
+*/
+MagickExport MagickBooleanType JP2EncodeImage(const ImageInfo *image_info,
+  Image *image)
+{
+  Image
+    *jp2_image;
+
+  ImageInfo
+    *jp2_info;
+
+  size_t
+    length;
+
+  unsigned char
+    *blob;
+
+  /*
+    Write image as JP2 image to a temporary file.
+  */
+  jp2_image=CloneImage(image,0,0,MagickTrue,&image->exception);
+  if (jp2_image == (Image *) NULL)
+    ThrowWriterException(CoderError,image->exception.reason);
+  DestroyBlob(jp2_image);
+  jp2_image->blob=CloneBlobInfo((BlobInfo *) NULL);
+  jp2_info=CloneImageInfo(image_info);
+  *jp2_info->filename='\0';
+  (void) strcpy(jp2_info->magick,"JP2");
+  length=0;
+  blob=ImageToBlob(jp2_info,jp2_image,&length,&image->exception);
+  jp2_info=DestroyImageInfo(jp2_info);
+  if (blob == (void *) NULL)
+    ThrowWriterException(CoderError,image->exception.reason);
+  (void) WriteBlob(image,length,blob);
+  jp2_image=DestroyImage(jp2_image);
+  blob=(unsigned char *) RelinquishMagickMemory(blob);
+  return(MagickTrue);
+}
+#else
+MagickExport MagickBooleanType JP2EncodeImage(
+  const ImageInfo *magick_unused(image_info),Image *image)
+{
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  ThrowBinaryException(MissingDelegateError,"JP2LibraryIsNotAvailable",
+    image->filename);
+}
+#endif
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -1168,7 +1256,7 @@ MagickExport MagickBooleanType LZWEncodeImage(Image *image,const size_t length,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (unsigned char *) NULL);
   table=(TableType *) AcquireMagickMemory((1 << 12)*sizeof(*table));
   if (table == (TableType *) NULL)
@@ -1303,7 +1391,7 @@ MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (unsigned char *) NULL);
   packbits=(unsigned char *) AcquireMagickMemory(128);
   if (packbits == (unsigned char *) NULL)
@@ -1445,7 +1533,7 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   compressed_packets=(size_t) (1.001*length+12);
   compressed_pixels=(unsigned char *) AcquireMagickMemory(compressed_packets);
   if (compressed_pixels == (unsigned char *) NULL)
@@ -1477,13 +1565,14 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
   return((MagickBooleanType) (status == Z_OK));
 }
 #else
-MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
-  const unsigned long quality,unsigned char *pixels)
+MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,
+  const size_t magick_unused(length),const unsigned long magick_unused(quality),
+  unsigned char *magick_unused(pixels))
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   ThrowBinaryException(MissingDelegateError,"ZLIBLibraryIsNotAvailable",
     image->filename);
 }
