@@ -17,13 +17,13 @@
 %                                 July 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -43,11 +43,11 @@
 #include "magick/blob.h"
 #include "magick/client.h"
 #include "magick/configure.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/hashmap.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
-#include "magick/nt_feature.h"
 #include "magick/semaphore.h"
 #include "magick/string_.h"
 #include "magick/token.h"
@@ -56,7 +56,7 @@
 /*
   Define declarations.
 */
-#define ConfigureFilename  "configure.mgk"
+#define ConfigureFilename  "configure.xml"
 
 /*
   Static declarations.
@@ -65,6 +65,7 @@ static const char
   *ConfigureMap = (char *)
     "<?xml version=\"1.0\"?>"
     "<configuremap>"
+    "  <configure stealth=\"True\" />"
     "</configuremap>";
 
 static LinkedListInfo
@@ -123,11 +124,11 @@ MagickExport void DestroyConfigureList(void)
 {
   AcquireSemaphoreInfo(&configure_semaphore);
   if (configure_list != (LinkedListInfo *) NULL)
-    DestroyLinkedList(configure_list,DestroyConfigureElement);
+    configure_list=DestroyLinkedList(configure_list,DestroyConfigureElement);
   configure_list=(LinkedListInfo *) NULL;
   instantiate_configure=MagickFalse;
-  RelinquishSemaphoreInfo(&configure_semaphore);
-  DestroySemaphoreInfo(&configure_semaphore);
+  RelinquishSemaphoreInfo(configure_semaphore);
+  configure_semaphore=DestroySemaphoreInfo(configure_semaphore);
 }
 
 /*
@@ -146,7 +147,7 @@ MagickExport void DestroyConfigureList(void)
 %
 %  The format of the DestroyProfiles method is:
 %
-%      void DestroyConfigureOptions(Image *image)
+%      LinkedListInfo *DestroyConfigureOptions(Image *image)
 %
 %  A description of each parameter follows:
 %
@@ -157,15 +158,14 @@ MagickExport void DestroyConfigureList(void)
 
 static void *DestroyOptions(void *option)
 {
-  DestroyStringInfo((StringInfo *) option);
-  return((void *) NULL);
+  return(DestroyStringInfo((StringInfo *) option));
 }
 
-MagickExport void DestroyConfigureOptions(LinkedListInfo *options)
+MagickExport LinkedListInfo *DestroyConfigureOptions(LinkedListInfo *options)
 {
   assert(options != (LinkedListInfo *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
-  DestroyLinkedList(options,DestroyOptions);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  return(DestroyLinkedList(options,DestroyOptions));
 }
 
 /*
@@ -207,28 +207,29 @@ MagickExport const ConfigureInfo *GetConfigureInfo(const char *name,
   assert(exception != (ExceptionInfo *) NULL);
   if ((configure_list == (LinkedListInfo *) NULL) ||
       (instantiate_configure == MagickFalse))
-    (void) InitializeConfigureList(exception);
+    if (InitializeConfigureList(exception) == MagickFalse)
+      return((const ConfigureInfo *) NULL);
   if ((configure_list == (LinkedListInfo *) NULL) ||
-      (IsLinkedListEmpty(configure_list) == MagickTrue))
+      (IsLinkedListEmpty(configure_list) != MagickFalse))
     return((const ConfigureInfo *) NULL);
   if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
-    return((const ConfigureInfo *) GetElementFromLinkedList(configure_list,0));
+    return((const ConfigureInfo *) GetValueFromLinkedList(configure_list,0));
   /*
     Search for named configure.
   */
   AcquireSemaphoreInfo(&configure_semaphore);
   ResetLinkedListIterator(configure_list);
-  p=(const ConfigureInfo *) GetNextElementInLinkedList(configure_list);
+  p=(const ConfigureInfo *) GetNextValueInLinkedList(configure_list);
   while (p != (const ConfigureInfo *) NULL)
   {
     if (LocaleCompare(name,p->name) == 0)
       break;
-    p=(const ConfigureInfo *) GetNextElementInLinkedList(configure_list);
+    p=(const ConfigureInfo *) GetNextValueInLinkedList(configure_list);
   }
   if (p == (ConfigureInfo *) NULL)
     (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
-      "UnrecognizedConfigure",name);
-  RelinquishSemaphoreInfo(&configure_semaphore);
+      "NoSuchElement","`%s'",name);
+  RelinquishSemaphoreInfo(configure_semaphore);
   return(p);
 }
 
@@ -249,7 +250,7 @@ MagickExport const ConfigureInfo *GetConfigureInfo(const char *name,
 %  The format of the GetConfigureInfoList function is:
 %
 %      const ConfigureInfo **GetConfigureInfoList(const char *pattern,
-%        unsigned long *number_options)
+%        unsigned long *number_options,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -258,6 +259,7 @@ MagickExport const ConfigureInfo *GetConfigureInfo(const char *name,
 %    o number_options:  This integer returns the number of configure options in
 %    the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -283,14 +285,11 @@ static int ConfigureInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const ConfigureInfo **GetConfigureInfoList(const char *pattern,
-  unsigned long *number_options)
+  unsigned long *number_options,ExceptionInfo *exception)
 {
   const ConfigureInfo
     **options;
 
-  ExceptionInfo
-    exception;
-
   register const ConfigureInfo
     *p;
 
@@ -301,12 +300,10 @@ MagickExport const ConfigureInfo **GetConfigureInfoList(const char *pattern,
     Allocate configure list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_options != (unsigned long *) NULL);
   *number_options=0;
-  GetExceptionInfo(&exception);
-  p=GetConfigureInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetConfigureInfo("*",exception);
   if (p == (const ConfigureInfo *) NULL)
     return((const ConfigureInfo **) NULL);
   options=(const ConfigureInfo **) AcquireMagickMemory((size_t)
@@ -318,15 +315,15 @@ MagickExport const ConfigureInfo **GetConfigureInfoList(const char *pattern,
   */
   AcquireSemaphoreInfo(&configure_semaphore);
   ResetLinkedListIterator(configure_list);
-  p=(const ConfigureInfo *) GetNextElementInLinkedList(configure_list);
+  p=(const ConfigureInfo *) GetNextValueInLinkedList(configure_list);
   for (i=0; p != (const ConfigureInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
       options[i++]=p;
-    p=(const ConfigureInfo *) GetNextElementInLinkedList(configure_list);
+    p=(const ConfigureInfo *) GetNextValueInLinkedList(configure_list);
   }
-  RelinquishSemaphoreInfo(&configure_semaphore);
+  RelinquishSemaphoreInfo(configure_semaphore);
   qsort((void *) options,(size_t) i,sizeof(*options),ConfigureInfoCompare);
   options[i]=(ConfigureInfo *) NULL;
   *number_options=(unsigned long) i;
@@ -350,7 +347,7 @@ MagickExport const ConfigureInfo **GetConfigureInfoList(const char *pattern,
 %  The format of the GetConfigureList function is:
 %
 %      char **GetConfigureList(const char *pattern,
-%        unsigned long *number_options)
+%        unsigned long *number_options,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -358,6 +355,8 @@ MagickExport const ConfigureInfo **GetConfigureInfoList(const char *pattern,
 %
 %    o number_options:  This integer returns the number of options in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
+%
 %
 */
 
@@ -381,14 +380,11 @@ static int ConfigureCompare(const void *x,const void *y)
 #endif
 
 MagickExport char **GetConfigureList(const char *pattern,
-  unsigned long *number_options)
+  unsigned long *number_options,ExceptionInfo *exception)
 {
   char
     **options;
 
-  ExceptionInfo
-    exception;
-
   register const ConfigureInfo
     *p;
 
@@ -399,30 +395,29 @@ MagickExport char **GetConfigureList(const char *pattern,
     Allocate configure list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_options != (unsigned long *) NULL);
   *number_options=0;
-  GetExceptionInfo(&exception);
-  p=GetConfigureInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetConfigureInfo("*",exception);
   if (p == (const ConfigureInfo *) NULL)
     return((char **) NULL);
   AcquireSemaphoreInfo(&configure_semaphore);
-  RelinquishSemaphoreInfo(&configure_semaphore);
+  RelinquishSemaphoreInfo(configure_semaphore);
   options=(char **) AcquireMagickMemory((size_t)
     (GetNumberOfElementsInLinkedList(configure_list)+1)*sizeof(*options));
   if (options == (char **) NULL)
     return((char **) NULL);
   AcquireSemaphoreInfo(&configure_semaphore);
-  p=(const ConfigureInfo *) GetNextElementInLinkedList(configure_list);
+  ResetLinkedListIterator(configure_list);
+  p=(const ConfigureInfo *) GetNextValueInLinkedList(configure_list);
   for (i=0; p != (const ConfigureInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
-      options[i++]=AcquireString(p->name);
-    p=(const ConfigureInfo *) GetNextElementInLinkedList(configure_list);
+      options[i++]=ConstantString(AcquireString(p->name));
+    p=(const ConfigureInfo *) GetNextValueInLinkedList(configure_list);
   }
-  RelinquishSemaphoreInfo(&configure_semaphore);
+  RelinquishSemaphoreInfo(configure_semaphore);
   qsort((void *) options,(size_t) i,sizeof(*options),ConfigureCompare);
   options[i]=(char *) NULL;
   *number_options=(unsigned long) i;
@@ -473,7 +468,7 @@ MagickExport LinkedListInfo *GetConfigureOptions(const char *filename,
     *xml;
 
   assert(filename != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),filename);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   assert(exception != (ExceptionInfo *) NULL);
   (void) CopyMagickString(path,filename,MaxTextExtent);
   /*
@@ -484,18 +479,18 @@ MagickExport LinkedListInfo *GetConfigureOptions(const char *filename,
   if (paths != (LinkedListInfo *) NULL)
     {
       ResetLinkedListIterator(paths);
-      element=(const char *) GetNextElementInLinkedList(paths);
+      element=(const char *) GetNextValueInLinkedList(paths);
       while (element != (const char *) NULL)
       {
         (void) FormatMagickString(path,MaxTextExtent,"%s%s",element,filename);
         (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
           "Searching for configure file: \"%s\"",path);
-        xml=FileToStringInfo(path);
+        xml=ConfigureFileToStringInfo(path);
         if (xml != (StringInfo *) NULL)
-          (void) AppendElementToLinkedList(options,xml);
-        element=(const char *) GetNextElementInLinkedList(paths);
+          (void) AppendValueToLinkedList(options,xml);
+        element=(const char *) GetNextValueInLinkedList(paths);
       }
-      DestroyLinkedList(paths,RelinquishMagickMemory);
+      paths=DestroyLinkedList(paths,RelinquishMagickMemory);
     }
 #if defined(__WINDOWS__)
   {
@@ -506,14 +501,15 @@ MagickExport LinkedListInfo *GetConfigureOptions(const char *filename,
     if (blob != (char *) NULL)
       {
         xml=StringToStringInfo(blob);
-        (void) AppendElementToLinkedList(options,xml);
+        (void) CopyMagickString(xml->path,filename,MaxTextExtent);
+        (void) AppendValueToLinkedList(options,xml);
         blob=(char *) RelinquishMagickMemory(blob);
       }
   }
 #endif
   if (GetNumberOfElementsInLinkedList(options) == 0)
-    (void) ThrowMagickException(exception,GetMagickModule(),ConfigureError,
-      "UnableToOpenConfigureFile",filename);
+    (void) ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
+      "UnableToOpenConfigureFile","`%s'",filename);
   ResetLinkedListIterator(options);
   return(options);
 }
@@ -555,7 +551,7 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
     *paths;
 
   assert(filename != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),filename);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   assert(exception != (ExceptionInfo *) NULL);
   (void) CopyMagickString(path,filename,MaxTextExtent);
   paths=NewLinkedList(0);
@@ -584,19 +580,22 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
         if ((q >= path) && (*q != *DirectorySeparator))
           (void) ConcatenateMagickString(path,DirectorySeparator,
             MaxTextExtent);
-        (void) AppendElementToLinkedList(paths,AcquireString(path));
+        (void) AppendValueToLinkedList(paths,AcquireString(path));
         p=strchr(p+1,DirectoryListSeparator);
       }
   }
 #if defined(UseInstalledMagick)
 #if defined(MagickLibConfigPath)
-  (void) AppendElementToLinkedList(paths,AcquireString(MagickLibConfigPath));
+  (void) AppendValueToLinkedList(paths,AcquireString(MagickLibConfigPath));
 #endif
 #if defined(MagickShareConfigPath)
-  (void) AppendElementToLinkedList(paths,AcquireString(MagickShareConfigPath));
+  (void) AppendValueToLinkedList(paths,AcquireString(MagickShareConfigPath));
+#endif
+#if defined(MagickDocumentPath)
+  (void) AppendValueToLinkedList(paths,AcquireString(MagickDocumentPath));
 #endif
 #if defined(MagickSharePath)
-  (void) AppendElementToLinkedList(paths,AcquireString(MagickSharePath));
+  (void) AppendValueToLinkedList(paths,AcquireString(MagickSharePath));
 #endif
 #if defined(__WINDOWS__) && !(defined(MagickLibConfigPath) || defined(MagickShareConfigPath))
   {
@@ -609,14 +608,11 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
     */
     registry_key="ConfigurePath";
     key_value=NTRegistryKeyLookup(registry_key);
-    if (key_value == (char *) NULL)
-      ThrowMagickException(exception,GetMagickModule(),ConfigureError,
-        "UnableToGetRegistryKey",registry_key);
-    else
+    if (key_value != (char *) NULL)
       {
         (void) FormatMagickString(path,MaxTextExtent,"%s%s",key_value,
           DirectorySeparator);
-        (void) AppendElementToLinkedList(paths,AcquireString(path));
+        (void) AppendValueToLinkedList(paths,AcquireString(path));
       }
   }
 #endif
@@ -634,14 +630,14 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
 #if !defined(POSIX)
         (void) FormatMagickString(path,MaxTextExtent,"%s%s",home,
           DirectorySeparator);
-        (void) AppendElementToLinkedList(paths,AcquireString(path));
+        (void) AppendValueToLinkedList(paths,AcquireString(path));
 #else
         (void) FormatMagickString(path,MaxTextExtent,"%s/lib/%s/",home,
           MagickLibConfigSubDir);
-        (void) AppendElementToLinkedList(paths,AcquireString(path));
+        (void) AppendValueToLinkedList(paths,AcquireString(path));
         (void) FormatMagickString(path,MaxTextExtent,"%s/share/%s/",home,
           MagickShareConfigSubDir);
-        (void) AppendElementToLinkedList(paths,AcquireString(path));
+        (void) AppendValueToLinkedList(paths,AcquireString(path));
 #endif
       }
     }
@@ -650,7 +646,7 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
 #if !defined(POSIX)
       (void) FormatMagickString(path,MaxTextExtent,"%s%s",GetClientPath(),
         DirectorySeparator);
-      (void) AppendElementToLinkedList(paths,AcquireString(path));
+      (void) AppendValueToLinkedList(paths,AcquireString(path));
 #else
       char
         prefix[MaxTextExtent];
@@ -662,10 +658,10 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
       ChopPathComponents(prefix,1);
       (void) FormatMagickString(path,MaxTextExtent,"%s/lib/%s/",prefix,
         MagickLibConfigSubDir);
-      (void) AppendElementToLinkedList(paths,AcquireString(path));
+      (void) AppendValueToLinkedList(paths,AcquireString(path));
       (void) FormatMagickString(path,MaxTextExtent,"%s/share/%s/",prefix,
         MagickShareConfigSubDir);
-      (void) AppendElementToLinkedList(paths,AcquireString(path));
+      (void) AppendValueToLinkedList(paths,AcquireString(path));
 #endif
     }
 #endif
@@ -681,13 +677,47 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
         */
         (void) FormatMagickString(path,MaxTextExtent,"%s%s%s",home,
           *home == '/' ? "/.magick" : "",DirectorySeparator);
-        (void) AppendElementToLinkedList(paths,AcquireString(path));
+        (void) AppendValueToLinkedList(paths,AcquireString(path));
       }
   }
+#if defined(__WINDOWS__)
+  {
+    char
+      module_path[MaxTextExtent];
+
+    if (NTGetModulePath("CORE_RL_magick_.dll",module_path) != MagickFalse)
+      {
+        char
+          *element;
+
+        /*
+          Search module path.
+        */
+        (void) FormatMagickString(path,MaxTextExtent,"%s%s",module_path,
+          DirectorySeparator);
+        element=(char *) RemoveElementByValueFromLinkedList(paths,path);
+        if (element != (char *) NULL)
+          element=(char *) RelinquishMagickMemory(element);
+        (void) AppendValueToLinkedList(paths,AcquireString(path));
+      }
+    if (NTGetModulePath("Magick.dll",module_path) != MagickFalse)
+      {
+        /*
+          Search PerlMagick module path.
+        */
+        (void) FormatMagickString(path,MaxTextExtent,"%s%s",module_path,
+          DirectorySeparator);
+        (void) AppendValueToLinkedList(paths,AcquireString(path));
+        (void) FormatMagickString(path,MaxTextExtent,"%s%s",module_path,
+          "\\inc\\lib\\auto\\Image\\Magick\\");
+        (void) AppendValueToLinkedList(paths,AcquireString(path));
+      }
+  }
+#endif
   /*
     Search current directory.
   */
-  (void) AppendElementToLinkedList(paths,AcquireString(""));
+  (void) AppendValueToLinkedList(paths,AcquireString(""));
   return(paths);
 }
 
@@ -715,7 +745,7 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
 */
 MagickExport const char *GetConfigureValue(const ConfigureInfo *configure_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(configure_info != (ConfigureInfo *) NULL);
   assert(configure_info->signature == MagickSignature);
   return(configure_info->value);
@@ -745,10 +775,6 @@ MagickExport const char *GetConfigureValue(const ConfigureInfo *configure_info)
 */
 static MagickBooleanType InitializeConfigureList(ExceptionInfo *exception)
 {
-  MagickBooleanType
-    active;
-
-  active=MagickFalse;
   if ((configure_list == (LinkedListInfo *) NULL) &&
       (instantiate_configure == MagickFalse))
     {
@@ -756,12 +782,10 @@ static MagickBooleanType InitializeConfigureList(ExceptionInfo *exception)
       if ((configure_list == (LinkedListInfo *) NULL) &&
           (instantiate_configure == MagickFalse))
         {
+          (void) LoadConfigureLists(ConfigureFilename,exception);
           instantiate_configure=MagickTrue;
-          active=MagickTrue;
         }
-      RelinquishSemaphoreInfo(&configure_semaphore);
-      if (active == MagickTrue)
-        (void) LoadConfigureLists(ConfigureFilename,exception);
+      RelinquishSemaphoreInfo(configure_semaphore);
     }
   return(configure_list != (LinkedListInfo *) NULL ? MagickTrue : MagickFalse);
 }
@@ -795,7 +819,9 @@ MagickExport MagickBooleanType ListConfigureInfo(FILE *file,
   ExceptionInfo *exception)
 {
   const char
-    *path;
+    *name,
+    *path,
+    *value;
 
   const ConfigureInfo
     **configure_info;
@@ -811,7 +837,7 @@ MagickExport MagickBooleanType ListConfigureInfo(FILE *file,
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  configure_info=GetConfigureInfoList("*",&number_options);
+  configure_info=GetConfigureInfoList("*",&number_options,exception);
   if (configure_info == (const ConfigureInfo **) NULL)
     return(MagickFalse);
   path=(const char *) NULL;
@@ -829,12 +855,17 @@ MagickExport MagickBooleanType ListConfigureInfo(FILE *file,
           "------------------------------\n");
       }
     path=configure_info[i]->path;
-    (void) fprintf(file,"%s",configure_info[i]->name);
-    for (j=(long) strlen(configure_info[i]->name); j <= 12; j++)
+    name="unknown";
+    if (configure_info[i]->name != (char *) NULL)
+      name=configure_info[i]->name;
+    (void) fprintf(file,"%s",name);
+    for (j=(long) strlen(name); j <= 12; j++)
       (void) fprintf(file," ");
     (void) fprintf(file," ");
+    value="unknown";
     if (configure_info[i]->value != (char *) NULL)
-      (void) fprintf(file,"%s",configure_info[i]->value);
+      value=configure_info[i]->value;
+    (void) fprintf(file,"%s",value);
     (void) fprintf(file,"\n");
   }
   (void) fflush(file);
@@ -888,9 +919,6 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
   MagickStatusType
     status;
 
-  size_t
-    length;
-
   /*
     Load the configure map file.
   */
@@ -915,17 +943,26 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
     /*
       Interpret XML.
     */
-    GetToken(q,&q,token);
+    GetMagickToken(q,&q,token);
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+      {
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -936,15 +973,15 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (*token != '=')
             continue;
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
               if (depth > 200)
                 (void) ThrowMagickException(exception,GetMagickModule(),
-                  ConfigureError,"IncludeElementNestedTooDeeply",token);
+                  ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
               else
                 {
                   char
@@ -956,7 +993,7 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
                   (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=(char *) FileToBlob(path,&length,exception);
+                  xml=FileToString(path,~0,exception);
                   status|=LoadConfigureList(xml,path,depth+1,exception);
                   xml=(char *) RelinquishMagickMemory(xml);
                 }
@@ -975,7 +1012,7 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
           ThrowMagickFatalException(ResourceLimitFatalError,
             "MemoryAllocationFailed",filename);
         (void) ResetMagickMemory(configure_info,0,sizeof(*configure_info));
-        configure_info->path=AcquireString(filename);
+        configure_info->path=ConstantString(AcquireString(filename));
         configure_info->signature=MagickSignature;
         continue;
       }
@@ -983,17 +1020,18 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
       continue;
     if (LocaleCompare(keyword,"/>") == 0)
       {
-        status=AppendElementToLinkedList(configure_list,configure_info);
+        status=AppendValueToLinkedList(configure_list,configure_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
-            ResourceLimitError,"MemoryAllocationFailed",configure_info->name);
+            ResourceLimitError,"MemoryAllocationFailed","`%s'",
+            configure_info->name);
         configure_info=(ConfigureInfo *) NULL;
       }
-    GetToken(q,(char **) NULL,token);
+    GetMagickToken(q,(char **) NULL,token);
     if (*token != '=')
       continue;
-    GetToken(q,&q,token);
-    GetToken(q,&q,token);
+    GetMagickToken(q,&q,token);
+    GetMagickToken(q,&q,token);
     switch (*keyword)
     {
       case 'N':
@@ -1001,7 +1039,18 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"name") == 0)
           {
-            configure_info->name=AcquireString(token);
+            configure_info->name=ConstantString(AcquireString(token));
+            break;
+          }
+        break;
+      }
+      case 'S':
+      case 's':
+      {
+        if (LocaleCompare((char *) keyword,"stealth") == 0)
+          {
+            configure_info->stealth=(MagickBooleanType)
+              (LocaleCompare(token,"True") == 0);
             break;
           }
         break;
@@ -1011,7 +1060,7 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"value") == 0)
           {
-            configure_info->value=AcquireString(token);
+            configure_info->value=ConstantString(AcquireString(token));
             break;
           }
         break;
@@ -1056,6 +1105,9 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
 static MagickBooleanType LoadConfigureLists(const char *filename,
   ExceptionInfo *exception)
 {
+#if defined(UseEmbeddableMagick)
+  return(LoadConfigureList(ConfigureMap,"built-in",0,exception));
+#else
   const StringInfo
     *option;
 
@@ -1067,16 +1119,19 @@ static MagickBooleanType LoadConfigureLists(const char *filename,
 
   status=MagickFalse;
   options=GetConfigureOptions(filename,exception);
-  option=(const StringInfo *) GetNextElementInLinkedList(options);
+  option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
     status|=LoadConfigureList((const char *) option->datum,option->path,0,
       exception);
-    option=(const StringInfo *) GetNextElementInLinkedList(options);
+    option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
-  DestroyConfigureOptions(options);
-  if ((configure_list == (LinkedListInfo *) NULL) || 
+  options=DestroyConfigureOptions(options);
+  if ((configure_list == (LinkedListInfo *) NULL) ||
       (IsLinkedListEmpty(configure_list) != MagickFalse))
     status|=LoadConfigureList(ConfigureMap,"built-in",0,exception);
+  else
+    (void) SetExceptionInfo(exception,UndefinedException);
   return(status != 0 ? MagickTrue : MagickFalse);
+#endif
 }
