@@ -16,13 +16,13 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -51,8 +51,10 @@
 #include "magick/configure.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/hashmap.h"
+#include "magick/list.h"
 #include "magick/memory_.h"
 #include "magick/resource_.h"
 #include "magick/semaphore.h"
@@ -63,16 +65,73 @@
 /*
   Define declarations.
 */
-#define DelegateFilename  "delegates.mgk"
+#define DelegateFilename  "delegates.xml"
 
 /*
   Declare delegate map.
 */
 static const char
-  *DelegateMap = (char *)
+  *DelegateMap = (const char *)
     "<?xml version=\"1.0\"?>"
     "<delegatemap>"
-    "  <delegate stealth=\"True\" />"
+    "  <delegate decode=\"autotrace\" stealth=\"True\" command='\"autotrace\" -output-format svg -output-file \"%o\" \"%i\"' />"
+    "  <delegate decode=\"browse\" stealth=\"True\" spawn=\"True\" command='\"htmlview\" http://www.imagemagick.org/'  />"
+    "  <delegate decode=\"cgm\" command='\"ralcgm\" -d ps -oC < \"%i\" > \"%o\" 2>/dev/null' />"
+    "  <delegate decode=\"crw\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"dcr\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"mrw\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"nef\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"orf\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"raf\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"x3f\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"dvi\" command='\"dvips\" -q -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"edit\" stealth=\"True\" command='\"xterm\" -title \"Edit Image Comment\" -e vi \"%o\"' />"
+    "  <delegate decode=\"emf\" command='\"wmf2eps\" -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"eps\" encode=\"pdf\" mode=\"bi\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 -sDEVICE=\"pdfwrite\" -sOutputFile=\"%o\" -f\"%i\"' />"
+    "  <delegate decode=\"eps\" encode=\"ps\" mode=\"bi\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 -sDEVICE=\"pswrite\" -sOutputFile=\"%o\" -f\"%i\"' />"
+    "  <delegate decode=\"fig\" command='\"fig2dev\" -L ps \"%i\" \"%o\"' />"
+    "  <delegate decode=\"gplt\" command='\"echo\" \"set size 1.25,0.62"
+    "    set terminal postscript portrait color solid; set output \"%o\"; load \"%i\"\" > \"%u\";\"gnuplot\" \"%u\"' />"
+    "  <delegate decode=\"gs-color\" stealth=\"True\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 \"-sDEVICE=pnmraw\" -dTextAlphaBits=%u -dGraphicsAlphaBits=%u \"-g%s\" \"-r%s\" %s \"-sOutputFile=%s\" \"-f%s\" \"-f%s\"' />"
+    "  <delegate decode=\"gs-cmyk\" stealth=\"True\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 \"-sDEVICE=bmpsep8\" -dTextAlphaBits=%u -dGraphicsAlphaBits=%u \"-g%s\" \"-r%s\" %s \"-sOutputFile=%s\" \"-f%s\" \"-f%s\"' />"
+    "  <delegate decode=\"gs-mono\" stealth=\"True\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 \"-sDEVICE=pbmraw\" -dTextAlphaBits=%u -dGraphicsAlphaBits=%u \"-g%s\" \"-r%s\" %s \"-sOutputFile=%s\" \"-f%s\" \"-f%s\"' />"
+    "  <delegate decode=\"hpg\" command='\"hp2xx\" -q -m eps -f `basename \"%o\"` \"%i\""
+    "    mv -f `basename \"%o\"` \"%o\"' />"
+    "  <delegate decode=\"hpgl\" command='if [ -e hp2xx -o -e /usr/bin/hp2xx ]; then"
+    "    hp2xx -q -m eps -f `basename \"%o\"` \"%i\""
+    "    mv -f `basename \"%o\"` \"%o"
+    "  else"
+    "    echo \"You need to install hp2xx to use HPGL files with ImageMagick.\""
+    "    exit 1"
+    "  fi' />"
+    "  <delegate decode=\"htm\" command='\"html2ps\" -U -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"html\" command='\"html2ps\" -U -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"https\" command='\"@WWWDecodeDelegateDefault@\" -q -O \"%o\" \"https:%i\"' />"
+    "  <delegate decode=\"ilbm\" command='\"ilbmtoppm\" \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"man\" command='\"groff\" -man -Tps \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"mpeg-decode\" stealth=\"True\" command='\"mpeg2decode\" -q -b \"%i\" -f -r -o3 \"%u%%d\"' />"
+    "  <delegate encode=\"mpeg-encode\" stealth=\"True\" command='\"mpeg2encode\" \"%i\" \"%o\"' />"
+    "  <delegate decode=\"pcl-color\" stealth=\"True\" command='\"pcl6\" -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 \"-sDEVICE=ppmraw\" -dTextAlphaBits=%u -dGraphicsAlphaBits=%u \"-g%s\" \"-r%s\" %s \"-sOutputFile=%s\" \"%s\"' />"
+    "  <delegate decode=\"pcl-cmyk\" stealth=\"True\" command='\"pcl6\" -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 \"-sDEVICE=@PCLCMYKDevice@\" -dTextAlphaBits=%u -dGraphicsAlphaBits=%u \"-g%s\" \"-r%s\" %s \"-sOutputFile=%s\" \"%s\"' />"
+    "  <delegate decode=\"pcl-mono\" stealth=\"True\" command='\"pcl6\" -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 \"-sDEVICE=pbmraw\" -dTextAlphaBits=%u -dGraphicsAlphaBits=%u \"-g%s\" \"-r%s\" %s \"-sOutputFile=%s\" \"%s\"' />"
+    "  <delegate decode=\"pdf\" encode=\"eps\" mode=\"bi\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 -sDEVICE=\"epswrite\" -sOutputFile=\"%o\" -f\"%i\"' />"
+    "  <delegate decode=\"pdf\" encode=\"ps\" mode=\"bi\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 -sDEVICE=\"pswrite\" -sOutputFile=\"%o\" -f\"%i\"' />"
+    "  <delegate decode=\"pnm\" encode=\"ilbm\" mode=\"encode\" command='\"ppmtoilbm\" -24if \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"pnm\" encode=\"launch\" mode=\"encode\" command='\"gimp\" \"%i\"' />"
+    "  <delegate decode=\"miff\" encode=\"win\" mode=\"encode\" command='\"display\" -immutable \"%i\"' />"
+    "  <delegate decode=\"pov\" command='\"povray\" \"+i\"%i\"\" +o\"%o\" +fn%q +w%w +h%h +a -q9 -kfi\"%s\" -kff\"%n\""
+    "    \"convert\" -concatenate \"%o*.png\" \"%o\"' />"
+    "  <delegate decode=\"ps\" encode=\"eps\" mode=\"bi\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 -sDEVICE=\"epswrite\" -sOutputFile=\"%o\" -f\"%i\"' />"
+    "  <delegate decode=\"ps\" encode=\"pdf\" mode=\"bi\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 -sDEVICE=\"pdfwrite\" -sOutputFile=\"%o\" -f\"%i\"' />"
+    "  <delegate decode=\"ps\" encode=\"print\" mode=\"encode\" command='lpr \"%i\"' />"
+    "  <delegate decode=\"rad\" command='\"ra_ppm\" -g 1.0 \"%i\" \"%o\"' />"
+    "  <delegate decode=\"rgba\" encode=\"rle\" mode=\"encode\" command='\"modify\" -flip -size %wx%h \"rgba:%i\""
+    "    \"rawtorle\" -w %w -h %h -n 4 -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"scan\" command='\"scanimage\" -d \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"shtml\" command='\"html2ps\" -U -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"txt\" encode=\"ps\" mode=\"bi\" command='\"enscript\" -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"wmf\" command='\"wmf2eps\" -o \"%o\" \"%i\"' />"
+    "  <delegate encode=\"show\" stealth=\"True\" spawn=\"True\" command='\"display\" -immutable -delay 0 -window_group %g -title \"%l of %f\" \"tmp:%i\"' />"
     "</delegatemap>";
 
 /*
@@ -81,11 +140,11 @@ static const char
 static LinkedListInfo
   *delegate_list = (LinkedListInfo *) NULL;
 
-static MagickBooleanType
-  instantiate_delegate = MagickFalse;
-
 static SemaphoreInfo
   *delegate_semaphore = (SemaphoreInfo *) NULL;
+
+static volatile MagickBooleanType
+  instantiate_delegate = MagickFalse;
 
 /*
   Forward declaractions.
@@ -136,11 +195,10 @@ MagickExport void DestroyDelegateList(void)
 {
   AcquireSemaphoreInfo(&delegate_semaphore);
   if (delegate_list != (LinkedListInfo *) NULL)
-    DestroyLinkedList(delegate_list,DestroyDelegate);
-  delegate_list=(LinkedListInfo *) NULL;
+    delegate_list=DestroyLinkedList(delegate_list,DestroyDelegate);
   instantiate_delegate=MagickFalse;
-  RelinquishSemaphoreInfo(&delegate_semaphore);
-  DestroySemaphoreInfo(&delegate_semaphore);
+  RelinquishSemaphoreInfo(delegate_semaphore);
+  delegate_semaphore=DestroySemaphoreInfo(delegate_semaphore);
 }
 
 /*
@@ -199,25 +257,26 @@ MagickExport char *GetDelegateCommand(const ImageInfo *image_info,Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   delegate_info=GetDelegateInfo(decode,encode,exception);
   if (delegate_info == (const DelegateInfo *) NULL)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),DelegateError,
-        "NoTagFound",decode ? decode : encode);
+        "NoTagFound","`%s'",decode ? decode : encode);
       return((char *) NULL);
     }
   commands=StringToList(delegate_info->commands);
   if (commands == (char **) NULL)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed",decode ? decode : encode);
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",
+        decode ? decode : encode);
       return((char *) NULL);
     }
   command=TranslateText(image_info,image,commands[0]);
   if (command == (char *) NULL)
     (void) ThrowMagickException(exception,GetMagickModule(),ResourceLimitError,
-      "MemoryAllocationFailed",commands[0]);
+      "MemoryAllocationFailed","`%s'",commands[0]);
   /*
     Free resources.
   */
@@ -251,7 +310,7 @@ MagickExport char *GetDelegateCommand(const ImageInfo *image_info,Image *image,
 */
 MagickExport const char *GetDelegateCommands(const DelegateInfo *delegate_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(delegate_info != (DelegateInfo *) NULL);
   assert(delegate_info->signature == MagickSignature);
   return(delegate_info->commands);
@@ -293,35 +352,36 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
   register const DelegateInfo
     *p;
 
+  assert(exception != (ExceptionInfo *) NULL);
   if ((delegate_list == (LinkedListInfo *) NULL) ||
       (instantiate_delegate == MagickFalse))
     if (InitializeDelegateList(exception) == MagickFalse)
       return((const DelegateInfo *) NULL);
   if ((delegate_list == (LinkedListInfo *) NULL) ||
-      (IsLinkedListEmpty(delegate_list) == MagickTrue))
+      (IsLinkedListEmpty(delegate_list) != MagickFalse))
     return((const DelegateInfo *) NULL);
   if ((LocaleCompare(decode,"*") == 0) && (LocaleCompare(encode,"*") == 0))
-    return((const DelegateInfo *) GetElementFromLinkedList(delegate_list,0));
+    return((const DelegateInfo *) GetValueFromLinkedList(delegate_list,0));
   /*
     Search for named delegate.
   */
   AcquireSemaphoreInfo(&delegate_semaphore);
   ResetLinkedListIterator(delegate_list);
-  p=(const DelegateInfo *) GetNextElementInLinkedList(delegate_list);
+  p=(const DelegateInfo *) GetNextValueInLinkedList(delegate_list);
   while (p != (const DelegateInfo *) NULL)
   {
     if (p->mode > 0)
       {
         if (LocaleCompare(p->decode,decode) == 0)
           break;
-        p=(const DelegateInfo *) GetNextElementInLinkedList(delegate_list);
+        p=(const DelegateInfo *) GetNextValueInLinkedList(delegate_list);
         continue;
       }
     if (p->mode < 0)
       {
         if (LocaleCompare(p->encode,encode) == 0)
           break;
-        p=(const DelegateInfo *) GetNextElementInLinkedList(delegate_list);
+        p=(const DelegateInfo *) GetNextValueInLinkedList(delegate_list);
         continue;
       }
     if (LocaleCompare(decode,p->decode) == 0)
@@ -333,9 +393,9 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
     if (LocaleCompare(decode,p->decode) == 0)
       if (LocaleCompare(encode,"*") == 0)
         break;
-    p=(const DelegateInfo *) GetNextElementInLinkedList(delegate_list);
+    p=(const DelegateInfo *) GetNextValueInLinkedList(delegate_list);
   }
-  RelinquishSemaphoreInfo(&delegate_semaphore);
+  RelinquishSemaphoreInfo(delegate_semaphore);
   return( p);
 }
 
@@ -355,7 +415,7 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
 %  The delegate of the GetDelegateInfoList function is:
 %
 %      const DelegateInfo **GetDelegateInfoList(const char *pattern,
-%        unsigned long *number_delegates)
+%        unsigned long *number_delegates,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -364,6 +424,7 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
 %    o number_delegates:  This integer returns the number of delegates in the
 %      list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -382,8 +443,12 @@ static int DelegateInfoCompare(const void *x,const void *y)
   if (LocaleCompare((*p)->path,(*q)->path) == 0)
     {
       if ((*p)->decode == (char *) NULL)
-        return(strcmp((*p)->encode,(*q)->encode));
-      return(strcmp((*p)->decode,(*q)->decode));
+        if (((*p)->encode != (char *) NULL) &&
+            ((*q)->encode != (char *) NULL))
+          return(strcmp((*p)->encode,(*q)->encode));
+      if (((*p)->decode != (char *) NULL) &&
+          ((*q)->decode != (char *) NULL))
+        return(strcmp((*p)->decode,(*q)->decode));
     }
   return(LocaleCompare((*p)->path,(*q)->path));
 }
@@ -393,14 +458,11 @@ static int DelegateInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const DelegateInfo **GetDelegateInfoList(const char *pattern,
-  unsigned long *number_delegates)
+  unsigned long *number_delegates,ExceptionInfo *exception)
 {
   const DelegateInfo
     **delegates;
 
-  ExceptionInfo
-    exception;
-
   register const DelegateInfo
     *p;
 
@@ -411,16 +473,14 @@ MagickExport const DelegateInfo **GetDelegateInfoList(const char *pattern,
     Allocate delegate list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_delegates != (unsigned long *) NULL);
   *number_delegates=0;
-  GetExceptionInfo(&exception);
-  p=GetDelegateInfo("*","*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetDelegateInfo("*","*",exception);
   if (p == (const DelegateInfo *) NULL)
     return((const DelegateInfo **) NULL);
   delegates=(const DelegateInfo **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(delegate_list)+1)*sizeof(DelegateInfo *));
+    (GetNumberOfElementsInLinkedList(delegate_list)+1)*sizeof(*delegates));
   if (delegates == (const DelegateInfo **) NULL)
     return((const DelegateInfo **) NULL);
   /*
@@ -428,18 +488,17 @@ MagickExport const DelegateInfo **GetDelegateInfoList(const char *pattern,
   */
   AcquireSemaphoreInfo(&delegate_semaphore);
   ResetLinkedListIterator(delegate_list);
-  p=(const DelegateInfo *) GetNextElementInLinkedList(delegate_list);
+  p=(const DelegateInfo *) GetNextValueInLinkedList(delegate_list);
   for (i=0; p != (const DelegateInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         ((GlobExpression(p->decode,pattern) != MagickFalse) ||
          (GlobExpression(p->encode,pattern) != MagickFalse)))
       delegates[i++]=p;
-    p=(const DelegateInfo *) GetNextElementInLinkedList(delegate_list);
+    p=(const DelegateInfo *) GetNextValueInLinkedList(delegate_list);
   }
-  RelinquishSemaphoreInfo(&delegate_semaphore);
-  qsort((void *) delegates,(size_t) i,sizeof(DelegateInfo **),
-    DelegateInfoCompare);
+  RelinquishSemaphoreInfo(delegate_semaphore);
+  qsort((void *) delegates,(size_t) i,sizeof(*delegates),DelegateInfoCompare);
   delegates[i]=(DelegateInfo *) NULL;
   *number_delegates=(unsigned long) i;
   return(delegates);
@@ -462,7 +521,7 @@ MagickExport const DelegateInfo **GetDelegateInfoList(const char *pattern,
 %  The format of the GetDelegateList function is:
 %
 %      char **GetDelegateList(const char *pattern,
-%        unsigned long *number_delegates)
+%        unsigned long *number_delegates,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -471,6 +530,7 @@ MagickExport const DelegateInfo **GetDelegateInfoList(const char *pattern,
 %    o number_delegates:  This integer returns the number of delegates
 %      in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -480,12 +540,12 @@ extern "C" {
 
 static int DelegateCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+  p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
@@ -494,14 +554,11 @@ static int DelegateCompare(const void *x,const void *y)
 #endif
 
 MagickExport char **GetDelegateList(const char *pattern,
-  unsigned long *number_delegates)
+  unsigned long *number_delegates,ExceptionInfo *exception)
 {
   char
     **delegates;
 
-  ExceptionInfo
-    exception;
-
   register const DelegateInfo
     *p;
 
@@ -512,31 +569,31 @@ MagickExport char **GetDelegateList(const char *pattern,
     Allocate delegate list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_delegates != (unsigned long *) NULL);
   *number_delegates=0;
-  GetExceptionInfo(&exception);
-  p=GetDelegateInfo("*","*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetDelegateInfo("*","*",exception);
   if (p == (const DelegateInfo *) NULL)
     return((char **) NULL);
   delegates=(char **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(delegate_list)+1)*sizeof(char *));
+    (GetNumberOfElementsInLinkedList(delegate_list)+1)*sizeof(*delegates));
   if (delegates == (char **) NULL)
     return((char **) NULL);
   AcquireSemaphoreInfo(&delegate_semaphore);
   ResetLinkedListIterator(delegate_list);
-  p=(const DelegateInfo *) GetNextElementInLinkedList(delegate_list);
+  p=(const DelegateInfo *) GetNextValueInLinkedList(delegate_list);
   for (i=0; p != (const DelegateInfo *) NULL; )
   {
-    if ((p->stealth == MagickFalse) && (GlobExpression(p->decode,pattern) != MagickFalse))
-      delegates[i++]=AcquireString(p->decode);
-    if ((p->stealth == MagickFalse) && (GlobExpression(p->encode,pattern) != MagickFalse))
-      delegates[i++]=AcquireString(p->encode);
-    p=(const DelegateInfo *) GetNextElementInLinkedList(delegate_list);
+    if ((p->stealth == MagickFalse) &&
+        (GlobExpression(p->decode,pattern) != MagickFalse))
+      delegates[i++]=ConstantString(AcquireString(p->decode));
+    if ((p->stealth == MagickFalse) &&
+        (GlobExpression(p->encode,pattern) != MagickFalse))
+      delegates[i++]=ConstantString(AcquireString(p->encode));
+    p=(const DelegateInfo *) GetNextValueInLinkedList(delegate_list);
   }
-  RelinquishSemaphoreInfo(&delegate_semaphore);
-  qsort((void *) delegates,(size_t) i,sizeof(char **),DelegateCompare);
+  RelinquishSemaphoreInfo(delegate_semaphore);
+  qsort((void *) delegates,(size_t) i,sizeof(*delegates),DelegateCompare);
   delegates[i]=(char *) NULL;
   *number_delegates=(unsigned long) i;
   return(delegates);
@@ -566,7 +623,7 @@ MagickExport char **GetDelegateList(const char *pattern,
 */
 MagickExport long GetDelegateMode(const DelegateInfo *delegate_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(delegate_info != (DelegateInfo *) NULL);
   assert(delegate_info->signature == MagickSignature);
   return(delegate_info->mode);
@@ -596,10 +653,6 @@ MagickExport long GetDelegateMode(const DelegateInfo *delegate_info)
 */
 static MagickBooleanType InitializeDelegateList(ExceptionInfo *exception)
 {
-  MagickBooleanType
-    active;
-
-  active=MagickFalse;
   if ((delegate_list == (LinkedListInfo *) NULL) &&
       (instantiate_delegate == MagickFalse))
     {
@@ -607,14 +660,12 @@ static MagickBooleanType InitializeDelegateList(ExceptionInfo *exception)
       if ((delegate_list == (LinkedListInfo *) NULL) &&
           (instantiate_delegate == MagickFalse))
         {
+          (void) LoadDelegateLists(DelegateFilename,exception);
           instantiate_delegate=MagickTrue;
-          active=MagickTrue;
         }
-      RelinquishSemaphoreInfo(&delegate_semaphore);
-      if (active == MagickTrue)
-        (void) LoadDelegateLists(DelegateFilename,exception);
+      RelinquishSemaphoreInfo(delegate_semaphore);
     }
-  return(delegate_list != (LinkedListInfo *) NULL);
+  return(delegate_list != (LinkedListInfo *) NULL ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -629,8 +680,8 @@ static MagickBooleanType InitializeDelegateList(ExceptionInfo *exception)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  InvokeDelegate replaces any embedded formatting characters with the
-%  appropriate image attribute and executes the resulting command.  False is
-%  returned if the commands execute with success otherwise True.
+%  appropriate image attribute and executes the resulting command.  MagickFalse
+%  is returned if the commands execute with success otherwise MagickTrue.
 %
 %  The format of the InvokeDelegate method is:
 %
@@ -673,7 +724,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   temporary=(MagickBooleanType) (*image->filename == '\0');
   if (temporary != MagickFalse)
     if (AcquireUniqueFilename(image->filename) == MagickFalse)
@@ -689,7 +740,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
       if (temporary != MagickFalse)
         (void) RelinquishUniqueFileResource(image->filename);
       (void) ThrowMagickException(exception,GetMagickModule(),DelegateError,
-        "NoTagFound",decode ? decode : encode);
+        "NoTagFound","`%s'",decode ? decode : encode);
       return(MagickFalse);
     }
   if (*image_info->filename == '\0')
@@ -705,8 +756,10 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
       image_info->temporary=MagickTrue;
     }
   if (delegate_info->mode != 0)
-    if (((decode != MagickFalse)&& (delegate_info->encode != (char *) NULL)) ||
-        ((encode != MagickFalse) && (delegate_info->decode != (char *) NULL)))
+    if (((decode != (const char *) NULL) &&
+         (delegate_info->encode != (char *) NULL)) ||
+        ((encode != (const char *) NULL) &&
+         (delegate_info->decode != (char *) NULL)))
       {
         char
           filename[MaxTextExtent],
@@ -743,7 +796,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
             if (temporary != MagickFalse)
               (void) RelinquishUniqueFileResource(image->filename);
             (void) ThrowMagickException(exception,GetMagickModule(),
-              DelegateError,"DelegateFailed",decode ? decode : encode);
+              DelegateError,"DelegateFailed","`%s'",decode ? decode : encode);
             return(MagickFalse);
           }
         LocaleUpper(magick);
@@ -758,7 +811,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
         (void) SetImageInfo(clone_info,MagickTrue,exception);
         (void) CopyMagickString(clone_info->filename,image_info->filename,
           MaxTextExtent);
-        for (p=image; p != (Image *) NULL; p=p->next)
+        for (p=image; p != (Image *) NULL; p=GetNextImageInList(p))
         {
           (void) FormatMagickString(p->filename,MaxTextExtent,"%s:%s",
             delegate_info->decode,filename);
@@ -769,9 +822,9 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
               (void) RelinquishUniqueFileResource(image_info->zero);
               if (temporary != MagickFalse)
                 (void) RelinquishUniqueFileResource(image->filename);
-              DestroyImageInfo(clone_info);
+              clone_info=DestroyImageInfo(clone_info);
               (void) ThrowMagickException(exception,GetMagickModule(),
-                DelegateError,"DelegateFailed",decode ? decode : encode);
+                DelegateError,"DelegateFailed","`%s'",decode ? decode : encode);
               return(MagickFalse);
             }
           if (clone_info->adjoin != MagickFalse)
@@ -779,7 +832,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
         }
         (void) RelinquishUniqueFileResource(image_info->unique);
         (void) RelinquishUniqueFileResource(image_info->zero);
-        DestroyImageInfo(clone_info);
+        clone_info=DestroyImageInfo(clone_info);
       }
   /*
     Invoke delegate.
@@ -791,7 +844,8 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
       if (temporary != MagickFalse)
         (void) RelinquishUniqueFileResource(image->filename);
       (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed",decode ? decode : encode);
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",
+        decode ? decode : encode);
       return(MagickFalse);
     }
   command=(char *) NULL;
@@ -820,14 +874,15 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
     */
     if (delegate_info->spawn != MagickFalse)
       (void) ConcatenateString(&command," &");
-    status=(MagickBooleanType) SystemCommand(image_info->verbose,command);
+    status=SystemCommand(image_info->verbose,command) != 0 ?
+      MagickTrue : MagickFalse;
     command=(char *) RelinquishMagickMemory(command);
     (void) RelinquishUniqueFileResource(image_info->unique);
     (void) RelinquishUniqueFileResource(image_info->zero);
     if (status != MagickFalse)
       {
         (void) ThrowMagickException(exception,GetMagickModule(),DelegateError,
-          "DelegateFailed",commands[i]);
+          "DelegateFailed","`%s'",commands[i]);
         return(MagickFalse);
       }
     commands[i]=(char *) RelinquishMagickMemory(commands[i]);
@@ -907,12 +962,8 @@ MagickExport MagickBooleanType InvokePostscriptDelegate(
   gs_func_struct.delete_instance=gsapi_delete_instance;
 #endif
   if (gs_func == (GhostscriptVectors *) NULL)
-    {
-      if (verbose)
-        (void) fputs(command,stdout);
-      return(SystemCommand(verbose,command));
-    }
-  if (verbose)
+    return(SystemCommand(verbose,command));
+  if (verbose != MagickFalse)
     {
       (void) fputs("[ghostscript library]",stdout);
       (void) fputs(strchr(command,' '),stdout);
@@ -933,11 +984,15 @@ MagickExport MagickBooleanType InvokePostscriptDelegate(
   for (i=0; i < (long) argc; i++)
     argv[i]=(char *) RelinquishMagickMemory(argv[i]);
   argv=(char **) RelinquishMagickMemory(argv);
-  if ((status == 0) || (status == -101))
-    return(MagickFalse);
+  if ((status == 0) || (status <= -100))
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "Ghostscript returns status %d, exit code %d",status,code);
+      return(MagickFalse);
+    }
   return(MagickTrue);
 #else
-  return((MagickBooleanType) SystemCommand(verbose,command));
+  return(SystemCommand(verbose,command) != 0 ? MagickTrue : MagickFalse);
 #endif
 }
 
@@ -990,7 +1045,7 @@ MagickExport MagickBooleanType ListDelegateInfo(FILE *file,
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  delegate_info=GetDelegateInfoList("*",&number_delegates);
+  delegate_info=GetDelegateInfoList("*",&number_delegates,exception);
   if (delegate_info == (const DelegateInfo **) NULL)
     return(MagickFalse);
   path=(const char *) NULL;
@@ -1027,7 +1082,8 @@ MagickExport MagickBooleanType ListDelegateInfo(FILE *file,
       (void) fprintf(file,"                     %s\n",commands[j]);
     }
     for (j=0; commands[j] != (char *) NULL; j++)
-      commands[i]=(char *) RelinquishMagickMemory(commands[j]);
+      commands[j]=(char *) RelinquishMagickMemory(commands[j]);
+    commands=(char **) RelinquishMagickMemory(commands);
   }
   (void) fflush(file);
   delegate_info=(const DelegateInfo **)
@@ -1077,7 +1133,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
   DelegateInfo
     *delegate_info = (DelegateInfo *) NULL;
 
-  MagickBooleanType
+  MagickStatusType
     status;
 
   /*
@@ -1098,23 +1154,32 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
         }
     }
   status=MagickTrue;
-  token=AcquireString((char *) xml);
+  token=AcquireString(xml);
   for (q=(char *) xml; *q != '\0'; )
   {
     /*
       Interpret XML.
     */
-    GetToken(q,&q,token);
+    GetMagickToken(q,&q,token);
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+      {
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -1125,15 +1190,15 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (*token != '=')
             continue;
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
               if (depth > 200)
                 (void) ThrowMagickException(exception,GetMagickModule(),
-                  ConfigureError,"IncludeElementNestedTooDeeply",token);
+                  ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
               else
                 {
                   char
@@ -1145,7 +1210,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
                   (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
+                  xml=FileToString(path,~0,exception);
                   status|=LoadDelegateList(xml,path,depth+1,exception);
                   xml=(char *) RelinquishMagickMemory(xml);
                 }
@@ -1159,12 +1224,12 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
           Allocate memory for the delegate list.
         */
         delegate_info=(DelegateInfo *)
-          AcquireMagickMemory(sizeof(DelegateInfo));
+          AcquireMagickMemory(sizeof(*delegate_info));
         if (delegate_info == (DelegateInfo *) NULL)
           ThrowMagickFatalException(ResourceLimitFatalError,
             "MemoryAllocationFailed",filename);
-        (void) ResetMagickMemory(delegate_info,0,sizeof(DelegateInfo));
-        delegate_info->path=AcquireString(filename);
+        (void) ResetMagickMemory(delegate_info,0,sizeof(*delegate_info));
+        delegate_info->path=ConstantString(AcquireString(filename));
         delegate_info->signature=MagickSignature;
         continue;
       }
@@ -1172,18 +1237,18 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
       continue;
     if (LocaleCompare(keyword,"/>") == 0)
       {
-        status=AppendElementToLinkedList(delegate_list,delegate_info);
+        status=AppendValueToLinkedList(delegate_list,delegate_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
-            ResourceLimitError,"MemoryAllocationFailed",
+            ResourceLimitError,"MemoryAllocationFailed","`%s'",
             delegate_info->commands);
         delegate_info=(DelegateInfo *) NULL;
       }
-    GetToken(q,(char **) NULL,token);
+    GetMagickToken(q,(char **) NULL,token);
     if (*token != '=')
       continue;
-    GetToken(q,&q,token);
-    GetToken(q,&q,token);
+    GetMagickToken(q,&q,token);
+    GetMagickToken(q,&q,token);
     switch (*keyword)
     {
       case 'C':
@@ -1191,7 +1256,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"command") == 0)
           {
-            delegate_info->commands=AcquireString(token);
+            delegate_info->commands=ConstantString(AcquireString(token));
 #if defined(__WINDOWS__)
             if (strchr(delegate_info->commands,'@') != (char *) NULL)
               {
@@ -1212,7 +1277,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"decode") == 0)
           {
-            delegate_info->decode=AcquireString(token);
+            delegate_info->decode=ConstantString(AcquireString(token));
             delegate_info->mode=1;
             break;
           }
@@ -1223,7 +1288,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"encode") == 0)
           {
-            delegate_info->encode=AcquireString(token);
+            delegate_info->encode=ConstantString(AcquireString(token));
             delegate_info->mode=(-1);
             break;
           }
@@ -1268,7 +1333,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
   token=(char *) RelinquishMagickMemory(token);
   if (delegate_list == (LinkedListInfo *) NULL)
     return(MagickFalse);
-  return(status);
+  return(status != 0 ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -1301,26 +1366,33 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
 static MagickBooleanType LoadDelegateLists(const char *filename,
   ExceptionInfo *exception)
 {
+#if defined(UseEmbeddableMagick)
+  return(LoadDelegateList(DelegateMap,"built-in",0,exception));
+#else
   const StringInfo
     *option;
 
   LinkedListInfo
     *options;
 
-  MagickBooleanType
+  MagickStatusType
     status;
 
   status=MagickFalse;
   options=GetConfigureOptions(filename,exception);
-  option=(const StringInfo *) GetNextElementInLinkedList(options);
+  option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
     status|=LoadDelegateList((const char *) option->datum,option->path,0,
       exception);
-    option=(const StringInfo *) GetNextElementInLinkedList(options);
+    option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
-  DestroyConfigureOptions(options);
-  if (status == MagickFalse)
-    status=LoadDelegateList(DelegateMap,"built-in",0,exception);
-  return(status);
+  options=DestroyConfigureOptions(options);
+  if ((delegate_list == (LinkedListInfo *) NULL) || 
+      (IsLinkedListEmpty(delegate_list) != MagickFalse))
+    status|=LoadDelegateList(DelegateMap,"built-in",0,exception);
+  else
+    (void) SetExceptionInfo(exception,UndefinedException);
+  return(status != 0 ? MagickTrue : MagickFalse);
+#endif
 }
