@@ -17,13 +17,13 @@
 %                                 August 1996                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -41,11 +41,14 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/draw.h"
 #include "magick/gem.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
-#include "magick/random.h"
+#include "magick/pixel-private.h"
+#include "magick/random_.h"
 #include "magick/signature.h"
 
 /*
@@ -78,7 +81,6 @@ MagickExport void Contrast(const int sign,Quantum *red,Quantum *green,
   Quantum *blue)
 {
   double
-    alpha,
     brightness,
     hue,
     saturation;
@@ -93,9 +95,7 @@ MagickExport void Contrast(const int sign,Quantum *red,Quantum *green,
   saturation=0.0;
   brightness=0.0;
   TransformHSB(*red,*green,*blue,&hue,&saturation,&brightness);
-  alpha=0.5+MagickEpsilon;
-  brightness+=
-    alpha*sign*(alpha*(sin(MagickPI*(brightness-alpha))+1.0)-brightness);
+  brightness+=0.5*sign*(0.5*(sin(MagickPI*(brightness-0.5))+1.0)-brightness);
   if (brightness > 1.0)
     brightness=1.0;
   else
@@ -184,7 +184,7 @@ MagickExport unsigned long GetOptimalKernelWidth1D(const double radius,
   register long
     u;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(sigma != 0.0);
   if (radius > 0.0)
     return(Max(2*((unsigned long) radius)+1,3));
@@ -195,7 +195,7 @@ MagickExport unsigned long GetOptimalKernelWidth1D(const double radius,
       normalize+=exp(-((double) u*u)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma);
     u=width/2;
     value=exp(-((double) u*u)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma)/normalize;
-    if ((long) (MaxRGB*value) <= 0)
+    if ((long) (QuantumRange*value) <= 0)
       break;
     width+=2;
   }
@@ -218,7 +218,7 @@ MagickExport unsigned long GetOptimalKernelWidth2D(const double radius,
     v;
 
   assert(sigma != 0.0);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (radius > 0.0)
     return(Max(2*((unsigned long) radius)+1,3));
   for (width=5; ;)
@@ -234,7 +234,7 @@ MagickExport unsigned long GetOptimalKernelWidth2D(const double radius,
     }
     v=width/2;
     value=exp(-((double) v*v)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma)/normalize;
-    if ((long) (MaxRGB*value) <= 0)
+    if ((long) (QuantumRange*value) <= 0)
       break;
     width+=2;
   }
@@ -293,13 +293,13 @@ MagickExport void HSBTransform(const double hue,const double saturation,
   assert(blue != (Quantum *) NULL);
   if (saturation == 0.0)
     {
-      *red=RoundToQuantum(MaxRGB*brightness);
+      *red=RoundToQuantum(QuantumRange*brightness);
       *green=(*red);
       *blue=(*red);
       return;
     }
   h=6.0*(hue-floor(hue));
-  f=h-floor(h);
+  f=h-floor((double) h);
   p=brightness*(1.0-saturation);
   q=brightness*(1.0-saturation*f);
   t=brightness*(1.0-(saturation*(1.0-f)));
@@ -308,44 +308,44 @@ MagickExport void HSBTransform(const double hue,const double saturation,
     case 0:
     default:
     {
-      *red=RoundToQuantum(MaxRGB*brightness);
-      *green=RoundToQuantum(MaxRGB*t);
-      *blue=RoundToQuantum(MaxRGB*p);
+      *red=RoundToQuantum(QuantumRange*brightness);
+      *green=RoundToQuantum(QuantumRange*t);
+      *blue=RoundToQuantum(QuantumRange*p);
       break;
     }
     case 1:
     {
-      *red=RoundToQuantum(MaxRGB*q);
-      *green=RoundToQuantum(MaxRGB*brightness);
-      *blue=RoundToQuantum(MaxRGB*p);
+      *red=RoundToQuantum(QuantumRange*q);
+      *green=RoundToQuantum(QuantumRange*brightness);
+      *blue=RoundToQuantum(QuantumRange*p);
       break;
     }
     case 2:
     {
-      *red=RoundToQuantum(MaxRGB*p);
-      *green=RoundToQuantum(MaxRGB*brightness);
-      *blue=RoundToQuantum(MaxRGB*t);
+      *red=RoundToQuantum(QuantumRange*p);
+      *green=RoundToQuantum(QuantumRange*brightness);
+      *blue=RoundToQuantum(QuantumRange*t);
       break;
     }
     case 3:
     {
-      *red=RoundToQuantum(MaxRGB*p);
-      *green=RoundToQuantum(MaxRGB*q);
-      *blue=RoundToQuantum(MaxRGB*brightness);
+      *red=RoundToQuantum(QuantumRange*p);
+      *green=RoundToQuantum(QuantumRange*q);
+      *blue=RoundToQuantum(QuantumRange*brightness);
       break;
     }
     case 4:
     {
-      *red=RoundToQuantum(MaxRGB*t);
-      *green=RoundToQuantum(MaxRGB*p);
-      *blue=RoundToQuantum(MaxRGB*brightness);
+      *red=RoundToQuantum(QuantumRange*t);
+      *green=RoundToQuantum(QuantumRange*p);
+      *blue=RoundToQuantum(QuantumRange*brightness);
       break;
     }
     case 5:
     {
-      *red=RoundToQuantum(MaxRGB*brightness);
-      *green=RoundToQuantum(MaxRGB*p);
-      *blue=RoundToQuantum(MaxRGB*q);
+      *red=RoundToQuantum(QuantumRange*brightness);
+      *green=RoundToQuantum(QuantumRange*p);
+      *blue=RoundToQuantum(QuantumRange*q);
       break;
     }
   }
@@ -399,9 +399,9 @@ MagickExport void HSLTransform(const double hue,const double saturation,
   assert(blue != (Quantum *) NULL);
   if (saturation == 0.0)
     {
-      *red=(Quantum) (MaxRGB*luminosity+0.5);
-      *green=(Quantum) (MaxRGB*luminosity+0.5);
-      *blue=(Quantum) (MaxRGB*luminosity+0.5);
+      *red=(Quantum) (QuantumRange*luminosity+0.5);
+      *green=(Quantum) (QuantumRange*luminosity+0.5);
+      *blue=(Quantum) (QuantumRange*luminosity+0.5);
       return;
     }
   v=(luminosity <= 0.5) ? (luminosity*(1.0+saturation)) :
@@ -419,9 +419,9 @@ MagickExport void HSLTransform(const double hue,const double saturation,
     case 5: r=v; g=y; b=z; break;
     default: r=v; g=x; b=y; break;
   }
-  *red=RoundToQuantum(MaxRGB*r);
-  *green=RoundToQuantum(MaxRGB*g);
-  *blue=RoundToQuantum(MaxRGB*b);
+  *red=RoundToQuantum(QuantumRange*r);
+  *green=RoundToQuantum(QuantumRange*g);
+  *blue=RoundToQuantum(QuantumRange*b);
 }
 
 /*
@@ -475,9 +475,9 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
   v=1.0-blackness;
   if (hue == 0.0)
     {
-      *red=(Quantum) (MaxRGB*v+0.5);
-      *green=(Quantum) (MaxRGB*v+0.5);
-      *blue=(Quantum) (MaxRGB*v+0.5);
+      *red=(Quantum) (QuantumRange*v+0.5);
+      *green=(Quantum) (QuantumRange*v+0.5);
+      *blue=(Quantum) (QuantumRange*v+0.5);
       return;
     }
   i=(long) floor(hue);
@@ -496,9 +496,9 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
     case 4: r=n; g=whiteness; b=v; break;
     case 5: r=v; g=whiteness; b=n; break;
   }
-  *red=(Quantum) (MaxRGB*r+0.5);
-  *green=(Quantum) (MaxRGB*g+0.5);
-  *blue=(Quantum) (MaxRGB*b+0.5);
+  *red=(Quantum) (QuantumRange*r+0.5);
+  *green=(Quantum) (QuantumRange*g+0.5);
+  *blue=(Quantum) (QuantumRange*b+0.5);
 }
 
 /*
@@ -652,8 +652,8 @@ MagickExport void Hull(const long x_offset,const long y_offset,
 %
 %  The format of the InterpolateColor method is:
 %
-%      PixelPacket InterpolateColor(const Image *image,const double x_offset,
-%        const double y_offset,ExceptionInfo *exception)
+%      MagickPixelPacket InterpolateColor(const Image *image,
+%        const double x_offset,const double y_offset,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -664,38 +664,143 @@ MagickExport void Hull(const long x_offset,const long y_offset,
 %
 %
 */
-MagickExport PixelPacket InterpolateColor(const Image *image,
+
+MagickExport MagickPixelPacket InterpolateColor(const Image *image,
   const double x_offset,const double y_offset,ExceptionInfo *exception)
 {
-  MagickRealType
-    alpha,
-    beta;
+  MagickPixelPacket
+    pixel,
+    pixels[4];
 
-  PixelPacket
-    color;
+  MagickRealType
+    alpha[4],
+    gamma;
+
+  PointInfo
+    delta;
 
   register const PixelPacket
     *p;
 
+  register IndexPacket
+    *indexes;
+
+  register long
+    i;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  GetMagickPixelPacket(image,&pixel);
   p=AcquireImagePixels(image,(long) x_offset,(long) y_offset,2,2,exception);
   if (p == (const PixelPacket *) NULL)
-    return(AcquireOnePixel(image,(long) x_offset,(long) y_offset,exception));
-  alpha=x_offset-floor(x_offset);
-  beta=y_offset-floor(y_offset);
-  color.red=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].red+
-    alpha*p[1].red)+beta*((1.0-alpha)*p[2].red+alpha*p[3].red)+0.5);
-  color.green=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].green+
-    alpha*p[1].green)+beta*((1.0-alpha)*p[2].green+alpha*p[3].green)+0.5);
-  color.blue=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].blue+
-    alpha*p[1].blue)+beta*((1.0-alpha)*p[2].blue+alpha*p[3].blue)+0.5);
-  if (image->matte == MagickFalse)
-    color.opacity=p[0].opacity;
-  else
-    color.opacity=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].opacity+alpha*
-      p[1].opacity)+beta*((1.0-alpha)*p[2].opacity+alpha*p[3].opacity)+0.5);
-  return(color);
+    return(pixel);
+  for (i=0; i < 4L; i++)
+  {
+    indexes=GetIndexes(image);
+    alpha[i]=1.0;
+    if (image->matte != MagickFalse)
+      alpha[i]=((MagickRealType) QuantumRange-p->opacity)/QuantumRange;
+    GetMagickPixelPacket(image,pixels+i);
+    SetMagickPixelPacket(p,indexes,pixels+i);
+    pixels[i].red*=alpha[i];
+    pixels[i].green*=alpha[i];
+    pixels[i].blue*=alpha[i];
+    if (image->colorspace == CMYKColorspace)
+      pixels[i].index*=alpha[i];
+    p++;
+  }
+  delta.x=x_offset-floor(x_offset);
+  delta.y=y_offset-floor(y_offset);
+  gamma=(((1.0-delta.y)*((1.0-delta.x)*alpha[0]+delta.x*alpha[1])+
+    delta.y*((1.0-delta.x)*alpha[2]+delta.x*alpha[3])));
+  gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+  pixel.red=(gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].red+delta.x*
+    pixels[1].red)+delta.y*((1.0-delta.x)*pixels[2].red+delta.x*
+    pixels[3].red)));
+  pixel.green=(gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].green+delta.x*
+    pixels[1].green)+delta.y*((1.0-delta.x)*pixels[2].green+delta.x*
+    pixels[3].green)));
+  pixel.blue=(gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].blue+delta.x*
+    pixels[1].blue)+delta.y*((1.0-delta.x)*pixels[2].blue+delta.x*
+    pixels[3].blue)));
+  if (image->matte != MagickFalse)
+    pixel.opacity=((1.0-delta.y)*((1.0-delta.x)*pixels[0].opacity+delta.x*
+      pixels[1].opacity)+delta.y*((1.0-delta.x)*pixels[2].opacity+delta.x*
+      pixels[3].opacity));
+  if (image->colorspace == CMYKColorspace)
+    pixel.index=(gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].index+delta.x*
+      pixels[1].index)+delta.y*((1.0-delta.x)*pixels[2].index+delta.x*
+      pixels[3].index)));
+  return(pixel);
+}
+
+MagickExport MagickPixelPacket IInterpolateColor(const Image *image,
+  const double x_offset,const double y_offset,ExceptionInfo *exception)
+{
+  MagickPixelPacket
+    pixel,
+    pixels[4];
+
+  MagickRealType
+    alpha[4],
+    gamma;
+
+  PointInfo
+    delta;
+
+  register const PixelPacket
+    *p;
+
+  register IndexPacket
+    *indexes;
+
+  register long
+    i;
+
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  GetMagickPixelPacket(image,&pixel);
+  p=AcquireImagePixels(image,(long) x_offset,(long) y_offset,2,2,exception);
+  if (p == (const PixelPacket *) NULL)
+    return(pixel);
+  indexes=GetIndexes(image);
+  for (i=0; i < 4L; i++)
+  {
+    alpha[i]=1.0;
+    if (image->matte != MagickFalse)
+      alpha[i]=((MagickRealType) QuantumRange-p->opacity)/QuantumRange;
+    GetMagickPixelPacket(image,pixels+i);
+    SetMagickPixelPacket(p,indexes+i,pixels+i);
+    pixels[i].red*=alpha[i];
+    pixels[i].green*=alpha[i];
+    pixels[i].blue*=alpha[i];
+    if (image->colorspace == CMYKColorspace)
+      pixels[i].index*=alpha[i];
+    p++;
+  }
+  delta.x=x_offset-(long) x_offset;
+  delta.y=y_offset-(long) y_offset;
+  gamma=(alpha[0]+(alpha[2]-alpha[0])*delta.y+(alpha[1]-alpha[0])*delta.x+
+    (alpha[0]+alpha[3]-alpha[2]-alpha[1])*delta.x*delta.y);
+  gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+  pixel.red=gamma*(pixels[0].red+(pixels[2].red-pixels[0].red)*delta.y+
+    (pixels[1].red-pixels[0].red)*delta.x+(pixels[0].red+pixels[3].red-
+    pixels[2].red-pixels[1].red)*delta.x*delta.y);
+  pixel.green=gamma*(pixels[0].green+(pixels[2].green-pixels[0].green)*
+    delta.y+(pixels[1].green-pixels[0].green)*delta.x+(pixels[0].green+
+    pixels[3].green-pixels[2].green-pixels[1].green)*delta.x*delta.y);
+  pixel.blue=gamma*(pixels[0].blue+(pixels[2].blue-pixels[0].blue)*delta.y+
+    (pixels[1].blue-pixels[0].blue)*delta.x+(pixels[0].blue+pixels[3].blue-
+    pixels[2].blue-pixels[1].blue)*delta.x*delta.y);
+  if (image->matte != MagickFalse)
+    pixel.opacity=(pixels[0].opacity+(pixels[2].opacity-pixels[0].opacity)*
+      delta.y+(pixels[1].opacity-pixels[0].opacity)*delta.x+(pixels[0].opacity+
+      pixels[3].opacity-pixels[2].opacity-pixels[1].opacity)*delta.x*delta.y);
+  if (image->colorspace == CMYKColorspace)
+    pixel.index=gamma*(pixels[0].index+(pixels[2].index-pixels[0].index)*
+      delta.y+(pixels[1].index-pixels[0].index)*delta.x+(pixels[0].index+
+      pixels[3].index-pixels[2].index-pixels[1].index)*delta.x*delta.y);
+  return(pixel);
 }
 
 /*
@@ -703,19 +808,19 @@ MagickExport PixelPacket InterpolateColor(const Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   M o d u l a t e                                                           %
+%   M o d u l a t e H S B                                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Modulate() modulates the hue, saturation, and brightness of an image.
+%  ModulateHSB() modulates the hue, saturation, and brightness of an image.
 %
-%  The format of the Modulate method is:
+%  The format of the ModulateHSB method is:
 %
-%      void Modulate(const double percent_hue,const double percent_saturation,
-%        const double percent_brightness,Quantum *red,Quantum *green,
-%        Quantum *blue)
+%      void ModulateHSB(const double percent_hue,
+%        const double percent_saturation,const double percent_brightness,
+%        Quantum *red,Quantum *green,Quantum *blue)
 %
 %  A description of each parameter follows:
 %
@@ -726,7 +831,7 @@ MagickExport PixelPacket InterpolateColor(const Image *image,
 %
 %
 */
-MagickExport void Modulate(const double percent_hue,
+MagickExport void ModulateHSB(const double percent_hue,
   const double percent_saturation,const double percent_brightness,
   Quantum *red,Quantum *green,Quantum *blue)
 {
@@ -742,9 +847,13 @@ MagickExport void Modulate(const double percent_hue,
   assert(green != (Quantum *) NULL);
   assert(blue != (Quantum *) NULL);
   TransformHSB(*red,*green,*blue,&hue,&saturation,&brightness);
-  brightness*=0.01*percent_brightness;
+  hue+=0.5*(0.01*percent_hue-1.0);
+  while (hue < 0.0)
+    hue+=1.0;
+  while (hue > 1.0)
+    hue-=1.0;
   saturation*=0.01*percent_saturation;
-  hue*=0.01*percent_hue;
+  brightness*=0.01*percent_brightness;
   HSBTransform(hue,saturation,brightness,red,green,blue);
 }
 
@@ -753,6 +862,114 @@ MagickExport void Modulate(const double percent_hue,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M o d u l a t e H S L                                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ModulateHSL() modulates the hue, saturation, and brightness of an image.
+%
+%  The format of the ModulateHSL method is:
+%
+%      void ModulateHSL(const double percent_hue,
+%        const double percent_saturation,const double percent_luminosity,
+%        Quantum *red,Quantum *green,Quantum *blue)
+%
+%  A description of each parameter follows:
+%
+%    o percent_hue, percent_saturation, percent_luminosity: A double value
+%      representing the percent change in a component of the HSL color space.
+%
+%    o red, green, blue: A pointer to a pixel component of type Quantum.
+%
+%
+*/
+MagickExport void ModulateHSL(const double percent_hue,
+  const double percent_saturation,const double percent_luminosity,
+  Quantum *red,Quantum *green,Quantum *blue)
+{
+  double
+    hue,
+    luminosity,
+    saturation;
+
+  /*
+    Increase or decrease color luminosity, saturation, or hue.
+  */
+  assert(red != (Quantum *) NULL);
+  assert(green != (Quantum *) NULL);
+  assert(blue != (Quantum *) NULL);
+  TransformHSL(*red,*green,*blue,&hue,&saturation,&luminosity);
+  hue+=0.5*(0.01*percent_hue-1.0);
+  while (hue < 0.0)
+    hue+=1.0;
+  while (hue > 1.0)
+    hue-=1.0;
+  saturation*=0.01*percent_saturation;
+  luminosity*=0.01*percent_luminosity;
+  HSLTransform(hue,saturation,luminosity,red,green,blue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M o d u l a t e H W B                                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ModulateHWB() modulates the hue, whiteness, and blackness of an image.
+%
+%  The format of the ModulateHWB method is:
+%
+%      void ModulateHWB(const double percent_hue,
+%        const double percent_whiteness,const double percent_blackness,
+%        Quantum *red,Quantum *green,Quantum *blue)
+%
+%  A description of each parameter follows:
+%
+%    o percent_hue, percent_whiteness, percent_blackness: A double value
+%      representing the percent change in a component of the HWB color space.
+%
+%    o red, green, blue: A pointer to a pixel component of type Quantum.
+%
+%
+*/
+MagickExport void ModulateHWB(const double percent_hue,
+  const double percent_whiteness,const double percent_blackness,
+  Quantum *red,Quantum *green,Quantum *blue)
+{
+  double
+    blackness,
+    hue,
+    whiteness;
+
+  /*
+    Increase or decrease color blackness, whiteness, or hue.
+  */
+  assert(red != (Quantum *) NULL);
+  assert(green != (Quantum *) NULL);
+  assert(blue != (Quantum *) NULL);
+  TransformHWB(*red,*green,*blue,&hue,&whiteness,&blackness);
+  hue+=0.5*(0.01*percent_hue-1.0);
+  while (hue < 0.0)
+    hue+=1.0;
+  while (hue > 1.0)
+    hue-=1.0;
+  blackness*=0.01*percent_blackness;
+  whiteness*=0.01*percent_whiteness;
+  HWBTransform(hue,whiteness,blackness,red,green,blue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   T r a n s f o r m H S B                                                   %
 %                                                                             %
 %                                                                             %
@@ -797,22 +1014,22 @@ MagickExport void TransformHSB(const Quantum red,const Quantum green,
   min=(MagickRealType) (red < green ? red : green);
   if ((MagickRealType) blue < min)
     min=(MagickRealType) blue;
-  *brightness=QuantumScale*max;
+  *brightness=(double) (QuantumScale*max);
   if (max == 0.0)
     {
       *saturation=0.0;
       *hue=0.0;
       return;
     }
-  *saturation=1.0-min/max;
+  *saturation=(double) (1.0-min/max);
   delta=max-min;
   if ((MagickRealType) red == max)
-    *hue=(green-(MagickRealType) blue)/delta;
+    *hue=(double) ((green-(MagickRealType) blue)/delta);
   else
     if ((MagickRealType) green == max)
-      *hue=2.0+(blue-(MagickRealType) red)/delta;
+      *hue=(double) (2.0+(blue-(MagickRealType) red)/delta);
     else
-      *hue=4.0+(red-(MagickRealType) green)/delta;
+      *hue=(double) (4.0+(red-(MagickRealType) green)/delta);
   *hue/=6.0;
   if (*hue < 0.0)
     *hue+=1.0;
@@ -864,25 +1081,26 @@ MagickExport void TransformHSL(const Quantum red,const Quantum green,
   assert(hue != (double *) NULL);
   assert(saturation != (double *) NULL);
   assert(luminosity != (double *) NULL);
-  r=(MagickRealType) red/MaxRGB;
-  g=(MagickRealType) green/MaxRGB;
-  b=(MagickRealType) blue/MaxRGB;
+  r=(MagickRealType) red/QuantumRange;
+  g=(MagickRealType) green/QuantumRange;
+  b=(MagickRealType) blue/QuantumRange;
   max=Max(r,Max(g,b));
   min=Min(r,Min(g,b));
   *hue=0.0;
   *saturation=0.0;
-  *luminosity=(min+max)/2.0;
+  *luminosity=(double) ((min+max)/2.0);
   delta=max-min;
   if (delta == 0.0)
     return;
-  *saturation=delta/((*luminosity < 0.5) ? (min+max) : (2.0-max-min));
+  *saturation=(double) (delta/((*luminosity < 0.5) ? (min+max) :
+    (2.0-max-min)));
   if (r == max)
-    *hue=(g == min ? 5.0+(max-b)/delta : 1.0-(max-g)/delta);
+    *hue=(double) (g == min ? 5.0+(max-b)/delta : 1.0-(max-g)/delta);
   else
     if (g == max)
-      *hue=(b == min ? 1.0+(max-r)/delta : 3.0-(max-b)/delta);
+      *hue=(double) (b == min ? 1.0+(max-r)/delta : 3.0-(max-b)/delta);
     else
-      *hue=(r == min ? 3.0+(max-g)/delta : 5.0-(max-r)/delta);
+      *hue=(double) (r == min ? 3.0+(max-g)/delta : 5.0-(max-r)/delta);
   *hue/=6.0;
 }
 
@@ -936,7 +1154,7 @@ MagickExport void TransformHWB(const Quantum red,const Quantum green,
   assert(blackness != (double *) NULL);
   w=Min(red,Min(green,blue));
   v=Max(red,Max(green,blue));
-  *blackness=(double) (MaxRGB-v)/MaxRGB;
+  *blackness=(double) (QuantumRange-v)/QuantumRange;
   if (v == w)
     {
       *hue=0.0;
@@ -946,8 +1164,8 @@ MagickExport void TransformHWB(const Quantum red,const Quantum green,
   f=(red == w) ? (MagickRealType) green-blue : ((green == w) ?
     (MagickRealType) blue-red : (MagickRealType) red-green);
   i=(red == w) ? 3 : ((green == w) ? 5 : 1);
-  *hue=(MagickRealType) (i-f/(v-w));
-  *whiteness=(MagickRealType) w/MaxRGB;
+  *hue=(double) (i-f/(v-w));
+  *whiteness=(double) w/QuantumRange;
 }
 
 /*
