@@ -17,13 +17,13 @@
 %                              January 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -41,7 +41,9 @@
 */
 #include "magick/studio.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/draw.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/geometry.h"
 #include "magick/memory_.h"
 #include "magick/string_.h"
@@ -186,7 +188,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
       if (LocaleNCompare(p,"0x",2) == 0)
         *width=(unsigned long) strtol(p,&p,10);
       else
-        *width=(unsigned long) (strtod(p,&p)+0.5);
+        *width=(unsigned long) floor(strtod(p,&p)+0.5);
       if (p != q)
         flags|=WidthValue;
     }
@@ -199,7 +201,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
             Parse height.
           */
           q=p;
-          *height=(unsigned long) (strtod(p,&p)+0.5);
+          *height=(unsigned long) floor(strtod(p,&p)+0.5);
           if (p != q)
             flags|=HeightValue;
         }
@@ -212,7 +214,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
       if (*p == '-')
         flags|=XNegative;
       q=p;
-      *x=(long) (strtod(p,&p)+0.5);
+      *x=(long) ceil(strtod(p,&p)-0.5);
       if (p != q)
         flags|=XValue;
       if ((*p == '+') || (*p == '-'))
@@ -223,7 +225,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
           if (*p == '-')
             flags|=YNegative;
           q=p;
-          *y=(long) (strtod(p,&p)+0.5);
+          *y=(long) ceil(strtod(p,&p)-0.5);
           if (p != q)
             flags|=YValue;
         }
@@ -340,7 +342,7 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
     i;
 
   assert(page_geometry != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),page_geometry);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",page_geometry);
   page=AcquireString(page_geometry);
   for (i=0; *PageSizes[i] != (char *) NULL; i++)
     if (LocaleNCompare(PageSizes[i][0],page,strlen(PageSizes[i][0])) == 0)
@@ -525,6 +527,9 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
     pedantic_geometry[MaxTextExtent],
     *q;
 
+  double
+    value;
+
   MagickStatusType
     flags;
 
@@ -609,7 +614,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
     }
   }
   /*
-    Parse rho, sigma, xi, and psi.
+    Parse rho, sigma, xi, psi, and optionally chi.
   */
   p=pedantic_geometry;
   if (*p == '\0')
@@ -625,23 +630,37 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       */
       q=p;
       if (LocaleNCompare(p,"0x",2) == 0)
-        geometry_info->rho=(double) strtol(p,&p,10);
+        value=(double) strtol(p,&p,10);
       else
-        geometry_info->rho=strtod(p,&p);
+        value=strtod(p,&p);
       if (p != q)
-        flags|=RhoValue;
+        {
+          flags|=RhoValue;
+          geometry_info->rho=value;
+        }
     }
-  if ((*p == 'x') || (*p == 'X') || (*p == '/') || (*p == ','))
+  q=p;
+  if ((*p == 'x') || (*p == 'X') || (*p == ',') || (*p == '/'))
     {
       /*
         Parse sigma.
       */
       p++;
-      q=p;
-      geometry_info->sigma=strtod(p,&p);
-      if (p != q)
-        flags|=SigmaValue;
+      while (isspace((int) ((unsigned char) *p)) != 0)
+        p++;
+      if (((*q != 'x') && (*q != 'X')) || ((*p != '+') && (*p != '-')))
+        {
+          q=p;
+          value=strtod(p,&p);
+          if (p != q)
+            {
+              flags|=SigmaValue;
+              geometry_info->sigma=value;
+            }
+        }
     }
+  while (isspace((int) ((unsigned char) *p)) != 0)
+    p++;
   if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/'))
     {
       /*
@@ -650,13 +669,16 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       if ((*p == ',') || (*p == '/'))
         p++;
       q=p;
-      geometry_info->xi=strtod(p,&p);
+      value=strtod(p,&p);
       if (p != q)
         {
           flags|=XiValue;
           if (*q == '-')
             flags|=XiNegative;
+          geometry_info->xi=value;
         }
+      while (isspace((int) ((unsigned char) *p)) != 0)
+        p++;
       if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/'))
         {
           /*
@@ -665,14 +687,17 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
           if ((*p == ',') || (*p == '/'))
             p++;
           q=p;
-          geometry_info->psi=strtod(p,&p);
+          value=strtod(p,&p);
           if (p != q)
             {
               flags|=PsiValue;
               if (*q == '-')
                 flags|=PsiNegative;
+              geometry_info->psi=value;
             }
         }
+      while (isspace((int) ((unsigned char) *p)) != 0)
+        p++;
       if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/'))
         {
           /*
@@ -681,12 +706,13 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
           if ((*p == ',') || (*p == '/'))
             p++;
           q=p;
-          geometry_info->chi=strtod(p,&p);
+          value=strtod(p,&p);
           if (p != q)
             {
               flags|=ChiValue;
               if (*q == '-')
                 flags|=ChiNegative;
+              geometry_info->chi=value;
             }
         }
     }
@@ -719,24 +745,23 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
 %    o region_info: The region as defined by the geometry string with
 %      respect to the image dimensions and its gravity.
 %
-%
 */
 MagickExport MagickStatusType ParseGravityGeometry(Image *image,
   const char *geometry,RectangleInfo *region_info)
 {
-  char
-    region_geometry[MaxTextExtent];
-
   MagickStatusType
     flags;
 
+  unsigned long
+    height,
+    width;
+
   SetGeometry(image,region_info);
-  (void) FormatMagickString(region_geometry,MaxTextExtent,"%s!",geometry);
-  flags=ParseAbsoluteGeometry(region_geometry,region_info);
+  flags=ParseAbsoluteGeometry(geometry,region_info);
   if (flags == NoValue)
     {
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        OptionError,"InvalidGeometry",geometry);
+        OptionError,"InvalidGeometry","`%s'",geometry);
       return(flags);
     }
   if ((flags & PercentValue) != 0)
@@ -744,25 +769,40 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
       GeometryInfo
         geometry_info;
 
-      MagickRealType
-        x_scale,
-        y_scale;
-
       MagickStatusType
         flags;
 
+      PointInfo
+        scale;
+
       /*
         Geometry is a percentage of the image size.
       */
       flags=ParseGeometry(geometry,&geometry_info);
-      x_scale=geometry_info.rho;
+      scale.x=geometry_info.rho;
       if ((flags & RhoValue) == 0)
-        x_scale=100.0;
-      y_scale=geometry_info.sigma;
+        scale.x=100.0;
+      scale.y=geometry_info.sigma;
       if ((flags & SigmaValue) == 0)
-        y_scale=x_scale;
-      region_info->width=(unsigned long) ((x_scale*image->columns/100.0)+0.5);
-      region_info->height=(unsigned long) ((y_scale*image->rows/100.0)+0.5);
+        scale.y=scale.x;
+      region_info->width=(unsigned long) ((scale.x*image->columns/100.0)+0.5);
+      region_info->height=(unsigned long) ((scale.y*image->rows/100.0)+0.5);
+    }
+  width=region_info->width;
+  height=region_info->height;
+  if (width == 0)
+    {
+      if (image->page.width != 0)
+        width=image->page.width;
+      else
+        width=image->columns;
+    }
+  if (height == 0)
+    {
+      if (image->page.height != 0)
+        height=image->page.height;
+      else
+        height=image->rows;
     }
   switch (image->gravity)
   {
@@ -771,48 +811,48 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
       break;
     case NorthGravity:
     {
-      region_info->x+=(long) (image->columns/2-region_info->width/2);
+      region_info->x+=(long) (image->columns/2-width/2);
       break;
     }
     case NorthEastGravity:
     {
-      region_info->x=(long) (image->columns-region_info->width-region_info->x);
+      region_info->x=(long) (image->columns-width-region_info->x);
       break;
     }
     case WestGravity:
     {
-      region_info->y+=(long) (image->rows/2-region_info->height/2);
+      region_info->y+=(long) (image->rows/2-height/2);
       break;
     }
     case StaticGravity:
     case CenterGravity:
     default:
     {
-      region_info->x+=(long) (image->columns/2-region_info->width/2);
-      region_info->y+=(long) (image->rows/2-region_info->height/2);
+      region_info->x+=(long) (image->columns/2-width/2);
+      region_info->y+=(long) (image->rows/2-height/2);
       break;
     }
     case EastGravity:
     {
-      region_info->x=(long) (image->columns-region_info->width-region_info->x);
-      region_info->y+=(long) (image->rows/2-region_info->height/2);
+      region_info->x=(long) (image->columns-width-region_info->x);
+      region_info->y+=(long) (image->rows/2-height/2);
       break;
     }
     case SouthWestGravity:
     {
-      region_info->y=(long) (image->rows-region_info->height-region_info->y);
+      region_info->y=(long) (image->rows-height-region_info->y);
       break;
     }
     case SouthGravity:
     {
-      region_info->x+=(long) (image->columns/2-region_info->width/2);
-      region_info->y=(long) (image->rows-region_info->height-region_info->y);
+      region_info->x+=(long) (image->columns/2-width/2);
+      region_info->y=(long) (image->rows-height-region_info->y);
       break;
     }
     case SouthEastGravity:
     {
-      region_info->x=(long) (image->columns-region_info->width-region_info->x);
-      region_info->y=(long) (image->rows-region_info->height-region_info->y);
+      region_info->x=(long) (image->columns-width-region_info->x);
+      region_info->y=(long) (image->rows-height-region_info->y);
       break;
     }
   }
@@ -848,7 +888,6 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
 %    o width,height:  The width and height as determined by the geometry
 %      specification.
 %
-%
 */
 MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
   long *y,unsigned long *width,unsigned long *height)
@@ -872,7 +911,7 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
   assert(height != (unsigned long *) NULL);
   if ((geometry == (char *) NULL) || (*geometry == '\0'))
     return(NoValue);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),geometry);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",geometry);
   /*
     Parse geometry using GetGeometry.
   */
@@ -882,25 +921,24 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
   flags=GetGeometry(geometry,x,y,width,height);
   if ((flags & PercentValue) != 0)
     {
-      MagickRealType
-        x_scale,
-        y_scale;
-
       MagickStatusType
         flags;
 
+      PointInfo
+        scale;
+
       /*
         Geometry is a percentage of the image size.
       */
       flags=ParseGeometry(geometry,&geometry_info);
-      x_scale=geometry_info.rho;
+      scale.x=geometry_info.rho;
       if ((flags & RhoValue) == 0)
-        x_scale=100.0;
-      y_scale=geometry_info.sigma;
+        scale.x=100.0;
+      scale.y=geometry_info.sigma;
       if ((flags & SigmaValue) == 0)
-        y_scale=x_scale;
-      *width=(unsigned long) ((x_scale*former_width/100.0)+0.5);
-      *height=(unsigned long) ((y_scale*former_height/100.0)+0.5);
+        scale.y=scale.x;
+      *width=(unsigned long) ((scale.x*former_width/100.0)+0.5);
+      *height=(unsigned long) ((scale.y*former_height/100.0)+0.5);
       former_width=(*width);
       former_height=(*height);
     }
@@ -953,17 +991,23 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
         area,
         distance;
 
+      PointInfo
+        scale;
+
       /*
         Geometry is a maximum area in pixels.
       */
       (void) ParseGeometry(geometry,&geometry_info);
       area=geometry_info.rho;
       distance=sqrt((double) former_width*former_height);
-      if (((former_width/(distance/sqrt(area))) < *width) ||
-          ((former_height/(distance/sqrt(area))) < *height))
+      scale.x=(MagickRealType) former_width/(distance/sqrt((double) area));
+      scale.y=(MagickRealType) former_height/(distance/sqrt((double) area));
+      if ((scale.x < (MagickRealType) *width) ||
+          (scale.y < (MagickRealType) *height))
         {
-          *width=(unsigned long) (former_width/(distance/sqrt(area)));
-          *height=(unsigned long) (former_height/(distance/sqrt(area)));
+          *width=(unsigned long) (former_width/(distance/sqrt((double) area)));
+          *height=(unsigned long) (former_height/(distance/
+            sqrt((double) area)));
         }
       former_width=(*width);
       former_height=(*height);
@@ -997,24 +1041,23 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
 %    o region_info: The region as defined by the geometry string with
 %      respect to the image and its gravity.
 %
-%
 */
-MagickExport MagickStatusType ParsePageGeometry(Image *image,const char *geometry,
-  RectangleInfo *region_info)
+MagickExport MagickStatusType ParsePageGeometry(Image *image,
+  const char *geometry,RectangleInfo *region_info)
 {
-  char
-    region_geometry[MaxTextExtent];
-
   MagickStatusType
     flags;
 
   SetGeometry(image,region_info);
-  (void) FormatMagickString(region_geometry,MaxTextExtent,"%s!",geometry);
-  flags=ParseMetaGeometry(region_geometry,&region_info->x,&region_info->y,
-    &region_info->width,&region_info->height);
+  flags=ParseAbsoluteGeometry(geometry,region_info);
   if (flags == NoValue)
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),OptionError,
-      "InvalidGeometry",geometry);
+    {
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        OptionError,"InvalidGeometry","`%s'",geometry);
+      return(flags);
+    }
+  flags=ParseMetaGeometry(geometry,&region_info->x,&region_info->y,
+    &region_info->width,&region_info->height);
   return(flags);
 }
 
@@ -1056,7 +1099,7 @@ MagickExport MagickStatusType ParseSizeGeometry(Image *image,
     &region_info->width,&region_info->height);
   if (flags == NoValue)
     (void) ThrowMagickException(&image->exception,GetMagickModule(),OptionError,
-      "InvalidGeometry",geometry);
+      "InvalidGeometry","`%s'",geometry);
   return(flags);
 }
 
@@ -1090,11 +1133,15 @@ MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(geometry != (RectangleInfo *) NULL);
   (void) ResetMagickMemory(geometry,0,sizeof(*geometry));
   geometry->width=image->columns;
+  if (image->page.width != 0)
+    geometry->width=image->page.width;
   geometry->height=image->rows;
+  if (image->page.height != 0)
+    geometry->height=image->page.height;
 }
 
 /*
@@ -1123,6 +1170,6 @@ MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
 MagickExport void SetGeometryInfo(GeometryInfo *geometry_info)
 {
   assert(geometry_info != (GeometryInfo *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   (void) ResetMagickMemory(geometry_info,0,sizeof(*geometry_info));
 }
