@@ -17,13 +17,13 @@
 %                                 July 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -43,13 +43,14 @@
 #include "magick/blob.h"
 #include "magick/client.h"
 #include "magick/configure.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/hashmap.h"
 #include "magick/locale_.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
-#include "magick/nt_feature.h"
 #include "magick/semaphore.h"
+#include "magick/splay-tree.h"
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
@@ -57,7 +58,7 @@
 /*
   Define declarations.
 */
-#define LocaleFilename  "locale.mgk"
+#define LocaleFilename  "locale.xml"
 
 /*
   Static declarations.
@@ -66,16 +67,20 @@ static const char
   *LocaleMap = (char *)
     "<?xml version=\"1.0\"?>"
     "<localemap>"
-    "  <locale name=\"english\">"
+    "  <locale name=\"C\">"
+    "    <Exception>"
+    "     <Message name=\"\">"
+    "     </Message>"
+    "    </Exception>"
     "  </locale>"
     "</localemap>";
 
-static LinkedListInfo
-  *locale_list = (LinkedListInfo *) NULL;
-
 static SemaphoreInfo
   *locale_semaphore = (SemaphoreInfo *) NULL;
 
+static SplayTreeInfo
+  *locale_list = (SplayTreeInfo *) NULL;
+
 static volatile MagickBooleanType
   instantiate_locale = MagickFalse;
 
@@ -105,32 +110,14 @@ static MagickBooleanType
 %
 %
 */
-
-static void *DestroyLocaleElement(void *locale_info)
-{
-  register LocaleInfo
-    *p;
-
-  p=(LocaleInfo *) locale_info;
-  if (p->path != (char *) NULL)
-    p->path=(char *) RelinquishMagickMemory(p->path);
-  if (p->tag != (char *) NULL)
-    p->tag=(char *) RelinquishMagickMemory(p->tag);
-  if (p->message != (char *) NULL)
-    p->message=(char *) RelinquishMagickMemory(p->message);
-  p=(LocaleInfo *) RelinquishMagickMemory(p);
-  return((void *) NULL);
-}
-
 MagickExport void DestroyLocaleList(void)
 {
   AcquireSemaphoreInfo(&locale_semaphore);
-  if (locale_list != (LinkedListInfo *) NULL)
-    DestroyLinkedList(locale_list,DestroyLocaleElement);
-  locale_list=(LinkedListInfo *) NULL;
+  if (locale_list != (SplayTreeInfo *) NULL)
+    locale_list=DestroySplayTree(locale_list);
   instantiate_locale=MagickFalse;
-  RelinquishSemaphoreInfo(&locale_semaphore);
-  DestroySemaphoreInfo(&locale_semaphore);
+  RelinquishSemaphoreInfo(locale_semaphore);
+  locale_semaphore=DestroySemaphoreInfo(locale_semaphore);
 }
 
 /*
@@ -149,7 +136,7 @@ MagickExport void DestroyLocaleList(void)
 %
 %  The format of the DestroyProfiles method is:
 %
-%      void DestroyLocaleOptions(Image *image)
+%      LinkedListInfo *DestroyLocaleOptions(Image *image)
 %
 %  A description of each parameter follows:
 %
@@ -160,15 +147,14 @@ MagickExport void DestroyLocaleList(void)
 
 static void *DestroyOptions(void *message)
 {
-  DestroyStringInfo((StringInfo *) message);
-  return((void *) NULL);
+  return(DestroyStringInfo((StringInfo *) message));
 }
 
-MagickExport void DestroyLocaleOptions(LinkedListInfo *messages)
+MagickExport LinkedListInfo *DestroyLocaleOptions(LinkedListInfo *messages)
 {
   assert(messages != (LinkedListInfo *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
-  DestroyLinkedList(messages,DestroyOptions);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  return(DestroyLinkedList(messages,DestroyOptions));
 }
 
 /*
@@ -205,27 +191,31 @@ MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
     *p;
 
   assert(exception != (ExceptionInfo *) NULL);
-  if ((locale_list == (LinkedListInfo *) NULL) ||
+  if ((locale_list == (SplayTreeInfo *) NULL) ||
       (instantiate_locale == MagickFalse))
-    (void) InitializeLocaleList(exception);
-  if ((locale_list == (LinkedListInfo *) NULL) ||
-      (IsLinkedListEmpty(locale_list) == MagickTrue))
+    if (InitializeLocaleList(exception) == MagickFalse)
+      return((const LocaleInfo *) NULL);
+  if ((locale_list == (SplayTreeInfo *) NULL) ||
+      (GetNumberOfNodesInSplayTree(locale_list) == 0))
     return((const LocaleInfo *) NULL);
   if ((tag == (const char *) NULL) || (LocaleCompare(tag,"*") == 0))
-    return((const LocaleInfo *) GetElementFromLinkedList(locale_list,0));
+    {
+      ResetSplayTreeIterator(locale_list);
+      return((const LocaleInfo *) GetNextValueInSplayTree(locale_list));
+    }
   /*
     Search for named tag.
   */
   AcquireSemaphoreInfo(&locale_semaphore);
-  ResetLinkedListIterator(locale_list);
-  p=(const LocaleInfo *) GetNextElementInLinkedList(locale_list);
+  ResetSplayTreeIterator(locale_list);
+  p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
   while (p != (const LocaleInfo *) NULL)
   {
     if (LocaleCompare(tag,p->tag) == 0)
       break;
-    p=(const LocaleInfo *) GetNextElementInLinkedList(locale_list);
+    p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
   }
-  RelinquishSemaphoreInfo(&locale_semaphore);
+  RelinquishSemaphoreInfo(locale_semaphore);
   return(p);
 }
 
@@ -246,7 +236,7 @@ MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
 %  The format of the GetLocaleInfoList function is:
 %
 %      const LocaleInfo **GetLocaleInfoList(const char *pattern,
-%        unsigned long *number_messages)
+%        unsigned long *number_messages,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -255,6 +245,7 @@ MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
 %    o number_messages:  This integer returns the number of locale messages in
 %    the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -280,14 +271,11 @@ static int LocaleInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const LocaleInfo **GetLocaleInfoList(const char *pattern,
-  unsigned long *number_messages)
+  unsigned long *number_messages,ExceptionInfo *exception)
 {
   const LocaleInfo
     **messages;
 
-  ExceptionInfo
-    exception;
-
   register const LocaleInfo
     *p;
 
@@ -298,32 +286,30 @@ MagickExport const LocaleInfo **GetLocaleInfoList(const char *pattern,
     Allocate locale list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_messages != (unsigned long *) NULL);
   *number_messages=0;
-  GetExceptionInfo(&exception);
-  p=GetLocaleInfo_("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetLocaleInfo_("*",exception);
   if (p == (const LocaleInfo *) NULL)
     return((const LocaleInfo **) NULL);
   messages=(const LocaleInfo **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(locale_list)+1)*sizeof(*messages));
+    (GetNumberOfNodesInSplayTree(locale_list)+1)*sizeof(*messages));
   if (messages == (const LocaleInfo **) NULL)
     return((const LocaleInfo **) NULL);
   /*
     Generate locale list.
   */
   AcquireSemaphoreInfo(&locale_semaphore);
-  ResetLinkedListIterator(locale_list);
-  p=(const LocaleInfo *) GetNextElementInLinkedList(locale_list);
+  ResetSplayTreeIterator(locale_list);
+  p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
   for (i=0; p != (const LocaleInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->tag,pattern) != MagickFalse))
       messages[i++]=p;
-    p=(const LocaleInfo *) GetNextElementInLinkedList(locale_list);
+    p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
   }
-  RelinquishSemaphoreInfo(&locale_semaphore);
+  RelinquishSemaphoreInfo(locale_semaphore);
   qsort((void *) messages,(size_t) i,sizeof(*messages),LocaleInfoCompare);
   messages[i]=(LocaleInfo *) NULL;
   *number_messages=(unsigned long) i;
@@ -346,14 +332,17 @@ MagickExport const LocaleInfo **GetLocaleInfoList(const char *pattern,
 %
 %  The format of the GetLocaleList function is:
 %
-%      char **GetLocaleList(const char *pattern,unsigned long *number_messages)
+%      char **GetLocaleList(const char *pattern,unsigned long *number_messages,
+%        Exceptioninfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o pattern: Specifies a pointer to a text string containing a pattern.
 %
-%    o number_messages:  This integer returns the number of messages in the list.
+%    o number_messages:  This integer returns the number of messages in the
+%      list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -377,14 +366,11 @@ static int LocaleTagCompare(const void *x,const void *y)
 #endif
 
 MagickExport char **GetLocaleList(const char *pattern,
-  unsigned long *number_messages)
+  unsigned long *number_messages,ExceptionInfo *exception)
 {
   char
     **messages;
 
-  ExceptionInfo
-    exception;
-
   register const LocaleInfo
     *p;
 
@@ -395,29 +381,28 @@ MagickExport char **GetLocaleList(const char *pattern,
     Allocate locale list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_messages != (unsigned long *) NULL);
   *number_messages=0;
-  GetExceptionInfo(&exception);
-  p=GetLocaleInfo_("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetLocaleInfo_("*",exception);
   if (p == (const LocaleInfo *) NULL)
     return((char **) NULL);
   AcquireSemaphoreInfo(&locale_semaphore);
-  RelinquishSemaphoreInfo(&locale_semaphore);
+  RelinquishSemaphoreInfo(locale_semaphore);
   messages=(char **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(locale_list)+1)*sizeof(*messages));
+    (GetNumberOfNodesInSplayTree(locale_list)+1)*sizeof(*messages));
   if (messages == (char **) NULL)
     return((char **) NULL);
   AcquireSemaphoreInfo(&locale_semaphore);
-  p=(const LocaleInfo *) GetNextElementInLinkedList(locale_list);
+  p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
   for (i=0; p != (const LocaleInfo *) NULL; )
   {
-    if ((p->stealth == MagickFalse) && (GlobExpression(p->tag,pattern) != MagickFalse))
-      messages[i++]=AcquireString(p->tag);
-    p=(const LocaleInfo *) GetNextElementInLinkedList(locale_list);
+    if ((p->stealth == MagickFalse) &&
+        (GlobExpression(p->tag,pattern) != MagickFalse))
+      messages[i++]=ConstantString(AcquireString(p->tag));
+    p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
   }
-  RelinquishSemaphoreInfo(&locale_semaphore);
+  RelinquishSemaphoreInfo(locale_semaphore);
   qsort((void *) messages,(size_t) i,sizeof(*messages),LocaleTagCompare);
   messages[i]=(char *) NULL;
   *number_messages=(unsigned long) i;
@@ -514,7 +499,7 @@ MagickExport LinkedListInfo *GetLocaleOptions(const char *filename,
     *xml;
 
   assert(filename != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),filename);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   assert(exception != (ExceptionInfo *) NULL);
   (void) CopyMagickString(path,filename,MaxTextExtent);
   /*
@@ -525,18 +510,18 @@ MagickExport LinkedListInfo *GetLocaleOptions(const char *filename,
   if (paths != (LinkedListInfo *) NULL)
     {
       ResetLinkedListIterator(paths);
-      element=(const char *) GetNextElementInLinkedList(paths);
+      element=(const char *) GetNextValueInLinkedList(paths);
       while (element != (const char *) NULL)
       {
         (void) FormatMagickString(path,MaxTextExtent,"%s%s",element,filename);
         (void) LogMagickEvent(LocaleEvent,GetMagickModule(),
           "Searching for locale file: \"%s\"",path);
-        xml=FileToStringInfo(path);
+        xml=ConfigureFileToStringInfo(path);
         if (xml != (StringInfo *) NULL)
-          (void) AppendElementToLinkedList(messages,xml);
-        element=(const char *) GetNextElementInLinkedList(paths);
+          (void) AppendValueToLinkedList(messages,xml);
+        element=(const char *) GetNextValueInLinkedList(paths);
       }
-      DestroyLinkedList(paths,RelinquishMagickMemory);
+      paths=DestroyLinkedList(paths,RelinquishMagickMemory);
     }
 #if defined(__WINDOWS__)
   {
@@ -547,14 +532,11 @@ MagickExport LinkedListInfo *GetLocaleOptions(const char *filename,
     if (blob != (char *) NULL)
       {
         xml=StringToStringInfo(blob);
-        (void) AppendElementToLinkedList(messages,xml);
+        (void) AppendValueToLinkedList(messages,xml);
         blob=(char *) RelinquishMagickMemory(blob);
       }
   }
 #endif
-  if (GetNumberOfElementsInLinkedList(messages) == 0)
-    (void) ThrowMagickException(exception,GetMagickModule(),ConfigureError,
-      "UnableToOpenLocaleFile",filename);
   ResetLinkedListIterator(messages);
   return(messages);
 }
@@ -583,7 +565,7 @@ MagickExport LinkedListInfo *GetLocaleOptions(const char *filename,
 */
 MagickExport const char *GetLocaleValue(const LocaleInfo *locale_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(locale_info != (LocaleInfo *) NULL);
   assert(locale_info->signature == MagickSignature);
   return(locale_info->message);
@@ -613,22 +595,12 @@ MagickExport const char *GetLocaleValue(const LocaleInfo *locale_info)
 */
 static MagickBooleanType InitializeLocaleList(ExceptionInfo *exception)
 {
-  MagickBooleanType
-    active;
-
-  active=MagickFalse;
-  if ((locale_list == (LinkedListInfo *) NULL) &&
+  if ((locale_list == (SplayTreeInfo *) NULL) &&
       (instantiate_locale == MagickFalse))
     {
       AcquireSemaphoreInfo(&locale_semaphore);
-      if ((locale_list == (LinkedListInfo *) NULL) &&
+      if ((locale_list == (SplayTreeInfo *) NULL) &&
           (instantiate_locale == MagickFalse))
-        {
-          instantiate_locale=MagickTrue;
-          active=MagickTrue;
-        }
-      RelinquishSemaphoreInfo(&locale_semaphore);
-      if (active == MagickTrue)
         {
           char
             *locale;
@@ -645,11 +617,11 @@ static MagickBooleanType InitializeLocaleList(ExceptionInfo *exception)
           if ((locale == (char *) NULL) || (*locale == '\0'))
             locale="C";
           (void) LoadLocaleLists(LocaleFilename,locale,exception);
-          if (GetNumberOfElementsInLinkedList(locale_list) == 0)
-            (void) LoadLocaleLists("english.mgk",locale,exception);
+          instantiate_locale=MagickTrue;
         }
+      RelinquishSemaphoreInfo(locale_semaphore);
     }
-  return((MagickBooleanType) (locale_list != (LinkedListInfo *) NULL));
+  return((MagickBooleanType) (locale_list != (SplayTreeInfo *) NULL));
 }
 
 /*
@@ -695,7 +667,7 @@ MagickExport MagickBooleanType ListLocaleInfo(FILE *file,
   if (file == (const FILE *) NULL)
     file=stdout;
   number_messages=0;
-  locale_info=GetLocaleInfoList("*",&number_messages);
+  locale_info=GetLocaleInfoList("*",&number_messages,exception);
   if (locale_info == (const LocaleInfo **) NULL)
     return(MagickFalse);
   path=(const char *) NULL;
@@ -775,6 +747,23 @@ static void ChopLocaleComponents(char *path,const unsigned long components)
         *p='\0';
         count++;
       }
+  if (count < (long) components)
+    *path='\0';
+}
+
+static void *DestroyLocaleNode(void *locale_info)
+{
+  register LocaleInfo
+    *p;
+
+  p=(LocaleInfo *) locale_info;
+  if (p->path != (char *) NULL)
+    p->path=(char *) RelinquishMagickMemory(p->path);
+  if (p->tag != (char *) NULL)
+    p->tag=(char *) RelinquishMagickMemory(p->tag);
+  if (p->message != (char *) NULL)
+    p->message=(char *) RelinquishMagickMemory(p->message);
+  return(RelinquishMagickMemory(p));
 }
 
 static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
@@ -801,10 +790,11 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
   */
   if (xml == (const char *) NULL)
     return(MagickFalse);
-  if (locale_list == (LinkedListInfo *) NULL)
+  if (locale_list == (SplayTreeInfo *) NULL)
     {
-      locale_list=NewLinkedList(0);
-      if (locale_list == (LinkedListInfo *) NULL)
+      locale_list=NewSplayTree(CompareSplayTreeString,RelinquishMagickMemory,
+        DestroyLocaleNode);
+      if (locale_list == (SplayTreeInfo *) NULL)
         {
           ThrowFileException(exception,ResourceLimitError,
             "MemoryAllocationFailed",filename);
@@ -821,17 +811,34 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
     /*
       Interpret XML.
     */
-    GetToken(q,&q,token);
+    GetMagickToken(q,&q,token);
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+      {
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+        {
+          GetMagickToken(q,&q,token);
+          while (isspace((int) ((unsigned char) *q)) != 0)
+            q++;
+        }
+        continue;
+      }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetToken(q,&q,token);
+        {
+          GetMagickToken(q,&q,token);
+          while (isspace((int) ((unsigned char) *q)) != 0)
+            q++;
+        }
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -842,10 +849,10 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (*token != '=')
             continue;
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (LocaleCompare(keyword,"locale") == 0)
             {
               if (LocaleCompare(locale,token) != 0)
@@ -856,7 +863,7 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
             {
               if (depth > 200)
                 (void) ThrowMagickException(exception,GetMagickModule(),
-                  ConfigureError,"IncludeElementNestedTooDeeply",token);
+                  ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
               else
                 {
                   char
@@ -869,7 +876,7 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
                   (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
+                  xml=FileToString(path,~0,exception);
                   status|=LoadLocaleList(xml,path,locale,depth+1,exception);
                   xml=(char *) RelinquishMagickMemory(xml);
                 }
@@ -885,10 +892,10 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
         while ((*token != '>') && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (*token != '=')
             continue;
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
         }
         continue;
       }
@@ -910,10 +917,10 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
         while ((*token != '>') && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (*token != '=')
             continue;
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (LocaleCompare(keyword,"name") == 0)
             {
               (void) ConcatenateMagickString(tag,token,MaxTextExtent);
@@ -932,14 +939,16 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
           ThrowMagickFatalException(ResourceLimitFatalError,
             "MemoryAllocationFailed",filename);
         (void) ResetMagickMemory(locale_info,0,sizeof(*locale_info));
-        locale_info->path=AcquireString(filename);
-        locale_info->tag=AcquireString(tag);
-        locale_info->message=AcquireString(message);
+        locale_info->path=ConstantString(AcquireString(filename));
+        locale_info->tag=ConstantString(AcquireString(tag));
+        locale_info->message=ConstantString(AcquireString(message));
         locale_info->signature=MagickSignature;
-        status=AppendElementToLinkedList(locale_list,locale_info);
+        status=AddValueToSplayTree(locale_list,
+          ConstantString(AcquireString(locale_info->tag)),locale_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
-            ResourceLimitError,"MemoryAllocationFailed",locale_info->tag);
+            ResourceLimitError,"MemoryAllocationFailed","`%s'",
+            locale_info->tag);
         (void) ConcatenateMagickString(tag,message,MaxTextExtent);
         (void) ConcatenateMagickString(tag,"\n",MaxTextExtent);
         q++;
@@ -961,7 +970,8 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
         if (*(keyword+1) == '/')
           {
             ChopLocaleComponents(tag,1);
-            (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
+            if (*tag != '\0')
+              (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
             continue;
           }
         token[strlen(token)-1]='\0';
@@ -970,12 +980,12 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
         (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
         continue;
       }
-    GetToken(q,(char **) NULL,token);
+    GetMagickToken(q,(char **) NULL,token);
     if (*token != '=')
       continue;
   }
   token=(char *) RelinquishMagickMemory(token);
-  if (locale_list == (LinkedListInfo *) NULL)
+  if (locale_list == (SplayTreeInfo *) NULL)
     return(MagickFalse);
   return(status != 0 ? MagickTrue : MagickFalse);
 }
@@ -1012,6 +1022,9 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
 static MagickBooleanType LoadLocaleLists(const char *filename,
   const char *locale,ExceptionInfo *exception)
 {
+#if defined(UseEmbeddableMagick)
+  return(LoadLocaleList(LocaleMap,"built-in",locale,0,exception));
+#else
   const StringInfo
     *option;
 
@@ -1023,16 +1036,30 @@ static MagickBooleanType LoadLocaleLists(const char *filename,
 
   status=MagickFalse;
   options=GetLocaleOptions(filename,exception);
-  option=(const StringInfo *) GetNextElementInLinkedList(options);
+  option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
     status|=LoadLocaleList((const char *) option->datum,option->path,locale,0,
       exception);
-    option=(const StringInfo *) GetNextElementInLinkedList(options);
+    option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
-  DestroyLocaleOptions(options);
-  if ((locale_list == (LinkedListInfo *) NULL) ||
-      (IsLinkedListEmpty(locale_list) != MagickFalse))
+  options=DestroyLocaleOptions(options);
+  if ((locale_list == (SplayTreeInfo *) NULL) ||
+      (GetNumberOfNodesInSplayTree(locale_list) == 0))
+    {
+      options=GetLocaleOptions("english.xml",exception);
+      option=(const StringInfo *) GetNextValueInLinkedList(options);
+      while (option != (const StringInfo *) NULL)
+      {
+        status|=LoadLocaleList((const char *) option->datum,option->path,locale,
+          0,exception);
+        option=(const StringInfo *) GetNextValueInLinkedList(options);
+      }
+      options=DestroyLocaleOptions(options);
+    }
+  if ((locale_list == (SplayTreeInfo *) NULL) ||
+      (GetNumberOfNodesInSplayTree(locale_list) == 0))
     status|=LoadLocaleList(LocaleMap,"built-in",locale,0,exception);
   return(status != 0 ? MagickTrue : MagickFalse);
+#endif
 }
