@@ -17,13 +17,13 @@
 %                                September 2002                               %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -43,7 +43,8 @@
 #include "magick/blob.h"
 #include "magick/client.h"
 #include "magick/configure.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/hashmap.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
@@ -53,11 +54,12 @@
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
+#include "magick/version.h"
 
 /*
   Define declarations.
 */
-#define LogFilename  "log.mgk"
+#define LogFilename  "log.xml"
 
 /*
   Typedef declarations.
@@ -73,7 +75,7 @@ typedef struct _EventInfo
 
 typedef struct _HandlerInfo
 {
-  char
+  const char
     *name;
 
   LogHandlerType
@@ -95,7 +97,7 @@ static const HandlerInfo
   };
 
 static const char
-  *LogMap = (char *)
+  *LogMap = (const char *)
     "<?xml version=\"1.0\"?>"
     "<logmap>"
     "  <log events=\"None\" />"
@@ -103,7 +105,7 @@ static const char
     "  <log filename=\"Magick-%d.log\" />"
     "  <log generations=\"3\" />"
     "  <log limit=\"2000\" />"
-    "  <log format=\"%t %r %u %c[%p]: %m/%f/%l/%d\n  %e\" />"
+    "  <log format=\"%t %r %u %v %d %c[%p]: %m/%f/%l/%d\n  %e\" />"
     "</logmap>";
 
 /*
@@ -174,11 +176,10 @@ MagickExport void DestroyLogList(void)
 {
   AcquireSemaphoreInfo(&log_semaphore);
   if (log_list != (LinkedListInfo *) NULL)
-    DestroyLinkedList(log_list,DestroyLogElement);
-  log_list=(LinkedListInfo *) NULL;
+    log_list=DestroyLinkedList(log_list,DestroyLogElement);
   instantiate_log=MagickFalse;
-  RelinquishSemaphoreInfo(&log_semaphore);
-  DestroySemaphoreInfo(&log_semaphore);
+  RelinquishSemaphoreInfo(log_semaphore);
+  log_semaphore=DestroySemaphoreInfo(log_semaphore);
 }
 
 /*
@@ -218,26 +219,26 @@ MagickExport const LogInfo *GetLogInfo(const char *name,
     if (InitializeLogList(exception) == MagickFalse)
       return((const LogInfo *) NULL);
   if ((log_list == (LinkedListInfo *) NULL) ||
-      (IsLinkedListEmpty(log_list) == MagickTrue))
+      (IsLinkedListEmpty(log_list) != MagickFalse))
     return((const LogInfo *) NULL);
   if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
-    return((const LogInfo *) GetElementFromLinkedList(log_list,0));
+    return((const LogInfo *) GetValueFromLinkedList(log_list,0));
   /*
     Search for named log.
   */
   AcquireSemaphoreInfo(&log_semaphore);
   ResetLinkedListIterator(log_list);
-  p=(const LogInfo *) GetNextElementInLinkedList(log_list);
+  p=(const LogInfo *) GetNextValueInLinkedList(log_list);
   while (p != (const LogInfo *) NULL)
   {
     if (LocaleCompare(name,p->name) == 0)
       break;
-    p=(const LogInfo *) GetNextElementInLinkedList(log_list);
+    p=(const LogInfo *) GetNextValueInLinkedList(log_list);
   }
   if (p == (LogInfo *) NULL)
     (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
-      "UnrecognizedLog",name);
-  RelinquishSemaphoreInfo(&log_semaphore);
+      "NoSuchElement","`%s'",name);
+  RelinquishSemaphoreInfo(log_semaphore);
   return(p);
 }
 
@@ -257,7 +258,7 @@ MagickExport const LogInfo *GetLogInfo(const char *name,
 %  The format of the GetLogInfoList function is:
 %
 %      const LogInfo **GetLogInfoList(const char *pattern,
-%        unsigned long *number_preferences)
+%        unsigned long *number_preferences,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -265,6 +266,7 @@ MagickExport const LogInfo *GetLogInfo(const char *name,
 %
 %    o number_preferences:  This integer returns the number of logs in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 #if defined(__cplusplus) || defined(c_plusplus)
@@ -289,14 +291,11 @@ static int LogInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const LogInfo **GetLogInfoList(const char *pattern,
-  unsigned long *number_preferences)
+  unsigned long *number_preferences,ExceptionInfo *exception)
 {
   const LogInfo
     **preferences;
 
-  ExceptionInfo
-    exception;
-
   register const LogInfo
     *p;
 
@@ -307,12 +306,10 @@ MagickExport const LogInfo **GetLogInfoList(const char *pattern,
     Allocate log list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_preferences != (unsigned long *) NULL);
   *number_preferences=0;
-  GetExceptionInfo(&exception);
-  p=GetLogInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetLogInfo("*",exception);
   if (p == (const LogInfo *) NULL)
     return((const LogInfo **) NULL);
   preferences=(const LogInfo **) AcquireMagickMemory((size_t)
@@ -324,15 +321,15 @@ MagickExport const LogInfo **GetLogInfoList(const char *pattern,
   */
   AcquireSemaphoreInfo(&log_semaphore);
   ResetLinkedListIterator(log_list);
-  p=(const LogInfo *) GetNextElementInLinkedList(log_list);
+  p=(const LogInfo *) GetNextValueInLinkedList(log_list);
   for (i=0; p != (const LogInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
       preferences[i++]=p;
-    p=(const LogInfo *) GetNextElementInLinkedList(log_list);
+    p=(const LogInfo *) GetNextValueInLinkedList(log_list);
   }
-  RelinquishSemaphoreInfo(&log_semaphore);
+  RelinquishSemaphoreInfo(log_semaphore);
   qsort((void *) preferences,(size_t) i,sizeof(*preferences),LogInfoCompare);
   preferences[i]=(LogInfo *) NULL;
   *number_preferences=(unsigned long) i;
@@ -354,7 +351,8 @@ MagickExport const LogInfo **GetLogInfoList(const char *pattern,
 %
 %  The format of the GetLogList function is:
 %
-%      char **GetLogList(const char *pattern,unsigned long *number_preferences)
+%      char **GetLogList(const char *pattern,unsigned long *number_preferences,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -362,6 +360,7 @@ MagickExport const LogInfo **GetLogInfoList(const char *pattern,
 %
 %    o number_preferences:  This integer returns the number of logs in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -371,12 +370,12 @@ extern "C" {
 
 static int LogCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+  p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
@@ -385,14 +384,11 @@ static int LogCompare(const void *x,const void *y)
 #endif
 
 MagickExport char **GetLogList(const char *pattern,
-  unsigned long *number_preferences)
+  unsigned long *number_preferences,ExceptionInfo *exception)
 {
   char
     **preferences;
 
-  ExceptionInfo
-    exception;
-
   register const LogInfo
     *p;
 
@@ -403,12 +399,10 @@ MagickExport char **GetLogList(const char *pattern,
     Allocate log list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_preferences != (unsigned long *) NULL);
   *number_preferences=0;
-  GetExceptionInfo(&exception);
-  p=GetLogInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetLogInfo("*",exception);
   if (p == (const LogInfo *) NULL)
     return((char **) NULL);
   preferences=(char **) AcquireMagickMemory((size_t)
@@ -420,15 +414,15 @@ MagickExport char **GetLogList(const char *pattern,
   */
   AcquireSemaphoreInfo(&log_semaphore);
   ResetLinkedListIterator(log_list);
-  p=(const LogInfo *) GetNextElementInLinkedList(log_list);
+  p=(const LogInfo *) GetNextValueInLinkedList(log_list);
   for (i=0; p != (const LogInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
-      preferences[i++]=AcquireString(p->name);
-    p=(const LogInfo *) GetNextElementInLinkedList(log_list);
+      preferences[i++]=ConstantString(AcquireString(p->name));
+    p=(const LogInfo *) GetNextValueInLinkedList(log_list);
   }
-  RelinquishSemaphoreInfo(&log_semaphore);
+  RelinquishSemaphoreInfo(log_semaphore);
   qsort((void *) preferences,(size_t) i,sizeof(*preferences),LogCompare);
   preferences[i]=(char *) NULL;
   *number_preferences=(unsigned long) i;
@@ -459,22 +453,16 @@ MagickExport char **GetLogList(const char *pattern,
 */
 static MagickBooleanType InitializeLogList(ExceptionInfo *exception)
 {
-  MagickBooleanType
-    active;
-
-  active=MagickFalse;
   if ((log_list == (LinkedListInfo *) NULL) && (instantiate_log == MagickFalse))
     {
       AcquireSemaphoreInfo(&log_semaphore);
       if ((log_list == (LinkedListInfo *) NULL) &&
           (instantiate_log == MagickFalse))
         {
+          (void) LoadLogLists(LogFilename,exception);
           instantiate_log=MagickTrue;
-          active=MagickTrue;
         }
-      RelinquishSemaphoreInfo(&log_semaphore);
-      if (active == MagickTrue)
-        (void) LoadLogLists(LogFilename,exception);
+      RelinquishSemaphoreInfo(log_semaphore);
     }
   return(log_list != (LinkedListInfo *) NULL ? MagickTrue : MagickFalse);
 }
@@ -508,7 +496,7 @@ MagickExport MagickBooleanType IsEventLogging(void)
     exception;
 
   if ((log_list == (LinkedListInfo *) NULL) ||
-      (IsLinkedListEmpty(log_list) == MagickTrue))
+      (IsLinkedListEmpty(log_list) != MagickFalse))
     return(MagickFalse);
   GetExceptionInfo(&exception);
   log_info=GetLogInfo("*",&exception);
@@ -559,7 +547,7 @@ MagickExport MagickBooleanType ListLogInfo(FILE *file,ExceptionInfo *exception)
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  log_info=GetLogInfoList("*",&number_aliases);
+  log_info=GetLogInfoList("*",&number_aliases,exception);
   if (log_info == (const LogInfo **) NULL)
     return(MagickFalse);
   j=0;
@@ -669,11 +657,8 @@ static char *TranslateEvent(const LogEventType type,const char *module,
   time_meridian=localtime(&seconds);
   elapsed_time=GetElapsedTime(&log_info->timer);
   user_time=GetUserTime(&log_info->timer);
-  length=10*MaxTextExtent;
-  text=(char *) AcquireMagickMemory(length);
-  if (text == (char *) NULL)
-    return((char *) NULL);
-  *text='\0';
+  text=AcquireString((char *) NULL);
+  length=MaxTextExtent;
   if (log_info->format == (char *) NULL)
     return(text);
   if (LocaleCompare(log_info->format,"xml") == 0)
@@ -689,7 +674,7 @@ static char *TranslateEvent(const LogEventType type,const char *module,
         time_meridian->tm_mon+1,time_meridian->tm_mday,time_meridian->tm_hour,
         time_meridian->tm_min,time_meridian->tm_sec);
       (void) FormatMagickString(text,length,
-        "<record>\n"
+        "<entry>\n"
         "  <timestamp>%s</timestamp>\n"
         "  <elapsed-time>%ld:%02ld</elapsed-time>\n"
         "  <user-time>%0.3f</user-time>\n"
@@ -699,7 +684,7 @@ static char *TranslateEvent(const LogEventType type,const char *module,
         "  <line>%lu</line>\n"
         "  <domain>%s</domain>\n"
         "  <event>%s</event>\n"
-        "</record>\n",timestamp,(long) (elapsed_time/60.0),
+        "</entry>",timestamp,(long) (elapsed_time/60.0),
         (long) ceil(fmod(elapsed_time,60.0)),user_time,(long) getpid(),module,
         function,line,domain,event);
       return(text);
@@ -714,8 +699,8 @@ static char *TranslateEvent(const LogEventType type,const char *module,
     if ((size_t) (q-text+MaxTextExtent) >= length)
       {
         length<<=1;
-        text=(char *) ResizeMagickMemory(text,
-          (length+MaxTextExtent)*sizeof(*text));
+        text=(char *) ResizeMagickMemory(text,(length+MaxTextExtent)*
+          sizeof(*text));
         if (text == (char *) NULL)
           return((char *) NULL);
         q=text+strlen(text);
@@ -723,6 +708,7 @@ static char *TranslateEvent(const LogEventType type,const char *module,
     /*
       The format of the log is defined by embedding special format characters:
 
+        %c   client name
         %d   domain
         %e   event
         %f   function
@@ -732,6 +718,7 @@ static char *TranslateEvent(const LogEventType type,const char *module,
         %r   real CPU time
         %t   wall clock time
         %u   user CPU time
+        %v   version
         %%   percent sign
         \n   newline
         \r   carriage return
@@ -832,6 +819,12 @@ static char *TranslateEvent(const LogEventType type,const char *module,
           q++;
         break;
       }
+      case 'v':
+      {
+        (void) CopyMagickString(q,MagickLibVersionText,MaxTextExtent);
+        q+=strlen(MagickLibVersionText);
+        break;
+      }
       default:
       {
         *q++=(*p);
@@ -843,24 +836,26 @@ static char *TranslateEvent(const LogEventType type,const char *module,
   return(text);
 }
 
-MagickExport MagickBooleanType LogMagickEvent(const LogEventType type,
-  const char *module,const char *function,const unsigned long line,
-  const char *format,...)
+MagickBooleanType LogMagickEventList(const LogEventType type,const char *module,
+  const char *function,const unsigned long line,const char *format,
+  va_list operands)
 {
   char
-    *domain,
     event[MaxTextExtent],
     *text;
 
+  const char
+    *domain;
+
   ExceptionInfo
     exception;
 
+  int
+    n;
+
   LogInfo
     *log_info;
 
-  va_list
-    operands;
-
   if (IsEventLogging() == MagickFalse)
     return(MagickFalse);
   GetExceptionInfo(&exception);
@@ -869,40 +864,22 @@ MagickExport MagickBooleanType LogMagickEvent(const LogEventType type,
   AcquireSemaphoreInfo(&log_semaphore);
   if ((log_info->event_mask & type) == 0)
     {
-      RelinquishSemaphoreInfo(&log_semaphore);
+      RelinquishSemaphoreInfo(log_semaphore);
       return(MagickTrue);
     }
-  switch (type)
-  {
-    case AnnotateEvent: domain=(char *) "Annotate"; break;
-    case BlobEvent: domain=(char *) "Blob"; break;
-    case CacheEvent: domain=(char *) "Cache"; break;
-    case CoderEvent: domain=(char *) "Coder"; break;
-    case ConfigureEvent: domain=(char *) "Configure"; break;
-    case DeprecateEvent: domain=(char *) "Deprecate"; break;
-    case DrawEvent: domain=(char *) "Draw"; break;
-    case ExceptionEvent: domain=(char *) "Exception"; break;
-    case LocaleEvent: domain=(char *) "Locale"; break;
-    case ModuleEvent: domain=(char *) "Module"; break;
-    case ResourceEvent: domain=(char *) "Resource"; break;
-    case TraceEvent: domain=(char *) "Trace"; break;
-    case TransformEvent: domain=(char *) "Transform"; break;
-    case X11Event: domain=(char *) "X11"; break;
-    case UserEvent: domain=(char *) "User"; break;
-    default: domain=(char *) "UnknownEvent"; break;
-  }
-  va_start(operands,format);
+  domain=MagickOptionToMnemonic(MagickLogEventOptions,type);
 #if defined(HAVE_VSNPRINTF)
-  (void) vsnprintf(event,MaxTextExtent,format,operands);
+  n=vsnprintf(event,MaxTextExtent,format,operands);
 #else
-  (void) vsprintf(event,format,operands);
+  n=vsprintf(event,format,operands);
 #endif
-  va_end(operands);
+  if  (n < 0)
+    event[MaxTextExtent-1]='\0';
   text=TranslateEvent(type,module,function,line,domain,event);
   if (text == (char *) NULL)
     {
       (void) ContinueTimer((TimerInfo *) &log_info->timer);
-      RelinquishSemaphoreInfo(&log_semaphore);
+      RelinquishSemaphoreInfo(log_semaphore);
       return(MagickFalse);
     }
   if ((log_info->handler_mask & ConsoleHandler) != 0)
@@ -942,21 +919,37 @@ MagickExport MagickBooleanType LogMagickEvent(const LogEventType type,
           log_info->file=fopen(filename,"w");
           if (log_info->file == (FILE *) NULL)
             {
-              RelinquishSemaphoreInfo(&log_semaphore);
+              RelinquishSemaphoreInfo(log_semaphore);
               return(MagickFalse);
             }
           log_info->generation++;
-          (void) fprintf(log_info->file,"<?xml version=\"1.0\"?>\n");
+          (void) fprintf(log_info->file,
+            "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n");
           (void) fprintf(log_info->file,"<log>\n");
         }
       (void) fprintf(log_info->file,"%s\n",text);
       (void) fflush(log_info->file);
     }
-  text=(char *) RelinquishMagickMemory(text);
-  (void) ContinueTimer((TimerInfo *) &log_info->timer);
-  RelinquishSemaphoreInfo(&log_semaphore);
+  text=(char  *) RelinquishMagickMemory(text);
+  (void)  ContinueTimer((TimerInfo *) &log_info->timer);
+  RelinquishSemaphoreInfo(log_semaphore);
   return(MagickTrue);
 }
+
+MagickBooleanType LogMagickEvent(const LogEventType type,const char *module,
+  const char *function,const unsigned long line,const char *format,...)
+{
+  va_list
+    operands;
+
+  MagickBooleanType
+    status;
+   
+  va_start(operands,format);
+  status=LogMagickEventList(type,module,function,line,format,operands);
+  va_end(operands);
+  return(status);
+}
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1025,17 +1018,26 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
     /*
       Interpret XML.
     */
-    GetToken(q,&q,token);
+    GetMagickToken(q,&q,token);
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+      {
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -1046,15 +1048,15 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (*token != '=')
             continue;
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
               if (depth > 200)
                 (void) ThrowMagickException(exception,GetMagickModule(),
-                  ConfigureError,"IncludeElementNestedTooDeeply",token);
+                  ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
               else
                 {
                   char
@@ -1066,7 +1068,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
                   (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
+                  xml=FileToString(path,~0,exception);
                   status|=LoadLogList(xml,path,depth+1,exception);
                   xml=(char *) RelinquishMagickMemory(xml);
                 }
@@ -1084,7 +1086,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
           ThrowMagickFatalException(ResourceLimitFatalError,
             "MemoryAllocationFailed",filename);
         (void) ResetMagickMemory(log_info,0,sizeof(*log_info));
-        log_info->path=AcquireString(filename);
+        log_info->path=ConstantString(AcquireString(filename));
         GetTimerInfo((TimerInfo *) &log_info->timer);
         log_info->signature=MagickSignature;
         continue;
@@ -1093,17 +1095,17 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
       continue;
     if (LocaleCompare(keyword,"</logmap>") == 0)
       {
-        status=AppendElementToLinkedList(log_list,log_info);
+        status=AppendValueToLinkedList(log_list,log_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
-            ResourceLimitError,"MemoryAllocationFailed",filename);
+            ResourceLimitError,"MemoryAllocationFailed","`%s'",filename);
         log_info=(LogInfo *) NULL;
       }
-    GetToken(q,(char **) NULL,token);
+    GetMagickToken(q,(char **) NULL,token);
     if (*token != '=')
       continue;
-    GetToken(q,&q,token);
-    GetToken(q,&q,token);
+    GetMagickToken(q,&q,token);
+    GetMagickToken(q,&q,token);
     switch (*keyword)
     {
       case 'E':
@@ -1125,7 +1127,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
             if (log_info->filename != (char *) NULL)
               log_info->filename=(char *)
                 RelinquishMagickMemory(log_info->filename);
-            log_info->filename=AcquireString(token);
+            log_info->filename=ConstantString(AcquireString(token));
             break;
           }
         if (LocaleCompare((char *) keyword,"format") == 0)
@@ -1133,7 +1135,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
             if (log_info->format != (char *) NULL)
               log_info->format=(char *)
                 RelinquishMagickMemory(log_info->format);
-            log_info->format=AcquireString(token);
+            log_info->format=ConstantString(AcquireString(token));
             break;
           }
         break;
@@ -1200,7 +1202,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The font file name.
+%    o filename: The log configuration filename.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -1209,6 +1211,9 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
 static MagickBooleanType LoadLogLists(const char *filename,
   ExceptionInfo *exception)
 {
+#if defined(UseEmbeddableMagick)
+  return(LoadLogList(LogMap,"built-in",0,exception));
+#else
   const StringInfo
     *option;
 
@@ -1220,17 +1225,20 @@ static MagickBooleanType LoadLogLists(const char *filename,
 
   status=MagickFalse;
   options=GetConfigureOptions(filename,exception);
-  option=(const StringInfo *) GetNextElementInLinkedList(options);
+  option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
     status|=LoadLogList((const char *) option->datum,option->path,0,exception);
-    option=(const StringInfo *) GetNextElementInLinkedList(options);
+    option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
-  DestroyConfigureOptions(options);
+  options=DestroyConfigureOptions(options);
   if ((log_list == (LinkedListInfo *) NULL) || 
       (IsLinkedListEmpty(log_list) != MagickFalse))
     status|=LoadLogList(LogMap,"built-in",0,exception);
+  else
+    (void) SetExceptionInfo(exception,UndefinedException);
   return(status != 0 ? MagickTrue : MagickFalse);
+#endif
 }
 
 /*
@@ -1307,7 +1315,7 @@ static LogHandlerType ParseLogHandlers(const char *handlers)
 %  other events are ignored.  By default, no debug is enabled.  This method
 %  returns the previous log event mask.
 %
-%  The format of the AcquireString method is:
+%  The format of the SetLogEventMask method is:
 %
 %      LogEventType SetLogEventMask(const char *events)
 %
@@ -1322,21 +1330,22 @@ MagickExport LogEventType SetLogEventMask(const char *events)
   ExceptionInfo
     exception;
 
-  LogEventType
-    event_mask;
-
   LogInfo
     *log_info;
 
+  long
+    option;
+
   GetExceptionInfo(&exception);
   log_info=(LogInfo *) GetLogInfo("*",&exception);
   DestroyExceptionInfo(&exception);
+  option=ParseMagickOption(MagickLogEventOptions,MagickTrue,events);
   AcquireSemaphoreInfo(&log_semaphore);
-  event_mask=(LogEventType)
-    ParseMagickOption(MagickLogEventOptions,MagickTrue,events);
-  log_info=(LogInfo *) GetElementFromLinkedList(log_list,0);
-  log_info->event_mask=event_mask;
-  RelinquishSemaphoreInfo(&log_semaphore);
+  log_info=(LogInfo *) GetValueFromLinkedList(log_list,0);
+  log_info->event_mask=(LogEventType) option;
+  if (option == -1)
+    log_info->event_mask=UndefinedEvents;
+  RelinquishSemaphoreInfo(log_semaphore);
   return(log_info->event_mask);
 }
 
@@ -1378,6 +1387,6 @@ MagickExport void SetLogFormat(const char *format)
   AcquireSemaphoreInfo(&log_semaphore);
   if (log_info->format != (char *) NULL)
     log_info->format=(char *) RelinquishMagickMemory(log_info->format);
-  log_info->format=AcquireString(format);
-  RelinquishSemaphoreInfo(&log_semaphore);
+  log_info->format=ConstantString(AcquireString(format));
+  RelinquishSemaphoreInfo(log_semaphore);
 }
