@@ -16,13 +16,13 @@
 %                                 July 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -42,7 +42,8 @@
 #include "magick/blob.h"
 #include "magick/client.h"
 #include "magick/configure.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/hashmap.h"
 #include "magick/magic.h"
 #include "magick/memory_.h"
@@ -54,13 +55,13 @@
 /*
   Define declarations.
 */
-#define MagicFilename  "magic.mgk"
+#define MagicFilename  "magic.xml"
 
 /*
   Static declarations.
 */
 static const char
-  *MagicMap = (char *)
+  *MagicMap = (const char *)
     "<?xml version=\"1.0\"?>"
     "<magicmap>"
     "  <magic name=\"AVI\" offset=\"0\" target=\"RIFF\" />"
@@ -73,11 +74,14 @@ static const char
     "  <magic name=\"BMP\" offset=\"0\" target=\"CP\" />"
     "  <magic name=\"BMP\" offset=\"0\" target=\"IC\" />"
     "  <magic name=\"BMP\" offset=\"0\" target=\"PI\" />"
+    "  <magic name=\"CIN\" offset=\"0\" target=\"\\200\\052\\137\\327\" />"
     "  <magic name=\"CGM\" offset=\"0\" target=\"BEGMF\" />"
     "  <magic name=\"DCM\" offset=\"128\" target=\"DICM\" />"
     "  <magic name=\"DCX\" offset=\"0\" target=\"\\261\\150\\336\\72\" />"
     "  <magic name=\"DIB\" offset=\"0\" target=\"\\050\\000\" />"
     "  <magic name=\"DOT\" offset=\"0\" target=\"digraph\" />"
+    "  <magic name=\"DPX\" offset=\"0\" target=\"SDPX\" />"
+    "  <magic name=\"DPX\" offset=\"0\" target=\"XPDS\" />"
     "  <magic name=\"EMF\" offset=\"40\" target=\"\\040\\105\\115\\106\\000\\000\\001\\000\" />"
     "  <magic name=\"EPT\" offset=\"0\" target=\"\\305\\320\\323\\306\" />"
     "  <magic name=\"FAX\" offset=\"0\" target=\"DFAX\" />"
@@ -89,14 +93,13 @@ static const char
     "  <magic name=\"GPLT\" offset=\"0\" target=\"#!/usr/local/bin/gnuplot\" />"
     "  <magic name=\"HDF\" offset=\"1\" target=\"HDF\" />"
     "  <magic name=\"HPGL\" offset=\"0\" target=\"IN;\" />"
-    "  <magic name=\"HPGL\" offset=\"0\" target=\"\\033E\\033\" />"
     "  <magic name=\"HTML\" offset=\"1\" target=\"HTML\" />"
     "  <magic name=\"HTML\" offset=\"1\" target=\"html\" />"
     "  <magic name=\"ILBM\" offset=\"8\" target=\"ILBM\" />"
     "  <magic name=\"IPTCWTEXT\" offset=\"0\" target=\"\\062\\000#\\000\\060\\000=\\000\\042\\000&\\000#\\000\\060\\000;\\000&\\000#\\000\\062\\000;\\000\\042\\000\" />"
     "  <magic name=\"IPTCTEXT\" offset=\"0\" target=\"2#0=\\042&#0;&#2;\\042\" />"
     "  <magic name=\"IPTC\" offset=\"0\" target=\"\\034\\002\" />"
-    "  <magic name=\"JNG\" offset=\"0\" target=\"\\213JNG\\r\\n\\032\n\" />"
+    "  <magic name=\"JNG\" offset=\"0\" target=\"\\213JNG\\r\\n\\032\\n\" />"
     "  <magic name=\"JPEG\" offset=\"0\" target=\"\\377\\330\\377\" />"
     "  <magic name=\"JPC\" offset=\"0\" target=\"\\377\\117\" />"
     "  <magic name=\"JP2\" offset=\"4\" target=\"\\152\\120\\040\\040\\015\" />"
@@ -105,7 +108,6 @@ static const char
     "  <magic name=\"MNG\" offset=\"0\" target=\"\\212MNG\\r\\n\\032\\n\" />"
     "  <magic name=\"MPC\" offset=\"0\" target=\"id=MagickCache\" />"
     "  <magic name=\"MPEG\" offset=\"0\" target=\"\\000\\000\\001\\263\" />"
-    "  <magic name=\"MVG\" offset=\"0\" target=\"push graphic-context\" />"
     "  <magic name=\"PCD\" offset=\"2048\" target=\"PCD_\" />"
     "  <magic name=\"PCL\" offset=\"0\" target=\"\\033E\\033\" />"
     "  <magic name=\"PCX\" offset=\"0\" target=\"\\012\\002\" />"
@@ -130,6 +132,7 @@ static const char
     "  <magic name=\"PSD\" offset=\"0\" target=\"8BPS\" />"
     "  <magic name=\"PWP\" offset=\"0\" target=\"SFW95\" />"
     "  <magic name=\"RAD\" offset=\"0\" target=\"#?RADIANCE\" />"
+    "  <magic name=\"RAD\" offset=\"0\" target=\"VIEW= \" />"
     "  <magic name=\"RLE\" offset=\"0\" target=\"\\122\\314\" />"
     "  <magic name=\"SCT\" offset=\"0\" target=\"CT\" />"
     "  <magic name=\"SFW\" offset=\"0\" target=\"SFW94\" />"
@@ -211,11 +214,10 @@ MagickExport void DestroyMagicList(void)
 {
   AcquireSemaphoreInfo(&magic_semaphore);
   if (magic_list != (LinkedListInfo *) NULL)
-    DestroyLinkedList(magic_list,DestroyMagicElement);
-  magic_list=(LinkedListInfo *) NULL;
+    magic_list=DestroyLinkedList(magic_list,DestroyMagicElement);
   instantiate_magic=MagickFalse;
-  RelinquishSemaphoreInfo(&magic_semaphore);
-  DestroySemaphoreInfo(&magic_semaphore);
+  RelinquishSemaphoreInfo(magic_semaphore);
+  magic_semaphore=DestroySemaphoreInfo(magic_semaphore);
 }
 
 /*
@@ -260,10 +262,10 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
     if (InitializeMagicList(exception) == MagickFalse)
       return((const MagicInfo *) NULL);
   if ((magic_list == (LinkedListInfo *) NULL) ||
-      (IsLinkedListEmpty(magic_list) == MagickTrue))
+      (IsLinkedListEmpty(magic_list) != MagickFalse))
     return((const MagicInfo *) NULL);
   if ((magic == (const unsigned char *) NULL) || (length == 0))
-    return((const MagicInfo *) GetElementFromLinkedList(magic_list,0));
+    return((const MagicInfo *) GetValueFromLinkedList(magic_list,0));
   if (length == 0)
     return((const MagicInfo *) NULL);
   /*
@@ -271,16 +273,16 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
   */
   AcquireSemaphoreInfo(&magic_semaphore);
   ResetLinkedListIterator(magic_list);
-  p=(const MagicInfo *) GetNextElementInLinkedList(magic_list);
+  p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   while (p != (const MagicInfo *) NULL)
   {
     assert(p->offset >= 0);
     if (((size_t) (p->offset+p->length) <= length) &&
         (memcmp(magic+p->offset,p->magic,p->length) == 0))
       break;
-    p=(const MagicInfo *) GetNextElementInLinkedList(magic_list);
+    p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   }
-  RelinquishSemaphoreInfo(&magic_semaphore);
+  RelinquishSemaphoreInfo(magic_semaphore);
   return((const MagicInfo *) p);
 }
 
@@ -301,7 +303,7 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
 %  The magic of the GetMagicInfoList function is:
 %
 %      const MagicInfo **GetMagicInfoList(const char *pattern,
-%        unsigned long *number_aliases)
+%        unsigned long *number_aliases,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -310,6 +312,7 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
 %    o number_aliases:  This integer returns the number of magics in the
 %      list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -335,14 +338,11 @@ static int MagicInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
-  unsigned long *number_aliases)
+  unsigned long *number_aliases,ExceptionInfo *exception)
 {
   const MagicInfo
     **aliases;
 
-  ExceptionInfo
-    exception;
-
   register const MagicInfo
     *p;
 
@@ -353,12 +353,10 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
     Allocate magic list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_aliases != (unsigned long *) NULL);
   *number_aliases=0;
-  GetExceptionInfo(&exception);
-  p=GetMagicInfo((unsigned char *) "*",0,&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetMagicInfo((const unsigned char *) "*",0,exception);
   if (p == (const MagicInfo *) NULL)
     return((const MagicInfo **) NULL);
   aliases=(const MagicInfo **) AcquireMagickMemory((size_t)
@@ -370,15 +368,15 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
   */
   AcquireSemaphoreInfo(&magic_semaphore);
   ResetLinkedListIterator(magic_list);
-  p=(const MagicInfo *) GetNextElementInLinkedList(magic_list);
+  p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   for (i=0; p != (const MagicInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
       aliases[i++]=p;
-    p=(const MagicInfo *) GetNextElementInLinkedList(magic_list);
+    p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   }
-  RelinquishSemaphoreInfo(&magic_semaphore);
+  RelinquishSemaphoreInfo(magic_semaphore);
   qsort((void *) aliases,(size_t) i,sizeof(*aliases),MagicInfoCompare);
   aliases[i]=(MagicInfo *) NULL;
   *number_aliases=(unsigned long) i;
@@ -401,7 +399,8 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
 %
 %  The format of the GetMagicList function is:
 %
-%      char **GetMagicList(const char *pattern,unsigned long *number_aliases)
+%      char **GetMagicList(const char *pattern,unsigned long *number_aliases,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -410,6 +409,7 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
 %    o number_aliases:  This integer returns the number of image format aliases
 %      in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -419,12 +419,12 @@ extern "C" {
 
 static int MagicCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     *p,
     *q;
 
-  p=(char *) x;
-  q=(char *) y;
+  p=(const char *) x;
+  q=(const char *) y;
   return(LocaleCompare(p,q));
 }
 
@@ -433,14 +433,11 @@ static int MagicCompare(const void *x,const void *y)
 #endif
 
 MagickExport char **GetMagicList(const char *pattern,
-  unsigned long *number_aliases)
+  unsigned long *number_aliases,ExceptionInfo *exception)
 {
   char
     **aliases;
 
-  ExceptionInfo
-    exception;
-
   register const MagicInfo
     *p;
 
@@ -451,12 +448,10 @@ MagickExport char **GetMagicList(const char *pattern,
     Allocate configure list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_aliases != (unsigned long *) NULL);
   *number_aliases=0;
-  GetExceptionInfo(&exception);
-  p=GetMagicInfo((unsigned char *) "*",0,&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetMagicInfo((const unsigned char *) "*",0,exception);
   if (p == (const MagicInfo *) NULL)
     return((char **) NULL);
   aliases=(char **) AcquireMagickMemory((size_t)
@@ -465,15 +460,15 @@ MagickExport char **GetMagicList(const char *pattern,
     return((char **) NULL);
   AcquireSemaphoreInfo(&magic_semaphore);
   ResetLinkedListIterator(magic_list);
-  p=(const MagicInfo *) GetNextElementInLinkedList(magic_list);
+  p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   for (i=0; p != (const MagicInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
-      aliases[i++]=AcquireString(p->name);
-    p=(const MagicInfo *) GetNextElementInLinkedList(magic_list);
+      aliases[i++]=ConstantString(AcquireString(p->name));
+    p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   }
-  RelinquishSemaphoreInfo(&magic_semaphore);
+  RelinquishSemaphoreInfo(magic_semaphore);
   qsort((void *) aliases,(size_t) i,sizeof(*aliases),MagicCompare);
   aliases[i]=(char *) NULL;
   *number_aliases=(unsigned long) i;
@@ -504,7 +499,7 @@ MagickExport char **GetMagicList(const char *pattern,
 */
 MagickExport const char *GetMagicName(const MagicInfo *magic_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magic_info != (MagicInfo *) NULL);
   assert(magic_info->signature == MagickSignature);
   return(magic_info->name);
@@ -534,10 +529,6 @@ MagickExport const char *GetMagicName(const MagicInfo *magic_info)
 */
 static MagickBooleanType InitializeMagicList(ExceptionInfo *exception)
 {
-  MagickBooleanType
-    active;
-
-  active=MagickFalse;
   if ((magic_list == (LinkedListInfo *) NULL) &&
       (instantiate_magic == MagickFalse))
     {
@@ -545,12 +536,10 @@ static MagickBooleanType InitializeMagicList(ExceptionInfo *exception)
       if ((magic_list == (LinkedListInfo *) NULL) &&
           (instantiate_magic == MagickFalse))
         {
+          (void) LoadMagicLists(MagicFilename,exception);
           instantiate_magic=MagickTrue;
-          active=MagickTrue;
         }
-      RelinquishSemaphoreInfo(&magic_semaphore);
-      if (active == MagickTrue)
-        (void) LoadMagicLists(MagicFilename,exception);
+      RelinquishSemaphoreInfo(magic_semaphore);
     }
   return(magic_list != (LinkedListInfo *) NULL ? MagickTrue : MagickFalse);
 }
@@ -600,7 +589,7 @@ MagickExport MagickBooleanType ListMagicInfo(FILE *file,
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  magic_info=GetMagicInfoList("*",&number_aliases);
+  magic_info=GetMagicInfoList("*",&number_aliases,exception);
   if (magic_info == (const MagicInfo **) NULL)
     return(MagickFalse);
   j=0;
@@ -695,23 +684,32 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
         }
     }
   status=MagickTrue;
-  token=AcquireString((char *) xml);
+  token=AcquireString(xml);
   for (q=(char *) xml; *q != '\0'; )
   {
     /*
       Interpret XML.
     */
-    GetToken(q,&q,token);
+    GetMagickToken(q,&q,token);
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+      {
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -722,15 +720,15 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (*token != '=')
             continue;
-          GetToken(q,&q,token);
+          GetMagickToken(q,&q,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
               if (depth > 200)
                 (void) ThrowMagickException(exception,GetMagickModule(),
-                  ConfigureError,"IncludeElementNestedTooDeeply",token);
+                  ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
               else
                 {
                   char
@@ -742,7 +740,7 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
                   (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
+                  xml=FileToString(path,~0,exception);
                   status|=LoadMagicList(xml,path,depth+1,exception);
                   xml=(char *) RelinquishMagickMemory(xml);
                 }
@@ -760,7 +758,7 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
           ThrowMagickFatalException(ResourceLimitFatalError,
             "MemoryAllocationFailed",filename);
         (void) ResetMagickMemory(magic_info,0,sizeof(*magic_info));
-        magic_info->path=AcquireString(filename);
+        magic_info->path=ConstantString(AcquireString(filename));
         magic_info->signature=MagickSignature;
         continue;
       }
@@ -768,17 +766,18 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
       continue;
     if (LocaleCompare(keyword,"/>") == 0)
       {
-        status=AppendElementToLinkedList(magic_list,magic_info);
+        status=AppendValueToLinkedList(magic_list,magic_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
-            ResourceLimitError,"MemoryAllocationFailed",magic_info->name);
+            ResourceLimitError,"MemoryAllocationFailed","`%s'",
+            magic_info->name);
         magic_info=(MagicInfo *) NULL;
       }
-    GetToken(q,(char **) NULL,token);
+    GetMagickToken(q,(char **) NULL,token);
     if (*token != '=')
       continue;
-    GetToken(q,&q,token);
-    GetToken(q,&q,token);
+    GetMagickToken(q,&q,token);
+    GetMagickToken(q,&q,token);
     switch (*keyword)
     {
       case 'N':
@@ -786,7 +785,7 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"name") == 0)
           {
-            magic_info->name=AcquireString(token);
+            magic_info->name=ConstantString(AcquireString(token));
             break;
           }
         break;
@@ -796,7 +795,7 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"offset") == 0)
           {
-            magic_info->offset=(ssize_t) atol(token);
+            magic_info->offset=(MagickOffsetType) atol(token);
             break;
           }
         break;
@@ -905,6 +904,12 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
 static MagickBooleanType LoadMagicLists(const char *filename,
   ExceptionInfo *exception)
 {
+#if defined(UseEmbeddableMagick)
+  return(LoadMagicList(MagicMap,"built-in",0,exception));
+#else
+  char
+    path[MaxTextExtent];
+
   const StringInfo
     *option;
 
@@ -915,17 +920,24 @@ static MagickBooleanType LoadMagicLists(const char *filename,
     status;
 
   status=MagickFalse;
+  *path='\0';
   options=GetConfigureOptions(filename,exception);
-  option=(const StringInfo *) GetNextElementInLinkedList(options);
+  option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
+    (void) CopyMagickString(path,option->path,MaxTextExtent);
     status|=LoadMagicList((const char *) option->datum,option->path,0,
       exception);
-    option=(const StringInfo *) GetNextElementInLinkedList(options);
+    option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
-  DestroyConfigureOptions(options);
+  options=DestroyConfigureOptions(options);
   if ((magic_list == (LinkedListInfo *) NULL) || 
       (IsLinkedListEmpty(magic_list) != MagickFalse))
-    status|=LoadMagicList(MagicMap,"built-in",0,exception);
+    {
+      (void) ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
+        "UnableToOpenConfigureFile","`%s'",path);
+      status|=LoadMagicList(MagicMap,"built-in",0,exception);
+    }
   return(status != 0 ? MagickTrue : MagickFalse);
+#endif
 }
