@@ -17,13 +17,13 @@
 %                                 July 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -43,6 +43,11 @@
 %  sizes.  It is based on the paper, "Fast Memory Allocation using Lazy Fits."
 %  written by Yoo C. Chung.
 %
+%  By default, ANSI memory methods are called (e.g. malloc()).  Use the
+%  custom memory allocator by defining UseEmbeddableMagick.  The custom memory
+%  allocator has the advantage of using memory-mapped heap allocation which
+%  permits the virtual memory to be returned to the system in all cases, unlike
+%  the ANSI memory methods that typically only memory-map large allocations.
 %
 */
 
@@ -51,7 +56,9 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/blob-private.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/memory_.h"
 #include "magick/semaphore.h"
 #include "magick/string_.h"
@@ -59,15 +66,13 @@
 /*
   Define declarations.
 */
-#define UseANSICStandard  1
-
 #define BlockFooter(block,size) \
   ((size_t *) ((char *) (block)+(size)-2*sizeof(size_t)))
 #define BlockHeader(block)  ((size_t *) (block)-1)
 #define BlockSize  4096
 #define BlockThreshold  1024
 #define MaxBlockExponent  16
-#define MaxBlocks ((BlockThreshold/(2*sizeof(double)))+MaxBlockExponent+1)
+#define MaxBlocks ((BlockThreshold/(4*sizeof(size_t)))+MaxBlockExponent+1)
 #define MaxSegments  1024
 #define MemoryGuard  ((0xdeadbeef << 31)+0xdeafdeed)
 #define NextBlock(block)  ((char *) (block)+SizeOfBlock(block))
@@ -118,6 +123,7 @@ typedef struct _MemoryInfo
 /*
   Global declarations.
 */
+#if defined(UseEmbeddableMagick)
 static MemoryInfo
   memory_info;
 
@@ -130,12 +136,11 @@ static volatile DataSegmentInfo
 /*
   Forward declarations.
 */
-#if !defined(UseANSICStandard)
 static MagickBooleanType
   ExpandHeap(size_t);
 #endif
 
-#if !defined(UseANSICStandard)
+#if defined(UseEmbeddableMagick)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -169,22 +174,22 @@ static inline size_t AllocationPolicy(size_t size)
     The linear distribution.
   */
   assert(size != 0);
-  assert(size % (2*sizeof(double)) == 0);
+  assert(size % (4*sizeof(size_t)) == 0);
   if (size <= BlockThreshold)
-    return(size/(2*sizeof(double)));
+    return(size/(4*sizeof(size_t)));
   /*
     Check for the largest block size.
   */
-  if (size > (size_t) (BlockThreshold*(1L << MaxBlockExponent)))
+  if (size > (size_t) (BlockThreshold*(1L << (MaxBlockExponent-1L))))
     return(MaxBlocks-1L);
   /*
     Otherwise use a power of two distribution.
   */
-  blocksize=BlockThreshold/(2*sizeof(double));
+  blocksize=BlockThreshold/(4*sizeof(size_t));
   for ( ; size > BlockThreshold; size/=2)
     blocksize++;
-  assert(blocksize > (BlockThreshold/(2*sizeof(double))));
-  assert(blocksize < (MaxBlocks-1));
+  assert(blocksize > (BlockThreshold/(4*sizeof(size_t))));
+  assert(blocksize < (MaxBlocks-1L));
   return(blocksize);
 }
 
@@ -242,7 +247,7 @@ static void *AcquireBlock(size_t size)
   /*
     Find free block.
   */
-  size=(size+sizeof(size_t)+3*sizeof(double)-1) & -(2*sizeof(double));
+  size=(size_t) (size+sizeof(size_t)+6*sizeof(size_t)-1) & -(4*sizeof(size_t));
   i=AllocationPolicy(size);
   block=memory_info.blocks[i];
   while ((block != (void *) NULL) && (SizeOfBlock(block) < size))
@@ -312,7 +317,7 @@ MagickExport void *AcquireMagickMemory(const size_t size)
   register void
     *memory;
 
-#if defined(UseANSICStandard)
+#if !defined(UseEmbeddableMagick)
   memory=malloc(size == 0 ? 1UL : size);
 #else
   if (free_segments == (DataSegmentInfo *) NULL)
@@ -320,11 +325,11 @@ MagickExport void *AcquireMagickMemory(const size_t size)
       AcquireSemaphoreInfo(&memory_semaphore);
       if (free_segments == (DataSegmentInfo *) NULL)
         {
-          register int32_t
+          register long
             i;
 
-          assert(sizeof(double) > (size_t) (~SizeMask));
-          (void) ResetMagickMemory(&memory_info,0,sizeof(MemoryInfo));
+          assert(2*sizeof(size_t) > (size_t) (~SizeMask));
+          (void) ResetMagickMemory(&memory_info,0,sizeof(memory_info));
           memory_info.allocation=SegmentSize;
           memory_info.blocks[MaxBlocks]=(void *) (-1);
           for (i=0; i < MaxSegments; i++)
@@ -337,22 +342,16 @@ MagickExport void *AcquireMagickMemory(const size_t size)
           }
           free_segments=(&memory_info.segment_pool[0]);
         }
-      RelinquishSemaphoreInfo(&memory_semaphore);
+      RelinquishSemaphoreInfo(memory_semaphore);
     }
   AcquireSemaphoreInfo(&memory_semaphore);
   memory=AcquireBlock(size == 0 ? 1UL : size);
   if (memory == (void *) NULL)
     {
-      if (ExpandHeap(size == 0 ? 1UL : size) == MagickFalse)
-        {
-          RelinquishSemaphoreInfo(&memory_semaphore);
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",(char *) NULL);
-        }
-      memory=AcquireBlock(size == 0 ? 1UL : size);
-      assert(memory != (void *) NULL);
+      if (ExpandHeap(size == 0 ? 1UL : size) != MagickFalse)
+        memory=AcquireBlock(size == 0 ? 1UL : size);
     }
-  RelinquishSemaphoreInfo(&memory_semaphore);
+  RelinquishSemaphoreInfo(memory_semaphore);
 #endif
   return(memory);
 }
@@ -427,11 +426,12 @@ MagickExport void *CopyMagickMemory(void *destination,const void *source,
 */
 MagickExport void DestroyMagickMemory(void)
 {
+#if defined(UseEmbeddableMagick)
   register long
     i;
 
   AcquireSemaphoreInfo(&memory_semaphore);
-  RelinquishSemaphoreInfo(&memory_semaphore);
+  RelinquishSemaphoreInfo(memory_semaphore);
   for (i=0; i < (long) memory_info.number_segments; i++)
     if (memory_info.segments[i]->mapped == MagickFalse)
       free(memory_info.segments[i]->allocation);
@@ -439,11 +439,12 @@ MagickExport void DestroyMagickMemory(void)
       (void) UnmapBlob(memory_info.segments[i]->allocation,
         memory_info.segments[i]->length);
   free_segments=(DataSegmentInfo *) NULL;
-  (void) ResetMagickMemory(&memory_info,0,sizeof(MemoryInfo));
-  DestroySemaphoreInfo(&memory_semaphore);
+  (void) ResetMagickMemory(&memory_info,0,sizeof(memory_info));
+  memory_semaphore=DestroySemaphoreInfo(memory_semaphore);
+#endif
 }
 
-#if !defined(UseANSICStandard)
+#if defined(UseEmbeddableMagick)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -455,12 +456,12 @@ MagickExport void DestroyMagickMemory(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ExpandHeap() get more memory from the system.  It returns MagickTrue on success
-%  otherwise MagickFalse.
+%  ExpandHeap() get more memory from the system.  It returns MagickTrue on
+%  success otherwise MagickFalse.
 %
 %  The format of the ExpandHeap method is:
 %
-%      MagickBooleanType ExpandHeap(const size_t size)
+%      MagickBooleanType ExpandHeap(size_t size)
 %
 %  A description of each parameter follows:
 %
@@ -475,7 +476,7 @@ static MagickBooleanType ExpandHeap(size_t size)
   MagickBooleanType
     mapped;
 
-  register int32_t
+  register long
     i;
 
   register void
@@ -487,10 +488,10 @@ static MagickBooleanType ExpandHeap(size_t size)
   void
     *segment;
 
-  blocksize=((size+6*sizeof(double))+SegmentSize-1) & -SegmentSize;
+  blocksize=((size+12*sizeof(size_t))+SegmentSize-1) & -SegmentSize;
   assert(memory_info.number_segments < MaxSegments);
   segment=MapBlob(-1,IOMode,0,blocksize);
-  mapped=segment != (void *) NULL;
+  mapped=segment != (void *) NULL ? MagickTrue : MagickFalse;
   if (segment == (void *) NULL)
     segment=(void *) malloc(blocksize);
   if (segment == (void *) NULL)
@@ -501,19 +502,19 @@ static MagickBooleanType ExpandHeap(size_t size)
   segment_info->length=blocksize;
   segment_info->allocation=segment;
   segment_info->bound=(char *) segment+blocksize;
-  i=memory_info.number_segments-1;
+  i=(long) memory_info.number_segments-1;
   for ( ; (i >= 0) && (memory_info.segments[i]->allocation > segment); i--)
     memory_info.segments[i+1]=memory_info.segments[i];
   memory_info.segments[i+1]=segment_info;
   memory_info.number_segments++;
-  size=blocksize-6*sizeof(double);
-  block=(char *) segment_info->allocation+2*sizeof(double);
+  size=blocksize-12*sizeof(size_t);
+  block=(char *) segment_info->allocation+4*sizeof(size_t);
   *BlockHeader(block)=size | PreviousBlockBit;
   *BlockFooter(block,size)=size;
   InsertFreeBlock(block,AllocationPolicy(size));
   block=NextBlock(block);
   assert(block < segment_info->bound);
-  *BlockHeader(block)=sizeof(double);
+  *BlockHeader(block)=2*sizeof(size_t);
   *BlockHeader(NextBlock(block))=PreviousBlockBit;
   return(MagickTrue);
 }
@@ -546,10 +547,10 @@ MagickExport void *RelinquishMagickMemory(void *memory)
 {
   if (memory == (void *) NULL)
     return((void *) NULL);
-#if defined(UseANSICStandard)
+#if !defined(UseEmbeddableMagick)
   free(memory);
 #else
-  assert((SizeOfBlock(memory) % (2*sizeof(double))) == 0);
+  assert((SizeOfBlock(memory) % (4*sizeof(size_t))) == 0);
   assert((*BlockHeader(NextBlock(memory)) & PreviousBlockBit) != 0);
   AcquireSemaphoreInfo(&memory_semaphore);
   if ((*BlockHeader(memory) & PreviousBlockBit) == 0)
@@ -582,7 +583,7 @@ MagickExport void *RelinquishMagickMemory(void *memory)
   *BlockFooter(memory,SizeOfBlock(memory))=SizeOfBlock(memory);
   *BlockHeader(NextBlock(memory))&=(~PreviousBlockBit);
   InsertFreeBlock(memory,AllocationPolicy(SizeOfBlock(memory)));
-  RelinquishSemaphoreInfo(&memory_semaphore);
+  RelinquishSemaphoreInfo(memory_semaphore);
 #endif
   return((void *) NULL);
 }
@@ -647,7 +648,7 @@ MagickExport void *ResetMagickMemory(void *memory,int byte,const size_t size)
 %
 */
 
-#if !defined(UseANSICStandard)
+#if defined(UseEmbeddableMagick)
 static inline void *ResizeBlock(void *block,size_t size)
 {
   register void
@@ -674,7 +675,7 @@ MagickExport void *ResizeMagickMemory(void *memory,const size_t size)
 
   if (memory == (void *) NULL)
     return(AcquireMagickMemory(size));
-#if defined(UseANSICStandard)
+#if !defined(UseEmbeddableMagick)
   block=realloc(memory,size == 0 ? 1UL : size);
 #else
   AcquireSemaphoreInfo(&memory_semaphore);
@@ -683,15 +684,15 @@ MagickExport void *ResizeMagickMemory(void *memory,const size_t size)
     {
       if (ExpandHeap(size == 0 ? 1UL : size) == MagickFalse)
         {
-          RelinquishSemaphoreInfo(&memory_semaphore);
+          RelinquishSemaphoreInfo(memory_semaphore);
           memory=RelinquishMagickMemory(memory);
           ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",(char *) NULL);
+            "MemoryAllocationFailed",strerror(errno));
         }
       block=ResizeBlock(memory,size == 0 ? 1UL : size);
       assert(block != (void *) NULL);
     }
-  RelinquishSemaphoreInfo(&memory_semaphore);
+  RelinquishSemaphoreInfo(memory_semaphore);
 #endif
   if (block == (void *) NULL)
     memory=RelinquishMagickMemory(memory);
