@@ -1,5 +1,19 @@
 /*
-  ImageMagick Magick API Method Prefix.
+  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  dedicated to making software imaging solutions freely available.
+  
+  You may not use this file except in compliance with the License.
+  obtain a copy of the License at
+  
+    http://www.imagemagick.org/script/license.php
+  
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
+
+  ImageMagick API methods prefix.
 
   nm .libs/libMagick.a | grep ' T ' | \
     awk '{ printf("#define %s  PrependMagickMethod(%s)\n", $3, $3); }' | \
@@ -36,6 +50,7 @@ extern "C" {
 #define AcquireUniqueFilename  PrependMagickMethod(AcquireUniqueFilename)
 #define AcquireUniqueFileResource  PrependMagickMethod(AcquireUniqueFileResource)
 #define AdaptiveThresholdImage  PrependMagickMethod(AdaptiveThresholdImage)
+#define AddValueToSplayTree  PrependMagickMethod(AddValueToSplayTree)
 #define AddNoiseImage  PrependMagickMethod(AddNoiseImage)
 #define AffineTransformImage  PrependMagickMethod(AffineTransformImage)
 #define AllocateImageColormap  PrependMagickMethod(AllocateImageColormap)
@@ -47,7 +62,7 @@ extern "C" {
 #define AnimateImageCommand  PrependMagickMethod(AnimateImageCommand)
 #define AnimateImages  PrependMagickMethod(AnimateImages)
 #define AnnotateImage  PrependMagickMethod(AnnotateImage)
-#define AppendElementToLinkedList  PrependMagickMethod(AppendElementToLinkedList)
+#define AppendValueToLinkedList  PrependMagickMethod(AppendValueToLinkedList)
 #define AppendImageFormat  PrependMagickMethod(AppendImageFormat)
 #define AppendImages  PrependMagickMethod(AppendImages)
 #define AppendImageToList  PrependMagickMethod(AppendImageToList)
@@ -96,9 +111,13 @@ extern "C" {
 #define ColorFloodfillImage  PrependMagickMethod(ColorFloodfillImage)
 #define ColorizeImage  PrependMagickMethod(ColorizeImage)
 #define CombineImages  PrependMagickMethod(CombineImages)
+#define CompareHashmapStringInfo  PrependMagickMethod(CompareHashmapStringInfo)
+#define CompareHashmapString  PrependMagickMethod(CompareHashmapString)
 #define CompareImageChannels  PrependMagickMethod(CompareImageChannels)
 #define CompareImageCommand  PrependMagickMethod(CompareImageCommand)
 #define CompareImages  PrependMagickMethod(CompareImages)
+#define CompareSplayTreeStringInfo  PrependMagickMethod(CompareSplayTreeStringInfo)
+#define CompareSplayTreeString  PrependMagickMethod(CompareSplayTreeString)
 #define CompareStringInfo  PrependMagickMethod(CompareStringInfo)
 #define CompositeImageCommand  PrependMagickMethod(CompositeImageCommand)
 #define CompositeImage  PrependMagickMethod(CompositeImage)
@@ -108,6 +127,7 @@ extern "C" {
 #define ConcatenateString  PrependMagickMethod(ConcatenateString)
 #define ConjureImageCommand  PrependMagickMethod(ConjureImageCommand)
 #define ConsolidateCMYKImages  PrependMagickMethod(ConsolidateCMYKImages)
+#define ConstantString  PrependMagickMethod(ConstantString)
 #define ConstituteImage  PrependMagickMethod(ConstituteImage)
 #define ContinueTimer  PrependMagickMethod(ContinueTimer)
 #define ContrastImage  PrependMagickMethod(ContrastImage)
@@ -127,7 +147,6 @@ extern "C" {
 #define DeleteMagickRegistry  PrependMagickMethod(DeleteMagickRegistry)
 #define DescribeImage  PrependMagickMethod(DescribeImage)
 #define DespeckleImage  PrependMagickMethod(DespeckleImage)
-#define DestroyBlobInfo  PrependMagickMethod(DestroyBlobInfo)
 #define DestroyBlob  PrependMagickMethod(DestroyBlob)
 #define DestroyCacheInfo  PrependMagickMethod(DestroyCacheInfo)
 #define DestroyCacheNexus  PrependMagickMethod(DestroyCacheNexus)
@@ -163,6 +182,7 @@ extern "C" {
 #define DestroyRandomReservoir  PrependMagickMethod(DestroyRandomReservoir)
 #define DestroySemaphoreInfo  PrependMagickMethod(DestroySemaphoreInfo)
 #define DestroySemaphore  PrependMagickMethod(DestroySemaphore)
+#define DestroySplayTree  PrependMagickMethod(DestroySplayTree)
 #define DestroyStringInfo  PrependMagickMethod(DestroyStringInfo)
 #define DestroyStringList  PrependMagickMethod(DestroyStringList)
 #define DestroyString  PrependMagickMethod(DestroyString)
@@ -192,6 +212,7 @@ extern "C" {
 #define ExpandFilename  PrependMagickMethod(ExpandFilename)
 #define ExpandFilenames  PrependMagickMethod(ExpandFilenames)
 #define ExportImagePixels  PrependMagickMethod(ExportImagePixels)
+#define ExportQuantumPixels  PrependMagickMethod(ExportQuantumPixels)
 #define FileToBlob  PrependMagickMethod(FileToBlob)
 #define FileToStringInfo  PrependMagickMethod(FileToStringInfo)
 #define FileToString  PrependMagickMethod(FileToString)
@@ -250,8 +271,8 @@ extern "C" {
 #define GetDelegateMode  PrependMagickMethod(GetDelegateMode)
 #define GetDrawInfo  PrependMagickMethod(GetDrawInfo)
 #define GetElapsedTime  PrependMagickMethod(GetElapsedTime)
-#define GetElementFromLinkedList  PrependMagickMethod(GetElementFromLinkedList)
-#define GetEntryFromHashmap  PrependMagickMethod(GetEntryFromHashmap)
+#define GetValueFromLinkedList  PrependMagickMethod(GetValueFromLinkedList)
+#define GetValueFromHashmap  PrependMagickMethod(GetValueFromHashmap)
 #define GetExceptionInfo  PrependMagickMethod(GetExceptionInfo)
 #define GetExecutionPath  PrependMagickMethod(GetExecutionPath)
 #define GetFirstImageInList  PrependMagickMethod(GetFirstImageInList)
@@ -259,11 +280,13 @@ extern "C" {
 #define GetImageAttribute  PrependMagickMethod(GetImageAttribute)
 #define GetImageBoundingBox  PrependMagickMethod(GetImageBoundingBox)
 #define GetImageChannelDepth  PrependMagickMethod(GetImageChannelDepth)
+#define GetImageChannelDistortion  PrependMagickMethod(GetImageChannelDistortion)
 #define GetImageChannelExtrema  PrependMagickMethod(GetImageChannelExtrema)
 #define GetImageChannelMean  PrependMagickMethod(GetImageChannelMean)
 #define GetImageClipMask  PrependMagickMethod(GetImageClipMask)
 #define GetImageClippingPathAttribute  PrependMagickMethod(GetImageClippingPathAttribute)
 #define GetImageDepth  PrependMagickMethod(GetImageDepth)
+#define GetImageDistortion  PrependMagickMethod(GetImageDistortion)
 #define GetImageDynamicThreshold  PrependMagickMethod(GetImageDynamicThreshold)
 #define GetImageException  PrependMagickMethod(GetImageException)
 #define GetImageExtrema  PrependMagickMethod(GetImageExtrema)
@@ -288,7 +311,7 @@ extern "C" {
 #define GetImageType  PrependMagickMethod(GetImageType)
 #define GetImageVirtualPixelMethod  PrependMagickMethod(GetImageVirtualPixelMethod)
 #define GetIndexes  PrependMagickMethod(GetIndexes)
-#define GetLastElementInLinkedList  PrependMagickMethod(GetLastElementInLinkedList)
+#define GetLastValueInLinkedList  PrependMagickMethod(GetLastValueInLinkedList)
 #define GetLastImageInList  PrependMagickMethod(GetLastImageInList)
 #define GetLocaleExceptionMessage  PrependMagickMethod(GetLocaleExceptionMessage)
 #define GetLocaleInfoList  PrependMagickMethod(GetLocaleInfoList)
@@ -309,6 +332,7 @@ extern "C" {
 #define GetMagickDecoder  PrependMagickMethod(GetMagickDecoder)
 #define GetMagickDescription  PrependMagickMethod(GetMagickDescription)
 #define GetMagickEncoder  PrependMagickMethod(GetMagickEncoder)
+#define GetMagickEndianSupport  PrependMagickMethod(GetMagickEndianSupport)
 #define GetMagickGeometry  PrependMagickMethod(GetMagickGeometry)
 #define GetMagickHomeURL  PrependMagickMethod(GetMagickHomeURL)
 #define GetMagickInfoList  PrependMagickMethod(GetMagickInfoList)
@@ -326,19 +350,24 @@ extern "C" {
 #define GetMagickVersion  PrependMagickMethod(GetMagickVersion)
 #define GetMagicList  PrependMagickMethod(GetMagicList)
 #define GetMagicName  PrependMagickMethod(GetMagicName)
+#define GetMonitorHandler  PrependMagickMethod(GetMonitorHandler)
 #define GetMontageInfo  PrependMagickMethod(GetMontageInfo)
-#define GetNextElementInLinkedList  PrependMagickMethod(GetNextElementInLinkedList)
-#define GetNextEntryInHashmap  PrependMagickMethod(GetNextEntryInHashmap)
+#define GetMultilineTypeMetrics  PrependMagickMethod(GetMultilineTypeMetrics)
+#define GetNextValueInLinkedList  PrependMagickMethod(GetNextValueInLinkedList)
+#define GetNextKeyInHashmap  PrependMagickMethod(GetNextKeyInHashmap)
 #define GetNextImageInList  PrependMagickMethod(GetNextImageInList)
 #define GetNextImageOption  PrependMagickMethod(GetNextImageOption)
 #define GetNextImage  PrependMagickMethod(GetNextImage)
 #define GetNextImageProfile  PrependMagickMethod(GetNextImageProfile)
+#define GetNextValueInSplayTree  PrependMagickMethod(GetNextValueInSplayTree)
 #define GetNexusIndexes  PrependMagickMethod(GetNexusIndexes)
 #define GetNexusPixels  PrependMagickMethod(GetNexusPixels)
 #define GetNexus  PrependMagickMethod(GetNexus)
+#define GetValueFromSplayTree  PrependMagickMethod(GetValueFromSplayTree)
 #define GetNumberColors  PrependMagickMethod(GetNumberColors)
 #define GetNumberOfElementsInLinkedList  PrependMagickMethod(GetNumberOfElementsInLinkedList)
 #define GetNumberOfEntriesInHashmap  PrependMagickMethod(GetNumberOfEntriesInHashmap)
+#define GetNumberOfNodesInSplayTree  PrependMagickMethod(GetNumberOfNodesInSplayTree)
 #define GetNumberScenes  PrependMagickMethod(GetNumberScenes)
 #define GetOnePixel  PrependMagickMethod(GetOnePixel)
 #define GetOptimalKernelWidth1D  PrependMagickMethod(GetOptimalKernelWidth1D)
@@ -355,7 +384,7 @@ extern "C" {
 #define GetRandomValue  PrependMagickMethod(GetRandomValue)
 #define GetSignatureInfo  PrependMagickMethod(GetSignatureInfo)
 #define GetTimerInfo  PrependMagickMethod(GetTimerInfo)
-#define GetToken  PrependMagickMethod(GetToken)
+#define GetMagickToken  PrependMagickMethod(GetMagickToken)
 #define GetTypeInfoByFamily  PrependMagickMethod(GetTypeInfoByFamily)
 #define GetTypeInfoList  PrependMagickMethod(GetTypeInfoList)
 #define GetTypeInfo  PrependMagickMethod(GetTypeInfo)
@@ -375,19 +404,22 @@ extern "C" {
 #define Hull  PrependMagickMethod(Hull)
 #define HWBTransform  PrependMagickMethod(HWBTransform)
 #define IdentifyImageCommand  PrependMagickMethod(IdentifyImageCommand)
+#define IdentifyImage  PrependMagickMethod(IdentifyImage)
 #define IdentityAffine  PrependMagickMethod(IdentityAffine)
 #define ImageListToArray  PrependMagickMethod(ImageListToArray)
+#define ImagesToBlob  PrependMagickMethod(ImagesToBlob)
 #define ImageToBlob  PrependMagickMethod(ImageToBlob)
 #define ImageToFile  PrependMagickMethod(ImageToFile)
 #define ImplodeImage  PrependMagickMethod(ImplodeImage)
 #define ImportImageCommand  PrependMagickMethod(ImportImageCommand)
 #define ImportImagePixels  PrependMagickMethod(ImportImagePixels)
+#define ImportQuantumPixels  PrependMagickMethod(ImportQuantumPixels)
 #define InheritException  PrependMagickMethod(InheritException)
 #define InitializeMagick  PrependMagickMethod(InitializeMagick)
 #define InitializeMagickResources  PrependMagickMethod(InitializeMagickResources)
 #define InitializeSemaphore  PrependMagickMethod(InitializeSemaphore)
-#define InsertElementInLinkedList  PrependMagickMethod(InsertElementInLinkedList)
-#define InsertElementInSortedLinkedList  PrependMagickMethod(InsertElementInSortedLinkedList)
+#define InsertValueInLinkedList  PrependMagickMethod(InsertValueInLinkedList)
+#define InsertValueInSortedLinkedList  PrependMagickMethod(InsertValueInSortedLinkedList)
 #define InsertImageInList  PrependMagickMethod(InsertImageInList)
 #define InterpolateColor  PrependMagickMethod(InterpolateColor)
 #define InvokeDelegate  PrependMagickMethod(InvokeDelegate)
@@ -396,7 +428,6 @@ extern "C" {
 #define IsBlobExempt  PrependMagickMethod(IsBlobExempt)
 #define IsBlobSeekable  PrependMagickMethod(IsBlobSeekable)
 #define IsBlobTemporary  PrependMagickMethod(IsBlobTemporary)
-#define IsMagickColorSimilar  PrependMagickMethod(IsMagickColorSimilar)
 #define IsEventLogging  PrependMagickMethod(IsEventLogging)
 #define IsGeometry  PrependMagickMethod(IsGeometry)
 #define IsGlob  PrependMagickMethod(IsGlob)
@@ -404,6 +435,7 @@ extern "C" {
 #define IsHashmapEmpty  PrependMagickMethod(IsHashmapEmpty)
 #define IsImagesEqual  PrependMagickMethod(IsImagesEqual)
 #define IsLinkedListEmpty  PrependMagickMethod(IsLinkedListEmpty)
+#define IsMagickColorSimilar  PrependMagickMethod(IsMagickColorSimilar)
 #define IsMagickConflict  PrependMagickMethod(IsMagickConflict)
 #define IsMagickInstantiated  PrependMagickMethod(IsMagickInstantiated)
 #define IsMonochromeImage  PrependMagickMethod(IsMonochromeImage)
@@ -453,9 +485,12 @@ extern "C" {
 #define MedianFilterImage  PrependMagickMethod(MedianFilterImage)
 #define MinifyImage  PrependMagickMethod(MinifyImage)
 #define ModifyImage  PrependMagickMethod(ModifyImage)
+#define ModulateHSB  PrependMagickMethod(ModulateHSB)
+#define ModulateHSL  PrependMagickMethod(ModulateHSL)
+#define ModulateHWB  PrependMagickMethod(ModulateHWB)
 #define ModulateImage  PrependMagickMethod(ModulateImage)
-#define Modulate  PrependMagickMethod(Modulate)
 #define MogrifyImageCommand  PrependMagickMethod(MogrifyImageCommand)
+#define MogrifyImageInfo  PrependMagickMethod(MogrifyImageInfo)
 #define MogrifyImageList  PrependMagickMethod(MogrifyImageList)
 #define MogrifyImage  PrependMagickMethod(MogrifyImage)
 #define MogrifyImages  PrependMagickMethod(MogrifyImages)
@@ -472,15 +507,18 @@ extern "C" {
 #define NewHashmap  PrependMagickMethod(NewHashmap)
 #define NewImageList  PrependMagickMethod(NewImageList)
 #define NewLinkedList  PrependMagickMethod(NewLinkedList)
+#define NewMagickImage  PrependMagickMethod(NewMagickImage)
+#define NewSplayTree  PrependMagickMethod(NewSplayTree)
 #define NormalizeImageChannel  PrependMagickMethod(NormalizeImageChannel)
 #define NormalizeImage  PrependMagickMethod(NormalizeImage)
 #define OilPaintImage  PrependMagickMethod(OilPaintImage)
 #define OpaqueImage  PrependMagickMethod(OpaqueImage)
 #define OpenBlob  PrependMagickMethod(OpenBlob)
-#define OpenCache  PrependMagickMethod(OpenCache)
 #define OpenCacheView  PrependMagickMethod(OpenCacheView)
 #define OrderedDitherImage  PrependMagickMethod(OrderedDitherImage)
 #define PackbitsEncodeImage  PrependMagickMethod(PackbitsEncodeImage)
+#define PaintOpaqueImage  PrependMagickMethod(PaintOpaqueImage)
+#define PaintTransparentImage  PrependMagickMethod(PaintTransparentImage)
 #define ParseAbsoluteGeometry  PrependMagickMethod(ParseAbsoluteGeometry)
 #define ParseChannelOption  PrependMagickMethod(ParseChannelOption)
 #define ParseGeometry  PrependMagickMethod(ParseGeometry)
@@ -538,7 +576,6 @@ extern "C" {
 #define RegisterCAPTIONImage  PrependMagickMethod(RegisterCAPTIONImage)
 #define RegisterCINImage  PrependMagickMethod(RegisterCINImage)
 #define RegisterCIPImage  PrependMagickMethod(RegisterCIPImage)
-#define RegisterCLIPBOARDImage  PrependMagickMethod(RegisterCLIPBOARDImage)
 #define RegisterCLIPImage  PrependMagickMethod(RegisterCLIPImage)
 #define RegisterCMYKImage  PrependMagickMethod(RegisterCMYKImage)
 #define RegisterCUTImage  PrependMagickMethod(RegisterCUTImage)
@@ -547,19 +584,15 @@ extern "C" {
 #define RegisterDOTImage  PrependMagickMethod(RegisterDOTImage)
 #define RegisterDPSImage  PrependMagickMethod(RegisterDPSImage)
 #define RegisterDPXImage  PrependMagickMethod(RegisterDPXImage)
-#define RegisterEMFImage  PrependMagickMethod(RegisterEMFImage)
 #define RegisterEPTImage  PrependMagickMethod(RegisterEPTImage)
 #define RegisterFAXImage  PrependMagickMethod(RegisterFAXImage)
 #define RegisterFITSImage  PrependMagickMethod(RegisterFITSImage)
-#define RegisterFPXImage  PrependMagickMethod(RegisterFPXImage)
 #define RegisterGIFImage  PrependMagickMethod(RegisterGIFImage)
 #define RegisterGRADIENTImage  PrependMagickMethod(RegisterGRADIENTImage)
 #define RegisterGRAYImage  PrependMagickMethod(RegisterGRAYImage)
 #define RegisterHISTOGRAMImage  PrependMagickMethod(RegisterHISTOGRAMImage)
 #define RegisterHTMLImage  PrependMagickMethod(RegisterHTMLImage)
 #define RegisterICONImage  PrependMagickMethod(RegisterICONImage)
-#define RegisterJBIGImage  PrependMagickMethod(RegisterJBIGImage)
-#define RegisterJP2Image  PrependMagickMethod(RegisterJP2Image)
 #define RegisterJPEGImage  PrependMagickMethod(RegisterJPEGImage)
 #define RegisterLABELImage  PrependMagickMethod(RegisterLABELImage)
 #define RegisterMAGICKImage  PrependMagickMethod(RegisterMAGICKImage)
@@ -635,6 +668,7 @@ extern "C" {
 #define RelinquishMagickResource  PrependMagickMethod(RelinquishMagickResource)
 #define RelinquishSemaphoreInfo  PrependMagickMethod(RelinquishSemaphoreInfo)
 #define RelinquishUniqueFileResource  PrependMagickMethod(RelinquishUniqueFileResource)
+#define RemoteDisplayCommand  PrependMagickMethod(RemoteDisplayCommand)
 #define RemoveElementByValueFromLinkedList  PrependMagickMethod(RemoveElementByValueFromLinkedList)
 #define RemoveElementFromLinkedList  PrependMagickMethod(RemoveElementFromLinkedList)
 #define RemoveEntryFromHashmap  PrependMagickMethod(RemoveEntryFromHashmap)
@@ -644,12 +678,15 @@ extern "C" {
 #define RemoveImageProfile  PrependMagickMethod(RemoveImageProfile)
 #define RemoveLastElementFromLinkedList  PrependMagickMethod(RemoveLastElementFromLinkedList)
 #define RemoveLastImageFromList  PrependMagickMethod(RemoveLastImageFromList)
+#define RemoveNodeByValueFromSplayTree  PrependMagickMethod(RemoveNodeByValueFromSplayTree)
+#define RemoveNodeFromSplayTree  PrependMagickMethod(RemoveNodeFromSplayTree)
 #define ReplaceImageInList  PrependMagickMethod(ReplaceImageInList)
 #define ResetHashmapIterator  PrependMagickMethod(ResetHashmapIterator)
 #define ResetImageOptionIterator  PrependMagickMethod(ResetImageOptionIterator)
 #define ResetImageProfileIterator  PrependMagickMethod(ResetImageProfileIterator)
 #define ResetLinkedListIterator  PrependMagickMethod(ResetLinkedListIterator)
 #define ResetMagickMemory  PrependMagickMethod(ResetMagickMemory)
+#define ResetSplayTreeIterator  PrependMagickMethod(ResetSplayTreeIterator)
 #define ResetStringInfo  PrependMagickMethod(ResetStringInfo)
 #define ResetTimer  PrependMagickMethod(ResetTimer)
 #define ResizeImage  PrependMagickMethod(ResizeImage)
@@ -675,17 +712,20 @@ extern "C" {
 #define SetGeometryInfo  PrependMagickMethod(SetGeometryInfo)
 #define SetGeometry  PrependMagickMethod(SetGeometry)
 #define SetImageAttribute  PrependMagickMethod(SetImageAttribute)
+#define SetImageBackgroundColor  PrependMagickMethod(SetImageBackgroundColor)
 #define SetImageChannelDepth  PrependMagickMethod(SetImageChannelDepth)
 #define SetImageClipMask  PrependMagickMethod(SetImageClipMask)
 #define SetImageColorspace  PrependMagickMethod(SetImageColorspace)
 #define SetImageDepth  PrependMagickMethod(SetImageDepth)
+#define SetImageExtent  PrependMagickMethod(SetImageExtent)
 #define SetImageInfo  PrependMagickMethod(SetImageInfo)
+#define SetImageInfoProgressMonitor  PrependMagickMethod(SetImageInfoProgressMonitor)
 #define SetImageList  PrependMagickMethod(SetImageList)
 #define SetImageOpacity  PrependMagickMethod(SetImageOpacity)
 #define SetImageOption  PrependMagickMethod(SetImageOption)
 #define SetImagePixels  PrependMagickMethod(SetImagePixels)
-#define SetImage  PrependMagickMethod(SetImage)
 #define SetImageProfile  PrependMagickMethod(SetImageProfile)
+#define SetImageProgressMonitor  PrependMagickMethod(SetImageProgressMonitor)
 #define SetImageType  PrependMagickMethod(SetImageType)
 #define SetImageVirtualPixelMethod  PrependMagickMethod(SetImageVirtualPixelMethod)
 #define SetLogEventMask  PrependMagickMethod(SetLogEventMask)
@@ -700,6 +740,7 @@ extern "C" {
 #define SetStringInfo  PrependMagickMethod(SetStringInfo)
 #define SetWarningHandler  PrependMagickMethod(SetWarningHandler)
 #define ShadeImage  PrependMagickMethod(ShadeImage)
+#define ShadowImage  PrependMagickMethod(ShadowImage)
 #define SharpenImageChannel  PrependMagickMethod(SharpenImageChannel)
 #define SharpenImage  PrependMagickMethod(SharpenImage)
 #define ShaveImage  PrependMagickMethod(ShaveImage)
@@ -752,6 +793,7 @@ extern "C" {
 #define TransformHSL  PrependMagickMethod(TransformHSL)
 #define TransformHWB  PrependMagickMethod(TransformHWB)
 #define TransformImage  PrependMagickMethod(TransformImage)
+#define TransformImages  PrependMagickMethod(TransformImages)
 #define TransformRGBImage  PrependMagickMethod(TransformRGBImage)
 #define TranslateText  PrependMagickMethod(TranslateText)
 #define TransparentImage  PrependMagickMethod(TransparentImage)
@@ -765,7 +807,6 @@ extern "C" {
 #define UnregisterCAPTIONImage  PrependMagickMethod(UnregisterCAPTIONImage)
 #define UnregisterCINImage  PrependMagickMethod(UnregisterCINImage)
 #define UnregisterCIPImage  PrependMagickMethod(UnregisterCIPImage)
-#define UnregisterCLIPBOARDImage  PrependMagickMethod(UnregisterCLIPBOARDImage)
 #define UnregisterCLIPImage  PrependMagickMethod(UnregisterCLIPImage)
 #define UnregisterCMYKImage  PrependMagickMethod(UnregisterCMYKImage)
 #define UnregisterCUTImage  PrependMagickMethod(UnregisterCUTImage)
@@ -774,19 +815,15 @@ extern "C" {
 #define UnregisterDOTImage  PrependMagickMethod(UnregisterDOTImage)
 #define UnregisterDPSImage  PrependMagickMethod(UnregisterDPSImage)
 #define UnregisterDPXImage  PrependMagickMethod(UnregisterDPXImage)
-#define UnregisterEMFImage  PrependMagickMethod(UnregisterEMFImage)
 #define UnregisterEPTImage  PrependMagickMethod(UnregisterEPTImage)
 #define UnregisterFAXImage  PrependMagickMethod(UnregisterFAXImage)
 #define UnregisterFITSImage  PrependMagickMethod(UnregisterFITSImage)
-#define UnregisterFPXImage  PrependMagickMethod(UnregisterFPXImage)
 #define UnregisterGIFImage  PrependMagickMethod(UnregisterGIFImage)
 #define UnregisterGRADIENTImage  PrependMagickMethod(UnregisterGRADIENTImage)
 #define UnregisterGRAYImage  PrependMagickMethod(UnregisterGRAYImage)
 #define UnregisterHISTOGRAMImage  PrependMagickMethod(UnregisterHISTOGRAMImage)
 #define UnregisterHTMLImage  PrependMagickMethod(UnregisterHTMLImage)
 #define UnregisterICONImage  PrependMagickMethod(UnregisterICONImage)
-#define UnregisterJBIGImage  PrependMagickMethod(UnregisterJBIGImage)
-#define UnregisterJP2Image  PrependMagickMethod(UnregisterJP2Image)
 #define UnregisterJPEGImage  PrependMagickMethod(UnregisterJPEGImage)
 #define UnregisterLABELImage  PrependMagickMethod(UnregisterLABELImage)
 #define UnregisterMAGICKImage  PrependMagickMethod(UnregisterMAGICKImage)
@@ -876,8 +913,6 @@ extern "C" {
 #define WriteImage  PrependMagickMethod(WriteImage)
 #define WriteImages  PrependMagickMethod(WriteImages)
 #define WriteStream  PrependMagickMethod(WriteStream)
-#define XAnimateBackgroundImage  PrependMagickMethod(XAnimateBackgroundImage)
-#define XAnimateImages  PrependMagickMethod(XAnimateImages)
 #define XAnnotateImage  PrependMagickMethod(XAnnotateImage)
 #define XBestFont  PrependMagickMethod(XBestFont)
 #define XBestIconSize  PrependMagickMethod(XBestIconSize)
@@ -895,9 +930,7 @@ extern "C" {
 #define XDestroyResourceInfo  PrependMagickMethod(XDestroyResourceInfo)
 #define XDestroyWindowColors  PrependMagickMethod(XDestroyWindowColors)
 #define XDialogWidget  PrependMagickMethod(XDialogWidget)
-#define XDisplayBackgroundImage  PrependMagickMethod(XDisplayBackgroundImage)
 #define XDisplayImageInfo  PrependMagickMethod(XDisplayImageInfo)
-#define XDisplayImage  PrependMagickMethod(XDisplayImage)
 #define XDrawImage  PrependMagickMethod(XDrawImage)
 #define XError  PrependMagickMethod(XError)
 #define XFileBrowserWidget  PrependMagickMethod(XFileBrowserWidget)
@@ -922,16 +955,16 @@ extern "C" {
 #define XInfoWidget  PrependMagickMethod(XInfoWidget)
 #define XInitializeWindows  PrependMagickMethod(XInitializeWindows)
 #define XListBrowserWidget  PrependMagickMethod(XListBrowserWidget)
-#define XMagickMonitor  PrependMagickMethod(XMagickMonitor)
+#define XMagickProgressMonitor  PrependMagickMethod(XMagickProgressMonitor)
 #define XMakeCursor  PrependMagickMethod(XMakeCursor)
 #define XMakeImage  PrependMagickMethod(XMakeImage)
 #define XMakeMagnifyImage  PrependMagickMethod(XMakeMagnifyImage)
 #define XMakeStandardColormap  PrependMagickMethod(XMakeStandardColormap)
 #define XMakeWindow  PrependMagickMethod(XMakeWindow)
 #define XMenuWidget  PrependMagickMethod(XMenuWidget)
-#define XMonitorWidget  PrependMagickMethod(XMonitorWidget)
 #define XNoticeWidget  PrependMagickMethod(XNoticeWidget)
 #define XPreferencesWidget  PrependMagickMethod(XPreferencesWidget)
+#define XProgressMonitorWidget  PrependMagickMethod(XProgressMonitorWidget)
 #define XQueryColorDatabase  PrependMagickMethod(XQueryColorDatabase)
 #define XQueryPosition  PrependMagickMethod(XQueryPosition)
 #define XRefreshWindow  PrependMagickMethod(XRefreshWindow)
