@@ -10,20 +10,20 @@
 %                  M   M   OOO   DDDD    UUU   LLLLL  EEEEE                   %
 %                                                                             %
 %                                                                             %
-%                         ImageMagick Module Methods                           %
+%                         ImageMagick Module Methods                          %
 %                                                                             %
 %                              Software Design                                %
 %                              Bob Friesenhahn                                %
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -45,7 +45,8 @@
 #include "magick/coder.h"
 #include "magick/client.h"
 #include "magick/configure.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/log.h"
 #include "magick/hashmap.h"
 #include "magick/magic.h"
@@ -53,13 +54,11 @@
 #include "magick/memory_.h"
 #include "magick/module.h"
 #include "magick/semaphore.h"
+#include "magick/splay-tree.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
-#if defined(__WINDOWS__)
-# include "magick/nt_feature.h"
-#endif
 #if defined(SupportMagickModules)
 #if defined(HasLTDL)
 #include "ltdl.h"
@@ -87,8 +86,8 @@ typedef void *ModuleHandle;
 static SemaphoreInfo
   *module_semaphore = (SemaphoreInfo *) NULL;
 
-static LinkedListInfo
-  *module_list = (LinkedListInfo *) NULL;
+static SplayTreeInfo
+  *module_list = (SplayTreeInfo *) NULL;
 
 static volatile MagickBooleanType
   instantiate_module = MagickFalse;
@@ -102,7 +101,7 @@ static const ModuleInfo
 static MagickBooleanType
   GetMagickModulePath(const char *,MagickModuleType,char *,ExceptionInfo *),
   InitializeModuleList(ExceptionInfo *),
-  UnloadModule(const ModuleInfo *,ExceptionInfo *);
+  UnregisterModule(const ModuleInfo *,ExceptionInfo *);
 
 static void
   TagToCoderModuleName(const char *,char *),
@@ -128,28 +127,6 @@ static void
 %      void DestroyMagickList(void)
 %
 */
-
-static void *DestroyModuleElement(void *module_info)
-{
-  ExceptionInfo
-    exception;
-
-  register ModuleInfo
-    *p;
-
-  GetExceptionInfo(&exception);
-  p=(ModuleInfo *) module_info;
-  if (UnloadModule(p,&exception) == MagickFalse)
-    CatchException(&exception);
-  if (p->tag != (char *) NULL)
-    p->tag=(char *) RelinquishMagickMemory(p->tag);
-  if (p->path != (char *) NULL)
-    p->path=(char *) RelinquishMagickMemory(p->path);
-  p=(ModuleInfo *) RelinquishMagickMemory(p);
-  DestroyExceptionInfo(&exception);
-  return((void *) NULL);
-}
-
 MagickExport void DestroyModuleList(void)
 {
   /*
@@ -157,15 +134,14 @@ MagickExport void DestroyModuleList(void)
   */
   AcquireSemaphoreInfo(&module_semaphore);
 #if defined(SupportMagickModules)
-  if (module_list != (LinkedListInfo *) NULL)
-    DestroyLinkedList(module_list,DestroyModuleElement);
-  module_list=(LinkedListInfo *) NULL;
+  if (module_list != (SplayTreeInfo *) NULL)
+    module_list=DestroySplayTree(module_list);
   if (instantiate_module != MagickFalse)
     (void) lt_dlexit();
 #endif
   instantiate_module=MagickFalse;
-  RelinquishSemaphoreInfo(&module_semaphore);
-  DestroySemaphoreInfo(&module_semaphore);
+  RelinquishSemaphoreInfo(module_semaphore);
+  module_semaphore=DestroySemaphoreInfo(module_semaphore);
 }
 
 /*
@@ -220,7 +196,7 @@ MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
   assert(image != (Image **) NULL);
   assert((*image)->signature == MagickSignature);
   if ((*image)->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*image)->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",(*image)->filename);
 #if !defined(BuildMagickModules)
   status=ExecuteStaticModuleProcess(tag,image,argc,argv);
   if (status != MagickFalse)
@@ -238,7 +214,7 @@ MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
   if (handle == (ModuleHandle) NULL)
     {
       (void) ThrowMagickException(&(*image)->exception,GetMagickModule(),
-        ModuleError,"UnableToLoadModule",name,lt_dlerror());
+        ModuleError,"UnableToLoadModule","`%s': %s",name,lt_dlerror());
       return(status);
     }
   /*
@@ -307,12 +283,12 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
   register const ModuleInfo
     *p;
 
-  if ((module_list == (LinkedListInfo *) NULL) ||
+  if ((module_list == (SplayTreeInfo *) NULL) ||
       (instantiate_module == MagickFalse))
     if (InitializeModuleList(exception) == MagickFalse)
       return((const ModuleInfo *) NULL);
-  if ((module_list == (LinkedListInfo *) NULL) ||
-      (IsLinkedListEmpty(module_list) == MagickTrue))
+  if ((module_list == (SplayTreeInfo *) NULL) ||
+      (GetNumberOfNodesInSplayTree(module_list) == 0))
     return((const ModuleInfo *) NULL);
   if ((tag == (const char *) NULL) || (LocaleCompare(tag,"*") == 0))
     {
@@ -320,21 +296,22 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
       if (LocaleCompare(tag,"*") == 0)
         (void) OpenModules(exception);
 #endif
-      return((const ModuleInfo *) GetElementFromLinkedList(module_list,0));
+      ResetSplayTreeIterator(module_list);
+      return((const ModuleInfo *) GetNextValueInSplayTree(module_list));
     }
   AcquireSemaphoreInfo(&module_semaphore);
-  ResetLinkedListIterator(module_list);
-  p=(const ModuleInfo *) GetNextElementInLinkedList(module_list);
+  ResetSplayTreeIterator(module_list);
+  p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
   while (p != (const ModuleInfo *) NULL)
   {
     if (LocaleCompare(p->tag,tag) == 0)
       break;
-    p=(const ModuleInfo *) GetNextElementInLinkedList(module_list);
+    p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
   }
   if (p == (ModuleInfo *) NULL)
-    (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
-      "UnrecognizedModule",tag);
-  RelinquishSemaphoreInfo(&module_semaphore);
+    (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
+      "NoSuchElement","`%s'",tag);
+  RelinquishSemaphoreInfo(module_semaphore);
   return((const ModuleInfo *) p);
 }
 
@@ -354,7 +331,7 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
 %  The format of the GetModuleInfoList function is:
 %
 %      const ModuleInfo **GetModuleInfoList(const char *pattern,
-%        unsigned long *number_modules)
+%        unsigned long *number_modules,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -362,6 +339,7 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
 %
 %    o number_modules:  This integer returns the number of modules in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -387,14 +365,11 @@ static int ModuleInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
-  unsigned long *number_modules)
+  unsigned long *number_modules,ExceptionInfo *exception)
 {
   const ModuleInfo
     **modules;
 
-  ExceptionInfo
-    exception;
-
   register const ModuleInfo
     *p;
 
@@ -405,32 +380,30 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
     Allocate module list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_modules != (unsigned long *) NULL);
   *number_modules=0;
-  GetExceptionInfo(&exception);
-  p=GetModuleInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetModuleInfo("*",exception);
   if (p == (const ModuleInfo *) NULL)
     return((const ModuleInfo **) NULL);
   modules=(const ModuleInfo **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(module_list)+1)*sizeof(*modules));
+    (GetNumberOfNodesInSplayTree(module_list)+1)*sizeof(*modules));
   if (modules == (const ModuleInfo **) NULL)
     return((const ModuleInfo **) NULL);
   /*
     Generate module list.
   */
   AcquireSemaphoreInfo(&module_semaphore);
-  ResetLinkedListIterator(module_list);
-  p=(const ModuleInfo *) GetNextElementInLinkedList(module_list);
+  ResetSplayTreeIterator(module_list);
+  p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
   for (i=0; p != (const ModuleInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->tag,pattern) != MagickFalse))
       modules[i++]=p;
-    p=(const ModuleInfo *) GetNextElementInLinkedList(module_list);
+    p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
   }
-  RelinquishSemaphoreInfo(&module_semaphore);
+  RelinquishSemaphoreInfo(module_semaphore);
   qsort((void *) modules,(size_t) i,sizeof(*modules),ModuleInfoCompare);
   modules[i]=(ModuleInfo *) NULL;
   *number_modules=(unsigned long) i;
@@ -453,7 +426,8 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
 %
 %  The format of the GetModuleList function is:
 %
-%      char **GetModuleList(const char *pattern,unsigned long *number_modules)
+%      char **GetModuleList(const char *pattern,unsigned long *number_modules,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -462,6 +436,7 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
 %    o number_modules:  This integer returns the number of modules in the
 %      list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -471,12 +446,12 @@ extern "C" {
 
 static int ModuleCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+   p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
@@ -485,7 +460,7 @@ static int ModuleCompare(const void *x,const void *y)
 #endif
 
 MagickExport char **GetModuleList(const char *pattern,
-  unsigned long *number_modules)
+  unsigned long *number_modules,ExceptionInfo *exception)
 {
   char
     **modules,
@@ -496,9 +471,6 @@ MagickExport char **GetModuleList(const char *pattern,
   DIR
     *directory;
 
-  ExceptionInfo
-    exception;
-
   MagickBooleanType
     status;
 
@@ -515,10 +487,8 @@ MagickExport char **GetModuleList(const char *pattern,
     Locate all modules in the coder path.
   */
   TagToCoderModuleName("magick",filename);
-  GetExceptionInfo(&exception);
   status=GetMagickModulePath(filename,MagickCoderModule,module_path,
-    &exception);
-  DestroyExceptionInfo(&exception);
+    exception);
   if (status == MagickFalse)
     return((char **) NULL);
   GetPathComponent(module_path,HeadPath,path);
@@ -615,7 +585,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
     *p;
 
   assert(filename != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),filename);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   assert(path != (char *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
   (void) CopyMagickString(path,filename,MaxTextExtent);
@@ -679,8 +649,8 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
     (void) FormatMagickString(path,MaxTextExtent,"%s%s",directory,filename);
     if (IsAccessible(path) == MagickFalse)
       {
-        (void) ThrowMagickException(exception,GetMagickModule(),
-          ConfigureWarning,"UnableToOpenModuleFile",path);
+        ThrowFileException(exception,ConfigureWarning,"UnableToOpenModuleFile",
+          path);
         return(MagickFalse);
       }
     return(MagickTrue);
@@ -713,18 +683,18 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
     if (value == (char *) NULL)
       {
         ThrowMagickException(exception,GetMagickModule(),ConfigureError,
-          "RegistryKeyLookupFailed",key);
+          "RegistryKeyLookupFailed","`%s'",key);
         return (MagickFalse);
       }
     (void) FormatMagickString(path,MaxTextExtent,"%s%s%s",value,
       DirectorySeparator,filename);
     if (IsAccessible(path) == MagickFalse)
       {
-        ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
-          "UnableToOpenModuleFile",path);
+        ThrowFileException(exception,ConfigureWarning,"UnableToOpenModuleFile",
+          path);
         return(MagickFalse);
       }
-    return (MagickTrue);
+    return(MagickTrue);
   }
 #endif
 #endif
@@ -778,7 +748,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
 #if !defined(POSIX)
       (void) FormatMagickString(path,MaxTextExtent,"%s%s%s",GetClientPath(),
         DirectorySeparator,filename);
-# else
+#else
       char
         prefix[MaxTextExtent];
 
@@ -808,6 +778,19 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
       if (IsAccessible(path) != MagickFalse)
         return(MagickTrue);
     }
+#if defined(__WINDOWS__)
+  {
+    /*
+      Search module path.
+    */
+    if (NTGetModulePath("CORE_RL_magick_.dll",path) != MagickFalse)
+      if (IsAccessible(path) != MagickFalse)
+        return(MagickTrue);
+    if (NTGetModulePath("Magick.dll",path) != MagickFalse)
+      if (IsAccessible(path) != MagickFalse)
+        return(MagickTrue);
+  }
+#endif
   {
     const char
       *home;
@@ -830,8 +813,8 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
   if (IsAccessible(path) != MagickFalse)
     return(MagickTrue);
   if (exception->severity < ConfigureError)
-    ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
-      "UnableToOpenModuleFile",path);
+    ThrowFileException(exception,ConfigureWarning,"UnableToOpenModuleFile",
+      path);
   return(MagickFalse);
 #endif
 }
@@ -859,44 +842,70 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
 %
 %
 */
-static MagickBooleanType InitializeModuleList(ExceptionInfo *exception)
-{
-  MagickBooleanType
-    active;
 
-  active=MagickFalse;
-  if ((module_list == (LinkedListInfo *) NULL) &&
+static void *DestroyModuleNode(void *module_info)
+{
+  ExceptionInfo
+    exception;
+
+  register ModuleInfo
+    *p;
+
+  GetExceptionInfo(&exception);
+  p=(ModuleInfo *) module_info;
+  if (UnregisterModule(p,&exception) == MagickFalse)
+    CatchException(&exception);
+  if (p->tag != (char *) NULL)
+    p->tag=(char *) RelinquishMagickMemory(p->tag);
+  if (p->path != (char *) NULL)
+    p->path=(char *) RelinquishMagickMemory(p->path);
+  DestroyExceptionInfo(&exception);
+  return(RelinquishMagickMemory(p));
+}
+
+static MagickBooleanType InitializeModuleList(
+  ExceptionInfo *magick_unused(exception))
+{
+  if ((module_list == (SplayTreeInfo *) NULL) &&
       (instantiate_module == MagickFalse))
     {
       AcquireSemaphoreInfo(&module_semaphore);
-      if ((module_list == (LinkedListInfo *) NULL) &&
+      if ((module_list == (SplayTreeInfo *) NULL) &&
           (instantiate_module == MagickFalse))
         {
-          instantiate_module=MagickTrue;
-          active=MagickTrue;
-        }
-      RelinquishSemaphoreInfo(&module_semaphore);
-      if (active == MagickTrue)
-        {
+          MagickBooleanType
+            status;
+
           ModuleInfo
             *module_info;
 
+          module_list=NewSplayTree(CompareSplayTreeString,
+            RelinquishMagickMemory,DestroyModuleNode);
+          if (module_list == (SplayTreeInfo *) NULL)
+            ThrowMagickFatalException(ResourceLimitFatalError,
+              "MemoryAllocationFailed",strerror(errno));
           module_info=(ModuleInfo *) AcquireMagickMemory(sizeof(*module_info));
           if (module_info == (ModuleInfo *) NULL)
             ThrowMagickFatalException(ResourceLimitFatalError,
-              "UnableToAllocateModuleInfo",strerror(errno));
+              "MemoryAllocationFailed",strerror(errno));
           (void) ResetMagickMemory(module_info,0,sizeof(*module_info));
-          module_info->tag=AcquireString("[boot-strap]");
+          module_info->tag=ConstantString(AcquireString("[boot-strap]"));
           module_info->stealth=MagickTrue;
-          module_info->signature=MagickSignature;
           (void) time(&module_info->load_time);
-          (void) RegisterModule(module_info,exception);
+          module_info->signature=MagickSignature;
+          status=AddValueToSplayTree(module_list,
+            ConstantString(AcquireString(module_info->tag)),module_info);
+          if (status == MagickFalse)
+            ThrowMagickFatalException(ResourceLimitFatalError,
+              "MemoryAllocationFailed",strerror(errno));
           if (lt_dlinit() != 0)
             ThrowMagickFatalException(ModuleFatalError,
               "UnableToInitializeModuleLoader",lt_dlerror());
+          instantiate_module=MagickTrue;
         }
+      RelinquishSemaphoreInfo(module_semaphore);
     }
-  return(module_list != (LinkedListInfo *) NULL);
+  return(module_list != (SplayTreeInfo *) NULL ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -938,9 +947,17 @@ MagickExport MagickBooleanType ListModuleInfo(FILE *file,
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  module_info=GetModuleInfoList("*",&number_modules);
+  module_info=GetModuleInfoList("*",&number_modules,exception);
   if (module_info == (const ModuleInfo **) NULL)
     return(MagickFalse);
+  if (module_info[0]->path != (char *) NULL)
+    {
+      char
+        path[MaxTextExtent];
+
+      GetPathComponent(module_info[0]->path,HeadPath,path);
+      (void) fprintf(file,"\nPath: %s\n\n",path);
+    }
   (void) fprintf(file,"Module\n");
   (void) fprintf(file,"-------------------------------------------------"
     "------------------------------\n");
@@ -1011,8 +1028,9 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   module_info=(ModuleInfo *) GetModuleInfo(module,exception);
   if (module_info != (ModuleInfo *) NULL)
     return(MagickTrue);
-  if (exception->severity == ModuleError)
-    SetExceptionInfo(exception,UndefinedException);
+  if ((exception->severity == OptionWarning) ||
+      (exception->severity == ModuleError))
+    (void) SetExceptionInfo(exception,UndefinedException);
   (void) CopyMagickString(module_name,module,MaxTextExtent);
   p=GetCoderInfo(module,exception);
   if (p != (CoderInfo *) NULL)
@@ -1035,7 +1053,7 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   if (handle == (ModuleHandle) NULL)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
-        "UnableToLoadModule",path,lt_dlerror());
+        "UnableToLoadModule","`%s': %s",path,lt_dlerror());
       return(MagickFalse);
     }
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
@@ -1048,8 +1066,8 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     ThrowMagickFatalException(ResourceLimitFatalError,
       "UnableToAllocateModuleInfo",module_name);
   (void) ResetMagickMemory(module_info,0,sizeof(*module_info));
-  module_info->path=AcquireString(path);
-  module_info->tag=AcquireString(module_name);
+  module_info->path=ConstantString(AcquireString(path));
+  module_info->tag=ConstantString(AcquireString(module_name));
   module_info->signature=MagickSignature;
   module_info->handle=handle;
   (void) time(&module_info->load_time);
@@ -1063,12 +1081,12 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   if (module_info->register_module == (void (*)(void)) NULL)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
-        "UnableToRegisterImageFormat",module_name,lt_dlerror());
+        "UnableToRegisterImageFormat","`%s': %s",module_name,lt_dlerror());
       return(MagickFalse);
     }
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
     "Method \"%s\" in module \"%s\" at address %p",name,module_name,
-    module_info->register_module);
+    (void *) module_info->register_module);
   /*
     Locate and record UnregisterFORMATImage function
   */
@@ -1077,12 +1095,12 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   if (module_info->unregister_module == (void (*)(void)) NULL)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
-        "UnableToRegisterImageFormat",module_name,lt_dlerror());
+        "UnableToRegisterImageFormat","`%s': %s",module_name,lt_dlerror());
       return(MagickFalse);
     }
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
     "Method \"%s\" in module \"%s\" at address %p",name,module_name,
-    module_info->unregister_module);
+    (void *) module_info->unregister_module);
   /*
     Execute RegisterFORMATImage module.
   */
@@ -1128,7 +1146,7 @@ MagickExport MagickBooleanType OpenModules(ExceptionInfo *exception)
   */
   (void) GetMagickInfo((char *) NULL,exception);
   number_modules=0;
-  modules=GetModuleList("*",&number_modules);
+  modules=GetModuleList("*",&number_modules,exception);
   if (modules == (char **) NULL)
     return(MagickFalse);
   for (i=0; i < (long) number_modules; i++)
@@ -1179,21 +1197,14 @@ static const ModuleInfo *RegisterModule(const ModuleInfo *module_info,
 
   assert(module_info != (ModuleInfo *) NULL);
   assert(module_info->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),module_info->tag);
-  if (module_list == (LinkedListInfo *) NULL)
-    {
-      module_list=NewLinkedList(0);
-      if (module_list == (LinkedListInfo *) NULL)
-        {
-          ThrowFileException(exception,ResourceLimitError,
-            "MemoryAllocationFailed",strerror(errno));
-          return((const ModuleInfo *) NULL);
-        }
-    }
-  status=AppendElementToLinkedList(module_list,module_info);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",module_info->tag);
+  if (module_list == (SplayTreeInfo *) NULL)
+    return((const ModuleInfo *) NULL);
+  status=AddValueToSplayTree(module_list,
+    ConstantString(AcquireString(module_info->tag)),module_info);
   if (status == MagickFalse)
-    (void) ThrowMagickException(exception,GetMagickModule(),
-      ResourceLimitError,"MemoryAllocationFailed",module_info->tag);
+    (void) ThrowMagickException(exception,GetMagickModule(),ResourceLimitError,
+      "MemoryAllocationFailed","`%s'",module_info->tag);
   return(module_info);
 }
 
@@ -1225,7 +1236,7 @@ static const ModuleInfo *RegisterModule(const ModuleInfo *module_info,
 static void TagToCoderModuleName(const char *tag,char *name)
 {
   assert(tag != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),tag);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",tag);
   assert(name != (char *) NULL);
 #if defined(HasLTDL)
   (void) FormatMagickString(name,MaxTextExtent,"%s.la",tag);
@@ -1237,11 +1248,9 @@ static void TagToCoderModuleName(const char *tag,char *name)
   else
     {
 #if defined(_DEBUG)
-      (void) FormatMagickString(name,MaxTextExtent,"IM_MOD_DB_%s_.dll",
-        tag);
+      (void) FormatMagickString(name,MaxTextExtent,"IM_MOD_DB_%s_.dll",tag);
 #else
-      (void) FormatMagickString(name,MaxTextExtent,"IM_MOD_RL_%s_.dll",
-        tag);
+      (void) FormatMagickString(name,MaxTextExtent,"IM_MOD_RL_%s_.dll",tag);
 #endif
     }
 #endif
@@ -1276,7 +1285,7 @@ static void TagToCoderModuleName(const char *tag,char *name)
 static void TagToFilterModuleName(const char *tag,char *name)
 {
   assert(tag != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),tag);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",tag);
   assert(name != (char *) NULL);
 #if !defined(HasLTDL)
   (void) FormatMagickString(name,MaxTextExtent,"%s.dll",tag);
@@ -1320,7 +1329,7 @@ static void TagToModuleName(const char *tag,const char *format,char *module)
     name[MaxTextExtent];
 
   assert(tag != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),tag);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",tag);
   assert(format != (const char *) NULL);
   assert(module != (char *) NULL);
   (void) CopyMagickString(name,tag,MaxTextExtent);
@@ -1344,18 +1353,18 @@ static void TagToModuleName(const char *tag,const char *format,char *module)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   U n l o a d M o d u l e                                                   %
+%   U n R e g i s t e r M o d u l e                                           %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  UnloadModule() unloads a module, and invokes its de-registration module.
+%  UnregisterModule() unloads a module, and invokes its de-registration module.
 %  Returns MagickTrue on success, and MagickFalse if there is an error.
 %
-%  The format of the UnloadModule module is:
+%  The format of the UnregisterModule module is:
 %
-%      MagickBooleanType UnloadModule(const ModuleInfo *module_info,
+%      MagickBooleanType UnregisterModule(const ModuleInfo *module_info,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
@@ -1366,7 +1375,7 @@ static void TagToModuleName(const char *tag,const char *format,char *module)
 %
 %
 */
-static MagickBooleanType UnloadModule(const ModuleInfo *module_info,
+static MagickBooleanType UnregisterModule(const ModuleInfo *module_info,
   ExceptionInfo *exception)
 {
   char
@@ -1376,7 +1385,7 @@ static MagickBooleanType UnloadModule(const ModuleInfo *module_info,
     Locate and execute UnregisterFORMATImage module.
   */
   assert(module_info != (const ModuleInfo *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),module_info->tag);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",module_info->tag);
   assert(exception != (ExceptionInfo *) NULL);
   if (module_info->unregister_module == NULL)
     return(MagickTrue);
@@ -1384,7 +1393,7 @@ static MagickBooleanType UnloadModule(const ModuleInfo *module_info,
   if (lt_dlclose((ModuleHandle) module_info->handle) != 0)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
-        "UnableToCloseModule",name,lt_dlerror());
+        "UnableToCloseModule","`%s': %s",name,lt_dlerror());
       return(MagickFalse);
     }
   return(MagickTrue);
@@ -1400,8 +1409,10 @@ MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
 #if !defined(BuildMagickModules)
   {
     MagickBooleanType
-      (*method)(Image **,const int,char **) = NULL;
+      AnalyzeImage(Image **,const int,char **),
+      (*method)(Image **,const int,char **);
 
+    method=NULL;
     if (LocaleCompare("analyze",tag) == 0)
       method=AnalyzeImage;
     if (method != NULL)
