@@ -17,13 +17,13 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -42,9 +42,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
 #include "magick/cache.h"
+#include "magick/cache-private.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/memory_.h"
 #include "magick/semaphore.h"
 #include "magick/string_.h"
@@ -52,7 +55,8 @@
 /*
   Typedef declaractions.
 */
-typedef CacheInfo StreamInfo;
+typedef CacheInfo
+  StreamInfo;
 
 /*
   Declare pixel cache interfaces.
@@ -186,12 +190,12 @@ static const PixelPacket *AcquirePixelStream(const Image *image,const long x,
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((x < 0) || (y < 0) || ((x+(long) columns) > (long) image->columns) ||
       ((y+(long) rows) > (long) image->rows) || (columns == 0) || (rows == 0))
     {
       (void) ThrowMagickException(exception,GetMagickModule(),StreamError,
-        "ImageDoesNotContainTheStreamGeometry",image->filename);
+        "ImageDoesNotContainTheStreamGeometry","`%s'",image->filename);
       return((PixelPacket *) NULL);
     }
   stream_info=(StreamInfo *) image->cache;
@@ -199,7 +203,7 @@ static const PixelPacket *AcquirePixelStream(const Image *image,const long x,
   if (stream_info->type == UndefinedCache)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),StreamError,
-        "PixelCacheIsNotOpen",image->filename);
+        "PixelCacheIsNotOpen","`%s'",image->filename);
       return((PixelPacket *) NULL);
     }
   /*
@@ -258,21 +262,21 @@ static void DestroyPixelStream(Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   stream_info=(StreamInfo *) image->cache;
   assert(stream_info->signature == MagickSignature);
-  AcquireSemaphoreInfo((SemaphoreInfo **) &stream_info->semaphore);
+  AcquireSemaphoreInfo(&stream_info->semaphore);
   stream_info->reference_count--;
   if (stream_info->reference_count > 0)
     {
-      RelinquishSemaphoreInfo((SemaphoreInfo **) &stream_info->semaphore);
+      RelinquishSemaphoreInfo(stream_info->semaphore);
       return;
     }
   if (stream_info->pixels != (PixelPacket *) NULL)
     stream_info->pixels=(PixelPacket *)
       RelinquishMagickMemory(stream_info->pixels);
-  RelinquishSemaphoreInfo((SemaphoreInfo **) &stream_info->semaphore);
-  DestroySemaphoreInfo((SemaphoreInfo **) &stream_info->semaphore);
+  RelinquishSemaphoreInfo(stream_info->semaphore);
+  stream_info->semaphore=DestroySemaphoreInfo(stream_info->semaphore);
   stream_info=(StreamInfo *) RelinquishMagickMemory(stream_info);
 }
 
@@ -311,7 +315,7 @@ static IndexPacket *GetIndexesFromStream(const Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   stream_info=(StreamInfo *) image->cache;
   assert(stream_info->signature == MagickSignature);
   return(stream_info->indexes);
@@ -398,7 +402,7 @@ static PixelPacket *GetPixelStream(Image *image,const long x,const long y,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   pixels=SetPixelStream(image,x,y,columns,rows);
   return(pixels);
 }
@@ -438,7 +442,7 @@ static PixelPacket *GetPixelsFromStream(const Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   stream_info=(StreamInfo *) image->cache;
   assert(stream_info->signature == MagickSignature);
   return(stream_info->pixels);
@@ -477,6 +481,9 @@ static PixelPacket *GetPixelsFromStream(const Image *image)
 MagickExport Image *ReadStream(const ImageInfo *image_info,StreamHandler stream,
   ExceptionInfo *exception)
 {
+  CacheMethods
+    cache_methods;
+
   Image
     *image;
 
@@ -489,17 +496,26 @@ MagickExport Image *ReadStream(const ImageInfo *image_info,StreamHandler stream,
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   read_info=CloneImageInfo(image_info);
   GetCacheInfo(&read_info->cache);
-  SetPixelCacheMethods(read_info->cache,AcquirePixelStream,GetPixelStream,
-    SetPixelStream,SyncPixelStream,GetPixelsFromStream,GetIndexesFromStream,
-    AcquireOnePixelFromStream,GetOnePixelFromStream,DestroyPixelStream);
+  GetCacheMethods(&cache_methods);
+  cache_methods.acquire_pixel_handler=AcquirePixelStream;
+  cache_methods.get_pixel_handler=GetPixelStream;
+  cache_methods.set_pixel_handler=SetPixelStream;
+  cache_methods.sync_pixel_handler=SyncPixelStream;
+  cache_methods.get_pixels_from_handler=GetPixelsFromStream;
+  cache_methods.get_indexes_from_handler=GetIndexesFromStream;
+  cache_methods.acquire_one_pixel_from_handler=AcquireOnePixelFromStream;
+  cache_methods.get_one_pixel_from_handler=GetOnePixelFromStream;
+  cache_methods.destroy_pixel_handler=DestroyPixelStream;
+  SetPixelCacheMethods(read_info->cache,&cache_methods);
   read_info->stream=stream;
   image=ReadImage(read_info,exception);
-  DestroyImageInfo(read_info);
+  read_info=DestroyImageInfo(read_info);
   return(image);
 }
 
@@ -560,14 +576,15 @@ static PixelPacket *SetPixelStream(Image *image,const long x,const long y,
       ((y+(long) rows) > (long) image->rows) || (columns == 0) || (rows == 0))
     {
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        StreamError,"ImageDoesNotContainTheStreamGeometry",image->filename);
+        StreamError,"ImageDoesNotContainTheStreamGeometry","`%s'",
+        image->filename);
       return((PixelPacket *) NULL);
     }
   stream_handler=GetBlobStreamHandler(image);
   if (stream_handler == (StreamHandler) NULL)
     {
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        StreamError,"NoStreamHandlerIsDefined",image->filename);
+        StreamError,"NoStreamHandlerIsDefined","`%s'",image->filename);
       return((PixelPacket *) NULL);
     }
   stream_info=(StreamInfo *) image->cache;
@@ -654,14 +671,14 @@ static MagickBooleanType SyncPixelStream(Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   stream_info=(StreamInfo *) image->cache;
   assert(stream_info->signature == MagickSignature);
   stream_handler=GetBlobStreamHandler(image);
   if (stream_handler == (StreamHandler) NULL)
     {
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        StreamError,"NoStreamHandlerIsDefined",image->filename);
+        StreamError,"NoStreamHandlerIsDefined","`%s'",image->filename);
       return(MagickFalse);
     }
   status=stream_handler(image,stream_info->pixels,(size_t)
@@ -709,12 +726,13 @@ MagickExport MagickBooleanType WriteStream(const ImageInfo *image_info,
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   write_info=CloneImageInfo(image_info);
   write_info->stream=stream;
   status=WriteImage(write_info,image);
-  DestroyImageInfo(write_info);
+  write_info=DestroyImageInfo(write_info);
   return(status);
 }
