@@ -17,13 +17,13 @@
 %                               August 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -42,7 +42,9 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/blob-private.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/list.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
@@ -103,19 +105,16 @@ static const unsigned char
 %
 %  A description of each parameter follows:
 %
-%    o allocated_string:  Method AcquireString returns a copy of the source
-%      string.
-%
 %    o source: A character string.
 %
-%
 */
 
-static size_t CheckOverflowException(size_t length,size_t extend)
+static inline size_t CheckOverflowException(const size_t length,
+  const size_t extend)
 {
   if ((length+extend) < length)
     ThrowMagickFatalException(ResourceLimitFatalError,"UnableToAcquireString",
-      (char *) NULL);
+      strerror(errno));
   return(length+extend);
 }
 
@@ -221,13 +220,12 @@ MagickExport char *CloneString(char **destination,const char *source)
   size_t
     length;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(destination != (char **) NULL);
   if (source == (const char *) NULL)
     {
       if (*destination != (char *) NULL)
         *destination=(char *) RelinquishMagickMemory(*destination);
-      *destination=(char *) NULL;
       return(*destination);
     }
   if (*destination == (char *) NULL)
@@ -272,7 +270,7 @@ MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
   StringInfo
     *clone_info;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   clone_info=AcquireStringInfo(string_info->length);
@@ -293,13 +291,13 @@ MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  CompareStringInfo() returns MagickTrue if the length and datum for two strings are
-%  equal.
+%  CompareStringInfo() compares the two datums target and source.  It returns
+%  an integer less than, equal to, or greater than zero if target is found,
+%  respectively, to be less than, to match, or be greater than source.
 %
 %  The format of the CompareStringInfo method is:
 %
-%      MagickBooleanType CompareStringInfo(const StringInfo *target,
-%        const StringInfo *source)
+%      int CompareStringInfo(const StringInfo *target,const StringInfo *source)
 %
 %  A description of each parameter follows:
 %
@@ -308,19 +306,32 @@ MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
 %    o source: The source string.
 %
 */
-MagickExport MagickBooleanType CompareStringInfo(const StringInfo *target,
+MagickExport int CompareStringInfo(const StringInfo *target,
   const StringInfo *source)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  register unsigned char
+    *p,
+    *q;
+
+  register size_t
+    i;
+
   assert(target != (StringInfo *) NULL);
   assert(target->signature == MagickSignature);
   assert(source != (StringInfo *) NULL);
   assert(source->signature == MagickSignature);
-  if (target->length != source->length)
-    return(MagickFalse);
-  if (memcmp(target->datum,source->datum,target->length) != 0)
-    return(MagickFalse);
-  return(MagickTrue);
+  p=target->datum;
+  q=source->datum;
+  for (i=0; i < Min(source->length,target->length); i++)
+  {
+    if (*p != *q)
+      break;
+    p++;
+    q++;
+  }
+  if ((i == source->length) && (source->length == target->length))
+    return(0);
+  return(*p-(int) *q);
 }
 
 /*
@@ -478,7 +489,7 @@ MagickExport void ConcatenateStringInfo(StringInfo *string_info,
   size_t
     length;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   assert(source != (const StringInfo *) NULL);
@@ -494,6 +505,144 @@ MagickExport void ConcatenateStringInfo(StringInfo *string_info,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   C o n f i g u r e F i l e T o S t r i n g I n f o                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ConfigureFileToStringInfo() returns the contents of a configure file as a
+%  string.
+%
+%  The format of the ConfigureFileToStringInfo method is:
+%
+%      StringInfo *ConfigureFileToStringInfo(const char *filename)
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o filename: The filename.
+%
+*/
+MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
+{
+  char
+    *string;
+
+  int
+    file;
+
+  MagickOffsetType
+    offset;
+
+  size_t
+    length;
+
+  StringInfo
+    *string_info;
+
+  void
+    *map;
+
+  assert(filename != (const char *) NULL);
+  file=open(filename,O_RDONLY | O_BINARY);
+  if (file == -1)
+    return((StringInfo *) NULL);
+  offset=(MagickOffsetType) MagickSeek(file,0,SEEK_END);
+  if ((offset < 0) || (offset != (MagickOffsetType) ((ssize_t) offset)))
+    {
+      file=close(file)-1;
+      return((StringInfo *) NULL);
+    }
+  length=(size_t) offset;
+  string=(char *) AcquireMagickMemory((length+1)*sizeof(*string));
+  if (string == (char *) NULL)
+    {
+      file=close(file)-1;
+      return((StringInfo *) NULL);
+    }
+  map=MapBlob(file,ReadMode,0,length);
+  if (map != (void *) NULL)
+    {
+      (void) CopyMagickMemory(string,map,length);
+      (void) UnmapBlob(map,length);
+    }
+  else
+    {
+      register size_t
+        i;
+
+      ssize_t
+        count;
+
+      (void) MagickSeek(file,0,SEEK_SET);
+      for (i=0; i < length; i+=count)
+      {
+        count=read(file,string+i,(size_t) Min(length-i,(size_t)
+          MagickMaxBufferSize));
+        if (count <= 0)
+          {
+            count=0;
+            if (errno != EINTR)
+              break;
+          }
+      }
+      if (i < length)
+        {
+          file=close(file)-1;
+          string=(char *) RelinquishMagickMemory(string);
+          return((StringInfo *) NULL);
+        }
+    }
+  string[length]='\0';
+  file=close(file)-1;
+  string_info=AcquireStringInfo(0);
+  (void) CopyMagickString(string_info->path,filename,MaxTextExtent);
+  string_info->length=strlen(string)+1;
+  string_info->datum=(unsigned char *) string;
+  return(string_info);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   C o n s t a n t S t r i n g                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ConstantString() trims a string of any excess memory.  This method is useful
+%  for strings that will not change over the life of the string.
+%
+%  The format of the ConstantString method is:
+%
+%      char *ConstantString(char *source)
+%
+%  A description of each parameter follows:
+%
+%    o source: A character string.
+%
+*/
+MagickExport char *ConstantString(char *source)
+{
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  if (source == (char *) NULL)
+    return((char *) NULL);
+  source=(char *) ResizeMagickMemory(source,(strlen(source)+1)*sizeof(*source));
+  if (source == (char *) NULL)
+    ThrowMagickFatalException(ResourceLimitFatalError,"UnableToAcquireString",
+      strerror(errno));
+  return(source);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   C o p y M a g i c k S t r i n g                                           %
 %                                                                             %
 %                                                                             %
@@ -568,17 +717,19 @@ MagickExport size_t CopyMagickString(char *destination,const char *source,
 %
 %  The format of the DestroyString method is:
 %
-%      void DestroyString(char *string)
+%      char *DestroyString(char *string)
 %
 %  A description of each parameter follows:
 %
 %    o string: The string.
 %
 */
-MagickExport void DestroyString(char *string)
+MagickExport char *DestroyString(char *string)
 {
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string != (char *) NULL);
   string=(char *) RelinquishMagickMemory(string);
+  return(string);
 }
 
 /*
@@ -596,22 +747,24 @@ MagickExport void DestroyString(char *string)
 %
 %  The format of the DestroyStringInfo method is:
 %
-%      void DestroyStringInfo(StringInfo *string_info)
+%      StringInfo *DestroyStringInfo(StringInfo *string_info)
 %
 %  A description of each parameter follows:
 %
 %    o string_info: The string info.
 %
 */
-MagickExport void DestroyStringInfo(StringInfo *string_info)
+MagickExport StringInfo *DestroyStringInfo(StringInfo *string_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   if (string_info->datum != (unsigned char *) NULL)
     string_info->datum=(unsigned char *)
       RelinquishMagickMemory(string_info->datum);
+  string_info->signature=(~MagickSignature);
   string_info=(StringInfo *) RelinquishMagickMemory(string_info);
+  return(string_info);
 }
 
 /*
@@ -629,22 +782,23 @@ MagickExport void DestroyStringInfo(StringInfo *string_info)
 %
 %  The format of the DestroyStringList method is:
 %
-%      void DestroyStringList(char **list)
+%      char **DestroyStringList(char **list)
 %
 %  A description of each parameter follows:
 %
 %    o list: The string list.
 %
 */
-MagickExport void DestroyStringList(char **list)
+MagickExport char **DestroyStringList(char **list)
 {
   register long
     i;
 
   assert(list != (char **) NULL);
   for (i=0; list[i] != (char *) NULL; i++)
-    DestroyString(list[i]);
+    list[i]=DestroyString(list[i]);
   list=(char **) RelinquishMagickMemory(list);
+  return(list);
 }
 
 /*
@@ -669,7 +823,7 @@ MagickExport void DestroyStringList(char **list)
 %
 %  A description of each parameter follows:
 %
-%    o allocated_string:  Method EscapeString returns the escaped string.
+%    o allocate_string:  Method EscapeString returns the escaped string.
 %
 %    o source: A character string.
 %
@@ -690,7 +844,7 @@ MagickExport char *EscapeString(const char *source,const char escape)
   size_t
     length;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(source != (const char *) NULL);
   length=CheckOverflowException(strlen(source),1);
   for (p=source; *p != '\0'; p++)
@@ -730,83 +884,28 @@ MagickExport char *EscapeString(const char *source,const char escape)
 %
 %  The format of the FileToString method is:
 %
-%      char *FileToString(const char *filename)
+%      char *FileToString(const char *filename,const size_t extent,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o filename: The filename.
 %
+%    o extent: Maximum length of the string.
+%
+%    o exception: Return any errors or warnings in this structure.
 %
 */
-MagickExport char *FileToString(const char *filename)
+MagickExport char *FileToString(const char *filename,const size_t extent,
+  ExceptionInfo *exception)
 {
-  char
-    *string;
-
-  int
-    file;
-
-  MagickOffsetType
-    offset;
-
   size_t
     length;
 
-  void
-    *map;
-
   assert(filename != (const char *) NULL);
-  file=open(filename,O_RDONLY | O_BINARY);
-  if (file == -1)
-    return((char *) NULL);
-  offset=(MagickOffsetType) MagickSeek(file,0,SEEK_END);
-  if ((offset < 0) || (offset != (MagickOffsetType) ((ssize_t) offset)))
-    {
-      (void) close(file);
-      return((char *) NULL);
-    }
-  length=(size_t) offset;
-  string=(char *) AcquireMagickMemory((length+1)*sizeof(*string));
-  if (string == (char *) NULL)
-    {
-      (void) close(file);
-      return((char *) NULL);
-    }
-  map=MapBlob(file,ReadMode,0,length);
-  if (map != (void *) NULL)
-    {
-      (void) CopyMagickMemory(string,map,length);
-      (void) UnmapBlob(map,length);
-    }
-  else
-    {
-      register size_t
-        i;
-
-      ssize_t
-        count;
-
-      (void) MagickSeek(file,0,SEEK_SET);
-      for (i=0; i < length; i+=count)
-      {
-        count=read(file,string+i,Min(length-i,(size_t) MagickMaxBufferSize));
-        if (count <= 0)
-          {
-            count=0;
-            if (errno != EINTR)
-              break;
-          }
-      }
-      if (i < length)
-        {
-          (void) close(file);
-          string=(char *) RelinquishMagickMemory(string);
-          return((char *) NULL);
-        }
-    }
-  string[length]='\0';
-  (void) close(file);
-  return(string);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  return((char *) FileToBlob(filename,extent,&length,exception));
 }
 
 /*
@@ -824,33 +923,35 @@ MagickExport char *FileToString(const char *filename)
 %
 %  The format of the FileToStringInfo method is:
 %
-%      void *FileToStringInfo(const char *filename,ExceptionInfo *exception)
+%      StringInfo *FileToStringInfo(const char *filename,const size_t extent,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o filename: The filename.
 %
+%    o extent: Maximum length of the string.
+%
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
-MagickExport StringInfo *FileToStringInfo(const char *filename)
+MagickExport StringInfo *FileToStringInfo(const char *filename,
+  const size_t extent,ExceptionInfo *exception)
 {
-  char
-    *string;
-
   StringInfo
     *string_info;
 
   assert(filename != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),filename);
-  string=FileToString(filename);
-  if (string == (char *) NULL)
-    return((StringInfo *) NULL);
-  string_info=AcquireStringInfo(strlen(string)+1);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  string_info=AcquireStringInfo(0);
   (void) CopyMagickString(string_info->path,filename,MaxTextExtent);
-  SetStringInfoDatum(string_info,(unsigned char *) string);
-  string=(char *) RelinquishMagickMemory(string);
+  string_info->datum=FileToBlob(filename,extent,&string_info->length,exception);
+  if (string_info->datum == (unsigned char *) NULL)
+    {
+      string_info=DestroyStringInfo(string_info);
+      return((StringInfo *) NULL);
+    }
   return(string_info);
 }
 
@@ -869,7 +970,7 @@ MagickExport StringInfo *FileToStringInfo(const char *filename)
 %
 %  The format of the FormatMagickString method is:
 %
-%      int FormatMagickString(char *string,const size_t length,
+%      long FormatMagickString(char *string,const size_t length,
 %        const char *format,...)
 %
 %  A description of each parameter follows.
@@ -882,25 +983,37 @@ MagickExport StringInfo *FileToStringInfo(const char *filename)
 %   o format:  A string describing the format to use to write the remaining
 %     arguments.
 %
-%
 */
-MagickExport int FormatMagickString(char *string,const size_t length,
-  const char *format,...)
+
+MagickExport long FormatMagickStringList(char *string,const size_t length,
+  const char *format,va_list operands)
 {
   int
-    count;
+    n;
+
+#if defined(HAVE_VSNPRINTF)
+  n=vsnprintf(string,length,format,operands);
+#else
+  n=vsprintf(string,format,operands);
+#endif
+  if (n < 0)
+    string[length-1]='\0';
+  return((long) n);
+}
+
+MagickExport long FormatMagickString(char *string,const size_t length,
+  const char *format,...)
+{
+  long
+    n;
 
   va_list
     operands;
 
   va_start(operands,format);
-#if defined(HAVE_VSNPRINTF)
-  count=vsnprintf(string,length,format,operands);
-#else
-  count=vsprintf(string,format,operands);
-#endif
+  n=(long) FormatMagickStringList(string,length,format,operands);
   va_end(operands);
-  return(count);
+  return(n);
 }
 
 /*
@@ -923,7 +1036,7 @@ MagickExport int FormatMagickString(char *string,const size_t length,
 %
 %  The format of the LocaleCompare method is:
 %
-%      int LocaleCompare(const char *p,const char *q)
+%      long LocaleCompare(const char *p,const char *q)
 %
 %  A description of each parameter follows:
 %
@@ -933,14 +1046,14 @@ MagickExport int FormatMagickString(char *string,const size_t length,
 %
 %
 */
-MagickExport int LocaleCompare(const char *p,const char *q)
+MagickExport long LocaleCompare(const char *p,const char *q)
 {
   if (p == (char *) NULL)
     return(-1);
   if (q == (char *) NULL)
     return(1);
 #if defined(HAVE_STRCASECMP)
-  return(strcasecmp(p,q));
+  return((long) strcasecmp(p,q));
 #else
   {
     register unsigned char
@@ -956,7 +1069,7 @@ MagickExport int LocaleCompare(const char *p,const char *q)
       p++;
       q++;
     }
-    return(AsciiMap[c]-AsciiMap[d]);
+    return((long) AsciiMap[c]-AsciiMap[d]);
   }
 #endif
 }
@@ -1019,7 +1132,7 @@ MagickExport void LocaleLower(char *string)
 %
 %  The format of the LocaleNCompare method is:
 %
-%      int LocaleNCompare(const char *p,const char *q,const size_t n)
+%      long LocaleNCompare(const char *p,const char *q,const size_t n)
 %
 %  A description of each parameter follows:
 %
@@ -1031,14 +1144,15 @@ MagickExport void LocaleLower(char *string)
 %
 %
 */
-MagickExport int LocaleNCompare(const char *p,const char *q,const size_t length)
+MagickExport long LocaleNCompare(const char *p,const char *q,
+  const size_t length)
 {
   if (p == (char *) NULL)
     return(-1);
   if (q == (char *) NULL)
     return(1);
 #if defined(HAVE_STRNCASECMP)
-  return(strncasecmp(p,q,length));
+  return((long) strncasecmp(p,q,length));
 #else
   {
     register size_t
@@ -1055,11 +1169,11 @@ MagickExport int LocaleNCompare(const char *p,const char *q,const size_t length)
       if (AsciiMap[c] != AsciiMap[d])
         return(AsciiMap[c]-AsciiMap[d]);
       if (c == '\0')
-        return(0);
+        return(0L);
       p++;
       q++;
     }
-    return(0);
+    return(0L);
   }
 #endif
 }
@@ -1113,45 +1227,77 @@ MagickExport void LocaleUpper(char *string)
 %
 %  The format of the PrintStringInfo method is:
 %
-%      void PrintStringInfo(const char *id,const StringInfo *string_info)
+%      void PrintStringInfo(FILE *file,const char *id,
+%        const StringInfo *string_info)
 %
 %  A description of each parameter follows:
 %
+%    o file: The file, typically stdout.
+%
 %    o id: The string id.
 %
 %    o string_info: The string info.
 %
 */
-MagickExport void PrintStringInfo(const char *id,const StringInfo *string_info)
+MagickExport void PrintStringInfo(FILE *file,const char *id,
+  const StringInfo *string_info)
 {
-  register unsigned char
-    *p,
-    *q;
+  register const char
+    *p;
 
+  register size_t
+    i,
+    j;
 
   assert(id != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),id);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",id);
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
-  q=string_info->datum+string_info->length;
-  for (p=string_info->datum; p < q; p++)
+  p=(char *) string_info->datum;
+  for (i=0; i < string_info->length; i++)
   {
-    if (((int) *p < 32) && (isspace((int) ((unsigned char) *p)) == 0))
-      break;
-    if (((int) *p) > 127)
+    if (((int) ((unsigned char) *p) < 32) &&
+        (isspace((int) ((unsigned char) *p)) == 0))
       break;
+    p++;
   }
-  (void) fprintf(stdout,"%s(%lu): ",id,(unsigned long) string_info->length);
-  if (p == q)
-    for (p=string_info->datum; p < q; p++)
-      (void) fprintf(stdout,"%c",(int) *p);
-  else
+  if (i == string_info->length)
     {
-      (void) fprintf(stdout,"0x");
-      for (p=string_info->datum; p < q; p++)
-        (void) fprintf(stdout,"%02lx",(unsigned long) *p);
+      (void) fputs((char *) string_info->datum,file);
+      (void) fputc('\n',file);
+      return;
     }
-  (void) fprintf(stdout,"\n");
+  /*
+    Convert string to a HEX list.
+  */
+  p=(char *) string_info->datum;
+  for (i=0; i < string_info->length; i+=0x14)
+  {
+    (void) fprintf(file,"0x%08lx: ",(unsigned long) (0x14*i));
+    for (j=1; j <= (size_t) Min(string_info->length-i,0x14); j++)
+    {
+      (void) fprintf(file,"%02lx",(unsigned long) (*(p+j)) & 0xff);
+      if ((j % 0x04) == 0)
+        (void) fputc(' ',file);
+    }
+    for ( ; j <= 0x14; j++)
+    {
+      (void) fputc(' ',file);
+      (void) fputc(' ',file);
+      if ((j % 0x04) == 0)
+        (void) fputc(' ',file);
+    }
+    (void) fputc(' ',file);
+    for (j=1; j <= (size_t) Min(string_info->length-i,0x14); j++)
+    {
+      if (isprint((int) ((unsigned char) *p)) != 0)
+        (void) fputc(*p,file);
+      else
+        (void) fputc('-',file);
+      p++;
+    }
+    (void) fputc('\n',file);
+  }
 }
 
 /*
@@ -1178,7 +1324,7 @@ MagickExport void PrintStringInfo(const char *id,const StringInfo *string_info)
 */
 MagickExport void ResetStringInfo(StringInfo *string_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   (void) ResetMagickMemory(string_info->datum,0,string_info->length);
@@ -1211,7 +1357,7 @@ MagickExport void ResetStringInfo(StringInfo *string_info)
 MagickExport void SetStringInfo(StringInfo *string_info,
   const StringInfo *source)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   assert(source != (StringInfo *) NULL);
@@ -1250,7 +1396,7 @@ MagickExport void SetStringInfo(StringInfo *string_info,
 MagickExport void SetStringInfoDatum(StringInfo *string_info,
   const unsigned char *source)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   assert(source != (const unsigned char *) NULL);
@@ -1284,7 +1430,7 @@ MagickExport void SetStringInfoDatum(StringInfo *string_info,
 MagickExport void SetStringInfoLength(StringInfo *string_info,
   const size_t length)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   string_info->length=length;
@@ -1328,7 +1474,7 @@ MagickExport StringInfo *SplitStringInfo(StringInfo *string_info,
   StringInfo
     *split_info;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   if (offset > string_info->length)
@@ -1368,9 +1514,9 @@ MagickExport StringInfo *StringToStringInfo(const char *string)
   StringInfo
     *string_info;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string != (const char *) NULL);
-  string_info=AcquireStringInfo(strlen(string));
+  string_info=AcquireStringInfo(strlen(string)+1);
   SetStringInfoDatum(string_info,(const unsigned char *) string);
   return(string_info);
 }
@@ -1449,7 +1595,7 @@ MagickExport char **StringToArgv(const char *text,int *argc)
   char
     **argv;
 
-  register char
+  register const char
     *p,
     *q;
 
@@ -1462,7 +1608,7 @@ MagickExport char **StringToArgv(const char *text,int *argc)
   /*
     Determine the number of arguments.
   */
-  for (p=(char *) text; *p != '\0'; )
+  for (p=text; *p != '\0'; )
   {
     while (isspace((int) ((unsigned char) *p)) != 0)
       p++;
@@ -1483,7 +1629,7 @@ MagickExport char **StringToArgv(const char *text,int *argc)
     Convert string to an ASCII list.
   */
   argv[0]=AcquireString("magick");
-  p=(char *) text;
+  p=text;
   for (i=1; i < (long) *argc; i++)
   {
     while (isspace((int) ((unsigned char) *p)) != 0)
@@ -1503,7 +1649,7 @@ MagickExport char **StringToArgv(const char *text,int *argc)
       else
         while ((isspace((int) ((unsigned char) *q)) == 0) && (*q != '\0'))
           q++;
-    argv[i]=(char *) AcquireMagickMemory(((q-p)+MaxTextExtent)*sizeof(*argv));
+    argv[i]=(char *) AcquireMagickMemory(((q-p)+MaxTextExtent)*sizeof(**argv));
     if (argv[i] == (char *) NULL)
       {
         for (i--; i >= 0; i--)
@@ -1623,7 +1769,7 @@ MagickExport char **StringToList(const char *text)
         if (*p == '\n')
           lines++;
       textlist=(char **)
-        AcquireMagickMemory((size_t) (lines+MaxTextExtent)*sizeof(*textlist));
+        AcquireMagickMemory((size_t) (lines+1)*sizeof(*textlist));
       if (textlist == (char **) NULL)
         ThrowMagickFatalException(ResourceLimitFatalError,"UnableToConvertText",
           text);
@@ -1657,7 +1803,7 @@ MagickExport char **StringToList(const char *text)
       */
       lines=(unsigned long) (strlen(text)/0x14)+1;
       textlist=(char **)
-        AcquireMagickMemory((size_t) (lines+MaxTextExtent)*sizeof(*textlist));
+        AcquireMagickMemory((size_t) (lines+1)*sizeof(*textlist));
       if (textlist == (char **) NULL)
         ThrowMagickFatalException(ResourceLimitFatalError,"UnableToConvertText",
           text);
@@ -1679,7 +1825,7 @@ MagickExport char **StringToList(const char *text)
           if ((j % 0x04) == 0)
             *q++=' ';
         }
-        for (; j <= 0x14; j++)
+        for ( ; j <= 0x14; j++)
         {
           *q++=' ';
           *q++=' ';
@@ -1739,8 +1885,6 @@ MagickExport void StripString(char *message)
   if (*message == '\0')
     return;
   length=strlen(message);
-  if (length == 1)
-    return;
   p=message;
   while (isspace((int) ((unsigned char) *p)) != 0)
     p++;
@@ -1771,11 +1915,11 @@ MagickExport void StripString(char *message)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  SubstituteString() performs string substitution on a buffer, replacing the
-%  buffer with the substituted version. Buffer must be allocated from the heap.
+%  buffer with the substituted version. Buffer must be allocate from the heap.
 %
 %  The format of the SubstituteString method is:
 %
-%      MagickBooleanType SubstituteString(char **buffer,const char* search,
+%      MagickBooleanType SubstituteString(char **buffer,const char *search,
 %        const char *replace)
 %
 %  A description of each parameter follows:
@@ -1792,21 +1936,23 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
   const char *search,const char *replace)
 {
   char
-    *destination,
     *result;
 
   const char
     *match,
     *source;
 
+  MagickOffsetType
+    destination_offset;
+
   size_t
-    allocated_length,
+    allocate_length,
     copy_length,
     replace_length,
     result_length,
     search_length;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(buffer != (char **) NULL);
   assert(*buffer != (char *) NULL);
   assert(search != (const char*) NULL);
@@ -1815,14 +1961,14 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
   match=strstr(source,search);
   if (match == (char *) NULL)
     return(MagickFalse);
-  allocated_length=CheckOverflowException(strlen(source),MaxTextExtent);
-  result=(char *) AcquireMagickMemory(allocated_length*sizeof(*result));
+  allocate_length=CheckOverflowException(strlen(source),MaxTextExtent);
+  result=(char *) AcquireMagickMemory(allocate_length*sizeof(*result));
   if (result == (char *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,"UnableToAcquireString",
       search);
   *result='\0';
   result_length=0;
-  destination=result;
+  destination_offset=0;
   replace_length=strlen(replace);
   search_length=strlen(search);
   while (match != (char *) NULL)
@@ -1834,33 +1980,33 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
     if (copy_length != 0)
       {
         result_length+=copy_length;
-        if ((result_length+MaxTextExtent) >= allocated_length)
+        if ((result_length+MaxTextExtent) >= allocate_length)
           {
-            allocated_length+=copy_length;
+            allocate_length+=copy_length;
             result=(char *) ResizeMagickMemory(result,
-              (allocated_length+MaxTextExtent)*sizeof(*result));
+              (allocate_length+MaxTextExtent)*sizeof(*result));
             if (result == (char *) NULL)
               ThrowMagickFatalException(ResourceLimitFatalError,
-              "UnableToAcquireString",search);
+                "UnableToAcquireString",search);
           }
-        (void) CopyMagickString(destination,source,copy_length+1);
-        destination+=copy_length;
+        (void) CopyMagickString(result+destination_offset,source,copy_length+1);
+        destination_offset+=copy_length;
       }
       /*
         Copy replacement.
       */
       result_length+=replace_length;
-      if ((result_length+MaxTextExtent) >= allocated_length)
+      if ((result_length+MaxTextExtent) >= allocate_length)
         {
-          allocated_length+=replace_length;
+          allocate_length+=replace_length;
           result=(char *) ResizeMagickMemory(result,
-            (allocated_length+MaxTextExtent)*sizeof(*result));
+            (allocate_length+MaxTextExtent)*sizeof(*result));
           if (result == (char *) NULL)
             ThrowMagickFatalException(ResourceLimitFatalError,
               "UnableToAcquireString",search);
         }
-      (void) strcat(destination,replace);
-      destination+=replace_length;
+      (void) strcat(result+destination_offset,replace);
+      destination_offset+=replace_length;
       /*
         Find next match.
       */
@@ -1873,17 +2019,17 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
   */
   copy_length=strlen(source);
   result_length+=copy_length;
-  if ((result_length+MaxTextExtent) >= allocated_length)
+  if ((result_length+MaxTextExtent) >= allocate_length)
     {
-      allocated_length+=copy_length;
+      allocate_length+=copy_length;
       result=(char *) ResizeMagickMemory(result,
-        (allocated_length+MaxTextExtent)*sizeof(*result));
+        (allocate_length+MaxTextExtent)*sizeof(*result));
       if (result == (char *) NULL)
         ThrowMagickFatalException(ResourceLimitFatalError,
           "UnableToAcquireString",search);
     }
-  (void) strcat(destination,source);
-  *buffer=(char *) RelinquishMagickMemory(*buffer);
+  (void) strcat(result+destination_offset,source);
+  (void) RelinquishMagickMemory(*buffer);
   *buffer=result;
   return(MagickTrue);
 }
