@@ -1,5 +1,19 @@
 /*
-  ImageMagick Application Programming Interface declarations.
+  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  dedicated to making software imaging solutions freely available.
+  
+  You may not use this file except in compliance with the License.
+  obtain a copy of the License at
+  
+    http://www.imagemagick.org/script/license.php
+  
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
+
+  ImageMagick private application programming interface declarations.
 */
 #ifndef _MAGICK_STUDIO_H
 #define _MAGICK_STUDIO_H
@@ -34,9 +48,9 @@ extern "C" {
 #if !defined(_MAGICK_CONFIG_H)
 # define _MAGICK_CONFIG_H
 # if !defined(vms) && !defined(macintosh)
-#  include "magick/magick_config.h"
+#  include "magick/magick-config.h"
 # else
-#  include "magick_config.h"
+#  include "magick-config.h"
 # endif
 # if defined(__cplusplus) || defined(c_plusplus)
 #  undef inline
@@ -111,14 +125,8 @@ extern "C" {
 # define MagickGlobal
 #endif
 
-#if defined(__cplusplus) || defined(c_plusplus)
-# define storage_class  c_class
-#else
-# define storage_class  class
-#endif
-
 #define MagickSignature  0xabacadabUL
-#define MaxTextExtent  ((size_t) 4096)
+#define MaxTextExtent  4096
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -161,9 +169,22 @@ extern size_t
   strlcpy(char *,const char *,size_t);
 #endif
 
-#if defined(HAVE_VSNPRINTF) && !defined(HAVE_VSNPRINTF_PROTOTYPE)
-extern int
-  vsnprintf(char *,size_t,const char *,va_list);
+#if !defined(magick_attribute)
+#  if !defined(__GNUC__)
+#    define magick_attribute(x)  /* nothing */
+#  else
+#    define magick_attribute  __attribute__
+#  endif
+#endif
+
+#if !defined(magick_unused)
+#  if defined(__GNUC__)
+#     define magick_unused(x)  magick_unused_ ## x __attribute__((unused))
+#  elif defined(__LCLINT__)
+#    define magick_unused(x) /*@unused@*/ x
+#  else
+#    define magick_unused(x) x
+#  endif
 #endif
 
 #if defined(__WINDOWS__) || defined(POSIX)
@@ -197,7 +218,7 @@ extern int
 # if !defined(S_ISREG)
 #  define S_ISREG(mode) (((mode) & S_IFMT) == S_IFREG)
 # endif
-# include "magick/ImageMagick.h"
+# include "magick/magick-type.h"
 # if !defined(__WINDOWS__)
 #  include <sys/time.h>
 # if defined(HAVE_SYS_TIMES_H)
@@ -217,7 +238,7 @@ extern int
 #  endif
 #  include <unix.h>
 # endif
-# include "magick/ImageMagick.h"
+# include "magick/magick-type.h"
 #endif
 
 #if defined(S_IRUSR) && defined(S_IWUSR)
@@ -229,7 +250,7 @@ extern int
 #endif
 
 #if defined(__WINDOWS__)
-# include "magick/nt_base.h"
+# include "magick/nt-base.h"
 #endif
 #if defined(macintosh)
 # include "magick/mac.h"
@@ -336,7 +357,7 @@ extern int
 # undef BuildMagickModules
 # define BuildMagickModules
 #endif
-
+
 /*
   I/O defines.
 */
@@ -348,6 +369,14 @@ extern int
 #define MagickTell(file) tell(file)
 #endif
 
+/*
+  Magick defines.
+*/
+#define AbsoluteValue(x)  ((x) < 0 ? -(x) : (x))
+#define Max(x,y)  (((x) > (y)) ? (x) : (y))
+#define Min(x,y)  (((x) < (y)) ? (x) : (y))
+#define Swap(x,y) ((x)^=(y), (y)^=(x), (x)^=(y))
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
