@@ -17,13 +17,13 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -43,7 +43,8 @@
 #include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/color.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/geometry.h"
 #include "magick/list.h"
 #include "magick/log.h"
@@ -52,6 +53,7 @@
 #include "magick/resource_.h"
 #include "magick/semaphore.h"
 #include "magick/signature.h"
+#include "magick/statistic.h"
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
@@ -98,7 +100,7 @@ MagickExport MagickBooleanType AcquireUniqueFilename(char *path)
   file=AcquireUniqueFileResource(path);
   if (file == -1)
     return(MagickFalse);
-  (void) close(file);
+  file=close(file)-1;
   return(MagickTrue);
 }
 
@@ -137,7 +139,7 @@ MagickExport void AppendImageFormat(const char *format,char *filename)
 
   assert(format != (char *) NULL);
   assert(filename != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),filename);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   if ((*format == '\0') || (*filename == '\0'))
     return;
   if (LocaleCompare(filename,"-") == 0)
@@ -199,7 +201,7 @@ MagickExport unsigned char *Base64Decode(const char *source,size_t *length)
   unsigned char
     *decode;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(source != (char *) NULL);
   assert(length != (size_t *) NULL);
   *length=0;
@@ -355,7 +357,7 @@ MagickExport char *Base64Encode(const unsigned char *blob,
     max_length,
     remainder;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(blob != (const unsigned char *) NULL);
   assert(blob_length != 0);
   assert(encode_length != (size_t *) NULL);
@@ -463,9 +465,6 @@ MagickExport void ExpandFilename(char *path)
   char
     expanded_path[MaxTextExtent];
 
-  register char
-    *p;
-
   if (path == (char *) NULL)
     return;
   if (*path != '~')
@@ -473,12 +472,15 @@ MagickExport void ExpandFilename(char *path)
   (void) CopyMagickString(expanded_path,path,MaxTextExtent);
   if ((*(path+1) == *DirectorySeparator) || (*(path+1) == '\0'))
     {
+      register const char
+        *p;
+
       /*
         Substitute ~ with $HOME.
       */
-      p=(char *) getenv("HOME");
-      if (p == (char *) NULL)
-        p=(char *) ".";
+      p=getenv("HOME");
+      if (p == (const char *) NULL)
+        p=".";
       (void) CopyMagickString(expanded_path,p,MaxTextExtent);
       (void) ConcatenateMagickString(expanded_path,path+1,MaxTextExtent);
     }
@@ -488,6 +490,9 @@ MagickExport void ExpandFilename(char *path)
       char
         username[MaxTextExtent];
 
+      register char
+        *p;
+
       struct passwd
         *entry;
 
@@ -554,7 +559,8 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
     **vector;
 
   long
-    count;
+    count,
+    parameters;
 
   register long
     i,
@@ -566,7 +572,7 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
   /*
     Allocate argument vector.
   */
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(argc != (int *) NULL);
   assert(argv != (char ***) NULL);
   for (i=1; i < (long) *argc; i++)
@@ -585,15 +591,20 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
   {
     option=(*argv)[i];
     vector[count++]=AcquireString(option);
-    if ((LocaleNCompare("+profile",option,8) == 0) ||
-        (LocaleNCompare("-format",option,7) == 0) ||
-        (LocaleNCompare("+define",option,7) == 0))
+    parameters=ParseMagickOption(MagickCommandOptions,MagickFalse,option);
+    if (parameters > 0)
       {
-        i++;
-        if (i == (long) *argc)
-          continue;
-        option=(*argv)[i];
-        vector[count++]=AcquireString(option);
+        /*
+          Do not expand command option parameters.
+        */
+        for (j=0; j < parameters; j++)
+        {
+          i++;
+          if (i == (long) *argc)
+            break;
+          option=(*argv)[i];
+          vector[count++]=AcquireString(option);
+        }
         continue;
       }
     if ((*option == '"') || (*option == '\''))
@@ -726,9 +737,19 @@ MagickExport void FormatSize(const MagickSizeType size,char *format)
       (void) FormatMagickString(format,MaxTextExtent,"%.2fgb",length);
       break;
     }
+    case 4:
+    {
+      (void) FormatMagickString(format,MaxTextExtent,"%.2ftb",length);
+      break;
+    }
+    case 5:
+    {
+      (void) FormatMagickString(format,MaxTextExtent,"%.2fpb",length);
+      break;
+    }
     default:
     {
-      (void) FormatMagickString(format,MaxTextExtent,"%.3ftb",length);
+      (void) FormatMagickString(format,MaxTextExtent,"%.3feb",length);
       break;
     }
   }
@@ -760,6 +781,7 @@ MagickExport void FormatSize(const MagickSizeType size,char *format)
 MagickExport MagickBooleanType GetExecutionPath(char *path)
 {
   *path='\0';
+  (void) getcwd(path,MaxTextExtent);
 #if defined(__WINDOWS__)
   return(NTGetExecutionPath(path));
 #endif
@@ -772,17 +794,12 @@ MagickExport MagickBooleanType GetExecutionPath(char *path)
     if (execution_path != (const char *) NULL)
       {
         if (*execution_path != *DirectorySeparator)
-          {
-            (void) getcwd(path,MaxTextExtent);
-            (void) ConcatenateMagickString(path,"/",MaxTextExtent);
-          }
+          (void) ConcatenateMagickString(path,"/",MaxTextExtent);
         (void) ConcatenateMagickString(path,execution_path,MaxTextExtent);
-        if (IsAccessible(path) != MagickFalse)
-          return(MagickTrue);
       }
   }
 #endif
-  return(MagickFalse);
+  return(MagickTrue);
 }
 
 /*
@@ -825,7 +842,7 @@ MagickExport void GetPathComponent(const char *path,PathType type,
     *p;
 
   assert(path != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),path);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",path);
   assert(component != (char *) NULL);
   if (*path == '\0')
     {
@@ -834,7 +851,7 @@ MagickExport void GetPathComponent(const char *path,PathType type,
     }
   (void) CopyMagickString(component,path,MaxTextExtent);
   *magick='\0';
-  if (IsAccessible(path) == MagickFalse)
+  if ((IsDirectory(path) < 0) && (IsAccessible(path) == MagickFalse))
     for (p=component; *p != '\0'; p++)
       if (*p == ':')
         {
@@ -887,8 +904,12 @@ MagickExport void GetPathComponent(const char *path,PathType type,
     case RootPath:
     {
       for (p=component+(strlen(component)-1); p > component; p--)
+      {
+        if (IsBasenameSeparator(*p))
+          break;
         if (*p == '.')
           break;
+      }
       if (*p == '.')
         *p='\0';
       break;
@@ -1076,12 +1097,12 @@ extern "C" {
 
 static int FileCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+  p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
@@ -1108,11 +1129,11 @@ MagickExport char **ListFiles(const char *directory,const char *pattern,
     Open directory.
   */
   assert(directory != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),directory);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",directory);
   assert(pattern != (const char *) NULL);
   assert(number_entries != (unsigned long *) NULL);
   *number_entries=0;
-  current_directory=opendir((char *) directory);
+  current_directory=opendir(directory);
   if (current_directory == (DIR *) NULL)
     return((char **) NULL);
   /*
@@ -1260,10 +1281,12 @@ MagickExport int SystemCommand(const MagickBooleanType verbose,
   int
     status;
 
+  if (verbose != MagickFalse)
+    (void) fputs(command,stdout);
 #if defined(POSIX)
   status=system(command);
 #elif defined(__WINDOWS__)
-  status=NTSystemComman(command);
+  status=NTSystemCommand(command);
 #elif defined(macintosh)
   status=MACSystemCommand(command);
 #elif defined(vms)
@@ -1271,9 +1294,17 @@ MagickExport int SystemCommand(const MagickBooleanType verbose,
 #else
 #  error No suitable system() method.
 #endif
-  if (verbose != MagickFalse)
-    MagickError(DelegateError,command,status < 0 ? strerror(errno) :
-      (char *) NULL);
+  if (status < 0)
+    {
+      ExceptionInfo
+        exception;
+
+      GetExceptionInfo(&exception);
+      (void) ThrowMagickException(&exception,GetMagickModule(),DelegateError,
+        "`%s': %s",command,strerror(errno));
+      CatchException(&exception);
+      DestroyExceptionInfo(&exception);
+    }
   return(status);
 }
 
@@ -1334,13 +1365,14 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((embed_text == (const char *) NULL) || (*embed_text == '\0'))
     return((char *) NULL);
   text=(char *) embed_text;
-  if ((*text == '@') && (IsAccessible(text+1) != MagickFalse))
+  if ((*text == '@') &&
+      ((*(text+1) == '-') || (IsAccessible(text+1) != MagickFalse)))
     {
-      text=(char *) FileToBlob(text+1,&length,&image->exception);
+      text=FileToString(text+1,~0,&image->exception);
       if (text == (char *) NULL)
         return((char *) NULL);
     }
@@ -1349,7 +1381,7 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
   */
   text_info=CloneImageInfo(image_info);
   translate_text=AcquireString(text);
-  length=MaxTextExtent;
+  length=strlen(text)+MaxTextExtent;
   p=text;
   for (q=translate_text; *p != '\0'; p++)
   {
@@ -1378,6 +1410,12 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
         p++;
         continue;
       }
+    if (*p == '\\')
+      {
+        p++;
+        *q++=(*p++);
+        continue;
+      }
     if (*p != '%')
       {
         *q++=(*p);
@@ -1386,12 +1424,35 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
     p++;
     switch (*p)
     {
+      case 'O':
+      {
+        (void) FormatMagickString(q,MaxTextExtent,"%+ld%+ld",image->page.x,
+          image->page.y);
+        while (*q != '\0')
+          q++;
+        break;
+      }
+      case 'P':
+      {
+        (void) FormatMagickString(q,MaxTextExtent,"%lux%lu",image->page.width,
+          image->page.height);
+        while (*q != '\0')
+          q++;
+        break;
+      }
       case 'b':
       {
         char
           format[MaxTextExtent];
 
-        FormatSize(GetBlobSize(image),format);
+        MagickSizeType
+          length;
+
+        length=GetBlobSize(image);
+        (void) FormatMagickString(format,MaxTextExtent,"%lu",
+          (unsigned long) length);
+        if (length != (MagickSizeType) ((size_t) length))
+          FormatSize(length,format);
         (void) ConcatenateMagickString(q,format,MaxTextExtent);
         while (*q != '\0')
           q++;
@@ -1399,8 +1460,8 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
       }
       case 'c':
       {
-        attribute=GetImageAttribute(image,"comment");
-        if (attribute == (ImageAttribute *) NULL)
+        attribute=GetImageAttribute(image,"Comment");
+        if (attribute == (const ImageAttribute *) NULL)
           break;
         (void) CopyMagickString(q,attribute->value,MaxTextExtent);
         q+=strlen(attribute->value);
@@ -1460,7 +1521,7 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
       case 'h':
       {
         (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          image->magick_rows != 0 ? image->magick_rows : image->rows);
+          image->rows != 0 ? image->rows : image->magick_rows);
         while (*q != '\0')
           q++;
         break;
@@ -1482,7 +1543,7 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
       case 'l':
       {
         attribute=GetImageAttribute(image,"label");
-        if (attribute == (ImageAttribute *) NULL)
+        if (attribute == (const ImageAttribute *) NULL)
           break;
         (void) CopyMagickString(q,attribute->value,MaxTextExtent);
         q+=strlen(attribute->value);
@@ -1517,8 +1578,8 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
           page;
 
         p=image;
-        for (page=1; p->previous != (Image *) NULL; page++)
-          p=p->previous;
+        for (page=1; GetPreviousImageInList(p) != (Image *) NULL; page++)
+          p=GetPreviousImageInList(p);
         (void) FormatMagickString(q,MaxTextExtent,"%lu",page);
         while (*q != '\0')
           q++;
@@ -1526,16 +1587,22 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
       }
       case 'q':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%u",QuantumDepth);
+        (void) FormatMagickString(q,MaxTextExtent,"%lu",image->depth);
         while (*q != '\0')
           q++;
         break;
       }
       case 'r':
       {
+        ColorspaceType
+          colorspace;
+
+        colorspace=image->colorspace;
+        if (IsGrayImage(image,&image->exception) != MagickFalse)
+          colorspace=GRAYColorspace;
         (void) FormatMagickString(q,MaxTextExtent,"%s%s%s",
           MagickOptionToMnemonic(MagickClassOptions,(long) image->storage_class),
-          MagickOptionToMnemonic(MagickColorspaceOptions,(long) image->colorspace),
+          MagickOptionToMnemonic(MagickColorspaceOptions,(long) colorspace),
           image->matte != MagickFalse ? "Matte" : "");
         while (*q != '\0')
           q++;
@@ -1560,7 +1627,7 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
       case 'w':
       {
         (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          image->magick_columns != 0 ? image->magick_columns : image->columns);
+          image->columns != 0 ? image->columns : image->magick_columns);
         while (*q != '\0')
           q++;
         break;
@@ -1621,7 +1688,7 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
             break;
           }
         attribute=GetImageInfoAttribute(text_info,image,key);
-        if (attribute == (ImageAttribute *) NULL)
+        if (attribute == (const ImageAttribute *) NULL)
           break;
         offset=(MagickOffsetType) strlen(attribute->value);
         if ((size_t) (q-translate_text+offset+1) >= length)
@@ -1652,8 +1719,8 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
       case '#':
       {
         (void) SignatureImage(image);
-        attribute=GetImageAttribute(image,"signature");
-        if (attribute == (ImageAttribute *) NULL)
+        attribute=GetImageAttribute(image,"Signature");
+        if (attribute == (const ImageAttribute *) NULL)
           break;
         (void) CopyMagickString(q,attribute->value,MaxTextExtent);
         q+=strlen(attribute->value);
@@ -1673,8 +1740,10 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
     }
   }
   *q='\0';
-  DestroyImageInfo(text_info);
+  text_info=DestroyImageInfo(text_info);
   if (text != (char *) embed_text)
     text=(char *) RelinquishMagickMemory(text);
+  (void) SubstituteString(&translate_text,"&lt;","<");
+  (void) SubstituteString(&translate_text,"&gt;",">");
   return(translate_text);
 }
