@@ -12,36 +12,24 @@
 %                                                                             %
 %                     VMS Utility Methods for ImageMagick                     %
 %                                                                             %
-%                                                                             %
 %                               Software Design                               %
 %                                 John Cristy                                 %
 %                                October 1994                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  E. I. du Pont de Nemours and Company be liable for any claim, damages or   %
-%  other liability, whether in an action of contract, tort or otherwise,      %
-%  arising from, out of or in connection with ImageMagick or the use or other %
-%  dealings in ImageMagick.                                                   %
-%                                                                             %
-%  Except as contained in this notice, the name of the E. I. du Pont de       %
-%  Nemours and Company shall not be used in advertising or otherwise to       %
-%  promote the sale, use or other dealings in ImageMagick without prior       %
-%  written authorization from the E. I. du Pont de Nemours and Company.       %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -50,15 +38,16 @@
 %
 */
 
-#if defined(vms) && !defined(_AXP_)
+#if defined(vms)
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "defines.h"
-#include "vms.h"
+#include "magick/studio.h"
+#include "magick/string_.h"
+#include "magick/memory_.h"
+#include "magick/vms.h"
 
-#if !defined(__VMS_VER) || (__VMS_VER < 70000000)
+#if !defined(_AXP_) && (!defined(__VMS_VER) || (__VMS_VER < 70000000))
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -70,8 +59,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method closedir closes the named directory stream and frees the DIR
-%  structure.
+%  closedir() closes the named directory stream and frees the DIR structure.
 %
 %  The format of the closedir method is:
 %
@@ -84,9 +72,11 @@
 */
 void closedir(DIR *directory)
 {
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(directory != (DIR *) NULL);
-  free(directory->pattern);
-  free((char *) directory);
+  directory->pattern=(char *) RelinquishMagickMemory(directory->pattern);
+  directory=(char *) RelinquishMagickMemory(directory);
 }
 
 /*
@@ -94,14 +84,14 @@ void closedir(DIR *directory)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   o p e n e d i r                                                           %
+%   o p e n d i r                                                             %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method opendir opens the directory named by filename and associates
-%  a directory stream with it.
+%  opendir() opens the directory named by filename and associates a directory
+%  stream with it.
 %
 %  The format of the opendir method is:
 %
@@ -121,7 +111,7 @@ DIR *opendir(char *name)
   /*
     Allocate memory for handle and the pattern.
   */
-  directory=(DIR *) AllocateMemory(sizeof(*directory));
+  directory=(DIR *) AcquireMagickMemory(sizeof(DIR));
   if (directory == (DIR *) NULL)
     {
       errno=ENOMEM;
@@ -129,18 +119,17 @@ DIR *opendir(char *name)
     }
   if (strcmp(".",name) == 0)
     name="";
-  directory->pattern=AllocateMemory((unsigned int) (strlen(name)
-    +sizeof("*.*")+1));
+  directory->pattern=(char *) AcquireMagickMemory(strlen(name)+sizeof("*.*")+1);
   if (directory->pattern == (char *) NULL)
     {
-      FreeMemory((char *) directory);
+      directory=(char *) RelinquishMagickMemory(directory);
       errno=ENOMEM;
       return(NULL);
     }
   /*
     Initialize descriptor.
   */
-  (void) sprintf(directory->pattern,"%.1024s*.*",name);
+  (void) FormatMagickString(directory->pattern,MaxTextExtent,"%s*.*",name);
   directory->context=0;
   directory->pat.dsc$a_pointer=directory->pattern;
   directory->pat.dsc$w_length=strlen(directory->pattern);
@@ -160,9 +149,8 @@ DIR *opendir(char *name)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method readdir returns a pointer to a structure representing the
-%  directory entry at the current position in the directory stream to
-%  which entry refers.
+%  readdir() returns a pointer to a structure representing the directory entry
+%  at the current position in the directory stream to which entry refers.
 %
 %  The format of the readdir
 %
@@ -206,22 +194,77 @@ struct dirent *readdir(DIR *directory)
   */
   buffer[sizeof(buffer)-1]='\0';
   for (p=buffer; *p; p++)
-    if (isupper(*p))
+    if (isupper((unsigned char) *p))
       *p=tolower(*p);
   /*
     Skip any directory component and just copy the name.
   */
   p=buffer;
-  while (!isspace(*p))
+  while (isspace((unsigned char) *p) == 0)
     p++;
   *p='\0';
   p=strchr(buffer,']');
   if (p)
-    (void) strcpy(directory->entry.d_name,p+1);
+    (void) CopyMagickString(directory->entry.d_name,p+1,MaxTextExtent);
   else
-    (void) strcpy(directory->entry.d_name,buffer);
+    (void) CopyMagickString(directory->entry.d_name,buffer,MaxTextExtent);
   directory->entry.d_namlen=strlen(directory->entry.d_name);
   return(&directory->entry);
 }
-#endif
-#endif
+#endif /* !defined(_AXP_) && (!defined(__VMS_VER) || (__VMS_VER < 70000000)) */
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   I s M a g i c k C o n f l i c t                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  VMSIsMagickConflict() returns true if the image format conflicts with a
+%  logical drive (.e.g. SYS$SCRATCH:).
+%
+%  Contributed by Forrest Cahoon (forrest@wiredaemons.com)
+%
+%  The format of the VMSIsMagickConflict method is:
+%
+%      MagickBooleanType VMSIsMagickConflict(const char *magick)
+%
+%  A description of each parameter follows:
+%
+%    o magick: Specifies the image format.
+%
+%
+*/
+MagickExport MagickBooleanType VMSIsMagickConflict(const char *magick)
+{
+  ile3
+    item_list[2];
+
+  int
+    device_class,
+    status;
+
+  struct dsc$descriptor_s
+    device;
+
+  assert(magick != (char *) NULL);
+  device.dsc$w_length=strlen(magick);
+  device.dsc$a_pointer=(char *) magick;
+  device.dsc$b_class=DSC$K_CLASS_S;
+  device.dsc$b_dtype=DSC$K_DTYPE_T;
+  item_list[0].ile3$w_length=sizeof(device_class);
+  item_list[0].ile3$w_code=DVI$_DEVCLASS;
+  item_list[0].ile3$ps_bufaddr=&device_class;
+  item_list[0].ile3$ps_retlen_addr=NULL;
+  (void) ResetMagickMemory(&item_list[1],0,sizeof(item_list[1]));
+  status=sys$getdviw(0,0,&device,&item_list,0,0,0,0);
+  if ((status == SS$_NONLOCAL) ||
+      ((status & 0x01) && (device_class & (DC$_DISK | DC$_TAPE))))
+    return(MagickTrue);
+  return(MagickFalse);
+}
+#endif /* defined(vms) */
